/*
 * arch/ppc/common/bootinfo.c
 *
 * General bootinfo record utilities
 * Author: MontaVista Software <source@mvista.com>
 *
 * Copyright 2002 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/types.h>
#include <asm/bootinfo.h>

#include "nonstdio.h"

static struct bi_record * birec = NULL;

static struct bi_record *
__bootinfo_build(struct bi_record *rec, unsigned long tag, unsigned long size,
		 void *data)
{
	/* set the tag */
	rec->tag = tag;

	/* if the caller has any data, copy it */
	if (size)
		memcpy(rec->data, (char *)data, size);

	/* set the record size */
	rec->size = sizeof(struct bi_record) + size;

	/* advance to the next available space */
	rec = (struct bi_record *)((unsigned long)rec + rec->size);

	return rec;
}

struct bi_record *
bootinfo_init(unsigned long image_size) {

	struct bi_record *rec = bootinfo_addr(image_size);

	/* save start of birec area */
	birec = rec;

	/* create an empty list */
	rec = __bootinfo_build(rec, BI_FIRST, 0, NULL);
	(void) __bootinfo_build(rec, BI_LAST, 0, NULL);

	/* return birec start to caller */
	return birec;
}

void
bootinfo_append(unsigned long tag, unsigned long size, void * data)
{

	struct bi_record *rec = birec;

	/* paranoia */
	if ((rec == NULL) || (rec->tag != BI_FIRST))
		return;

	/* find the last entry in the list */
	while (rec->tag != BI_LAST)
		rec = (struct bi_record *)((ulong)rec + rec->size);

	/* overlay BI_LAST record with new one and tag on a new BI_LAST */
	rec = __bootinfo_build(rec, tag, size, data);
	(void) __bootinfo_build(rec, BI_LAST, 0, NULL);
}
