/*
 * arch/ppc/platforms/arctic2.c  Platform setup for the IBM Arctic-2 reference platform
 *					with the Subzero core card and Beech personality card
 * 				      Based on beech.c by Bishop Brock 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Copyright (C) 2002, International Business Machines Corporation
 * All Rights Reserved.
 *
 * Ken Inoue 
 * IBM Thomas J. Watson Research Center
 * keninoue@us.ibm.com
 *
 * David Gibson
 * IBM Ozlabs, Canberra, Australia
 * arctic@gibson.dropbear.id.au
 */

#include <linux/blk.h>
#include <linux/config.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/param.h>
#include <linux/rtc.h>
#include <linux/string.h>

#include <asm/delay.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/page.h>
#include <asm/processor.h>
#include <asm/system.h>
#include <asm/time.h>

#include <platforms/arctic2.h>

/* Virtual address of the PCCF macro, which needs to be ioremap()ed
 * and initialized by the board setup code. */
volatile u16 *pccf_4xx_macro_vaddr;
unsigned long pccf_4xx_io_base;
unsigned long pccf_4xx_mem_base;
EXPORT_SYMBOL(pccf_4xx_macro_vaddr);
EXPORT_SYMBOL(pccf_4xx_io_base);
EXPORT_SYMBOL(pccf_4xx_mem_base);

void __init
board_setup_arch(void)
{
}

/* All Arctic-2 core physical memory resources are direct-mapped at boot time. 
*  Need to change that. */

void __init
board_io_mapping(void)
{
	/* FIXME: yuck.  Need to get rid of these */
	io_block_mapping(ARCTIC2_FPGA8_VADDR, ARCTIC2_FPGA8_PADDR,
			 ARCTIC2_FPGA8_SIZE, _PAGE_IO);
	io_block_mapping(ARCTIC2_FPGA16_VADDR, ARCTIC2_FPGA16_PADDR,
			 ARCTIC2_FPGA16_SIZE, _PAGE_IO);
	ibm405lp_setup_pccf(&pccf_4xx_macro_vaddr, &pccf_4xx_io_base,
			    &pccf_4xx_mem_base);
}

void __init
board_setup_irq(void)
{
}

void __init
board_init(void)
{
#ifdef CONFIG_PPC_RTC
	ppc_md.time_init = ibm405lp_time_init;
	ppc_md.set_rtc_time = ibm405lp_set_rtc_time;
	ppc_md.get_rtc_time = ibm405lp_get_rtc_time;
#endif

	/* Set up the EBC, then Disable the LCD controller, which may have been
	   left on by the BIOS. */

	subzero_core_ebc_setup();

	/* Configure the Arctic-II specific EBC banks */

	/* Bank 1: 16-bit FPGA peripherals (ethernet data, SDIO, USB, DOC)
	 * 1MB, RW, 16-bit at 0xf1000000-0xf10fffff */
	/* The access parameters are programmed assuming a 33Mhz EBC
	   clock, which is true for nearly all the operating points we
	   have defined:
	   	BME=0, TWT=5, CSN=0, OEN=1, WBN=1, WBF=1 TH=4
		RE=1, SOR=0, BEM=0, PEN=0
	 */
	mtdcri(DCRN_EBC0, BnAP(1), 0x02815900);
	mtdcri(DCRN_EBC0, BnCR(1), ARCTIC2_FPGA16_PADDR | 0x1a000);

	/* Bank 2: 8-bit FPGA peripherals (switch/control, ethernet regs, TCPA)
	 * 1MB, RW, 8-bit at 0xf8000000-0xf80fffff */
	mtdcri(DCRN_EBC0, BnAP(2), 0x02815580);
	mtdcri(DCRN_EBC0, BnCR(2), ARCTIC2_FPGA8_PADDR | 0x18000);

	mtdcri(DCRN_LCD0, DER, 0);
}

/*
 * Local variables:
 * c-basic-offset: 8
 * End: */
