/*
 * arch/ppc/platforms/arctic2.h   Platform definitions for the IBM Arctic-II
 *				based on beech.h by Bishop Brock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Copyright (C) 2002, International Business Machines Corporation
 * All Rights Reserved.
 *
 * Ken Inoue 
 * IBM Thomas J. Watson Research Center
 * keninoue@us.ibm.com
 *
 * David Gibson
 * IBM Ozlabs, Canberra, Australia
 * arctic@gibson.dropbear.id.au
 * 
 */

#ifdef __KERNEL__
#ifndef __ASM_ARCTIC2_H__
#define __ASM_ARCTIC2_H__

#include <platforms/subzero.h>

#ifndef __ASSEMBLY__

#define ARCTIC2_FPGA8_PADDR	(0xf8000000)
#define ARCTIC2_FPGA8_VADDR	ARCTIC2_FPGA8_PADDR
#define ARCTIC2_FPGA8_SIZE	(768*1024)

#define ARCTIC2_FPGA16_PADDR	(0xf9000000)
#define ARCTIC2_FPGA16_VADDR	ARCTIC2_FPGA16_PADDR
#define ARCTIC2_FPGA16_SIZE	(1024*1024)

/* Arctic II uses the internal clock for UART. Note that the OPB
   frequency must be more than 2x the UART clock frequency. At OPB
   frequencies less than this the serial port will not function due to
   the way that SerClk is sampled.  We use 11.1111MHz as the frequency
   because it can be generated from a wide range of OPB frequencies we
   want to use. */

#define PPC4xx_SERCLK_FREQ 11111111

#define BASE_BAUD (PPC4xx_SERCLK_FREQ / 16)

#define RTC_DVBITS	RTC_DVBITS_33KHZ	/* 33kHz RTC */

#define PPC4xx_MACHINE_NAME	"IBM Arctic II"

#include <asm/pccf_4xx.h>
#define _IO_BASE		(pccf_4xx_io_base)
#define _ISA_MEM_BASE		(pccf_4xx_mem_base)

#endif /* !__ASSEMBLY__ */
#endif /* __ASM_ARCTIC2_H__ */
#endif /* __KERNEL__ */
