/*
 * arch/ppc/platforms/cxd4707_pci.c
 *
 * CXD4707 ES1.x PLB-PCI bridge
 *
 *  Copyright 2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/threads.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <linux/snsc_nlver.h>
#include <asm/io.h>
#include <asm/ibm4xx.h>
#include <asm/mmu.h>
#include <asm/ocp.h>
#include <asm/system.h>
#include <asm/todc.h>

#define NAME "CXD4707 PCI-2.2: " 

/* 
 * PCI-2.2 releated setups for CXD4707
 */

static void *pcil0_reg_base;
static void *pcic0_addr, *pcic0_data;

#undef CXD4707_PCI_DEBUG


#define PCIL0_WRITEL(value, offset) \
	(writel(value, (u32)pcil0_reg_base+offset))

#define PCIL0_READL(offset) \
	(readl((u32)pcil0_reg_base+offset))


#define pcic0_cfg_read(val, addr, type, op)	*val = op((type)(addr))
#define pcic0_cfg_write(val, addr, type, op)	op((type *)(addr), (val))
#define PCIC0_OP(rw, size, type, op, mask)				\
void								\
pcic0_##rw##_##size(type val, int offset) 				\
{									\
	writel( (1<<31) | (offset & 0xfc), pcic0_addr);			\
	pcic0_cfg_##rw(val, pcic0_data + (offset & mask), type, op); \
}

PCIC0_OP(read,  byte,  u8 *,  in_8,    3)
PCIC0_OP(read,  word,  u16 *, in_le16, 2)
PCIC0_OP(read,  dword, u32 *, in_le32, 0)
PCIC0_OP(write, byte,  u8,    out_8,    3)
PCIC0_OP(write, word,  u16,   out_le16, 2)
PCIC0_OP(write, dword, u32,   out_le32, 0)


#define PCIC0_WRITEL(value, offset) \
	( \
	 writel(value, (u32)pcic0_data))

#define PCIC0_READL(offset) \
	(writel(((1<<31) | (offset & 0xfc)), (u32)pcic0_addr); \
	 readl(value, (u32)pcic0_data))

#ifdef CXD4707_PCI_DEBUG
void
cxd4707_dump_pci(void)
{
	u_int32_t dword;
	u_int16_t word;
	u_int8_t  byte;

	pcic0_read_word(&word, 0x00);
	printk("  PCI Vendor ID                  = 0x%04x\n", word);
	pcic0_read_word(&word, 0x02);
	printk("  PCI Device ID                  = 0x%04x\n", word);
	pcic0_read_word(&word, 0x04);
	printk("  PCI Command                    = 0x%04x\n", word);
	pcic0_read_word(&word, 0x06);
	printk("  PCI Status                     = 0x%04x\n", word);
	pcic0_read_byte(&byte, 0x08);
	printk("  PCI Revision ID                = 0x%02x\n", byte);
	pcic0_read_dword(&dword, 0x08);
	printk("  PCI Class                      = 0x%06x\n", (dword>>8) & 0xffffff);
	pcic0_read_byte(&byte, 0x0c);
	printk("  PCI Cache Line Size            = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x0d);
	printk("  PCI Latency Timer              = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x0e);
	printk("  PCI Header Type                = 0x%02x\n", byte);
	pcic0_read_dword(&dword, 0x0f);
	printk("  PCI Built-in Self Test Control = 0x%08x\n", dword);
	pcic0_read_dword(&dword, 0X14);
	printk("  PCI PTM1 BAR                   = 0x%08x\n", dword);
	pcic0_read_dword(&dword, 0x18);
	printk("  PCI PTM2 BAR                   = 0x%08x\n", dword);
	pcic0_read_word(&word, 0x2c);
	printk("  PCI Subsystem Vencor ID        = 0x%02x\n", word);
	pcic0_read_word(&word, 0x2e);
	printk("  PCI Subsystem ID               = 0x%04x\n", word);
	pcic0_read_byte(&byte, 0x34);
	printk("  PCI Capabilities Pointer       = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x3c);
	printk("  PCI Interrupt Line             = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x3d);
	printk("  PCI interrupt Pin              = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x3e);
	printk("  PCI Minimum Grant              = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x3f);
	printk("  PCI Maximum Latency            = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x44);
	printk("  PCI Interrupt Control/Status   = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x48);
	printk("  Error Enable                   = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x49);
	printk("  Error Status                   = 0x%02x\n", byte);
	pcic0_read_word(&word, 0x4a);
	printk("  Bridge Option1                 = 0x%04x\n", word);
	pcic0_read_dword(&dword, 0x4c);
	printk("  PLB Slave Error Syndrome0      = 0x%08x\n", dword);
	pcic0_read_dword(&dword, 0x50);
	printk("  PLB Slave Error Syndrome1      = 0x%08x\n", dword);
	pcic0_read_dword(&dword, 0x54);
	printk("  PLB Slave Error Address        = 0x%08x\n", dword);
	pcic0_read_byte(&byte, 0x58);
	printk("  Capability Identifier          = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x59);
	printk("  Next Item Pointer              = 0x%02x\n", byte);
	pcic0_read_word(&word, 0x5a);
	printk("  Power Management Capabilities  = 0x%04x\n", word);
	pcic0_read_word(&word, 0x5c);
	printk("  Power Management Ctrl/Status   = 0x%04x\n", word);
	pcic0_read_byte(&byte, 0x5e);
	printk("  PMCSR PCI-PCI Bridge Sup. Ext. = 0x%02x\n", byte);
	pcic0_read_byte(&byte, 0x5f);
	printk("  Data                           = 0x%02x\n", byte);
	pcic0_read_word(&word, 0x60);
	printk("  Bridge Option2                 = 0x%04x\n", word);
	pcic0_read_byte(&byte, 0x64);
	printk("  PM State Change Req.           = 0x%02x\n", byte);

}
#endif /* CXD4707_PCI_DEBUG */

void __init
cxd4707_setup_pci(void)
{
	extern unsigned long total_memory; /* init.c */
	pcil0_reg_base	= ioremap64(CXD4707_PCI_REG_BASE, CXD4707_PCI_REG_SIZE);
	pcic0_addr	= ioremap64(CXD4707_PCI_CFGA_PLB32, 4);
	pcic0_data	= ioremap64(CXD4707_PCI_CFGD_PLB32, 4);

	printk(NAME "pcil0_reg_base = 0x%08x\n", (unsigned int)pcil0_reg_base);
	printk(NAME "pcic0_addr = 0x%08x, pcic0_data = 0x%08x\n", 
	       (unsigned int)pcic0_addr, (unsigned int)pcic0_data);

	/* Disable all windows */
	PCIL0_WRITEL(0, PCIL0_PMM0LA);
	PCIL0_WRITEL(0, PCIL0_PMM0MA);
	PCIL0_WRITEL(0, PCIL0_PMM0PCILA);
	PCIL0_WRITEL(0, PCIL0_PMM0PCIHA);

	PCIL0_WRITEL(0, PCIL0_PMM1LA);
	PCIL0_WRITEL(0, PCIL0_PMM1MA);
	PCIL0_WRITEL(0, PCIL0_PMM1PCILA);
	PCIL0_WRITEL(0, PCIL0_PMM1PCIHA);

	PCIL0_WRITEL(0, PCIL0_PMM2LA);
	PCIL0_WRITEL(0, PCIL0_PMM2MA);
	PCIL0_WRITEL(0, PCIL0_PMM2PCILA);
	PCIL0_WRITEL(0, PCIL0_PMM2PCIHA);

	PCIL0_WRITEL(0, PCIL0_PTM1MS);
	PCIL0_WRITEL(0, PCIL0_PTM2MS);

	PCIL0_WRITEL(0, PCIL0_PTM1LA);
	PCIL0_WRITEL(0, PCIL0_PTM2LA);

	/* Setup 1GB PLB->PCI outbound mem window (0_8000_0000->8000_0000) */
	PCIL0_WRITEL((u_int32_t)PCI_PLB_BASE, PCIL0_PMM0LA); 	/* PLB addr. */
	PCIL0_WRITEL(0x80000000, PCIL0_PMM0PCILA);	/* lo addr. 0x8000_0000 */
	PCIL0_WRITEL(0x00000000, PCIL0_PMM0PCIHA);	/* hi addr. 0x0000_0000 */
	PCIL0_WRITEL(0xc0000001, PCIL0_PMM0MA);		/* 1GB, non-prefech, enable */

	/* Setup PCI->PLB inbound memory window at 0, enable MSIs */
	PCIL0_WRITEL(0x00000000, PCIL0_PTM1LA);	/* PCI addr.0x0000_0000 */
	PCIL0_WRITEL((~(cxd4707_find_end_of_memory()-1)|0x1), PCIL0_PTM1MS);

	eieio();

	/* Setup PCI->PLB for PCI side */
	pcic0_write_dword(0x00000001, PCIC0_PTM1BAR);

	pcic0_write_word(0x1014,	PCIC0_VENDID); /* IBM */
	pcic0_write_word(0x0156, 	PCIC0_DEVID); /* PCI Bridge for 405GP */
	pcic0_write_dword(0x06000001,	PCIC0_REVID); /* Rev and Class */

#ifdef CXD4707_PCI_DEBUG
	cxd4707_dump_pci();
#endif
}

