/*
 * cxd4707_pci.h
 *
 *  PCI module definitions for CXD4707 ES1.x
 *
 *  Copyright 2003,2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CXD4707_PCI_H__
#define __CXD4707_PCI_H__

#include <linux/config.h>
#include <platforms/ibm440.h>

#define CXD4707_PCI_LOWER_IO	0x00000000
#define CXD4707_PCI_UPPER_IO	0x0000ffff
#define CXD4707_PCI_LOWER_MEM	0x80000000
#define CXD4707_PCI_UPPER_MEM	0xbfffffff

#define CXD4707_PCI_CFGA_PLB32	0xdec00000
#define CXD4707_PCI_CFGD_PLB32	0xdec00004

#define CXD4707_PCI_IO_BASE	0xd8000000
#define CXD4707_PCI_IO_SIZE	0x00010000
#define CXD4707_PCI_MEM_OFFSET	0x00000000

#define CXD4707_PCI_REG_BASE	0xDF400000
#define CXD4707_PCI_REG_SIZE	0x40

#define CXD4707_PCI_ADDR	CXD4707_PCI_REG_BASE


// Bridge registers
#define PCIL0_PMM0LA		0x000	// Range 0 PLB Local address
#define PCIL0_PMM0MA		0x004	//             Mask/Attribute
#define PCIL0_PMM0PCILA		0x008	//         PCI Low address
#define PCIL0_PMM0PCIHA		0x00C	//             High address

#define PCIL0_PMM1LA		0x010	// Range 1 PLB Local address
#define PCIL0_PMM1MA		0x014	//             Mask/Attribute
#define PCIL0_PMM1PCILA		0x018	//         PCI Low address
#define PCIL0_PMM1PCIHA		0x01C	//             High address

#define PCIL0_PMM2LA		0x020	// Range 2 PLB Local address
#define PCIL0_PMM2MA		0x024	//             Mask/Attribute
#define PCIL0_PMM2PCILA		0x028	//         PCI Low address
#define PCIL0_PMM2PCIHA		0x02C	//             High address

#define PCIL0_PTM1MS		0x030	// Region 1 Size/Attribute
#define PCIL0_PTM1LA		0x034	//          Local address
#define PCIL0_PTM2MS		0x038	// Region 2 Size/Attribute
#define PCIL0_PTM2LA		0x03C	//          Local address


// PCI Registers accessed indirectly
#define PCIC0_VENDID		0x00	// R/W PCI Vendor ID 
#define PCIC0_DEVID		0x02	// R/W PCI Device ID 
#define PCIC0_CMD		0x04	// R/W PCI Command 
#define PCIC0_STATUS		0x06	// R/W PCI Status 
#define PCIC0_REVID		0x08	// R/W PCI Revision ID 
#define PCIC0_ICLS		0x09	// R/W PCI Class 
#define PCIC0_CACHELS		0x0c	// R   PCI Cache Line Size 
#define PCIC0_LATTIM		0x0d	// R/W PCI Latency Timer 
#define PCIC0_HDTYPE		0x0e	// R   PCI Header Type  
#define PCIC0_BISTC		0x0f	// R   PCI Built-in Self Test Control 
#define PCIC0_PTM1BAR		0x14	// R/W PCI PTM1 BAR 
#define PCIC0_PTM2BAR		0x18	// R/W PCI PTM2 BAR 
#define PCIC0_SUBSYSVENID	0x2c	// R/W PCI Subsystem Vencor ID 
#define PCIC0_SUBSYSID		0x2e	// R/W PCI Subsystem ID 
#define PCIC0_CAP		0x34	// R   PCI Capabilities Pointer 
#define PCIC0_INTLN		0x3c	// R/W PCI Interrupt Line 
#define PCIC0_INTPN		0x3d	// R   PCI interrupt Pin 
#define PCIC0_MINGNT		0x3e	// R   PCI Minimum Grant 
#define PCIC0_MAXLTNCY		0x3f	// R   PCI Maximum Latency 
#define PCIC0_ICS		0x44	// R/W PCI Interrupt Control/Status 
#define PCIC0_ERREN		0x48	// R/W Error Enable 
#define PCIC0_ERRSTS		0x49	// R/W Error Status 
#define PCIC0_BRDGOPT1		0x4a	// R/W Bridge Option1 
#define PCIC0_PLBSESR0		0x4c	// R/W PLB Slave Error Syndrome0 
#define PCIC0_PLBSESR1		0x50	// R/W PLB Slave Error Syndrome1 
#define PCIC0_PLBSEAR		0x54	// R/W PLB Slave Error Address 
#define PCIC0_CAPID		0x58	// R   Capability Identifier 
#define PCIC0_NEXTITEMPTR	0x59	// R   Next Item Pointer 
#define PCIC0_PMC		0x5a	// R   Power Management Capabilities 
#define PCIC0_PMCSR		0x5c	// R/W Power Management Control/Status 
#define PCIC0_PMCSRBSE		0x5e	// R   PMCSR PCI-to-PCI Bridge Support Extensions 
#define PCIC0_DATA		0x5f	// R   Data 
#define PCIC0_BRDGOPT2		0x60	// R/W Bridge Option2 
#define PCIC0_PMSCRR		0x64	// R/W Power Management State Change Request 



#endif // __CXD4707_PCI_H__ */
