/*
 * arch/ppc/platforms/cxd4707_pcix.c
 *
 * CXD4707 ES2.x PLB-PCIX bridge
 *
 *  Copyright 2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/threads.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <linux/bitops.h>
#include <linux/snsc_nlver.h>
#include <asm/io.h>
#include <asm/ibm4xx.h>
#include <asm/mmu.h>
#include <asm/ocp.h>
#include <asm/system.h>
#include <asm/todc.h>
#include <asm/machdep.h>

#define NAME "CXD4707 PCI-X: " 

/* 
 * PCI-X releated setups for CXD4707-ES2.0 and later
 */

static void *pcix0_reg_base;

#undef CXD4707_PCI_DEBUG

#define PCIX_WRITEW(value, offset) \
	(writew(value, (u16)pcix0_reg_base+offset))

#define PCIX_WRITEL(value, offset) \
	(writel(value, (u32)pcix0_reg_base+offset))

#ifdef CXD4707_PCI_DEBUG
void
cxd4707_dump_pci(void)
{
	/* n/a */
}
#endif /* CXD4707_PCI_DEBUG */

void __init
cxd4707_setup_pci(void)
{
	unsigned int memsize;

	if ( ppc_md.progress )
		ppc_md.progress("cxd4707_setup_pci() (PCI-X): in", 0x3eab);

	pcix0_reg_base	= ioremap64(CXD4707_PCI_REG_BASE, CXD4707_PCI_REG_SIZE);
	printk(NAME "pcix_reg_base = 0x%08x\n", (unsigned int)pcix0_reg_base);

	/* Disable all windows */
	PCIX_WRITEL(0, PCIX0_POM0SA);
	PCIX_WRITEL(0, PCIX0_POM1SA);
	PCIX_WRITEL(0, PCIX0_POM2SA);
	PCIX_WRITEL(0, PCIX0_PIM0SA);
	PCIX_WRITEL(0, PCIX0_PIM1SA);
	PCIX_WRITEL(0, PCIX0_PIM2SA);

	/* Setup 1GB PLB->PCI outbound mem window (0_8000_0000->8000_0000) */
	PCIX_WRITEL(0x00000000, PCIX0_POM0LAH);
	PCIX_WRITEL(0x80000000, PCIX0_POM0LAL);
	PCIX_WRITEL(0x00000000, PCIX0_POM0PCIAH);
	PCIX_WRITEL(0x80000000, PCIX0_POM0PCIAL);
	PCIX_WRITEL(0xc0000001, PCIX0_POM0SA);

	/* Setup PCI->PLB inbound memory window at 0, enable MSIs */
	PCIX_WRITEL(0x00000000, PCIX0_PIM0LAH);
	PCIX_WRITEL(0x00000000, PCIX0_PIM0LAL);

	/* PIMx must be power of two */
	memsize = cxd4707_find_end_of_memory();
	if (hweight32(memsize) > 1) 
		memsize = 1 << fls(memsize);
	PCIX_WRITEL((~(memsize - 1) | 0x3),  PCIX0_PIM0SA); /* prefetch, enable */

	eieio();

#if 0
	PCIX_WRITEW(0x1014,	PCIX0_VENDID); /* IBM */
	PCIX_WRITEW(0x01ef, 	PCIX0_DEVID); /* PCI Bridge for 440GP */
	PCIX_WRITEL(0x06000001,	PCIX0_REVID); /* Rev and Class */
#endif

#ifdef CXD4707_PCI_DEBUG
	cxd4707_dump_pci();
#endif
}

