/*
 * arch/ppc/platforms/cxd4707_pcixerr.c
 *
 * CXD4707 ES2.x PLB-PCIX bridge SERR#/PERR# handler driver
 *
 *  Copyright 2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/threads.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <linux/snsc_nlver.h>
#include <asm/io.h>
#include <asm/ibm4xx.h>
#include <asm/mmu.h>
#include <asm/ocp.h>
#include <asm/system.h>
#include <asm/todc.h>
#include <asm/machdep.h>

/* ======================================================================= *
 * Constants                                                               *
 * ======================================================================= */

#define NAME "PCI-X Error Handler : " 
#define DEBUG 0

/* ======================================================================= *
 * Debug print function                                                    *
 *   set permutations of:                                                  *
 *     0 : disables all debug output                                       *
 *     1 : Print output. (Default)                                         *
 *     2 : Verbose, dump registers                                         *
 *     4 : Print entry of functions.                                       *
 * ======================================================================= */

#if DEBUG & 1
#define PRINTK(format, args...)                    \
          printk("%s():%d " format ,            \
		 __FUNCTION__ , __LINE__ , ##args)
#else
#define PRINTK(format, args...) 
#endif

#if DEBUG & 2
#define PRINTK2(format, args...)                   \
          printk("%s():%d " format ,            \
		 __FUNCTION__ , __LINE__ , ##args)
#else
#define PRINTK2(format, args...) 
#endif

#if DEBUG & 4
#define ENTRY()                                    \
          printk("%s():%d\n",                   \
		 __FUNCTION__ , __LINE__ )
#else
#define ENTRY() 
#endif

#define ERROR(format, args...)                     \
          printk(KERN_ERR "%s(): " format ,     \
		 __FUNCTION__ , ##args)


/* ======================================================================= *
 * Global variable                                                         *
 * ======================================================================= */
static void *pcix0_reg_base;
static u_int32_t pcix_erren_save; /* save old value of PCICx_ERREN */
int intr_count = 0;

/* ======================================================================= *
 * Defines and macros                                                      *
 * ======================================================================= */

/* 
 * PCI-X read/write routines
 */

#define PCIX_WRITEL(value, offset) \
	(writel(value, (u32)pcix0_reg_base+offset))

#define PCIX_READL(offset) \
	(readl((u32)pcix0_reg_base+offset))

#define PCIX_WRITEW(value, offset) \
	(writew(value, (u32)pcix0_reg_base+offset))

#define PCIX_READW(offset) \
	(readw((u32)pcix0_reg_base+offset))


/* 
 * PCI-X Error INT
 */
#define PCIX_ERR_INT	59


/* 
 * Bit assignments for :
 *   -. PCICx_CMD, PCICx_STATUS, 
 *   -. PCICx_ERREN, PCICx_ERRSTS
 *   -. PCIX_STS
 */
#define SET_BIT(x,s)	(((x) & 0x1) << (s))
#define GET_BIT(v,s)	(((v) >> (s)) & 0x1)

/* PCICx_CMD */
#define PER_BIT		6
#define PER(x)		SET_BIT(x, PER_BIT)

/* PCICx_STATUS */
#define DEPE_BIT	15
#define SSE_BIT		14
#define RMA_BIT		13
#define RTA_BIT		12
#define STA_BIT		11
#define DPE_BIT		8

#define DEPE(x)		SET_BIT(x, DEPE_BIT)
#define SSE(x)		SET_BIT(x, SSE_BIT)
#define RMA(x)		SET_BIT(x, RMA_BIT)
#define RTA(x)		SET_BIT(x, RTA_BIT)
#define STA(x)		SET_BIT(x, STA_BIT)
#define DPE(x)		SET_BIT(x, DPE_BIT)

#define IS_DEPE(v)	GET_BIT(v, DEPE_BIT)
#define IS_SSE(v)	GET_BIT(v, SSE_BIT)
#define IS_RMA(v)	GET_BIT(v, RMA_BIT)
#define IS_RTA(v)	GET_BIT(v, RTA_BIT)
#define IS_STA(v)	GET_BIT(v, STA_BIT)
#define IS_DPE(v)	GET_BIT(v, DPE_BIT)

/* PCICx_ERREN */
#define REL_BIT		18
#define MRWE_BIT	17
#define EAE_BIT		16
#define SDTE_BIT	15
#define USEE_BIT	14
#define RERE_BIT	12
#define WERE_BIT	11
#define MQRE_BIT	10
#define TRE_BIT		9
#define PO_BIT		8
#define IDPE_BIT	7
#define MAEE_BIT	5
#define TAEE_BIT	4
#define STEE_BIT	3
#define APEE_BIT	2
#define DRTE_BIT	1
#define SEE_BIT		0

#define REL(x)		SET_BIT(x, REL_BIT)
#define MRWE(x)		SET_BIT(x, MRWE_BIT)
#define EAE(x)		SET_BIT(x, EAE_BIT)
#define SDTE(x)		SET_BIT(x, SDTE_BIT)
#define USEE(x)		SET_BIT(x, USEE_BIT)
#define RERE(x)		SET_BIT(x, RERE_BIT)
#define WERE(x)		SET_BIT(x, WERE_BIT)
#define MQRE(x)		SET_BIT(x, MQRE_BIT)
#define TRE(x)		SET_BIT(x, TRE_BIT)
#define PO(x)		SET_BIT(x, PO_BIT)
#define IDPE(x)		SET_BIT(x, IDPE_BIT)
#define MAEE(x)		SET_BIT(x, MAEE_BIT)
#define TAEE(x)		SET_BIT(x, TAEE_BIT)
#define STEE(x)		SET_BIT(x, STEE_BIT)
#define APEE(x)		SET_BIT(x, APEE_BIT)
#define DRTE(x)		SET_BIT(x, DRTE_BIT)
#define SEE(x)		SET_BIT(x, SEE_BIT)

/* PCICx_ERRSTS */
#define MRWE_BIT	17
#define EIS_BIT		16
#define SDTE_BIT	15
#define MRER_BIT	12
#define MWER_BIT	11
#define MQR_BIT		10
#define TO_BIT		9
#define IPED_BIT	7
#define APED_BIT	2
#define DTE_BIT		1
#define PSEE_BIT	0

#define MRWE(x)		SET_BIT(x, MRWE_BIT)
#define EIS(x)		SET_BIT(x, EIS_BIT)
#define SDTE(x)		SET_BIT(x, SDTE_BIT)
#define MRER(x)		SET_BIT(x, MRER_BIT)
#define MWER(x)		SET_BIT(x, MWER_BIT)
#define MQR(x)		SET_BIT(x, MQR_BIT)
#define TO(x)		SET_BIT(x, TO_BIT)
#define IPED(x)		SET_BIT(x, IPED_BIT)
#define APED(x)		SET_BIT(x, APED_BIT)
#define DTE(x)		SET_BIT(x, DTE_BIT)
#define PSEE(x)		SET_BIT(x, PSEE_BIT)

#define IS_MRWE(v)	GET_BIT(v, MRWE_BIT)
#define IS_EIS(v)	GET_BIT(v, EIS_BIT)
#define IS_SDTE(v)	GET_BIT(v, SDTE_BIT)
#define IS_MRER(v)	GET_BIT(v, MRER_BIT)
#define IS_MWER(v)	GET_BIT(v, MWER_BIT)
#define IS_MQR(v)	GET_BIT(v, MQR_BIT)
#define IS_TO(v)	GET_BIT(v, TO_BIT)
#define IS_IPED(v)	GET_BIT(v, IPED_BIT)
#define IS_APED(v)	GET_BIT(v, APED_BIT)
#define IS_DTE(v)	GET_BIT(v, DTE_BIT)
#define IS_PSEE(v)	GET_BIT(v, PSEE_BIT)

/* PCIX0_STS */
#define SPEM_BIT	29
#define USC_BIT		19

#define SPEM(x)		SET_BIT(x, SPEM_BIT)
#define USC(x)		SET_BIT(x, USC_BIT)

#define IS_SPEM(v)	GET_BIT(v, SPEM_BIT)
#define IS_USC(v)	GET_BIT(v, USC_BIT)


/* 
 * Error Enable definitions set to PCICx_ERREN
 */
#define SET_ERREN							\
	(REL(1)		| 	/* Route Error Locally as ERROR_INT */  \
	 MRWE(0)	| 	/* MRdErr/MWrErr assert on errors   */	\
	 EAE(1))	 	/* ERROR_INT enable                 */

#define SET_SERR							\
	SEE(1)			/* Receive PCI_SERR# as Error       */

#define SET_PARITY							\
	(PO(1)		| 	/* Parity Error (all types)         */	\
	IDPE(1))		/* Inbound Write Data Parity        */	\

#define SET_PLB_ERR							\
	(RERE(1)	| 	/* MRdErr Receive                   */	\
	WERE(1)		| 	/* MWrErr Receive                   */	\
	MQRE(1))	 	/* MIRQ                             */ 

#define SET_PCIX							\
	(USEE(1)	| 	/* Unexpected Split Completion      */	\
	STEE(1)		| 	/* Split Transaction Error (PCI-X)  */	\
	SDTE(1))	 	/* Outbout Split Timeout (PCI-X)    */ 

#define SET_PCIX_PARITY							\
	APEE(1)		 	/* Addr/Attrib Parity Error (PCI-X) */ 

#define SET_PCI_ABORT							\
	(MAEE(1)	| 	/* Master Abort Error               */	\
	TAEE(1))	 	/* Target Abort Error               */ 

#define SET_TIMEOUT							\
	(TRE(1)		| 	/* Timeout Receive                  */	\
	DRTE(1))	 	/* Delayed Read Discard Timer       */ 


#define CLEAR_PCI_STAT     \
	(DEPE(1) | SSE(1) | RMA(1) | RTA(1) | STA(1) | DPE(1))

#define CLEAR_PCIX_STAT    \
	(SPEM(1) | USC(1))

#define CLEAR_PCI_ESTAT \
	(MRWE(1) | EIS(1) | SDTE(1) | MRER(1) | MWER(1)	|  \
	 MQR(1) | TO(1)	| IPED(1) | APED(1) | DTE(1) | PSEE(1))

#define STOP_ERR \
	(REL(0) | EAE(0))


/* ======================================================================= *
 * Linux driver interface                                                  *
 * ======================================================================= */

static int
pcix_err_print(void)
{
	int retval = 1;		/* 0 = if recoverble  */
	u_int32_t xstat, estat, besr, bearl, bearh;
	u_int16_t stat;

	stat  = PCIX_READW(PCIX0_STATUS); /* PCI */
	xstat = PCIX_READL(PCIX0_STS);    /* PCI-X */
	estat = PCIX_READL(PCIX0_ERRSTS); /* Error Status */
	besr  = PCIX_READL(PCIX0_PLBBESR);
	bearl = PCIX_READL(PCIX0_PLBBEARL);
	bearh = PCIX_READL(PCIX0_PLBBEARH);

	/* 
	 * Errors while the bridge is PLB Master
	 */
	if (IS_MRER(estat))
		printk("While PLB Master, deteced PLB Read MErr.\n");

	if (IS_MWER(estat))
		printk("While PLB Master, deteced PLB Write MErr.\n");

	if (IS_MQR(estat))
		printk("While PLB Master, deteced PLB MIRQ.\n");

	/* 
	 * Errors while the bridge is PCI Initiator
	 */
	if (IS_RMA(stat))
		printk("While PCI Initiator, detected Master Abort.\n");

	if (IS_STA(stat))
		printk("While PCI Initiator, detected Target Abort.\n");

	if (IS_DPE(stat))
		printk("While PCI Initiator, detected PCI Data Parity Error.\n");

	/* 
	 * Errors while the bridge is PCI Initiator/Target
	 */
	if (IS_DEPE(stat))
		printk("While PCI Initiator, detected PCI Bus Parity Error.\n");

	/* 
	 * Errors while the bridge is PCI-X Initiator
	 */
	if (IS_SPEM(xstat))
		printk("While PCI-X Initiator, received Split Transaction Error.\n");

	/* 
	 * Errors while the bridge is PCI Target
	 */
	if (IS_APED(estat))
		printk("While PCI target, detected Address/Attribyte Parity Error.\n");

	if (IS_DTE(estat))
		printk("While PCI target, Delayed Read Discard Timer expired.\n");

	/* 
	 * misc
	 */
	if (IS_USC(xstat))
		printk("Unexpected Split Completion Received on PCI-X\n");

	if (IS_SDTE(estat))
		printk("PCI-X Outbout Split Discard Timer Expired\n");

	if (IS_PSEE(estat))
		printk("PCI_SERR# is asserted\n");


	/* dump registers */
	printk("ERRSTS:0x%08x PLB_BESR:0x%08x PLB_BEARL:0x%08x PLB_BEARH:0x%08x\n", 
	       estat, besr, bearl, bearh);

	return retval;
}

static void
pcix_err_clear(void)
{
	PCIX_WRITEW(CLEAR_PCI_STAT,  PCIX0_STATUS);
	PCIX_WRITEL(CLEAR_PCIX_STAT, PCIX0_STS);
	PCIX_WRITEL(CLEAR_PCI_ESTAT, PCIX0_ERRSTS);
}

void
cxd4707_pcixerr(int irq, void *dev_id, struct pt_regs *regs)
{
	extern void die(const char *,struct pt_regs *,long);

	int retval;

	retval = pcix_err_print();
	pcix_err_clear();

#ifdef CONFIG_PCIXERR_DIE
	if (retval)
		die("PCI Error Detected.", regs, 0xdeadbeef);
#else
	printk("PCI Error Detected.");
#endif

	return;
}

static int
cxd4707_pcixerr_init(void)
{
	if (request_irq(PCIX_ERR_INT, cxd4707_pcixerr, SA_INTERRUPT, 
			"PCI ERR", &intr_count)) {
		printk("request_irq() failed\n");
		return -1;
	}

	pcix0_reg_base	= ioremap64(CXD4707_PCI_REG_BASE, CXD4707_PCI_REG_SIZE);
	PRINTK2("pcix_reg_base   = 0x%08x\n", (unsigned int)pcix0_reg_base);

	pcix_erren_save = PCIX_READL(PCIX0_ERREN);
	PRINTK2("pcix_erren_save = 0x%08x\n", pcix_erren_save);

#if defined(CONFIG_PCIXERR_PARITY) || defined(CONFIG_PCIXERR_PCIXPARITY)
	PCIX_WRITEW(PCIX_READW(PCIX0_CMD)|PER(1), PCIX0_CMD);
#endif
	PCIX_WRITEL(
#ifdef CONFIG_PCIXERR_SERR
		SET_SERR	| 
#endif
#ifdef CONFIG_PCIXERR_PARITY
		SET_PARITY	| 
#endif
#ifdef CONFIG_PCIXERR_PLBERR
		SET_PLB_ERR	|
#endif
#ifdef CONFIG_PCIXERR_PCIXERR
		SET_PCIX	|
#endif
#ifdef CONFIG_PCIXERR_PCIXPARITY
		SET_PCIX_PARITY	|
#endif
#ifdef CONFIG_PCIXERR_ABORT
		SET_PCI_ABORT	|
#endif
#ifdef CONFIG_PCIXERR_TIMEOUT
		SET_TIMEOUT	|
#endif
		SET_ERREN, PCIX0_ERREN);


	PRINTK2("pcix_erren_new  = 0x%08x\n", PCIX_READL(PCIX0_ERREN));

	PRINTK(NAME ": loaded. (" __TIME__ ")\n");
	return 0;
}

static void
cxd4707_pcixerr_exit(void)
{
	PCIX_WRITEL(pcix_erren_save, PCIX0_ERREN);
	iounmap(pcix0_reg_base);
	free_irq(PCIX_ERR_INT, &intr_count);

	PRINTK(NAME ": unloaded.\n");
	return ;
}

module_init(cxd4707_pcixerr_init);
module_exit(cxd4707_pcixerr_exit);

MODULE_AUTHOR("Sony HENC");
MODULE_DESCRIPTION("IBM PowerPC 4xx PCI-X Error handler");
MODULE_LICENSE("GPL");

