/* 
 * Register description for CXD-4707
 *
 *  Copyright 2003,2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/io.h>

/* 
 * Register tables
 */
static ppc4xx_reg_t cxd4707_processor[] =
{
	{"PVR",			0x11f},
	{"PIR",			0x11e},
	{"CCR0",		0x3b3},
	{"CCR1",		0x378},
	{"RSTCFG",		0x39b},
	{"SPRG0",		0x110},
	{"SPRG1",		0x111},
	{"SPRG2",		0x112},
	{"SPRG3",		0x113},
	{"SPRG4R",		0x104},
	{"SPRG5R",		0x105},
	{"SPRG6R",		0x106},
	{"SPRG7R",		0x107},
	{"SPRG4W",		0x114},
	{"SPRG5W",		0x115},
	{"SPRG6W",		0x116},
	{"SPRG7W",		0x117},
	{"XER",			0x001},
	{"LR",			0x008},
	{"CTR",			0x009},
	{"USPRG0",		0x100},
	{NULL}
};

/* Proceccor core cache related */
static ppc4xx_reg_t cxd4707_processor_cache[] =
{
	/* Cache Control */
	{"DNV0",		0x390},
	{"DNV1",		0x391},
	{"DNV2",		0x392},
	{"DNV3",		0x393},
	{"DTV0",		0x394},
	{"DTV1",		0x395},
	{"DTV2",		0x396},
	{"DTV3",		0x397},
	{"DVLIM",		0x398},
	{"INV0",		0x370},
	{"INV1",		0x371},
	{"INV2",		0x372},
	{"INV3",		0x373},
	{"ITV0",		0x374},
	{"ITV1",		0x375},
	{"ITV2",		0x376},
	{"ITV3",		0x377},
	{"IVLIM",		0x399},

	/* Cache debug */
	{"DCDBTRL",		0x39c},
	{"DCDBTRH",		0x39d},
	{"ICDBTRL",		0x39e},
	{"ICDBTRH",		0x39f},
	{"ICDBDR",		0x3d3},
	{NULL}
};

/* Proceccor core interrupt processing related */
static ppc4xx_reg_t cxd4707_processor_intr[] =
{
	{"ESR",			0x03e},
	{"DEAR",		0x03d},
	{"SRR0",		0x01a},
	{"SRR1",		0x01b},
	{"MCSRR0",		0x23a},
	{"MCSRR1",		0x23b},
	{"MCSR",		0x23c},
	{"CSRR0",		0x03a},
	{"CSRR1",		0x03b},
	{"IVOR0",		0x190},
	{"IVOR1",		0x191},
	{"IVOR2",		0x192},
	{"IVOR3",		0x193},
	{"IVOR4",		0x194},
	{"IVOR5",		0x195},
	{"IVOR6",		0x196},
	{"IVOR7",		0x197},
	{"IVOR8",		0x198},
	{"IVOR9",		0x199},
	{"IVOR10",		0x19a},
	{"IVOR11",		0x19b},
	{"IVOR12",		0x19c},
	{"IVOR13",		0x19d},
	{"IVOR14",		0x19e},
	{"IVOR15",		0x19f},
	{"IVPR",		0x03f},
	{NULL}
};

/* Proceccor core timer related */
static ppc4xx_reg_t cxd4707_processor_timer[] =
{
	{"DEC",			0x016},
	{"DECAR",		0x036},
	{"TBLR",		0x10c},
	{"TBUR",		0x10d},
	{"TBLW",		0x11c},
	{"TBUW",		0x11d},
	{"TCR",			0x154},
	{"TSR",			0x150},
	{NULL}
};

/* Proceccor core debug related */
static ppc4xx_reg_t cxd4707_processor_debug[] =
{
	{"DAC1",		0x13c},
	{"DAC2",		0x13d},
	{"DBCR0",		0x134},
	{"DBCR1",		0x135},
	{"DBCR2",		0x136},
	{"DBDR",		0x3f3},
	{"DBSR",		0x130},
	{"DVC1",		0x13e},
	{"DVC2",		0x13f},
	{"IAC1",		0x138},
	{"IAC2",		0x139},
	{"IAC3",		0x13a},
	{"IAC4",		0x13b},
	{NULL}
};

/* Proceccor core mmu related */
static ppc4xx_reg_t cxd4707_processor_mmu[] =
{
	{"PID",			0x030},
	{"MMUCR",		0x3b2},
	{NULL}
};

/* On chip bus */
static ppc4xx_reg_t cxd4707_opb0[] =
{
	{"OPB0_BCTRL",		0x0a8},
	{"OPB0_BSTAT",		0x0a9},
	{"OPB0_BEARL",		0x0aa},
	{"OPB0_BEARH",		0x0ab},
	{"OPB0_REVID",		0x0ac},
	{NULL}
};

static ppc4xx_reg_t cxd4707_opb1[] =
{
	{"OPB1_BCTRL",		0x078},
	{"OPB1_BSTAT",		0x079},
	{"OPB1_BEARL",		0x07a},
	{"OPB1_BEARH",		0x07b},
	{"OPB1_REVID",		0x07c},
	{NULL}
};

static ppc4xx_reg_t cxd4707_plbc[] =
{
	{"PLBC_PCBI",		0x080},
	{"PLBC_P0ACR",		0x081},
	{"PLBC_P0ESRL",		0x082},
	{"PLBC_P0ESRH",		0x083},
	{"PLBC_P0BEARL",	0x084},
	{"PLBC_P0BEARH",	0x085},
	{"PLBC_P0BESRL",	0x086},
	{"PLBC_P0BESRH",	0x087},
	{"PLBC_PCBC",		0x088},
	{"PLBC_P1ACR",		0x089},
	{"PLBC_P1ESRL",		0x08a},
	{"PLBC_P1ESRH",		0x08b},
	{"PLBC_P1BEARL",	0x08c},
	{"PLBC_P1BEARH",	0x08d},
	{"PLBC_P1BESRL",	0x08e},
	{"PLBC_P1BESRH",	0x08f},
	{NULL}
};

static ppc4xx_reg_t cxd4707_pob0[] =
{
	{"POB0_BESR0",		0x090},
	{"POB0_BESR0W",		0x091},
	{"POB0_BEARL",		0x092},
	{"POB0_BEARH",		0x093},
	{"POB0_BESR1",		0x094},
	{"POB0_BESR1W",		0x095},
	{"POB0_CONFG",		0x096},
	{"POB0_CONFGW",		0x097},
	{"POB0_LATENCY",	0x098},
	{"POB0_LATENCYW",	0x099},
	{"POB0_REVID",		0x09a},
	{NULL}
};

static ppc4xx_reg_t cxd4707_pob1[] =
{
	{"POB1_BESR0",		0x0f0},
	{"POB1_BESR0W",		0x0f1},
	{"POB1_BEARL",		0x0f2},
	{"POB1_BEARH",		0x0f3},
	{"POB1_BESR1",		0x0f4},
	{"POB1_BESR1W",		0x0f5},
	{"POB1_CONFG",		0x0f6},
	{"POB1_CONFGW",		0x0f7},
	{"POB1_LATENCY",	0x0f8},
	{"POB1_LATENCYW",	0x0f9},
	{"POB1_REVID",		0x0fa},
	{NULL}
};

/* Clocking, Power Management and Chip Control */
static ppc4xx_reg_t cxd4707_cpc[] =
{
	{"CPC0_SR",		0x0b0},
	{"CPC0_ER",		0x0b1},
	{"CPC0_FR",		0x0b2},
	{"CPC0_SYS0",		0x0e0},
	{"CPC0_SYS1",		0x0e1},
	{"CPC0_CUST0",		0x0e2},
	{"CPC0_CUST1",		0x0e3},
	{"CPC0_STRP0",		0x0e4},
	{"CPC0_STRP1",		0x0e5},
	{"CPC0_STRP2",		0x0e6},
	{"CPC0_STRP3",		0x0e7},
	{"CPC0_GPIO",		0x0e8},
	{"CPC0_PLB",		0x0e9},
	{"CPC0_CR1",		0x0ea},
	{"CPC0_CR0",		0x0eb},
	{"CPC0_MIRQ0",		0x0ec},
	{"CPC0_MIRQ1",		0x0ed},
	{"CPC0_JTAGID",		0x0ef},
	{NULL}
};

/* Universal Interrupt Controllers */
static ppc4xx_reg_t cxd4707_uic0[] =
{
	{"UIC0_SR",		0x0c0},
	{"UIC0_SRS",		0x0c1},
	{"UIC0_ER",		0x0c2},
	{"UIC0_CR",		0x0c3},
	{"UIC0_PR",		0x0c4},
	{"UIC0_TR",		0x0c5},
	{"UIC0_MSR",		0x0c6},
	{"UIC0_VR",		0x0c7},
	{"UIC0_VCR",		0x0c8},
	{NULL}
};

static ppc4xx_reg_t cxd4707_uic1[] =
{
	{"UIC1_SR",		0x0d0},
	{"UIC1_SRS",		0x0d1},
	{"UIC1_ER",		0x0d2},
	{"UIC1_CR",		0x0d3},
	{"UIC1_PR",		0x0d4},
	{"UIC1_TR",		0x0d5},
	{"UIC1_MSR",		0x0d6},
	{"UIC1_VR",		0x0d7},
	{"UIC1_VCR",		0x0d8},
	{NULL}
};

static ppc4xx_reg_t cxd4707_uic2[] =
{
	{"UIC2_SR",		0x0e0},
	{"UIC2_SRS",		0x0e1},
	{"UIC2_ER",		0x0e2},
	{"UIC2_CR",		0x0e3},
	{"UIC2_PR",		0x0e4},
	{"UIC2_TR",		0x0e5},
	{"UIC2_MSR",		0x0e6},
	{"UIC2_VR",		0x0e7},
	{"UIC2_VCR",		0x0e8},
	{NULL}
};

/* Direct Memory Access */
static ppc4xx_reg_t cxd4707_dma0[] =
{
	{"DMA0_CR0",		0x100},
	{"DMA0_CT0",		0x101},
	{"DMA0_SAH0",		0x102},
	{"DMA0_SAL0",		0x103},
	{"DMA0_DAH0",		0x104},
	{"DMA0_DAL0",		0x105},
	{"DMA0_SGH0",		0x106},
	{"DMA0_SGL0",		0x107},
	{"DMA0_CR1",		0x108},
	{"DMA0_CT1",		0x109},
	{"DMA0_SAH1",		0x10a},
	{"DMA0_SAL1",		0x10b},
	{"DMA0_DAH1",		0x10c},
	{"DMA0_DAL1",		0x10d},
	{"DMA0_SGH1",		0x10e},
	{"DMA0_SGL1",		0x10f},
	{"DMA0_CR2",		0x110},
	{"DMA0_CT2",		0x111},
	{"DMA0_SAH2",		0x112},
	{"DMA0_SAL2",		0x113},
	{"DMA0_DAH2",		0x114},
	{"DMA0_DAL2",		0x115},
	{"DMA0_SGH2",		0x116},
	{"DMA0_SGL2",		0x117},
	{"DMA0_CR3",		0x118},
	{"DMA0_CT3",		0x119},
	{"DMA0_SAH3",		0x11a},
	{"DMA0_SAL3",		0x11b},
	{"DMA0_DAH3",		0x11c},
	{"DMA0_DAL3",		0x11d},
	{"DMA0_SGH3",		0x11e},
	{"DMA0_SGL3",		0x11f},
	{"DMA0_SR",		0x120},
	{"DMA0_SGC",		0x123},
	{"DMA0_SLP",		0x125},
	{"DMA0_POL",		0x126},
	{NULL}
};

static ppc4xx_reg_t cxd4707_dma1[] =
{
	{"DMA1_CR0",		0x140},
	{"DMA1_CT0",		0x141},
	{"DMA1_SAH0",		0x142},
	{"DMA1_SAL0",		0x143},
	{"DMA1_DAH0",		0x144},
	{"DMA1_DAL0",		0x145},
	{"DMA1_SGH0",		0x146},
	{"DMA1_SGL0",		0x147},
	{"DMA1_CR1",		0x148},
	{"DMA1_CT1",		0x149},
	{"DMA1_SAH1",		0x14a},
	{"DMA1_SAL1",		0x14b},
	{"DMA1_DAH1",		0x14c},
	{"DMA1_DAL1",		0x14d},
	{"DMA1_SGH1",		0x14e},
	{"DMA1_SGL1",		0x14f},
	{"DMA1_CR2",		0x150},
	{"DMA1_CT2",		0x151},
	{"DMA1_SAH2",		0x152},
	{"DMA1_SAL2",		0x153},
	{"DMA1_DAH2",		0x154},
	{"DMA1_DAL2",		0x155},
	{"DMA1_SGH2",		0x156},
	{"DMA1_SGL2",		0x157},
	{"DMA1_CR3",		0x158},
	{"DMA1_CT3",		0x159},
	{"DMA1_SAH3",		0x15a},
	{"DMA1_SAL3",		0x15b},
	{"DMA1_DAH3",		0x15c},
	{"DMA1_DAL3",		0x15d},
	{"DMA1_SGH3",		0x15e},
	{"DMA1_SGL3",		0x15f},
	{"DMA1_SR",		0x160},
	{"DMA1_SGC",		0x163},
	{"DMA1_SLP",		0x165},
	{"DMA1_POL",		0x166},
	{NULL}
};

/* Memory Access Layer */
static ppc4xx_reg_t cxd4707_mal[] =
{
	{"MAL0_CFG",		0x180},
	{"MAL0_ESR",		0x181},
	{"MAL0_IER",		0x182},
	{"MAL0_TXCASR",		0x184},
	{"MAL0_TXCARR",		0x185},
	{"MAL0_TXEOBISR",	0x186},
	{"MAL0_TXDEIR",		0x187},
	{"MAL0_RXCASR",		0x190},
	{"MAL0_RXCARR",		0x191},
	{"MAL0_RXEOBISR",	0x192},
	{"MAL0_RXDEIR",		0x193},
	{"MAL0_TXCTP0R",	0x1a0},
	{"MAL0_RXCTP0R",	0x1c0},
	{"MAL0_RCBS0",		0x1e0},
	{NULL}
};

/* PLB to AHB bridge */
static ppc4xx_reg_t cxd4707_plb2ahb[] =
{
	{"SEAR_ADDRU",		0x200},
	{"SEAR_ADDRL",		0x204},
	{"SESR",		0x208},
	{"AHB_TOPADDR",		0x20c},
	{"AHB_BTMADDR",		0x210},
	{"PLBAHB_REVID",	0x214},
	{"ABH_ATTRIB",		0x218},
	{NULL}
};

/* RTC */
static ppc4xx_reg_t cxd4707_rtc[] =
{
	{"RTC_SEC",		0x300},
	{"RTC_MIN",		0x302},
	{"RTC_MINALM",		0x303},
	{"RTC_HRS",		0x304},
	{"RTC_HRSALM",		0x305},
	{"RTC_DAY",		0x306},
	{"RTC_DATE",		0x307},
	{"RTC_MON",		0x308},
	{"RTC_YEAR",		0x309},
	{"RTC_REGA",		0x30a},
	{"RTC_REGB",		0x30b},
	{"RTC_REGC",		0x30c},
	{"RTC_REGD",		0x30d},
	{"RTC_CENT",		0x30e},
	{"RTC_ARR_0x30f",	0x30f},
	{"RTC_ARR_0x310",	0x310},
	{"RTC_ARR_0x311",	0x311},
	{"RTC_ARR_0x312",	0x312},
	{"RTC_ARR_0x313",	0x313},
	{"RTC_ARR_0x314",	0x314},
	{"RTC_ARR_0x315",	0x315},
	{"RTC_ARR_0x316",	0x316},
	{"RTC_ARR_0x317",	0x317},
	{"RTC_ARR_0x318",	0x318},
	{"RTC_ARR_0x319",	0x319},
	{"RTC_ARR_0x31a",	0x31a},
	{"RTC_ARR_0x31b",	0x31b},
	{"RTC_ARR_0x31c",	0x31c},
	{"RTC_ARR_0x31d",	0x31d},
	{"RTC_ARR_0x31e",	0x31e},
	{"RTC_ARR_0x31f",	0x31f},
	{"RTC_ARR_0x320",	0x320},
	{"RTC_ARR_0x321",	0x321},
	{"RTC_ARR_0x322",	0x322},
	{"RTC_ARR_0x323",	0x323},
	{"RTC_ARR_0x324",	0x324},
	{"RTC_ARR_0x325",	0x325},
	{"RTC_ARR_0x326",	0x326},
	{"RTC_ARR_0x327",	0x327},
	{"RTC_ARR_0x328",	0x328},
	{"RTC_ARR_0x329",	0x329},
	{"RTC_ARR_0x32a",	0x32a},
	{"RTC_ARR_0x32b",	0x32b},
	{"RTC_ARR_0x32c",	0x32c},
	{"RTC_ARR_0x32d",	0x32d},
	{"RTC_ARR_0x32e",	0x32e},
	{"RTC_ARR_0x32f",	0x32f},
	{"RTC_ARR_0x330",	0x330},
	{"RTC_ARR_0x331",	0x331},
	{"RTC_ARR_0x332",	0x332},
	{"RTC_ARR_0x333",	0x333},
	{"RTC_ARR_0x334",	0x334},
	{"RTC_ARR_0x335",	0x335},
	{"RTC_ARR_0x336",	0x336},
	{"RTC_ARR_0x337",	0x337},
	{"RTC_ARR_0x338",	0x338},
	{"RTC_ARR_0x339",	0x339},
	{"RTC_ARR_0x33a",	0x33a},
	{"RTC_ARR_0x33b",	0x33b},
	{"RTC_ARR_0x33c",	0x33c},
	{"RTC_ARR_0x33d",	0x33d},
	{"RTC_ARR_0x33e",	0x33e},
	{"RTC_ARR_0x33f",	0x33f},
	{"RTC_ARR_0x340",	0x340},
	{"RTC_ARR_0x341",	0x341},
	{"RTC_ARR_0x342",	0x342},
	{"RTC_ARR_0x343",	0x343},
	{"RTC_ARR_0x344",	0x344},
	{"RTC_ARR_0x345",	0x345},
	{"RTC_ARR_0x346",	0x346},
	{"RTC_ARR_0x347",	0x347},
	{"RTC_ARR_0x348",	0x348},
	{"RTC_ARR_0x349",	0x349},
	{"RTC_ARR_0x34a",	0x34a},
	{"RTC_ARR_0x34b",	0x34b},
	{"RTC_ARR_0x34c",	0x34c},
	{"RTC_ARR_0x34d",	0x34d},
	{"RTC_ARR_0x34e",	0x34e},
	{"RTC_ARR_0x34f",	0x34f},
	{"RTC_ARR_0x350",	0x350},
	{"RTC_ARR_0x351",	0x351},
	{"RTC_ARR_0x352",	0x352},
	{"RTC_ARR_0x353",	0x353},
	{"RTC_ARR_0x354",	0x354},
	{"RTC_ARR_0x355",	0x355},
	{"RTC_ARR_0x356",	0x356},
	{"RTC_ARR_0x357",	0x357},
	{"RTC_ARR_0x358",	0x358},
	{"RTC_ARR_0x359",	0x359},
	{"RTC_ARR_0x35a",	0x35a},
	{"RTC_ARR_0x35b",	0x35b},
	{"RTC_ARR_0x35c",	0x35c},
	{"RTC_ARR_0x35d",	0x35d},
	{"RTC_ARR_0x35e",	0x35e},
	{"RTC_ARR_0x35f",	0x35f},
	{"RTC_ARR_0x360",	0x360},
	{"RTC_ARR_0x361",	0x361},
	{"RTC_ARR_0x362",	0x362},
	{"RTC_ARR_0x363",	0x363},
	{"RTC_ARR_0x364",	0x364},
	{"RTC_ARR_0x365",	0x365},
	{"RTC_ARR_0x366",	0x366},
	{"RTC_ARR_0x367",	0x367},
	{"RTC_ARR_0x368",	0x368},
	{"RTC_ARR_0x369",	0x369},
	{"RTC_ARR_0x36a",	0x36a},
	{"RTC_ARR_0x36b",	0x36b},
	{"RTC_ARR_0x36c",	0x36c},
	{"RTC_ARR_0x36d",	0x36d},
	{"RTC_ARR_0x36e",	0x36e},
	{"RTC_ARR_0x36f",	0x36f},
	{"RTC_ARR_0x370",	0x370},
	{"RTC_ARR_0x371",	0x371},
	{"RTC_ARR_0x372",	0x372},
	{"RTC_ARR_0x373",	0x373},
	{"RTC_ARR_0x374",	0x374},
	{"RTC_ARR_0x375",	0x375},
	{"RTC_ARR_0x376",	0x376},
	{"RTC_ARR_0x377",	0x377},
	{"RTC_ARR_0x378",	0x378},
	{"RTC_ARR_0x379",	0x379},
	{"RTC_ARR_0x37a",	0x37a},
	{"RTC_ARR_0x37b",	0x37b},
	{"RTC_ARR_0x37c",	0x37c},
	{"RTC_ARR_0x37d",	0x37d},
	{"RTC_ARR_0x37e",	0x37e},
	{"RTC_ARR_0x37f",	0x37f},
	{"RTC_ARR_0x380",	0x380},
	{"RTC_ARR_0x381",	0x381},
	{"RTC_ARR_0x382",	0x382},
	{"RTC_ARR_0x383",	0x383},
	{"RTC_ARR_0x384",	0x384},
	{"RTC_ARR_0x385",	0x385},
	{"RTC_ARR_0x386",	0x386},
	{"RTC_ARR_0x387",	0x387},
	{"RTC_ARR_0x388",	0x388},
	{"RTC_ARR_0x389",	0x389},
	{"RTC_ARR_0x38a",	0x38a},
	{"RTC_ARR_0x38b",	0x38b},
	{"RTC_ARR_0x38c",	0x38c},
	{"RTC_ARR_0x38d",	0x38d},
	{"RTC_ARR_0x38e",	0x38e},
	{"RTC_ARR_0x38f",	0x38f},
	{"RTC_ARR_0x390",	0x390},
	{"RTC_ARR_0x391",	0x391},
	{"RTC_ARR_0x392",	0x392},
	{"RTC_ARR_0x393",	0x393},
	{"RTC_ARR_0x394",	0x394},
	{"RTC_ARR_0x395",	0x395},
	{"RTC_ARR_0x396",	0x396},
	{"RTC_ARR_0x397",	0x397},
	{"RTC_ARR_0x398",	0x398},
	{"RTC_ARR_0x399",	0x399},
	{"RTC_ARR_0x39a",	0x39a},
	{"RTC_ARR_0x39b",	0x39b},
	{"RTC_ARR_0x39c",	0x39c},
	{"RTC_ARR_0x39d",	0x39d},
	{"RTC_ARR_0x39e",	0x39e},
	{"RTC_ARR_0x39f",	0x39f},
	{"RTC_ARR_0x3a0",	0x3a0},
	{"RTC_ARR_0x3a1",	0x3a1},
	{"RTC_ARR_0x3a2",	0x3a2},
	{"RTC_ARR_0x3a3",	0x3a3},
	{"RTC_ARR_0x3a4",	0x3a4},
	{"RTC_ARR_0x3a5",	0x3a5},
	{"RTC_ARR_0x3a6",	0x3a6},
	{"RTC_ARR_0x3a7",	0x3a7},
	{"RTC_ARR_0x3a8",	0x3a8},
	{"RTC_ARR_0x3a9",	0x3a9},
	{"RTC_ARR_0x3aa",	0x3aa},
	{"RTC_ARR_0x3ab",	0x3ab},
	{"RTC_ARR_0x3ac",	0x3ac},
	{"RTC_ARR_0x3ad",	0x3ad},
	{"RTC_ARR_0x3ae",	0x3ae},
	{"RTC_ARR_0x3af",	0x3af},
	{"RTC_ARR_0x3b0",	0x3b0},
	{"RTC_ARR_0x3b1",	0x3b1},
	{"RTC_ARR_0x3b2",	0x3b2},
	{"RTC_ARR_0x3b3",	0x3b3},
	{"RTC_ARR_0x3b4",	0x3b4},
	{"RTC_ARR_0x3b5",	0x3b5},
	{"RTC_ARR_0x3b6",	0x3b6},
	{"RTC_ARR_0x3b7",	0x3b7},
	{"RTC_ARR_0x3b8",	0x3b8},
	{"RTC_ARR_0x3b9",	0x3b9},
	{"RTC_ARR_0x3ba",	0x3ba},
	{"RTC_ARR_0x3bb",	0x3bb},
	{"RTC_ARR_0x3bc",	0x3bc},
	{"RTC_ARR_0x3bd",	0x3bd},
	{"RTC_ARR_0x3be",	0x3be},
	{"RTC_ARR_0x3bf",	0x3bf},
	{"RTC_ARR_0x3c0",	0x3c0},
	{"RTC_ARR_0x3c1",	0x3c1},
	{"RTC_ARR_0x3c2",	0x3c2},
	{"RTC_ARR_0x3c3",	0x3c3},
	{"RTC_ARR_0x3c4",	0x3c4},
	{"RTC_ARR_0x3c5",	0x3c5},
	{"RTC_ARR_0x3c6",	0x3c6},
	{"RTC_ARR_0x3c7",	0x3c7},
	{"RTC_ARR_0x3c8",	0x3c8},
	{"RTC_ARR_0x3c9",	0x3c9},
	{"RTC_ARR_0x3ca",	0x3ca},
	{"RTC_ARR_0x3cb",	0x3cb},
	{"RTC_ARR_0x3cc",	0x3cc},
	{"RTC_ARR_0x3cd",	0x3cd},
	{"RTC_ARR_0x3ce",	0x3ce},
	{"RTC_ARR_0x3cf",	0x3cf},
	{"RTC_ARR_0x3d0",	0x3d0},
	{"RTC_ARR_0x3d1",	0x3d1},
	{"RTC_ARR_0x3d2",	0x3d2},
	{"RTC_ARR_0x3d3",	0x3d3},
	{"RTC_ARR_0x3d4",	0x3d4},
	{"RTC_ARR_0x3d5",	0x3d5},
	{"RTC_ARR_0x3d6",	0x3d6},
	{"RTC_ARR_0x3d7",	0x3d7},
	{"RTC_ARR_0x3d8",	0x3d8},
	{"RTC_ARR_0x3d9",	0x3d9},
	{"RTC_ARR_0x3da",	0x3da},
	{"RTC_ARR_0x3db",	0x3db},
	{"RTC_ARR_0x3dc",	0x3dc},
	{"RTC_ARR_0x3dd",	0x3dd},
	{"RTC_ARR_0x3de",	0x3de},
	{"RTC_ARR_0x3df",	0x3df},
	{"RTC_ARR_0x3e0",	0x3e0},
	{"RTC_ARR_0x3e1",	0x3e1},
	{"RTC_ARR_0x3e2",	0x3e2},
	{"RTC_ARR_0x3e3",	0x3e3},
	{"RTC_ARR_0x3e4",	0x3e4},
	{"RTC_ARR_0x3e5",	0x3e5},
	{"RTC_ARR_0x3e6",	0x3e6},
	{"RTC_ARR_0x3e7",	0x3e7},
	{"RTC_ARR_0x3e8",	0x3e8},
	{"RTC_ARR_0x3e9",	0x3e9},
	{"RTC_ARR_0x3ea",	0x3ea},
	{"RTC_ARR_0x3eb",	0x3eb},
	{"RTC_ARR_0x3ec",	0x3ec},
	{"RTC_ARR_0x3ed",	0x3ed},
	{"RTC_ARR_0x3ee",	0x3ee},
	{"RTC_ARR_0x3ef",	0x3ef},
	{"RTC_ARR_0x3f0",	0x3f0},
	{"RTC_ARR_0x3f1",	0x3f1},
	{"RTC_ARR_0x3f2",	0x3f2},
	{"RTC_ARR_0x3f3",	0x3f3},
	{"RTC_ARR_0x3f4",	0x3f4},
	{"RTC_ARR_0x3f5",	0x3f5},
	{"RTC_ARR_0x3f6",	0x3f6},
	{"RTC_ARR_0x3f7",	0x3f7},
	{"RTC_ARR_0x3f8",	0x3f8},
	{"RTC_ARR_0x3f9",	0x3f9},
	{"RTC_ARR_0x3fa",	0x3fa},
	{"RTC_ARR_0x3fb",	0x3fb},
	{"RTC_ARR_0x3fc",	0x3fc},
	{"RTC_ARR_0x3fd",	0x3fd},
	{"RTC_ARR_0x3fe",	0x3fe},
	{NULL}
};

/* CPR (Clock, Power-On, Reset) Registers */
#define CPR_ADDR		0x00c
#define CPR_DATA		0x00d
static ppc4xx_reg_t cxd4707_cpr[] =
{
	{"CPR0_PLL", 		0x20},
	{"CPR0_PLLCS",		0x40},
	{"CPR0_EBC0",		0x60},
	{"CPR0_MSPROD0",	0x80},
#ifdef CONFIG_CXD4707_ES_2_0
	{"CPR0_SCIDIV0",	0xa0},
	{"CPR0_SCIDIV1",	0xa1},
#endif
	{NULL}
};

/* SDR (System DCR) Registers */
#define SDR_ADDR		0x00e
#define SDR_DATA		0x00f
static ppc4xx_reg_t cxd4707_sdr[] =
{
	{"SDR0_DDRDL0",		0x0e0},
	{"SDR0_PST",		0x100},
	{"SDR0_AHB_MID",	0x110},
	{"SDR0_AHB_REQX",	0x111},
	{"SDR0_UART0",		0x120},
	{"SDR0_UART1",		0x121},
	{"SDR0_UART2",		0x122},
	{"SDR0_UART3",		0x123},
	{"SDR0_WOLEN",		0x130},
	{"SDR0_MSDPDDIS",	0x140},
	{"SDR0_CP440",		0x180},
	{"SDR0_SRST0",		0x200},
	{"SDR0_SRST1",		0x201},
	{"SDR0_SLIPIPE0",	0x220},
	{"SDR0_AMP0",		0x240},
	{"SDR0_MIRQ0",		0x260},
	{"SDR0_MIRQ1",		0x261},
	{"SDR0_PFC0",		0x280},
#ifdef CONFIG_CXD4707_ES_2_0
	{"SDR0_PIO2FIFOCH",	0x290},
	{"SDR0_CHAT",		0x300},
	{"SDR0_DREQRST",	0x320},
#endif
	{NULL}
};

/* DDR-SDRAM Controller DCRs */
#define SDRAM_ADDR		0x010
#define SDRAM_DATA		0x011
static ppc4xx_reg_t cxd4707_sdram[] =
{
	{"SDRAM0_BESRA",	0x00},
	{"SDRAM0_BESRA",	0x04},
	{"SDRAM0_BESRB",	0x08},
	{"SDRAM0_BESRB",	0x0c},
	{"SDRAM0_BEAR",		0x10},
	{"SDRAM0_MIRQ",		0x11},
	{"SDRAM0_MIRQ",		0x12},
	{"SDRAM0_SLIO",		0x18},
	{"SDRAM0_CFG0",		0x20},
	{"SDRAM0_CFG1",		0x21},
	{"SDRAM0_DEVOPT",	0x22},
	{"SDRAM0_MCSTS",	0x24},
	{"SDRAM0_RTR",		0x30},
	{"SDRAM0_PMIT",		0x34},
	{"SDRAM0_UABBA",	0x38},
	{"SDRAM0_B0CR",		0x40},
	{"SDRAM0_B1CR",		0x44},
	{"SDRAM0_B2CR",		0x48},
	{"SDRAM0_B3CR",		0x4c},
	{"SDRAM0_TR0",		0x80},
	{"SDRAM0_TR1",		0x81},
	{"SDRAM0_CLKTR",	0x82},
	{"SDRAM0_WRDTR",	0x83},
	{"SDRAM0_DLYCAL",	0x84},
	{"SDRAM0_ECCESR",	0x98},
	{"SDRAM0_CID",		0xa4},
	{"SDRAM0_RID",		0xa8},
	{NULL}
};

/* External Bus Controller DCRs */
#define EBC_ADDR		0x012
#define EBC_DATA		0x013
static ppc4xx_reg_t cxd4707_ebc[] =
{
	{"EBC0_B0CR",		0x00},
	{"EBC0_B1CR",		0x01},
	{"EBC0_B2CR",		0x02},
	{"EBC0_B3CR",		0x03},
	{"EBC0_B4CR",		0x04},
	{"EBC0_B5CR",		0x05},
	{"EBC0_B6CR",		0x06},
	{"EBC0_B7CR",		0x07},
	{"EBC0_B0AP",		0x10},
	{"EBC0_B1AP",		0x11},
	{"EBC0_B2AP",		0x12},
	{"EBC0_B3AP",		0x13},
	{"EBC0_B4AP",		0x14},
	{"EBC0_B5AP",		0x15},
	{"EBC0_B6AP",		0x16},
	{"EBC0_B7AP",		0x17},
	{"EBC0_BEAR",		0x20},
	{"EBC0_BESR",		0x21},
	{"EBC0_CFG",		0x23},
	{"EBC0_CID",		0x24},
	{NULL}
};

/* External Bus Master DCRs */
#define EBM_ADDR		0x014
#define EBM_DATA		0x015
static ppc4xx_reg_t cxd4707_ebm[] =
{
	{"EBM0_CTL",		0x00},
	{"EBM0_LCNT",		0x01},
	{"EBM0_BEAR",		0x02},
	{"EBM0_BESR",		0x03},
	{"EBM0_BEMR",		0x04},
	{"EBM0_UAR",		0x05},
	{"EBM0_UAM",		0x06},
	{"EBM0_SLPMD",		0x07},
	{"EBM0_FAIR",		0x08},
	{"EBM0_MISCSTS",	0x10},
	{"EBM0_CID",		0x11},
	{NULL}
};

/* MemoryStick Pro (MMIO) */
static ppc4xx_reg_t cxd4707_mspro[]=
{
	{"MSP0_CTRL",		0x000},
	{"MSP0_SYS",		0x004},
	{"MSP0_FLAG",		0x008},
	{"MSP0_MEMC",		0x00c},
	{"MSP0_GEN01",		0x010},
	{"MSP0_GEN23",		0x014},
	{"MSP0_GEN45",		0x018},
	{"MSP0_TIMER",		0x01c},
	{"MSP0_INSTQ",		0x020},
	{"MSP0_GFIFO",		0x024},
	{"MSP0_PBUF",		0x028},
	{"MSP0_VERSION",	0x02c},
	{"MSP0_CMDREG",		0x030},
	{"MSP0_DATREG",		0x034},
	{"MSP0_STAREG",		0x038},
	{"MSP0_HSYS",		0x03c},
	{NULL}
};


/* USB Host (MMIO) */
static ppc4xx_reg_t cxd4707_usbh0[]=
{
	{"USBH0_HCREVISION",		0x00},
	{"USBH0_HCCONTROL",		0x04},
	{"USBH0_HCCOMMANDSTATUS",	0x08},
	{"USBH0_HCINTERRUPTSTATUS",	0x0C},
	{"USBH0_HCINTERRUPTENABLE",	0x10},
	{"USBH0_HCINTERRUPTDISABLE",	0x14},
	{"USBH0_HCHCCA",		0x18},
	{"USBH0_HCPERIODCURRENTED",	0x1C},
	{"USBH0_HCCONTROLHEADED",	0x20},
	{"USBH0_HCCONTROLCURRENTED",	0x24},
	{"USBH0_HCBULKHEADED",		0x28},
	{"USBH0_HCBULKCURRENTED",	0x2C},
	{"USBH0_HCDONEHEAD",		0x30},
	{"USBH0_HCFMINTERVAL",		0x34},
	{"USBH0_HCFMREMAINING",		0x38},
	{"USBH0_HCFMNUMBER",		0x3C},
	{"USBH0_HCPERIODICSTART",	0x40},
	{"USBH0_HCLSTHRESHOLD",		0x44},
	{"USBH0_HCRHDESCRIPTORA",	0x48},
	{"USBH0_HCRHDESCRIPTORB",	0x4C},
	{"USBH0_HCRHSTATUS",		0x50},
	{"USBH0_HCRHPORTSTATUS1",	0x54},
	{"USBH0_HCRHPORTSTATUS2",	0x58},
	{"USBH0_HCRHPORTSTATUS3",	0x5C},
	{"USBH0_HCRHPORTSTATUS4",	0x60},
	{NULL}
};

/* USB Device (MMIO) */
static ppc4xx_reg_t cxd4707_usbd0[]=
{
	{"USBD0_SIR",		0x000},
	{"USBD0_BIR",		0x004},
	{"USBD0_BCR",		0x008},
	{"USBD0_CMDR",		0x00c},
	{"USBD0_IIR",		0x010},
	{"USBD0_IER",		0x014},
	{"USBD0_FNR",		0x018},
	{"USBD0_WWR",		0x040},
	{"USBD0_RWR",		0x044},
	{"USBD0_EP0CR",		0x080},
	{"USBD0_EP0SR",		0x084},
	{"USBD0_EP0IER",	0x088},
	{"USBD0_EP0BF1",	0x090},
	{"USBD0_EP0BF2",	0x0A0},
	{"USBD0_EP1CR",		0x100},
	{"USBD0_EP1ER",		0x104},
	{"USBD0_EP1IER",	0x108},
	{"USBD0_EP1SR",		0x10c},
	{"USBD0_EP1TR",		0x110},
	{"USBD0_EP1DI",		0x114},
	{"USBD0_EP1BUF",	0x180},
	{"USBD0_EP2CR",		0x200},
	{"USBD0_EP2ER",		0x204},
	{"USBD0_EP2IER",	0x208},
	{"USBD0_EP2SR",		0x20c},
	{"USBD0_EP2TR",		0x210},
	{"USBD0_EP2DO",		0x214},
	{"USBD0_EP2BUF",	0x280},
	{"USBD0_EP3CR",		0x300},
	{"USBD0_EP3SR",		0x304},
	{"USBD0_EP3IER",	0x308},
	{"USBD0_EP3BUF",	0x310},
	{NULL}
};

/* AHB to PLB Bridge (MMIO) */
static ppc4xx_reg_t cxd4707_ahb2plb0[]=
{
	{"A2P0_BCNTL",		0x00},
	{"A2P0_BSTAT",		0x04},
	{"A2P0_BSTATM",		0x08},
	{"A2P0_ERSR0",		0x0C},
	{"A2P0_ERSR1",		0x10},
	{"A2P0_MSC",		0x14},
	{"A2P0_WTEA",		0x18},
	{"A2P0_REVID",		0x1C},
	{NULL}
};

static ppc4xx_reg_t cxd4707_pio2fifo0[]=
{
	{"PIO0_CMD0",		0x00},
	{"PIO0_CMD1",		0x04},
	{"PIO0_CMD2",		0x08},
	{"PIO0_CMD3",		0x0C},
	{"PIO0_CMD4",		0x10},
	{"PIO0_CMD5",		0x14},
	{"PIO0_CMD6",		0x18},
	{"PIO0_CMD7",		0x1C},
	{"PIO0_CMD8",		0x20},
	{"PIO0_CMD9",		0x24},
	{"PIO0_CMD10",		0x28},
	{"PIO0_CMD11",		0x2C},
	{"PIO0_CMD12",		0x30},
	{"PIO0_CMD13",		0x34},
	{"PIO0_CMD14",		0x38},
	{"PIO0_CMD15",		0x3C},
	{"PIO0_CONTL",		0x40},
	{"PIO0_ADDR",		0x44},
//	{"PIO0_FIFODATA",	0x48},
	{"PIO0_TXCTL",		0x4C},
	{"PIO0_TXSTAT",		0x50},
	{"PIO0_RXCTL",		0x54},
	{"PIO0_RXSTAT",		0x58},
	{"PIO0_INTENA",		0x5C},
	{"PIO0_INTSTAT",	0x60},
	{NULL}
};

static ppc4xx_reg_t cxd4707_pio2fifo1[]=
{
	{"PIO1_CMD0",		0x00},
	{"PIO1_CMD1",		0x04},
	{"PIO1_CMD2",		0x08},
	{"PIO1_CMD3",		0x0C},
	{"PIO1_CMD4",		0x10},
	{"PIO1_CMD5",		0x14},
	{"PIO1_CMD6",		0x18},
	{"PIO1_CMD7",		0x1C},
	{"PIO1_CMD8",		0x20},
	{"PIO1_CMD9",		0x24},
	{"PIO1_CMD10",		0x28},
	{"PIO1_CMD11",		0x2C},
	{"PIO1_CMD12",		0x30},
	{"PIO1_CMD13",		0x34},
	{"PIO1_CMD14",		0x38},
	{"PIO1_CMD15",		0x3C},
	{"PIO1_CONTL",		0x40},
	{"PIO1_ADDR",		0x44},
//	{"PIO1_FIFODATA",	0x48},
	{"PIO1_TXCTL",		0x4C},
	{"PIO1_TXSTAT",		0x50},
	{"PIO1_RXCTL",		0x54},
	{"PIO1_RXSTAT",		0x58},
	{"PIO1_INTENA",		0x5C},
	{"PIO1_INTSTAT",	0x60},
	{NULL}
};

static ppc4xx_reg_t cxd4707_pio2fifo2[]=
{
	{"PIO2_CMD0",		0x00},
	{"PIO2_CMD1",		0x04},
	{"PIO2_CMD2",		0x08},
	{"PIO2_CMD3",		0x0C},
	{"PIO2_CMD4",		0x10},
	{"PIO2_CMD5",		0x14},
	{"PIO2_CMD6",		0x18},
	{"PIO2_CMD7",		0x1C},
	{"PIO2_CMD8",		0x20},
	{"PIO2_CMD9",		0x24},
	{"PIO2_CMD10",		0x28},
	{"PIO2_CMD11",		0x2C},
	{"PIO2_CMD12",		0x30},
	{"PIO2_CMD13",		0x34},
	{"PIO2_CMD14",		0x38},
	{"PIO2_CMD15",		0x3C},
	{"PIO2_CONTL",		0x40},
	{"PIO2_ADDR",		0x44},
//	{"PIO2_FIFODATA",	0x48},
	{"PIO2_TXCTL",		0x4C},
	{"PIO2_TXSTAT",		0x50},
	{"PIO2_RXCTL",		0x54},
	{"PIO2_RXSTAT",		0x58},
	{"PIO2_INTENA",		0x5C},
	{"PIO2_INTSTAT",	0x60},
	{NULL}
};

static ppc4xx_reg_t cxd4707_gpio0[]=
{
	{"GPIO0_OR",		0x00},
	{"GPIO0_TCR",		0x04},
	{"GPIO0_OS0",		0x08},
	{"GPIO0_OS1",		0x0C},
	{"GPIO0_TS0",		0x10},
	{"GPIO0_TS1",		0x14},
	{"GPIO0_ODR",		0x18},
	{"GPIO0_IR",		0x1C},
	{"GPIO0_R1",		0x20},
	{"GPIO0_R2",		0x24},
	{"GPIO0_R3",		0x28},
	{"GPIO0_IS10",		0x30},
	{"GPIO0_IS11",		0x34},
	{"GPIO0_IS20",		0x38},
	{"GPIO0_IS21",		0x3C},
	{"GPIO0_IS30",		0x40},
	{"GPIO0_IS31",		0x44},
	{NULL}
};

static ppc4xx_reg_t cxd4707_gpio1[]=
{
	{"GPIO1_OR",		0x00},
	{"GPIO1_TCR",		0x04},
	{"GPIO1_OS0",		0x08},
	{"GPIO1_OS1",		0x0C},
	{"GPIO1_TS0",		0x10},
	{"GPIO1_TS1",		0x14},
	{"GPIO1_ODR",		0x18},
	{"GPIO1_IR",		0x1C},
	{"GPIO1_R1",		0x20},
	{"GPIO1_R2",		0x24},
	{"GPIO1_R3",		0x28},
	{"GPIO1_IS10",		0x30},
	{"GPIO1_IS11",		0x34},
	{"GPIO1_IS20",		0x38},
	{"GPIO1_IS21",		0x3C},
	{"GPIO1_IS30",		0x40},
	{"GPIO1_IS31",		0x44},
	{NULL}
};

static ppc4xx_reg_t cxd4707_gpt[]=
{
	{"GPT0_TBC",		0x00},
	{"GPT0_CE",		0x04},
	{"GPT0_EC",		0x08},
	{"GPT0_SC",		0x0C},
	{"GPT0_OE",		0x10},
	{"GPT0_OL",		0x14},
	{"GPT0_IM",		0x18},
	{"GPT0_ISS",		0x1C},
	{"GPT0_ISC",		0x20},
	{"GPT0_IE",		0x24},
	{"GPT0_CAPT0",		0x40},
	{"GPT0_CAPT1",		0x44},
	{"GPT0_CAPT2",		0x48},
	{"GPT0_COMP0",		0x80},
	{"GPT0_COMP1",		0x84},
	{"GPT0_COMP2",		0x88},
	{"GPT0_COMP3",		0x8C},
	{"GPT0_COMP4",		0x90},
	{"GPT0_COMP5",		0x94},
	{"GPT0_COMP6",		0x98},
	{"GPT0_MASK0",		0xC0},
	{"GPT0_MASK1",		0xC4},
	{"GPT0_MASK2",		0xC8},
	{"GPT0_MASK3",		0xCC},
	{"GPT0_MASK4",		0xD0},
	{"GPT0_MASK5",		0xD4},
	{"GPT0_MASK6",		0xD8},
	{"GPT0_VFGCTL",		0x10C},
	{"GPT0_DCT0",		0x110},
	{"GPT0_INTRSTAT",	0x11C},
	{"GPT0_VFG0PER",	0x120},
	{"GPT0_VFG0PW",		0x124},
	{"GPT0_VFG1PER",	0x128},
	{"GPT0_VFG1PW",		0x12C},
	{"GPT0_VFG2PER",	0x130},
	{"GPT0_VFG2PW",		0x134},
	{"GPT0_VFG3PER",	0x138},
	{"GPT0_VFG3PW",		0x13C},
	{NULL}
};

static ppc4xx_reg_t cxd4707_emac[]=
{
	{"EMAC3_0_MODE0",	0x00},
	{"EMAC3_0_MODE1",	0x04},
	{"EMAC3_0_TMR0",	0x08},
	{"EMAC3_0_TMR1",	0x0C},
	{"EMAC3_0_RMR",		0x10},
	{"EMAC3_0_ISR",		0x14},
	{"EMAC3_0_IER",		0x18},
	{"EMAC3_0_IAH",		0x1C},
	{"EMAC3_0_IAL",		0x20},
	{"EMAC3_0_TPID",	0x24},
	{"EMAC3_0_TCI",		0x28},
	{"EMAC3_0_PTMR",	0x2C},
	{"EMAC3_0_IAHASH0",	0x30},
	{"EMAC3_0_IAHASH1",	0x34},
	{"EMAC3_0_IAHASH2",	0x38},
	{"EMAC3_0_IAHASH3",	0x3C},
	{"EMAC3_0_GAHASH0",	0x40},
	{"EMAC3_0_GAHASH1",	0x44},
	{"EMAC3_0_GAHASH2",	0x48},
	{"EMAC3_0_GAHASH3",	0x4C},
	{"EMAC3_0_LASTSAH",	0x50},
	{"EMAC3_0_LASTSAL",	0x54},
	{"EMAC3_0_IFG",		0x58},
	{"EMAC3_0_STACTL",	0x5C},
	{"EMAC3_0_TXTHR",	0x60},
	{"EMAC3_0_RXWMARK",	0x64},
	{"EMAC3_0_TXOCT",	0x68},
	{"EMAC3_0_RXOCT",	0x6C},
	{NULL}
};

#if 0				/* byte access specifier not implemented */
static ppc4xx_reg_t cxd4707_scp0[]=
{
	{"SCP0_SPMODE",		0x00},
	{"SCP0_RXDATA",		0x01},
	{"SCP0_TXDATA",		0x02},
	{"SCP0_SPCOM",		0x03},
	{"SCP0_STATUS",		0x04},
	{"SCP0_CDM",		0x06},
	{NULL}
};
                                                  
static ppc4xx_reg_t cxd4707_scp1[]=
{
	{"SCP1_SPMODE",		0x00},
	{"SCP1_RXDATA",		0x01},
	{"SCP1_TXDATA",		0x02},
	{"SCP1_SPCOM",		0x03},
	{"SCP1_STATUS",		0x04},
	{"SCP1_CDM",		0x06},
	{NULL}
};
#endif

static ppc4xx_reg_t cxd4707_iic0[]=
{
	{"IIC0_MDB",		0x00},
	{"IIC0_SDB",		0x02},
	{"IIC0_MAR",		0x04},
	{"IIC0_CTL",		0x06},
	{"IIC0_MCR",		0x07},
	{"IIC0_STS",		0x08},
	{"IIC0_EST",		0x09},
	{"IIC0_SAR",		0x0A},
	{"IIC0_CLK",		0x0C},
	{"IIC0_IER",		0x0D},
	{"IIC0_TCR",		0x0E},
	{"IIC0_ECS",		0x0F},
	{"IIC0_DCL",		0x10},
	{"IIC0_ISR",		0x11},
	{NULL}
};
                                                
static ppc4xx_reg_t cxd4707_iic1[]=
{
	{"IIC1_MDB",		0x00},
	{"IIC1_SDB",		0x02},
	{"IIC1_MAR",		0x04},
	{"IIC1_CTL",		0x06},
	{"IIC1_MCR",		0x07},
	{"IIC1_STS",		0x08},
	{"IIC1_EST",		0x09},
	{"IIC1_SAR",		0x0A},
	{"IIC1_CLK",		0x0C},
	{"IIC1_IER",		0x0D},
	{"IIC1_TCR",		0x0E},
	{"IIC1_ECS",		0x0F},
	{"IIC1_DCL",		0x10},
	{"IIC1_ISR",		0x11},
	{NULL}
};


#ifdef CONFIG_CXD4707_ES_2_0
/* PCI-X Bridge Local Config (MMIO) */
static ppc4xx_reg_t cxd4707_pcixl[]=
{
	{"PCIX0_BRDGOPT1",	0x040},
	{"PCIX0_BRDGOPT2",	0x044},
	{"PCIX0_ERREN",		0x050},
	{"PCIX0_ERRSTS",	0x054},
	{"PCIX0_PLBBESR",	0x058},
	{"PCIX0_PLBBEARL",	0x05C},
	{"PCIX0_PLBBEARH",	0x060},
	{"PCIX0_POM0LAL",	0x068},
	{"PCIX0_POM0LAH",	0x06C},
	{"PCIX0_POM0MA",	0x070},
	{"PCIX0_POM0PCIAL",	0x074},
	{"PCIX0_POM0PCIAH",	0x078},
	{"PCIX0_POM1LAL",	0x07C},
	{"PCIX0_POM1LAH",	0x080},
	{"PCIX0_POM1MA",	0x084},
	{"PCIX0_POM1PCIAL",	0x088},
	{"PCIX0_POM1PCIAH",	0x08C},
	{"PCIX0_POM2MA",	0x090},
	{"PCIX0_PIM0SA",	0x098},
	{"PCIX0_PIM0LAL",	0x09C},
	{"PCIX0_PIM0LAH",	0x0A0},
	{"PCIX0_PIM1SA",	0x0A4},
	{"PCIX0_PIM1LAL",	0x0A8},
	{"PCIX0_PIM1LAH",	0x0AC},
	{"PCIX0_PIM2SA",	0x0B0},
	{"PCIX0_PIM2LAL",	0x0B4},
	{"PCIX0_PIM2LAH",	0x0B8},
	{"PCIX0_OMCAPID",	0x0C0},
	{"PCIX0_OMNIPTR",	0x0C1},
	{"PCIX0_OMMC",		0x0C2},
	{"PCIX0_OMMA",		0x0C4},
	{"PCIX0_OMMUA",		0x0C8},
	{"PCIX0_OMMDATA",	0x0CC},
	{"PCIX0_OMMEOI",	0x0CE},
	{"PCIX0_PMCAPID",	0x0D0},
	{"PCIX0_PMNIPTR",	0x0D1},
	{"PCIX0_PMC",		0x0D2},
	{"PCIX0_PMCSR",		0x0D4},
	{"PCIX0_PMCSRBSE",	0x0D6},
//	{"PCIX0_PMDATA",	0x0D7},
	{"PCIX0_PMSCRR",	0x0D8},
	{"PCIX0_PCIXCAPID",	0x0DC},
	{"PCIX0_PCIXNIPTR",	0x0DD},
	{"PCIX0_PCIXCMD",	0x0DE},
	{"PCIX0_PCIXSTS",	0x0E0},
	{"PCIX0_PCIXIDR",	0x0E4},
	{"PCIX0_PCIXCID",	0x0E8},
	{"PCIX0_PCIXRID",	0x0EC},
	{NULL}
};

/* SSI (MMIO) */
static ppc4xx_reg_t cxd4707_ssi[]=
{
	{"SSI0_CTRLR0",		0x00},
	{"SSI0_SSIENR",		0x08},
	{"SSI0_MWCR",		0x0c},
	{"SSI0_TXFTLR",		0x18},
	{"SSI0_RXFTLR",		0x1c},
	{"SSI0_TXFLR",		0x20},
	{"SSI0_RXFLR",		0x24},
	{"SSI0_SR",		0x28},
	{"SSI0_IMR",		0x2c},
	{"SSI0_ISR",		0x30},
	{"SSI0_RISR",		0x34},
	{"SSI0_TXOICR",		0x38},
	{"SSI0_RXOICR",		0x3c},
	{"SSI0_RXUICR",		0x40},
	{"SSI0_ICR",		0x48},
	{"SSI0_DMACR",		0x4c},
	{"SSI0_DMATDLR",	0x50},
	{"SSI0_DMARDLR",	0x54},
	{"SSI0_IDR",		0x58},
	{"SSI0_VERSION",	0x5c},
	{"SSI0_DR",		0x60},
	{NULL}
};

/* SCI (MMIO) */
static ppc4xx_reg_t cxd4707_sci[]=
{
//w/o	{"SCI_SCTBF",		0x00},
//w/o	{"SCI_SCTB",		0x00},
//	{"SCI_SCRBF",		0x00},
//	{"SCI_SCRB",		0x00},
	{"SCI_SCCTL0",		0x04},
	{"SCI_SCCTL1",		0x05},
	{"SCI_SCINTEN0",	0x06},
	{"SCI_SCINTEN1",	0x07},
	{"SCI_SCINT0",		0x08},
	{"SCI_SSINT1",		0x09},
	{"SCI_SCSTAT",		0x0A},
	{"SCI_SCBMR",		0x0B},
	{"SCI_SCCLK",		0x0C},
	{"SCI_SCCTL3",		0x0D},
	{"SCI_SCETU0",		0x0E},
	{"SCI_SCETU1",		0x0F},
	{"SCI_SCRXLEN",		0x10},
	{"SCI_SCTXLEN",		0x12},
	{"SCI_SCCWT0",		0x14},
	{"SCI_SCCWT1",		0x15},
	{"SCI_SCEGT",		0x16},
	{"SCI_SCBWT",		0x18},
	{NULL}
};

#else  /* CONFIG_CXD4707_ES_2_0 */
static ppc4xx_reg_t cxd4707_pcil[]=
{
	{"PCIL0_PMM0LA",	0x00},
	{"PCIL0_PMM0MA",	0x04},
	{"PCIL0_PMM0PCILA",	0x08},
	{"PCIL0_PMM0PCIHA",	0x0C},
	{"PCIL0_PMM1LA",	0x10},
	{"PCIL0_PMM1MA",	0x14},
	{"PCIL0_PMM1PCILA",	0x18},
	{"PCIL0_PMM1PCIHA",	0x1C},
	{"PCIL0_PMM2LA",	0x20},
	{"PCIL0_PMM2MA",	0x24},
	{"PCIL0_PMM2PCILA",	0x28},
	{"PCIL0_PMM2PCIHA",	0x2C},
	{"PCIL0_PTM1MS",	0x30},
	{"PCIL0_PTM1LA",	0x34},
	{"PCIL0_PTM2MS",	0x38},
	{"PCIL0_PTM2LA",	0x3C},
	{NULL}
};

static ppc4xx_reg_t cxd4707_pio2fifo3[]=
{
	{"PIO3_CMD0",		0x00},
	{"PIO3_CMD1",		0x04},
	{"PIO3_CMD2",		0x08},
	{"PIO3_CMD3",		0x0C},
	{"PIO3_CMD4",		0x10},
	{"PIO3_CMD5",		0x14},
	{"PIO3_CMD6",		0x18},
	{"PIO3_CMD7",		0x1C},
	{"PIO3_CMD8",		0x20},
	{"PIO3_CMD9",		0x24},
	{"PIO3_CMD10",		0x28},
	{"PIO3_CMD11",		0x2C},
	{"PIO3_CMD12",		0x30},
	{"PIO3_CMD13",		0x34},
	{"PIO3_CMD14",		0x38},
	{"PIO3_CMD15",		0x3C},
	{"PIO3_CONTL",		0x40},
	{"PIO3_ADDR",		0x44},
//	{"PIO3_FIFODATA",	0x48},
	{"PIO3_TXCTL",		0x4C},
	{"PIO3_TXSTAT",		0x50},
	{"PIO3_RXCTL",		0x54},
	{"PIO3_RXSTAT",		0x58},
	{"PIO3_INTENA",		0x5C},
	{"PIO3_INTSTAT",	0x60},
	{NULL}
};

/* NAND Ctlr (MMIO) */
static ppc4xx_reg_t cxd4707_nand[]=
{
	{"NAND0_CFG",		0x00},
	{"NAND0_IMM",		0x04},
	{"NAND0_PRM",		0x08},
	{"NAND0_RADDR",		0x0C},
	{"NAND0_ECC",		0x10},
	{"NAND0_TCTRL",		0x14},
	{"NAND0_RWDR",		0x18},
	{NULL}
};

#endif /* CONFIG_CXD4707_ES_2_0 */



/* 
 * Module table
 */
static ppc4xx_reg_module_t ppc4xx_reg_proc_modules[] =
{
	{"CPU",		REG_SPR,	cxd4707_processor},
	{"CPU_CACHE",	REG_SPR,	cxd4707_processor_cache},
	{"CPU_INTR",	REG_SPR,	cxd4707_processor_intr},
	{"CPU_TIMER",	REG_SPR,	cxd4707_processor_timer},
	{"CPU_DEBUG",	REG_SPR,	cxd4707_processor_debug},
	{"CPU_MMU",	REG_SPR,	cxd4707_processor_mmu},
	{"OPB_0",	REG_DCR,	cxd4707_opb0},
	{"OPB_1",	REG_DCR,	cxd4707_opb1},
	{"PLBC",	REG_DCR,	cxd4707_plbc},
	{"POB_0",	REG_DCR,	cxd4707_pob0},
	{"POB_1",	REG_DCR,	cxd4707_pob1},
	{"CPC",		REG_DCR,	cxd4707_cpc},
	{"UIC_0",	REG_DCR,	cxd4707_uic0},
	{"UIC_1",	REG_DCR,	cxd4707_uic1},
	{"UIC_2",	REG_DCR,	cxd4707_uic2},
	{"DMA_0",	REG_DCR,	cxd4707_dma0},
	{"DMA_1",	REG_DCR,	cxd4707_dma1},
	{"MAL",		REG_DCR,	cxd4707_mal},
	{"PLB2AHB",	REG_DCR,	cxd4707_plb2ahb},
	{"RTC",		REG_DCR,	cxd4707_rtc},
	{"CPR",		REG_DCRI,	cxd4707_cpr,	CPR_ADDR,	CPR_DATA},
	{"SDR",		REG_DCRI,	cxd4707_sdr,	SDR_ADDR,	SDR_DATA},
	{"SDRAM",	REG_DCRI,	cxd4707_sdram,	SDRAM_ADDR, 	SDRAM_DATA},
	{"EBC",		REG_DCRI,	cxd4707_ebc,	EBC_ADDR,	EBC_DATA},
	{"EBM",		REG_DCRI,	cxd4707_ebm,	EBM_ADDR,	EBM_DATA},
	{"MSPro",	REG_MMIO,	cxd4707_mspro,	MSP0_REG_BASE,	MSP0_REG_SIZE},
	{"USBHost",	REG_MMIO,	cxd4707_usbh0,	USBH0_REG_BASE,	USBH0_REG_SIZE},
	{"USBDev",	REG_MMIO,	cxd4707_usbd0,	USBD0_REG_BASE,	USBD0_REG_SIZE},
	{"AHB2PLB",	REG_MMIO,	cxd4707_ahb2plb0, AHB2PLB0_REG_BASE, AHB2PLB0_REG_SIZE},
	{"GPIO0",	REG_MMIO,	cxd4707_gpio0,	CXD4707_GPIO0_ADDR, CXD4707_GPIO_SIZE},
	{"GPIO1",	REG_MMIO,	cxd4707_gpio1,	CXD4707_GPIO1_ADDR, CXD4707_GPIO_SIZE},
	{"GPT",		REG_MMIO,	cxd4707_gpt,	CXD4707_GPT_ADDR, CXD4707_GPT_SIZE},
	{"EMAC",	REG_MMIO,	cxd4707_emac,	CXD4707_EMAC0_ADDR, CXD4707_EMAC_SIZE},
//	{"SCP0",	REG_MMIO,	cxd4707_scp0,	CXD4707_SCP0_ADDR, CXD4707_SCP_SIZE},
//	{"SCP1",	REG_MMIO,	cxd4707_scp1,	CXD4707_SCP1_ADDR, CXD4707_SCP_SIZE},
	{"I2C0",	REG_MMIO,	cxd4707_iic0,	CXD4707_I2C0_ADDR, CXD4707_I2C_SIZE},
	{"I2C1",	REG_MMIO,	cxd4707_iic1,	CXD4707_I2C1_ADDR, CXD4707_I2C_SIZE},
#ifdef CONFIG_CXD4707_ES_2_0
	{"PCIXBridge",	REG_MMIO,	cxd4707_pcixl,	0xdec80000,	512},
	{"SSI",		REG_MMIO,	cxd4707_ssi,	CXD4707_SSI_ADDR, CXD4707_SSI_SIZE},
	{"SCI",		REG_MMIO,	cxd4707_sci,	CXD4707_SCI_ADDR, CXD4707_SCI_SIZE},
	{"PIO2FIFO0",	REG_MMIO,	cxd4707_pio2fifo0, CXD4707_PIO2FIFO0_ADDR, CXD4707_PIO2FIFO_SIZE},
	{"PIO2FIFO1",	REG_MMIO,	cxd4707_pio2fifo1, CXD4707_PIO2FIFO1_ADDR, CXD4707_PIO2FIFO_SIZE},
	{"PIO2FIFO2",	REG_MMIO,	cxd4707_pio2fifo2, CXD4707_PIO2FIFO2_ADDR, CXD4707_PIO2FIFO_SIZE},
#else  /* ES1.x */
	{"NAND",	REG_MMIO,	cxd4707_nand,	NAND0_REG_BASE,	NAND0_REG_SIZE},
	{"PCIBridge",	REG_MMIO,	cxd4707_pcil,	0xdf400000,	64},
	{"SCP0FIFO",	REG_MMIO,	cxd4707_pio2fifo0, CXD4707_SCP0FIFO_ADDR, CXD4707_PIO2FIFO_SIZE},
	{"SCP1FIFO",	REG_MMIO,	cxd4707_pio2fifo1, CXD4707_SCP1FIFO_ADDR, CXD4707_PIO2FIFO_SIZE},
	{"I2C0FIFO",	REG_MMIO,	cxd4707_pio2fifo2, CXD4707_I2C0FIFO_ADDR, CXD4707_PIO2FIFO_SIZE},
	{"I2C1FIFO",	REG_MMIO,	cxd4707_pio2fifo3, CXD4707_I2C1FIFO_ADDR, CXD4707_PIO2FIFO_SIZE},
#endif
	{NULL}
};

