/*
 * arch/ppc/platforms/cxd4707gb.c
 *
 * CXD4707 I/O descriptions
 *
 *  Copyright 2003-2006 Sony Corporation.
 *
 *
 * Matt Porter <mporter@mvista.com>
 * Armin Kuster <akuster@mvista.com>
 *
 * Copyright 2002 MontaVista Software Inc.
 *
 * Eugene Surovegin <eugene.surovegin@zultys.com> or <ebs@ebshome.net>
 * Copyright (c) 2003 Zultys Technologies
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/threads.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <linux/snsc_nlver.h>
#include <linux/pci.h>
#include <linux/snsc_early.h>
#include <asm/io.h>
#include <asm/ibm4xx.h>
#include <asm/mmu.h>
#include <asm/ocp.h>
#include <asm/system.h>
#include <asm/todc.h>
#include <asm/pci-bridge.h>
#include <asm/time.h>
#include <asm/machdep.h>
#ifdef CONFIG_PCI_CONF
#include <linux/pci_conf.h>
#endif
#ifdef CONFIG_SNSC_NBL_LOWADDR
#include <asm/bootinfo.h>
#endif
#ifdef CONFIG_SNSC_MPU4xx_MACADDR
#include <platforms/snsc-mpu4xx_macaddr.h>
#endif

#ifdef CONFIG_SNSC_MPU410
#include "snsc-mpu410.h"
#endif
#ifdef CONFIG_FRX
#include "frx.h"
#endif
#ifdef CONFIG_DTGNEXUS
#include "dtgnexus.h"
#endif
#ifdef CONFIG_DTG3G
#include "dtg3g.h"
#endif

#ifdef CONFIG_PCI_AUTO
static void cxd4707_pciauto_fixup_dev(struct pci_controller *hose,
				      int current_bus, int pci_devfn,
				      int *pciauto_upper_memspc, 
				      int *pciauto_upper_iospc);
#endif

#ifdef CONFIG_SERIAL_TEXT_DEBUG
extern void gen550_progress(char *, unsigned short);

#  if defined(CONFIG_SNSC_EARLY_PRINTK)
static void
cxd4707_early_puts(const char *str)
{
	if (ppc_md.progress)
		ppc_md.progress(str, 0x0);
}
#  endif /* CONFIG_SNSC_EARLY_PRINTK */

#endif /* CONFIG_SERIAL_TEXT_DEBUG */

#define NAME "CXD4707: " 

phys_addr_t fixup_bigphys_addr(phys_addr_t addr, phys_addr_t size)
{
	return addr;
};

/*
 * We only have onboard EMAC support for now.  I2C, DMA, timers,
 * etc. have yet to be enabled.
 */

struct ocp_def core_ocp[] = {
	{UART,	CXD4707_UART0_ADDR, UART0_INT, IBM_CPM_UART0},
	{UART,	CXD4707_UART1_ADDR, UART1_INT, IBM_CPM_UART1},
	{UART,	CXD4707_UART2_ADDR, UART2_INT, IBM_CPM_UART2},
	{UART,	CXD4707_UART3_ADDR, UART3_INT, IBM_CPM_UART3},
	{IIC,	CXD4707_IIC0_ADDR, IIC0_IRQ, IBM_CPM_IIC0},
	{IIC,	CXD4707_IIC1_ADDR, IIC1_IRQ, IBM_CPM_IIC1},
	{GPIO,	CXD4707_GPIO0_ADDR, OCP_IRQ_NA, IBM_CPM_GPIO0},
	{GPIO,	CXD4707_GPIO1_ADDR, OCP_IRQ_NA, IBM_CPM_GPIO1},
	{EMAC,	CXD4707_EMAC0_ADDR, BL_MAC_ETH0, OCP_CPM_NA},
	{PCI,   CXD4707_PCI_ADDR, OCP_IRQ_NA, IBM_CPM_PCI},
	{DMAC0,	0x0, DMA0_INT, IBM_CPM_DMA0},
	{DMAC0,	0x0, DMA1_INT, IBM_CPM_DMA0},
	{DMAC0,	0x0, DMA2_INT, IBM_CPM_DMA0},
	{DMAC0,	0x0, DMA3_INT, IBM_CPM_DMA0},
	{DMAC1,	0x0, DMA4_INT, IBM_CPM_DMA1},
	{DMAC1,	0x0, DMA5_INT, IBM_CPM_DMA1},
	{DMAC1,	0x0, DMA6_INT, IBM_CPM_DMA1},
	{DMAC1,	0x0, DMA7_INT, IBM_CPM_DMA1},
	{OCP_NULL_TYPE, 0x0, OCP_IRQ_NA, OCP_CPM_NA},

};

/*
 * Calculate clocks
 */
struct ibm440_clocks cxd4707_get_clocks(void)
{
	struct ibm440_clocks clk;
	u_int32_t cpr0_pll = mfdcri(DCRN_CPR0, PLL);
	u_int32_t cpr0_ebc0 = mfdcri(DCRN_CPR0, EBC0);

	switch ((cpr0_pll >> 30) & 0x3) {
	case 0x00:		/* CPU = 266 Mhz */
		clk.cpu = CXD4707_CPUCLK_266;
		break;
	default:
		printk(NAME "Strange value set for cpr0_pll = 0x%0x\n", cpr0_pll);
		/* FALLTHROUGH */
	case 0x01:		/* CPU = 399 Mhz */
		clk.cpu = CXD4707_CPUCLK;
		break;
	}

	/* PLB = 133 Mhz */
	clk.plb = CXD4707_PLBCLK;

	/* OPB = 66 Mhz */
	clk.opb =  CXD4707_OPBCLK;
	
	switch (cpr0_ebc0 & 0x1) {
	case 0x0:		/* EBC = 33 Mhz */
		clk.ebc =  CXD4707_EBCCLK;
		break;
	case 0x1:		/* EBC = 66 Mhz */
		clk.ebc =  CXD4707_EBCCLK*2;
		break;
	}

	return clk;
}
EXPORT_SYMBOL(cxd4707_get_clocks);

extern void gen550_init(int, struct serial_struct *);


void _cxd4707_init_uart_clk(int port)
{
	/* set uart diviser */
	switch (port) {
	case 0:
		mtdcri(DCRN_SDR0, UART0, UART_DIV);
		break;
	case 1:
		mtdcri(DCRN_SDR0, UART1, UART_DIV);
		break;
	case 2:
		mtdcri(DCRN_SDR0, UART2, UART_DIV);
		break;
	case 3:
		mtdcri(DCRN_SDR0, UART3, UART_DIV);
		break;
	}
	return;
}


/* 
   port is actual UART device number initialized.
   rs_num is the offset in the rs_table[].
 */
void cxd4707_init_uart(int port, int rs_num)
{
	struct serial_struct serial_req;
	int irq;
	phys_addr_t paddr;

	/* set uart clk divisor */
	_cxd4707_init_uart_clk(port);

	/* set uart irq, paddr */
	switch (port) {
	case 0:
		irq = UART0_INT;
		paddr = CXD4707_UART0_ADDR;
		break;
	case 1:
		irq = UART1_INT;
		paddr = CXD4707_UART1_ADDR;
		break;
	case 2:
		irq = UART2_INT;
		paddr = CXD4707_UART2_ADDR;
		break;
	case 3:
		irq = UART3_INT;
		paddr = CXD4707_UART3_ADDR;
		break;
	}

	/* Setup ioremapped serial port access */
	memset(&serial_req, 0, sizeof(serial_req));

	serial_req.line = port;
	serial_req.baud_base = BASE_BAUD;
	serial_req.irq = irq;
	serial_req.flags = ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST;
	serial_req.io_type = SERIAL_IO_MEM;
	serial_req.iomem_base = ioremap64(paddr, 8);
	serial_req.iomem_reg_shift = 0;

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	/* Configure debug serial access */
	gen550_init(rs_num, &serial_req);
#endif

	if (early_serial_setup(&serial_req) != 0) {
		printk(NAME "Early serial init of UART %d failed\n", port);
	}
}


#if defined (CONFIG_CXD4707_ES_1_0) || defined (CONFIG_CXD4707_ES_1_1)

static const char *pfc0_grp0_str[] = {
	"GPIO/INT0-15, SCP0-1, IIC1, UART2(Tx/Rx/RTS/CTS)",
	"IDE0",
	"GPIO/INT13-15, NAND, SCP0-1, IIC1, UART2(Tx/Rx/RTS/CTS)",
	"reserved"
};

static const char *pfc0_grp1_str[] = {
	"EBC32, GPIO20-23, GPIO/INT16-19, UART3(Tx/Rx)",
	"reserved",
	"GPIO20-41, GPIO/INT16-19, UART3(Tx/Rx)",
	"TRACE, GPIO20-25, GPIO/INT16-19, UART3(Tx/Rx)"
};

static const char *pfc0_grp2_str[] = {
	"UART2 Modem DCD/DSR/DTR + [RI] / [NAND ND_RDY,ND_WP(GPIO63)]",
	"EBC Master",
	"UART2 Modem DCD/DSR/DTR, NAND ND_RDY,ND_WP(GPIO63)",
	"reserved",
};

static const char *pfc0_grp3_str[] = {
	"USB Host",
	"USB Device, GPIO62",
	"reserved",
	"reserved"
};

#elif defined (CONFIG_CXD4707_ES_2_0)

static const char *pfc0_grp0_str[] = {
	"GPIO/INT0-15, SCP1",
	"NAND16",
	"GPIO/INT13-15, NAND8, SCP1",
	"reserved"
};

static const char *pfc0_grp1_str[] = {
	"EBC32",
	"reserved",
	"GPIO24-41",
	"GPIO24-25, TRACE"
};

static const char *pfc0_grp2_str[] = {
	"UART2 Modem DCD/DSR/DTR/RI, NAND ND_WP(GPIO63)",
	"EBC Master",
	"UART2 Modem DCD/DSR/DTR, NAND8 ND_RDY,ND_WP(GPIO63)",
	"UART2 Modem DCD, NAND16 ND_RDY,ND_WP(GPIO63)",
};

static const char *pfc0_grp3_str[] = {
	"USB Host",
	"USB Device, GPIO62",
	"reserved",
	"reserved",
};

static const char *pfc0_grp4_str[] = {
	"SCI",
	"UART1 TX/RX, EBC DMA DREQ/DACK0-1 (GPIO 49,56,57)"
};

static const char *pfc0_grp5_str[] = {
	"SSI",
	"SCP0"
};

static const char *pfc0_grp6_str[] = {
	"GPIO 60-61",
	"PCI REQ/GNT2"
};

static const char *pfc0_grp7_str[] = {
	"GPIO 58-59",
	"PCI REQ/GNT3"
};

#else

static const char *pfc0_grp0_str[] = 
                  *pfc0_grp1_str[] =
                  *pfc0_grp2_str[] = 
                  *pfc0_grp3_str[] = 
{
	"unknown (0b00)",
	"unknown (0b01)",
	"unknown (0b10)",
	"unknown (0b11)"
};

#endif


void cxd4707_read_pfc(void)
{
	u_int32_t sdr0_pfc0 = mfdcri(DCRN_SDR0, PFC0);

	printk("CXD4707 PinMux settings:\n");

	printk("          Grp0 = %s\n", pfc0_grp0_str[PFC0_GRP0_FUNC(sdr0_pfc0)]);
	printk("          Grp1 = %s\n", pfc0_grp1_str[PFC0_GRP1_FUNC(sdr0_pfc0)]);

	/* Group2 is forced by Group0 mux */
	if (PFC0_GRP0_FUNC(sdr0_pfc0) == PFC0_GRP0_NAND16) {
		printk("          Grp2 = %s\n", pfc0_grp2_str[PFC0_GRP2_NAND16]);
	} else if (PFC0_GRP0_FUNC(sdr0_pfc0) == PFC0_GRP0_NAND8) {
#if defined (CONFIG_CXD4707_ES_2_0)
		printk("          Grp2 = %s\n", pfc0_grp2_str[PFC0_GRP2_NAND8]);
#else
		printk("          Grp2 = %s\n", pfc0_grp2_str[PFC0_GRP2_NAND]);
#endif
	} else {
		printk("          Grp2 = %s\n", pfc0_grp2_str[PFC0_GRP2_FUNC(sdr0_pfc0)]);
	}
	printk("          Grp3 = %s\n", pfc0_grp3_str[PFC0_GRP3_FUNC(sdr0_pfc0)]);
#if defined (CONFIG_CXD4707_ES_2_0)
	printk("          Grp4 = %s\n", pfc0_grp4_str[PFC0_GRP4_FUNC(sdr0_pfc0)]);
	printk("          Grp5 = %s\n", pfc0_grp5_str[PFC0_GRP5_FUNC(sdr0_pfc0)]);
	printk("          Grp6 = %s\n", pfc0_grp6_str[PFC0_GRP6_FUNC(sdr0_pfc0)]);
	printk("          Grp7 = %s\n", pfc0_grp7_str[PFC0_GRP7_FUNC(sdr0_pfc0)]);
#endif

	return ;
}


/*
 * Read the CXD4707 memory controller to get size of system memory.
 */
unsigned long __init
cxd4707_find_end_of_memory(void)
{
	u32 i, bank_config;
	u32 mem_size = 0;

	for (i=0; i<4; i++)
	{
		switch (i)
		{
			case 0:
				bank_config = mfdcri(DCRN_SDRAM0, MB0CF);
				break;
			case 1:
				bank_config = mfdcri(DCRN_SDRAM0, MB1CF);
				break;
			case 2:
				bank_config = mfdcri(DCRN_SDRAM0, MB2CF);
				break;
			case 3:
				bank_config = mfdcri(DCRN_SDRAM0, MB3CF);
				break;
		}

		if (!(bank_config & SDRAM_CONFIG_BANK_ENABLE))
			continue;
		switch (SDRAM_CONFIG_BANK_SIZE(bank_config))
		{
			case SDRAM_CONFIG_SIZE_8M:
				mem_size += PPC440_MEM_SIZE_8M;
				break;
			case SDRAM_CONFIG_SIZE_16M:
				mem_size += PPC440_MEM_SIZE_16M;
				break;
			case SDRAM_CONFIG_SIZE_32M:
				mem_size += PPC440_MEM_SIZE_32M;
				break;
			case SDRAM_CONFIG_SIZE_64M:
				mem_size += PPC440_MEM_SIZE_64M;
				break;
			case SDRAM_CONFIG_SIZE_128M:
				mem_size += PPC440_MEM_SIZE_128M;
				break;
			case SDRAM_CONFIG_SIZE_256M:
				mem_size += PPC440_MEM_SIZE_256M;
				break;
			case SDRAM_CONFIG_SIZE_512M:
				mem_size += PPC440_MEM_SIZE_512M;
				break;
		}
	}
	return mem_size;
}


#ifdef CONFIG_CXD4707_RTC
/*
 * CXD4707 includes an MC146818-equivalent core accessed via a DCR
 */
void
cxd4707_todc_setup(void)
{
	todc_info->rtc_type = TODC_TYPE_MC146818; /* is compatible */
	todc_info->nvram_as0  = 0;
	todc_info->nvram_as1  = 0;
	todc_info->nvram_data = CXD4707_RTC_NVRAM;
	todc_info->as0_bits = 8;
	todc_info->nvram_size     = CXD4707_RTC_NVRAM_SIZE;
	todc_info->sw_flags       = 0;
	todc_info->year           = DCRN_RTC0_YEAR;
	todc_info->month          = DCRN_RTC0_MON;
	todc_info->day_of_month   = DCRN_RTC0_DATE;
	todc_info->day_of_week    = DCRN_RTC0_DAY;
	todc_info->hours          = DCRN_RTC0_HRS;
	todc_info->minutes        = DCRN_RTC0_MIN;
	todc_info->seconds        = DCRN_RTC0_SEC;
	todc_info->control_b      = DCRN_RTC0_REGA; /* 'control_b' is used for freq  */
	todc_info->control_a      = DCRN_RTC0_REGB; /* 'control_a' is used for ctrl */
	todc_info->watchdog       = DCRN_RTC0_REGC;
	todc_info->interrupts     = DCRN_RTC0_REGD;
	todc_info->alarm_date     = 0xff; /* n/a */
	todc_info->alarm_hour     = DCRN_RTC0_HRSALM;
	todc_info->alarm_minutes  = DCRN_RTC0_MINALM;
	todc_info->alarm_seconds  = DCRN_RTC0_SECALM;
	todc_info->century        = DCRN_RTC0_CENT;
	todc_info->flags          = 0xff; /* n/a */
	todc_info->nvram_addr_reg = 0;
	todc_info->nvram_data_reg = 0;
}


u_char
cxd4707_todc_read_val(int addr)
{
	unsigned char retval = 0;
	retval = (u_char)mfdcr(addr);
	return retval;
}

void
cxd4707_todc_write_val(int addr, unsigned char val)
{
	mtdcr(addr, val);
}

#endif /* CONFIG_CXD4707_RTC */

unsigned int
cxd4707_get_rev(void)
{
	unsigned int pvr;
	unsigned int rev = CXD4707_UNKNOWN;
	void *ahb_base;
	unsigned int *ms_ver;
	unsigned long flags;

	pvr = mfspr(PVR);

	switch (pvr) {
	case PVR_CXD4707_ES1x:	/* ES1.0, ES1.1 */
		
		local_irq_save(flags);

		ahb_base = ioremap64(MSP0_REG_BASE, MSP0_REG_SIZE);
		ms_ver = (unsigned int*)(ahb_base + MSP0_VERSION);

		switch (*ms_ver) {
		case 0x00000010: /* ES1.0, AHB is byte swapped */
			rev = CXD4707_ES_1_0;
		break;
		case 0x10000000:
			rev = CXD4707_ES_1_1;
		break;
		}

		iounmap(ahb_base);

		local_irq_restore(flags);
		break;

	case PVR_CXD4707_ES20:	/* ES2.0 */
		rev = CXD4707_ES_2_0;
		break;
	case PVR_CXD4707_ES21:	/* ES2.1 */
		rev = CXD4707_ES_2_1;
		break;
	default:
		break;
	}

	return rev;
}

EXPORT_SYMBOL(cxd4707_get_rev);

#define COMPILED_FOR "Compiled for CXD-4707 "
unsigned int
cxd4707_get_config_rev(void)
{
#if   defined CONFIG_CXD4707_ES_1_0
	return CXD4707_ES_1_0;
#elif defined CONFIG_CXD4707_ES_1_1
	return CXD4707_ES_1_1;
#elif defined CONFIG_CXD4707_ES_2_0
	return CXD4707_ES_2_0;
#else
	return CXD4707_UNKNOWN;
#endif
}

void cxd4707_probe_cpu(void)
{
	unsigned int cpurev, cfgrev;
	cpurev = cxd4707_get_rev();
	cfgrev = cxd4707_get_config_rev();

	switch (cpurev) {
	case CXD4707_ES_1_0:
		printk("CXD-4707 ES1.0 probed.\n");
		break;
	case CXD4707_ES_1_1:
		printk("CXD-4707 ES1.1 probed.\n");
		break;
	case CXD4707_ES_2_0:
		printk("CXD-4707 ES2.0 probed.\n");
		break;
	case CXD4707_ES_2_1:
		printk("CXD-4707 ES2.1 probed.\n");
		break;
	case CXD4707_UNKNOWN:
	default:
		printk("CXD-4707 Unknown revision probed.\n");
		break;
	}

	return ;
}

#ifdef CONFIG_PCI_CONF
#ifdef CONFIG_PCI_CONF_SPECIAL_DEVS
static pciconf_special_dev_t cxd4707_pciconf_special_devs[] = {
  {PCI_VENDOR_ID_SONY,	PCI_DEVICE_ID_SONY_NILE,	
   PCICONF_CONFTYPE_FLG_PRESET | PCICONF_CONFTYPE_NORMAL},
  {PCI_VENDOR_ID_INTEL,	PCI_DEVICE_ID_INTEL_82557,	
   PCICONF_CONFTYPE_FLG_PRESET | PCICONF_CONFTYPE_NORMAL},

  {PCI_VENDOR_ID_DEC,	PCI_DEVICE_ID_DEC_21152,	
   PCICONF_CONFTYPE_FLG_PRESET | PCICONF_CONFTYPE_PCIPCI_BRIDGE},

  {0x3388, 0x0021,	
   PCICONF_CONFTYPE_FLG_PRESET | PCICONF_CONFTYPE_PCIPCI_BRIDGE},

  {PCI_VENDOR_ID_SONY,	PCI_DEVICE_ID_SONY_WG,	
   PCICONF_CONFTYPE_FLG_PRESET | PCICONF_CONFTYPE_NORMAL},

  {0, 0, 0},
};

static int
cxd4707_pciconf_special_dev_to_conftype(struct pci_dev *dev, struct pci_ops *ops,
				     int phase)
{
	return pciconf_special_conftype(dev, ops, cxd4707_pciconf_special_devs);
}
pciconf_dev_to_conftype_t cxd4707_pciconf_dev_to_conftype =
	cxd4707_pciconf_special_dev_to_conftype;
#else /* !CONFIG_PCI_CONF_SPECIAL_DEVS */
pciconf_dev_to_conftype_t cxd4707_pciconf_dev_to_conftype = NULL;
#endif /* !CONFIG_PCI_CONF_SPECIAL_DEVS */
#endif /* CONFIG_PCI_CONF */

/* 
 * Common PCI setup code
 */
void __init
cxd4707_setup_hose(void)
{
	struct pci_controller *hose;
#ifdef CONFIG_PCI_CONF
	struct resource *pci_io_resources[2], *pci_mem_resources[2];
#endif
	if ( ppc_md.progress )
		ppc_md.progress("cxd4707_setup_hose(): in", 0x3eab);

	/* Configure windows on the PCI host bridge */
	cxd4707_setup_pci();

	hose = pcibios_alloc_controller();
	if (!hose)
		return;

	hose->first_busno	= 0;
	hose->last_busno	= 0xff;
	hose->pci_mem_offset	= CXD4707_PCI_MEM_OFFSET;

	pci_init_resource(&hose->io_resource, CXD4707_PCI_LOWER_IO,
			  CXD4707_PCI_UPPER_IO, IORESOURCE_IO,
			  "PCI host bridge");
	pci_init_resource(&hose->mem_resources[0], CXD4707_PCI_LOWER_MEM,
			  CXD4707_PCI_UPPER_MEM, IORESOURCE_MEM,
			  "PCI host bridge");

	hose->io_space.start	= CXD4707_PCI_LOWER_IO;
	hose->io_space.end	= CXD4707_PCI_UPPER_IO;
	hose->mem_space.start	= CXD4707_PCI_LOWER_MEM;
	hose->mem_space.end	= CXD4707_PCI_UPPER_MEM;
	isa_io_base 		= 
		(unsigned long)ioremap64(CXD4707_PCI_IO_BASE, CXD4707_PCI_IO_SIZE);
	hose->io_base_virt 	= (void *)isa_io_base;

	setup_indirect_pci(hose, CXD4707_PCI_CFGA_PLB32, CXD4707_PCI_CFGD_PLB32);

	hose->set_cfg_type 	= 1;
#ifdef CONFIG_PCI_CONF
	pci_io_resources[0]	= &hose->io_space;
	pci_io_resources[1]	= NULL;
	pci_mem_resources[0]	= &hose->mem_space;
	pci_mem_resources[1]	= NULL;
	hose->last_busno	= pciconf_conf(hose->first_busno,
					       &early_pci_ops,
					       cxd4707_pciconf_dev_to_conftype,
					       pci_io_resources,
					       pci_mem_resources);
#else  /* use old PCI AUTO */
	hose->last_busno 	= pciauto_bus_scan(hose, hose->first_busno);
#endif

	if ( ppc_md.progress )
		ppc_md.progress("cxd4707_setup_hose(): out", 0x3eab);
}

void __init
cxd4707_calibrate_decr(void)
{
	unsigned int freq;
	extern struct ibm440_clocks ibm440_clks;

	switch (PVR_REV(mfspr(PVR))) {
	case PVR_CXD4707_ES1x:	/* ES1.0, ES1.1 */
	case PVR_CXD4707_ES20:	/* ES2.0 */
	case PVR_CXD4707_ES21:	/* ES2.1 */		
	default:
		freq = ibm440_clks.cpu;
		break;
	}
	
	tb_ticks_per_jiffy = freq / HZ;
	tb_to_us = mulhwu_scale_factor(freq, 1000000);
#ifdef CONFIG_ILATENCY
	{
		extern long ticks_per_usec;
		ticks_per_usec = freq / 1000000;
	}
#endif

	/* Clear any pending timer interrupts */
	mtspr(SPRN_TSR, TSR_ENW | TSR_WIS | TSR_DIS | TSR_FIS);

	/* Enable decrementer interrupt */
	mtspr(SPRN_TCR, mfspr(SPRN_TCR) | TCR_DIE);
}

void __init
cxd4707_parse_bootinfo(unsigned long r3)
{
#ifdef CONFIG_SNSC_NBL_LOWADDR
	parse_bootinfo((struct bi_record *) r3);
#else
	parse_bootinfo((struct bi_record *)(r3 + KERNELBASE));
#endif
}

void 
cxd4707_platform_init(unsigned long r3, unsigned long r4,
	      unsigned long r5, unsigned long r6, unsigned long r7)
{
	cxd4707_parse_bootinfo(r3);

	ppc_md.find_end_of_memory = cxd4707_find_end_of_memory;
	ppc_md.setup_io_mappings  = NULL;
	ppc_md.calibrate_decr	= cxd4707_calibrate_decr;

#ifdef CONFIG_RTC7301_RTC
	ppc_md.get_rtc_time	= rtc7301_rtc_rtc_get_time;
#endif /* CONFIG_RTC7301_RTC */
#ifdef CONFIG_PPC_RTC
	ppc_md.time_init	= todc_time_init;
	ppc_md.set_rtc_time	= todc_set_rtc_time;
	ppc_md.get_rtc_time	= todc_get_rtc_time;
	ppc_md.nvram_read_val	= todc_direct_read_val;
	ppc_md.nvram_write_val	= todc_direct_write_val;
#endif /* CONFIG_PPC_RTC */
#ifdef CONFIG_CXD4707_RTC
	ppc_md.time_init	= todc_time_init;
	ppc_md.get_rtc_time	= todc_get_rtc_time;
	ppc_md.nvram_read_val	= cxd4707_todc_read_val;
	ppc_md.nvram_write_val	= cxd4707_todc_write_val;
#endif

#ifdef CONFIG_SERIAL_TEXT_DEBUG
	/* ppc_md.progress_init = ? */
	ppc_md.progress		= gen550_progress;
#  if defined(CONFIG_SNSC_EARLY_PRINTK)
	snsc_arch_early.puts	= cxd4707_early_puts;
#  endif
#endif /* CONFIG_SERIAL_TEXT_DEBUG */

#if defined(CONFIG_VT) && !defined(CONFIG_DUMMY_KEYB)
	/* USB keyboard support */
	ppc_md.kbd_setkeycode	= pckbd_setkeycode;
	ppc_md.kbd_getkeycode	= pckbd_getkeycode;
	ppc_md.kbd_translate	= pckbd_translate;
#endif

#ifdef CONFIG_SNSC_MPU4xx_MACADDR
	ppc_md.get_enetaddr	= mpu4xx_get_enetaddr;
#endif

#ifdef CONFIG_SNSC_ADD_NBL_TLB
	ppc_md.board_ioremap	= cxd4707_ioremap;
	ppc_md.board_iounmap	= cxd4707_iounmap;
#endif
#ifdef CONFIG_PCI_AUTO
	ppc_md.pciauto_fixup_dev = cxd4707_pciauto_fixup_dev;
#endif

}


#ifdef CONFIG_SNSC_ADD_NBL_TLB
static snsc_nbl_tlb_t cxd4707_nbl_tlb[] = {
	{CXD4707_AHB_BASE,  CXD4707_AHB_SIZE,  CXD4707_AHB_MAPPED_BASE},
	{CXD4707_OPB0_BASE, CXD4707_OPB0_SIZE, CXD4707_OPB0_MAPPED_BASE},
	{CXD4707_OPB1_BASE, CXD4707_OPB1_SIZE, CXD4707_OPB1_MAPPED_BASE},
	{CXD4707_ROM_BASE, CXD4707_ROM_SIZE, CXD4707_ROM_MAPPED_BASE},
	{0, 0, 0}
};

void *cxd4707_ioremap(phys_addr_t paddr, unsigned long size)
{
	snsc_nbl_tlb_t *t;
	u_int32_t *vaddr = NULL;

	paddr = paddr & PAGE_MASK;
	for (t = cxd4707_nbl_tlb; t->size != 0; t++) {
		if ((paddr >= t->paddr) && 
		    ((paddr + size) <= (t->paddr + t->size))) {
			vaddr = (void *)(t->vaddr + (u_int32_t)(paddr - t->paddr));
			break;
		}
	}
	return vaddr;
}

int cxd4707_iounmap(void *paddr)
{
	snsc_nbl_tlb_t *t;
	int retval = -1;

	for (t = cxd4707_nbl_tlb; t->size != 0; t++) {
		if (((u_int32_t)paddr >= t->paddr) && 
		    ((u_int32_t)paddr < (t->paddr + t->size))) {
			retval = 0;
			break;
		}
	}
	return retval;
}
#endif /* CONFIG_SNSC_ADD_NBL_TLB */

#ifdef CONFIG_EXCEPTION_MONITOR
int exception_check_mode = 3;
void 
set_exception_check(char *str)
{
	get_option(&str, &exception_check_mode);
}
__setup("em=", set_exception_check);
void (*exception_check)(int mode, struct pt_regs *regs) = NULL;
#endif

#ifdef CONFIG_PCI_AUTO
/* fixup routine for pciauto_bus_scan() in pci_auto.c */
static void 
cxd4707_pciauto_fixup_dev(struct pci_controller *hose,
			  int current_bus, int pci_devfn,
			  int *pciauto_upper_memspc, 
			  int *pciauto_upper_iospc)
{
	unsigned short vid, did;

	early_read_config_word(hose, current_bus, pci_devfn, 
			       PCI_VENDOR_ID, &vid);
	early_read_config_word(hose, current_bus, pci_devfn, 
			       PCI_DEVICE_ID, &did);
#ifdef CONFIG_SONY_WG_ADDR_23_21_WORKAROUND
#define WG_BAR_SIZE		(0x200000)
#define WG_FIXED_BAR_SIZE	(0x1000000)

	if ((vid == PCI_VENDOR_ID_SONY) && (did == PCI_DEVICE_ID_SONY_WG)) {
		unsigned int bar0;
		early_read_config_dword(hose, current_bus, pci_devfn,
					PCI_BASE_ADDRESS_0, &bar0);
		bar0 &= PCI_BASE_ADDRESS_MEM_MASK;
		if (bar0 == 0) {
			/* something wrong */
			return;
		}

		bar0 += WG_BAR_SIZE;	/* end of bar */
		bar0 = (bar0 - WG_FIXED_BAR_SIZE) & ~(WG_FIXED_BAR_SIZE - 1);

		early_write_config_dword(hose, current_bus, pci_devfn,
					 PCI_BASE_ADDRESS_0, bar0);
		*pciauto_upper_memspc = bar0;
	}
#endif /* CONFIG_SONY_WG_ADDR_23_21_WORKAROUND */
}

#endif /* CONFIG_PCI_AUTO */
