/*
 * cxd4707.h
 *
 *  Definitions for CXD4707 with PPC440 core.
 *
 *  Copyright 2003,2004 Sony Corporation.
 *
 *  Copyright 2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef __KERNEL__
#ifndef __ASM_CXD4707_H__
#define __ASM_CXD4707_H__

#include <linux/config.h>
#include <platforms/ibm440.h>

/* 
 * CXD-4707 revision 
 */
#define CXD4707_ES_1_0		0x10
#define CXD4707_ES_1_1		0x11
#define CXD4707_ES_2_0		0x20
#define CXD4707_ES_2_1		0x21
#define CXD4707_UNKNOWN		0xff

/* System Clock */
#define CXD4707_PCICLK		(33333333) 		/*  33.33 Mhz */
#define CXD4707_CPUCLK		(CXD4707_PCICLK*36/3)	/* 400.0 Mhz */
#define CXD4707_CPUCLK_266	(CXD4707_PCICLK*36/4.5)	/* 266.66 Mhz */
#define CXD4707_PLBCLK		(CXD4707_PCICLK*36/9)	/* 133.33 Mhz */
#define CXD4707_OPBCLK		(CXD4707_PCICLK*36/18)	/*  66.66 Mhz */
#define CXD4707_AHBCLK		(CXD4707_PCICLK*36/9)	/* 133.33 Mhz */
#define CXD4707_EBCCLK		(CXD4707_PCICLK*36/36)	/*  33.33 Mhz */


/* 
 * Clock, Power-On, Reset Registers (CPR) 
 */
#define DCRN_CPR0_CFGADDR	0x000c
#define DCRN_CPR0_CFGDATA	0x000d

#define DCRN_CPR0_PLL		0x020
#define DCRN_CPR0_PLLCS		0x040
#define DCRN_CPR0_EBC0		0x060
#define DCRN_CPR0_MSPROD0	0x080
#ifdef CONFIG_CXD4707_ES_2_0
#define DCRN_CPR0_SCIDIV0	0x0a0
#define DCRN_CPR0_SCIDIV1	0x0a1
#endif

/* 
 * System DCR Registers (SDR) 
*/
#define DCRN_SDR0_CFGADDR	0x000e
#define DCRN_SDR0_CFGDATA	0x000f

#define DCRN_SDR0_DDRDL0	0x0e0
#define DCRN_SDR0_PST   	0x100
#define DCRN_SDR0_AHB_MID	0x110
#define DCRN_SDR0_AHB_REQX  	0x111
#define DCRN_SDR0_UART0 	0x120
#define DCRN_SDR0_UART1 	0x121
#define DCRN_SDR0_UART2 	0x122
#define DCRN_SDR0_UART3 	0x123
#define DCRN_SDR0_WOLEN 	0x130
#define DCRN_SDR0_MSD1PDDIS	0x140
#define DCRN_SDR0_CP440 	0x180
#define DCRN_SDR0_SRST0 	0x200
#define DCRN_SDR0_SRST1 	0x201
#define DCRN_SDR0_SLPIPE0 	0x220
#define DCRN_SDR0_AMP0  	0x240
#define DCRN_SDR0_MIRQ0 	0x260
#define DCRN_SDR0_MIRQ1 	0x261
#define DCRN_SDR0_PFC0  	0x280
#ifdef CONFIG_CXD4707_ES_2_0
#define DCRN_SDR0_PIO2FIFO_CH	0x290
#define DCRN_SDR0_CHAT		0x300
#define DCRN_SDR0_NBURST	0x310
#define DCRN_SDR0_DREQRST	0x320
#endif

/* 
 * Pin Function bits in DCRN_SDR0_PFC0 
 */
#define PFC0_GRP0_MASK		0x00000030
#define PFC0_GRP0_FUNC(reg)	((reg & PFC0_GRP0_MASK) >> 4)
#define PFC0_GRP0_GPIO		0x0
#define PFC0_GRP0_IDE0		0x1
#define PFC0_GRP0_NAND16	0x1 /* ES2.0 */
#define PFC0_GRP0_NAND		0x2
#define PFC0_GRP0_NAND8		0x2 /* ES2.0 */
#define PFC0_GRP0_rsvd		0x3

#define PFC0_GRP1_MASK		0x0000000c
#define PFC0_GRP1_FUNC(reg)	((reg & PFC0_GRP1_MASK) >> 2)
#define PFC0_GRP1_EBC32		0x0
#define PFC0_GRP1_rsvd		0x1
#define PFC0_GRP1_GPIO		0x2 /* GPIO24-41 (ES1.x),  GPIO16-41 (ES-2.0) */
#define PFC0_GRP1_TRACE		0x3

#define PFC0_GRP2_MASK		0x00000002
#define PFC0_GRP2_FUNC(reg)	((reg & PFC0_GRP2_MASK) >> 1)
#define PFC0_GRP2_MODEM		0x0 /* no UART2_RI# if PFC0_GRP0_NAND */
#define PFC0_GRP2_EBC		0x1
#define PFC0_GRP2_NAND		0x2
#define PFC0_GRP2_NAND8		0x2 /* ES2.0 */
#define PFC0_GRP2_NAND16	0x3 /* ES2.0 */

#define PFC0_GRP3_MASK		0x00000001
#define PFC0_GRP3_FUNC(reg)	(reg & PFC0_GRP3_MASK)
#define PFC0_GRP3_USBH		0x0
#define PFC0_GRP3_USBD		0x1

#define PFC0_GRP4_MASK		0x00000040
#define PFC0_GRP4_FUNC(reg)	((reg & PFC0_GRP4_MASK) >> 6)
#define PFC0_GRP4_SCI		0x0
#define PFC0_GRP4_UART1		0x1

#define PFC0_GRP5_MASK		0x00000080
#define PFC0_GRP5_FUNC(reg)	((reg & PFC0_GRP5_MASK) >> 7)
#define PFC0_GRP5_SSI		0x0
#define PFC0_GRP5_SCP		0x1

#define PFC0_GRP6_MASK		0x00000200
#define PFC0_GRP6_FUNC(reg)	((reg & PFC0_GRP6_MASK) >> 9)
#define PFC0_GRP6_GPIO		0x0
#define PFC0_GRP6_PCI		0x1

#define PFC0_GRP7_MASK		0x00000100
#define PFC0_GRP7_FUNC(reg)	((reg & PFC0_GRP7_MASK) >> 8)
#define PFC0_GRP7_GPIO		0x0
#define PFC0_GRP7_PCI		0x1

/* 
 * DRAM controller DCRs 
 */
#define DCRN_SDRAM0_CFGADDR	0x010
#define DCRN_SDRAM0_CFGDATA	0x011

#define DCRN_SDRAM0_BESRA  	0x0000 // Bus Error Status Register A
#define DCRN_SDRAM0_BESRAM 	0x0004 //
#define DCRN_SDRAM0_BESRB  	0x0008 // Bus Error Status Register B
#define DCRN_SDRAM0_BESRBM 	0x000C //
#define DCRN_SDRAM0_BEAR   	0x0010 // Bus Error Address Register
#define DCRN_SDRAM0_WMIRQ  	0x0011 // Master Write Interrupt
#define DCRN_SDRAM0_WMIRQM 	0x0012 //
#define DCRN_SDRAM0_PLBOPT 	0x0018 // PLB Slave Interface Options
#define DCRN_SDRAM0_MCOPT1 	0x0020 // Memory Controller Options 1
#define DCRN_SDRAM0_MCOPT2 	0x0021 // Memory Controller Options 2
#define DCRN_SDRAM0_DDROPT 	0x0022 // DDR_SDRAM Device Options
#define DCRN_SDRAM0_MCSTAT 	0x0024 // Memory Controller Status
#define DCRN_SDRAM0_RTR    	0x0030 // Refresh Timer Register
#define DCRN_SDRAM0_PMIT   	0x0034 // Power Management Idle Timer
#define DCRN_SDRAM0_PUABA  	0x0038 // PLB UA Bus Base Address
#define DCRN_SDRAM0_MB0CF  	0x0040 // Memory Bank 0 Configuration
#define DCRN_SDRAM0_MB1CF  	0x0044 // Memory Bank 1 Configuration
#define DCRN_SDRAM0_MB2CF  	0x0048 // Memory Bank 2 Configuration
#define DCRN_SDRAM0_MB3CF  	0x004C // Memory Bank 3 Configuration
#define DCRN_SDRAM0_SDTR1  	0x0080 // DDR_SDRAM Timing Register 1
#define DCRN_SDRAM0_SDTR2  	0x0081 // DDR_SDRAM Timing Register 2
#define DCRN_SDRAM0_CLKTR  	0x0082 // DDR_SDRAM Clock Timing Register
#define DCRN_SDRAM0_WDRTR  	0x0083 // Write Data, DQS, DM Clock Timing Register
#define DCRN_SDRAM0_DLYCAL 	0x0084 // Delay Line Calibration Register
#define DCRN_SDRAM0_ECCCR  	0x0098 // ECC Error Status
#define DCRN_SDRAM0_CID    	0x00A4 // Core ID Register
#define DCRN_SDRAM0_RID    	0x00A8 // Revision ID Register

#define SDRAM_CONFIG_BANK_ENABLE	0x00000001
#define SDRAM_CONFIG_SIZE_MASK		0x000e0000
#define SDRAM_CONFIG_BANK_SIZE(reg)	((reg & SDRAM_CONFIG_SIZE_MASK) >> 17)
#define SDRAM_CONFIG_SIZE_8M		0x00000001
#define SDRAM_CONFIG_SIZE_16M		0x00000002
#define SDRAM_CONFIG_SIZE_32M		0x00000003
#define SDRAM_CONFIG_SIZE_64M		0x00000004
#define SDRAM_CONFIG_SIZE_128M		0x00000005
#define SDRAM_CONFIG_SIZE_256M		0x00000006
#define SDRAM_CONFIG_SIZE_512M		0x00000007

/* 
 * External BUS Controller Register 
 */
#define DCRN_EBC0_CFGADDR	0x0012
#define DCRN_EBC0_CFGDATA	0x0013

#define DCRN_EBC0_PB0CR		0x000 // Peripheral Bank 0 Configuration Register
#define DCRN_EBC0_PB1CR		0x001 // Peripheral Bank 1 Configuration Register
#define DCRN_EBC0_PB2CR		0x002 // Peripheral Bank 2 Configuration Register
#define DCRN_EBC0_PB3CR		0x003 // Peripheral Bank 3 Configuration Register
#define DCRN_EBC0_PB4CR		0x004 // Peripheral Bank 4 Configuration Register
#define DCRN_EBC0_PB5CR		0x005 // Peripheral Bank 5 Configuration Register
#define DCRN_EBC0_PB6CR		0x006 // Peripheral Bank 6 Configuration Register
#define DCRN_EBC0_PB7CR		0x007 // Peripheral Bank 7 Configuration Register
#define DCRN_EBC0_PB0AP		0x010 // Peripheral Bank 0 Access Parameter Registers
#define DCRN_EBC0_PB1AP		0x011 // Peripheral Bank 1 Access Parameter Registers
#define DCRN_EBC0_PB2AP		0x012 // Peripheral Bank 2 Access Parameter Registers
#define DCRN_EBC0_PB3AP		0x013 // Peripheral Bank 3 Access Parameter Registers
#define DCRN_EBC0_PB4AP		0x014 // Peripheral Bank 4 Access Parameter Registers
#define DCRN_EBC0_PB5AP		0x015 // Peripheral Bank 5 Access Parameter Registers
#define DCRN_EBC0_PB6AP		0x016 // Peripheral Bank 6 Access Parameter Registers
#define DCRN_EBC0_PB7AP		0x017 // Peripheral Bank 7 Access Parameter Registers
#define DCRN_EBC0_BEAR 		0x020 // Peripheral Bus Error Address Register
#define DCRN_EBC0_BESR 		0x021 // Peripheral Bus Error Status Register
#define DCRN_EBC0_CFG  		0x023 // External Bus Control Register
#define DCRN_EBC0_CID  		0x024 // EBCO ID Register

/* 
 * External BUS Master Interface Controller 
 */
/* see ibm440.h */
#define DCRN_EBM0_MISCS	0x010 // EBMI Miscellaneous Status 0

/* 
 * OPB to PLB Bridge 
 */
#define DCRN_O2P0_BCTRL		0x00A8 // OPB to PLB Bridge 0 Control Register
#define DCRN_O2P0_BSTAT		0x00A9 // OPB to PLB Bridge 0 Status Register
#define DCRN_O2P0_BEARL		0x00AA // OPB to PLB Bridge 0 Error Address Reg. Low
#define DCRN_O2P0_BEARH		0x00AB // OPB to PLB Bridge 0 Error Address Reg. High
#define DCRN_O2P0_REVID		0x00AC // OPB to PLB Bridge 0 Revision ID Register

#define DCRN_O2P1_BCTRL		0x0078 // OPB to PLB Bridge 1 Control Register
#define DCRN_O2P1_BSTAT		0x0079 // OPB to PLB Bridge 1 Status Register
#define DCRN_O2P1_BEARL		0x007A // OPB to PLB Bridge 1 Error Address Reg. Low
#define DCRN_O2P1_BEARH		0x007B // OPB to PLB Bridge 1 Error Address Reg. High
#define DCRN_O2P1_REVID		0x007C // OPB to PLB Bridge 1 Revision ID Register

/* 
 * PLB Arbiter 
 */
#define DCRN_PLB0_PCBI		0x0080 // PLB Crossbar ID/Revision Register
#define DCRN_PLB0_P0ACR		0x0081 // PLB0 Arbiter Control Register
#define DCRN_PLB0_P0ESRL	0x0082 // PLB0 Error Status Register Low
#define DCRN_PLB0_P0ESRH	0x0083 // PLB0 Error Status Register High
#define DCRN_PLB0_P0EARL	0x0084 // PLB0 Bus Error Address Register Low
#define DCRN_PLB0_P0EARH	0x0085 // PLB0 Bus Error Address Register High
#define DCRN_PLB0_P0ESRLS	0x0086 // PLB0 Bus Error Status Register Low
#define DCRN_PLB0_P0ESRHS	0x0087 // PLB0 Bus Error Status Register High
#define DCRN_PLB0_PCBC		0x0088 // PLB Crossbar Control Register
#define DCRN_PLB0_P1ACR		0x0089 // PLB1 Arbiter Control Register
#define DCRN_PLB0_P1ESRL	0x008A // PLB1 Error Status Register Low
#define DCRN_PLB0_P1ESRH	0x008B // PLB1 Error Status Register High
#define DCRN_PLB0_P1EARL	0x008C // PLB1 Bus Error Address Register Low
#define DCRN_PLB0_P1EARH	0x008D // PLB1 Bus Error Address Register High
#define DCRN_PLB0_P1ESRLS	0x008E // PLB1 Bus Error Status Register Low
#define DCRN_PLB0_P1ESRHS	0x008F // PLB1 Bus Error Status Register High

/* 
 * PLB to OPB Bridge Register 
 */
#define DCRN_P2O0_BESR0		0x0090 // P2O 0 Error Status Register 0 (MD:0,1,2,3)
#define DCRN_P2O0_BESR0S	0x0091 // P2O 0 Error Status Register 0 (MD:0,1,2,3) SET
#define DCRN_P2O0_BEARL		0x0092 // P2O 0 Error Address Register
#define DCRN_P2O0_BEARH		0x0093 // P2O 0 Error Address Register
#define DCRN_P2O0_BESR1		0x0094 // P2O 0 Error Status Register 1 (MD:4,5,6,7)
#define DCRN_P2O0_BESR1S	0x0095 // P2O 0 Error Status Register 1 (MD:4,5,6,7) SET
#define DCRN_P2O0_CONFG		0x0096 // P2O 0 Error Configuration Register
#define DCRN_P2O0_CONFGS	0x0097 // P2O 0 Error Configuration Register SET
#define DCRN_P2O0_LATENCY	0x0098 // P2O 0 Error Burst Latency Timer
#define DCRN_P2O0_LATENCYS	0x0099 // P2O 0 Error Burst Latency Timer SET
#define DCRN_P2O0_REVID		0x009A // P2O 0 Revision ID Register

#define DCRN_P2O1_BESR0		0x00F0 // P2O 1 Error Status Register 0 (MD:0,1,2,3)
#define DCRN_P2O1_BESR0S	0x00F1 // P2O 1 Error Status Register 0 (MD:0,1,2,3) SET
#define DCRN_P2O1_BEARL		0x00F2 // P2O 1 Error Address Register
#define DCRN_P2O1_BEARH		0x00F3 // P2O 1 Error Address Register
#define DCRN_P2O1_BESR1		0x00F4 // P2O 1 Error Status Register 1 (MD:4,5,6,7)
#define DCRN_P2O1_BESR1S	0x00F5 // P2O 1 Error Status Register 1 (MD:4,5,6,7) SET
#define DCRN_P2O1_CONFG		0x00F6 // P2O 1 Error Configuration Register
#define DCRN_P2O1_CONFGS	0x00F7 // P2O 1 Error Configuration Register SET
#define DCRN_P2O1_LATENCY	0x00F8 // P2O 1 Error Burst Latency Timer
#define DCRN_P2O1_LATENCYS	0x00F9 // P2O 1 Error Burst Latency Timer SET
#define DCRN_P2O1_REVID		0x00FA // P2O 1 Revision ID Register

/* 
 * UIC 
 */
#define DCRN_UIC0_BASE		0xc0
#define DCRN_UIC1_BASE		0xd0
#define DCRN_UIC2_BASE		0xe0
#define UIC0			DCRN_UIC0_BASE
#define UIC1			DCRN_UIC1_BASE
#define UIC2			DCRN_UIC2_BASE

#define UIC0_UIC1NC		30	/* UIC1 non-critical interrupt */
#define UIC0_UIC1CR		31	/* UIC1 critical interrupt */

#define UIC0_UIC2NC		28	/* UIC2 non-critical interrupt */
#define UIC0_UIC2CR		29	/* UIC2 critical interrupt */

#undef NR_UICS
#define NR_UICS 3
#define UIC_CASCADE1_MASK	0x0003	/* bits 30,31 */
#define UIC_CASCADE2_MASK	0x000c	/* bits 28,29 */


/* 
 * DMAC
 */
#define DCRN_DMA0_BASE		0x100
#define DCRN_DMA1_BASE		0x108
#define DCRN_DMA2_BASE		0x110
#define DCRN_DMA3_BASE		0x118
#define DCRN_DMA0_SR_BASE	0x120

#define DCRN_DMA4_BASE		0x140
#define DCRN_DMA5_BASE		0x148
#define DCRN_DMA6_BASE		0x150
#define DCRN_DMA7_BASE		0x158
#define DCRN_DMA1SR_BASE	0x160

#define DCRN_DMACR4		(DCRN_DMA4_BASE + 0x0)	/* DMA Channel Control 4 */
#define DCRN_DMACT4		(DCRN_DMA4_BASE + 0x1)  /* DMA Count 4 */
#define DCRN_DMASAH4		(DCRN_DMA4_BASE + 0x2)	/* DMA Src Addr High 4 */
#define DCRN_DMASA4		(DCRN_DMA4_BASE + 0x3)	/* DMA Src Addr Low 4 */
#define DCRN_DMADAH4		(DCRN_DMA4_BASE + 0x4)	/* DMA Dest Addr High 4 */
#define DCRN_DMADA4		(DCRN_DMA4_BASE + 0x5)	/* DMA Dest Addr Low 4 */
#define DCRN_ASGH4		(DCRN_DMA4_BASE + 0x6)	/* DMA SG Desc Addr High 4 */
#define DCRN_ASG4		(DCRN_DMA4_BASE + 0x7)	/* DMA SG Desc Addr Low 4 */

#define DCRN_DMACR5		(DCRN_DMA5_BASE + 0x0)	/* DMA Channel Control 5 */
#define DCRN_DMACT5		(DCRN_DMA5_BASE + 0x1)  /* DMA Count 5 */
#define DCRN_DMASAH5		(DCRN_DMA5_BASE + 0x2)	/* DMA Src Addr High 5 */
#define DCRN_DMASA5		(DCRN_DMA5_BASE + 0x3)	/* DMA Src Addr Low 5 */
#define DCRN_DMADAH5		(DCRN_DMA5_BASE + 0x4)	/* DMA Dest Addr High 5 */
#define DCRN_DMADA5		(DCRN_DMA5_BASE + 0x5)	/* DMA Dest Addr Low 5 */
#define DCRN_ASGH5		(DCRN_DMA5_BASE + 0x6)	/* DMA SG Desc Addr High 5 */
#define DCRN_ASG5		(DCRN_DMA5_BASE + 0x7)	/* DMA SG Desc Addr Low 5 */

#define DCRN_DMACR6		(DCRN_DMA6_BASE + 0x0)	/* DMA Channel Control 6 */
#define DCRN_DMACT6		(DCRN_DMA6_BASE + 0x1)  /* DMA Count 6 */
#define DCRN_DMASAH6		(DCRN_DMA6_BASE + 0x2)	/* DMA Src Addr High 6 */
#define DCRN_DMASA6		(DCRN_DMA6_BASE + 0x3)	/* DMA Src Addr Low 6 */
#define DCRN_DMADAH6		(DCRN_DMA6_BASE + 0x4)	/* DMA Dest Addr High 6 */
#define DCRN_DMADA6		(DCRN_DMA6_BASE + 0x5)	/* DMA Dest Addr Low 6 */
#define DCRN_ASGH6		(DCRN_DMA6_BASE + 0x6)	/* DMA SG Desc Addr High 6 */
#define DCRN_ASG6		(DCRN_DMA6_BASE + 0x7)	/* DMA SG Desc Addr Low 6 */

#define DCRN_DMACR7		(DCRN_DMA7_BASE + 0x0)	/* DMA Channel Control 7 */
#define DCRN_DMACT7		(DCRN_DMA7_BASE + 0x1)  /* DMA Count 7 */
#define DCRN_DMASAH7		(DCRN_DMA7_BASE + 0x2)	/* DMA Src Addr High 7 */
#define DCRN_DMASA7		(DCRN_DMA7_BASE + 0x3)	/* DMA Src Addr Low 7 */
#define DCRN_DMADAH7		(DCRN_DMA7_BASE + 0x4)	/* DMA Dest Addr High 7 */
#define DCRN_DMADA7		(DCRN_DMA7_BASE + 0x5)	/* DMA Dest Addr Low 7 */
#define DCRN_ASGH7		(DCRN_DMA7_BASE + 0x6)	/* DMA SG Desc Addr High 7 */
#define DCRN_ASG7		(DCRN_DMA7_BASE + 0x7)	/* DMA SG Desc Addr Low 7 */

#define DCRN_DMA1_SR		(DCRN_DMA1SR_BASE + 0x0) /* DMA1 Status Register */
#define DCRN_DMA1_ASGC		(DCRN_DMA1SR_BASE + 0x3) /* DMA1 Scatter/Gather */
#define DCRN_DMA1_SLP		(DCRN_DMA1SR_BASE + 0x5) /* DMA1 Sleep Register */
#define DCRN_DMA1_POL		(DCRN_DMA1SR_BASE + 0x6) /* DMA1 Polarity Register */

/* DMAC0 Int */
#define DMA0_INT		12
#define DMA1_INT		13
#define DMA2_INT		14
#define DMA3_INT		15

/* DMAC1 Int */
#define DMA4_INT		23
#define DMA5_INT		24
#define DMA6_INT		25
#define DMA7_INT		26

/* 
 * MAL 
 */
#define DCRN_MAL_BASE		0x180
#define DCRN_MALMDBR(base)	(base + 0x3)	/* MAL Debug Register */

/* 
 * PLB to AHB Bridge Register 
 */
#define DCRN_P2A0_SEARAU	0x200 // Upper 4 bits of the offending address.
#define DCRN_P2A0_SEARAL	0x204 // Lower 32 bits of the offending address.
#define DCRN_P2A0_SESR		0x208 // System Error Status Register.
#define DCRN_P2A0_TOPADDR	0x20C // Max. AHB address.
#define DCRN_P2A0_BOTADDR	0x210 // Min. AHB address.
#define DCRN_P2A0_REVID		0x214 // Revision ID Register.
#define DCRN_P2A0_ATTR		0x218 // Specifies AHB transaction attributes.

/* 
 * Real Time Clock 
 */
#define DCRN_RTC0_SEC		0x0300 // Seconds
#define DCRN_RTC0_SECALM	0x0301 // Seconds Alarm
#define DCRN_RTC0_MIN		0x0302 // Minutes
#define DCRN_RTC0_MINALM	0x0303 // Minutes Alarm
#define DCRN_RTC0_HRS		0x0304 // Hours
#define DCRN_RTC0_HRSALM	0x0305 // Hours Alarm
#define DCRN_RTC0_DAY		0x0306 // Day of Week
#define DCRN_RTC0_DATE		0x0307 // Date of Month
#define DCRN_RTC0_MON		0x0308 // Month
#define DCRN_RTC0_YEAR		0x0309 // Year
#define DCRN_RTC0_REGA		0x030A // RegisterA
#define DCRN_RTC0_REGB		0x030B // RegisterB
#define DCRN_RTC0_REGC		0x030C // RegisterC
#define DCRN_RTC0_REGD		0x030D // RegisterD
#define DCRN_RTC0_CENT		0x030E // Century
#define DCRN_RTC0_ARRY0F	0x030F // Register Array
#define DCRN_RTC0_ARRY40	0x0340 // Register Array
#define DCRN_RTC0_ARRY5A	0x035A // Register Array
#define DCRN_RTC0_ARRYFF	0x03FF // Register Array

#define	CXD4707_RTC_NVRAM	0x0310
#define	CXD4707_RTC_NVRAM_SIZE	242

/* 
 * Power Management 
 */
#define DCRN_CPMER		0x00B0 // CPM Enable Register
#define DCRN_CPMFR		0x00B1 // CPM Force Register
#define DCRN_CPMSR		0x00B2 // CPM Status Register

#define IBM_CPM_IIC0		0x80000000	/* IIC0 interface */
#define IBM_CPM_IIC1		0x40000000	/* IIC1 interface */
#define IBM_CPM_PCI		0x20000000	/* PCI bridge */
#define IBM_CPM_UART2		0x10000000	/* serial port 2 */

#define IBM_CPM_UART3		0x08000000	/* serial port 3 */
#define IBM_CPM_CPU		0x02000000	/* processor core */
#define IBM_CPM_DMA0		0x01000000	/* DMA controller 0 */

#define IBM_CPM_BGO0		0x00800000	/* PLB to OPB bus arbiter 0 */
#define IBM_CPM_BGI0		0x00400000	/* OPB to PLB bridge 0 */
#define IBM_CPM_EBC		0x00200000	/* External Bux Controller */
#define IBM_CPM_EBM		0x00100000	/* Ext Bus Master Interface */

#define IBM_CPM_DMC		0x00080000	/* DDR SDRAM peripheral controller */
#define IBM_CPM_PLB		0x00040000	/* PLB bus arbiter */
#define IBM_CPM_BGO1		0x00020000	/* PLB to OPB bus arbiter 1 */
#define IBM_CPM_BGI1		0x00010000	/* OPB to PLB bridge 1 */

#define IBM_CPM_DMA1		0x00004000	/* DMA controller 1 */
#define IBM_CPM_UIC2		0x00002000	/* Universal Interrupt Controller 2 */
#define IBM_CPM_UIC1		0x00001000	/* Universal Interrupt Controller 1 */

#define IBM_CPM_GPIO0		0x00000800	/* General Purpose IO 0 */
#define IBM_CPM_GPT		0x00000400	/* General Purpose Timers  */
#define IBM_CPM_UART0		0x00000200	/* serial port 0 */
#define IBM_CPM_UART1		0x00000100	/* serial port 1 */

#define IBM_CPM_UIC0		0x00000080	/* Universal Interrupt Controller */
#define IBM_CPM_PIO2FF		0x00000040	/* PIO2FIFO */
#define IBM_CPM_EMAC3		0x00000020	/* EMAC3 */
#define IBM_CPM_GPIO1		0x00000010	/* General Purpose IO 1 */

#define IBM_CPM_AH2PL		0x00000008	/* AHB2PLB */
#define IBM_CPM_USBH		0x00000002	/* USBHOSTAHB */
#define IBM_CPM_USBDL		0x00000001	/* USBDEVICEAHB */

#if 0				/* XXX: where do we use it? */
#define DFLT_IBM4xx_PM		~(IBM_CPM_UIC | IBM_CPM_UIC1 | IBM_CPM_CPU \
				| IBM_CPM_EBC | IBM_CPM_SRAM | IBM_CPM_BGO \
				| IBM_CPM_EBM | IBM_CPM_PLB | IBM_CPM_OPB \
				| IBM_CPM_TMRCLK | IBM_CPM_DMA | IBM_CPM_PCI)
#endif

/* 
 * Memory Stick / Memory Stick PRO Host Controller 
 */
#define MSP0_REG_BASE		0xc0000000ULL
#define MSP0_REG_SIZE		0x40

#define MSP0_CTRL		0x000 // Control Register
#define MSP0_SYS		0x004 // System Register
#define MSP0_FLAG		0x008 // Flag Register
#define MSP0_MEMC		0x00c // Memory Control Register
#define MSP0_GEN01		0x010 // General Register0,1
#define MSP0_GEN23		0x014 // General Register2,3
#define MSP0_GEN45		0x018 // General Register4,5
#define MSP0_TIMER		0x01c // Timer Control Register
#define MSP0_INSTQ		0x020 // Instruction Queue
#define MSP0_GFIFO		0x024 // General Data FIFO
#define MSP0_PBUF		0x028 // PageBuffer
#define MSP0_VERSION		0x02c // Version Register
#define MSP0_CMDREG		0x030 // Command Register
#define MSP0_DATREG		0x034 // Data Register
#define MSP0_STAREG		0x038 // Status Register
#define MSP0_HSYS		0x03c // System Register

/* 
 * NAND Flash 
 */
#if defined(CONFIG_CXD4707_ES_1_0) || defined(CONFIG_CXD4707_ES_1_1)

#define NAND0_REG_BASE		0xc0000100ULL
#define NAND0_REG_SIZE		0x100

#define NAND0_CFG		0x00 // EMC NAND Flash Configuration
#define NAND0_IMM		0x04 // EMC NAND Flash Immediate Access
#define NAND0_PRM		0x08 // EMC NAND Flash timing parameter
#define NAND0_RADDR		0x0C // EMC NAND Flash Row Address
#define NAND0_ECC		0x10 // EMC NAND Flash ECC
#define NAND0_TCTRL		0x14 // EMC NAND Flash Auto TP Tuning
#define NAND0_RWDR		0x18 // EMC NAND Flash Data R/W Register

#define NAND0_INT_N		54 // UIC1, No. 22
#define NAND0_RDY		55 // UIC1, No. 23
#define NAND0_INT		(NAND0_RDY)

#else	/* defined(CONFIG_CXD4707_ES_1_0) || defined(CONFIG_CXD4707_ES_1_1) */

#define NAND0_REG_BASE		0xc0010000ULL
#define NAND0_REG_SIZE		0x00060000

#define NAND0_INT_N		54 // UIC1, No. 22
#define NAND0_RDY		55 // UIC1, No. 23
#define NAND0_INT		(NAND0_RDY)

#endif	/* defined(CONFIG_CXD4707_ES_1_0) || defined(CONFIG_CXD4707_ES_1_1) */

#ifdef CONFIG_CWING
#undef NAND0_REG_BASE
#define NAND0_REG_BASE		0x0f0000000ULL
#undef NAND0_REG_SIZE
#define NAND0_REG_SIZE		0x00060000
#undef NAND0_INT 
#define NAND0_INT   		 85	// UIC2, No. 21
#endif	/* CONFIG_CWING */

/* 
 * USB11 OHCI Host 
 */
#ifdef CONFIG_CXD4707_ES_2_0
#define USBH0_REG_BASE		0xc0000400ULL
#else
#define USBH0_REG_BASE		0xc0000200ULL
#endif
#define USBH0_REG_SIZE		0xe00

#define USBH0_HCREVISION		0x00
#define USBH0_HCCONTROL			0x04
#define USBH0_HCCOMMANDSTATUS		0x08
#define USBH0_HCINTERRUPTSTATUS		0x0C
#define USBH0_HCINTERRUPTENABLE		0x10
#define USBH0_HCINTERRUPTDISABLE	0x14
#define USBH0_HCHCCA			0x18
#define USBH0_HCPERIODCURRENTED		0x1C
#define USBH0_HCCONTROLHEADED		0x20
#define USBH0_HCCONTROLCURRENTED	0x24
#define USBH0_HCBULKHEADED		0x28
#define USBH0_HCBULKCURRENTED		0x2C
#define USBH0_HCDONEHEAD		0x30
#define USBH0_HCFMINTERVAL		0x34
#define USBH0_HCFMREMAINING		0x38
#define USBH0_HCFMNUMBER		0x3C
#define USBH0_HCPERIODICSTART		0x40
#define USBH0_HCLSTHRESHOLD		0x44
#define USBH0_HCRHDESCRIPTORA		0x48
#define USBH0_HCRHDESCRIPTORB		0x4C
#define USBH0_HCRHSTATUS		0x50
#define USBH0_HCRHPORTSTATUS1		0x54
#define USBH0_HCRHPORTSTATUS2		0x58
#define USBH0_HCRHPORTSTATUS3		0x5C
#define USBH0_HCRHPORTSTATUS4		0x60

/* 
 * USB0FH07 (USB Target) 
 */
#define USBD0_REG_BASE		0xc0001000ULL
#define USBD0_REG_SIZE		0x1000

#define USBD0_SIR		0x000 // Specification Identify Register
#define USBD0_BIR		0x004 // Block Identify Register
#define USBD0_BCR		0x008 // Block Control Register
#define USBD0_CMDR		0x00c // Command Register
#define USBD0_IIR		0x010 // Interrupt Identify Register
#define USBD0_IER		0x014 // Interrupt Enable Register
#define USBD0_FNR		0x018 // Frame Number Register
#define USBD0_WWR		0x040 // Write Wait Register
#define USBD0_RWR		0x044 // Read Wait Register

#define USBD0_EP0CR		0x080 // Endpoint #0 Control Register
#define USBD0_EP0SR		0x084 // Endpoint #0 Status Register
#define USBD0_EP0IER		0x088 // Endpoint #0 Interrupt Enable Register
#define USBD0_EP0BF1		0x090 // Endpoint #0 Buffer
#define USBD0_EP0BF2		0x0A0 // Endpoint #0 Buffer

#define USBD0_EP1CR		0x100 // Endpoint #1 Control Register
#define USBD0_EP1ER		0x104 // Endpoint #1 Event Register
#define USBD0_EP1IER		0x108 // Endpoint #1 Interrupt Enable Register
#define USBD0_EP1SR		0x10c // Endpoint #1 Status Register
#define USBD0_EP1TR		0x110 // Endpoint #1 Traffic Register
#define USBD0_EP1DI		0x114 // Endpoint #1 Data In
#define USBD0_EP1BUF		0x180 // Endpoint #1 Buffer

#define USBD0_EP2CR		0x200 // Endpoint #2 Control Register
#define USBD0_EP2ER		0x204 // Endpoint #2 Event Register
#define USBD0_EP2IER		0x208 // Endpoint #2 Interrupt Enable Register
#define USBD0_EP2SR		0x20c // Endpoint #2 Status Register
#define USBD0_EP2TR		0x210 // Endpoint #2 Traffic Register
#define USBD0_EP2DO		0x214 // Endpoint #2 Data Out
#define USBD0_EP2BUF		0x280 // Endpoint #2 Buffer

#define USBD0_EP3CR		0x300 // Endpoint #3 Control Register
#define USBD0_EP3SR		0x304 // Endpoint #3 Status Register
#define USBD0_EP3IER		0x308 // Endpoint #3 Interrupt Enable Register
#define USBD0_EP3BUF		0x310 // Endpoint #3 Buffer

/* 
 * AHB to PLB Bridge 
 */
#define AHB2PLB0_REG_BASE	0xcfffff00ULL
#define AHB2PLB0_REG_SIZE	0x20

#define A2P0_BCNTL		0x00 // Bridge Control Register 
#define A2P0_BSTAT		0x04 // Bridge Status Register 
#define A2P0_BSTATM		0x08 // Bridge Interrupt Mask Register
#define A2P0_ERSR0		0x0C // Endianess Region Swap Register 0
#define A2P0_ERSR1		0x10 // Endianess Region Swap Register 1
#define A2P0_MSC		0x14 // Master Swap Control Registe
#define A2P0_WTEA		0x18 // PLB Write Transfer Error Address
#define A2P0_REVID		0x1C // AHB to PLB Revision ID Register

/* 
 * PLB-PCI bridge
 */
#define PCI_PLB_BASE		0x80000000LL
#define PCI_PLB_SIZE		0x40000000LL

/* DMA needs PLB address to r/w to PCI area */
#define TO_PLBADDR(x)		(PCI_PLB_BASE | (phys_addr_t)(x))

/* DMA needs to know what area it is tranferring to/from */
#define IS_PCI_REGION(x)	((PCI_PLB_BASE <= (x)) && \
				 (x) < (PCI_PLB_BASE + PCI_PLB_SIZE))

#if defined (CONFIG_CXD4707_ES_1_0) || defined (CONFIG_CXD4707_ES_1_1)
#  include <platforms/cxd4707_pci.h>
#elif defined (CONFIG_CXD4707_ES_2_0)
#  include <platforms/cxd4707_pcix.h>
#else
#  warning Unknown CXD4707 revision, cannot select PLB-PCI bridge
#endif

/* 
 * UART
 */
#define CXD4707_UART_BASE	0xE0000000ULL
#define CXD4707_UART0_ADDR	(CXD4707_UART_BASE + 0x200)
#define CXD4707_UART1_ADDR	(CXD4707_UART_BASE + 0x300)
#define CXD4707_UART2_ADDR	(CXD4707_UART_BASE + 0x0)
#define CXD4707_UART3_ADDR	(CXD4707_UART_BASE + 0x100)

#define UART0_INT		0
#define UART1_INT		1
#define UART2_INT		35
#define UART3_INT		36

/* 
 * OPB Arbiter 0
 */
#define CXD4707_OPB0_ADDR	0xE0000600ULL
#define CXD4707_OPB1_ADDR	0xE8000600ULL
#define CXD4707_OPB_SIZE	0x40

/* 
 * EMAC 
 */
#define CXD4707_EMAC0_ADDR	0xE0000800ULL
#define CXD4707_EMAC_SIZE	0x100

#define BL_MAC_WOL		61	/* WOL */
#define BL_MAL_SERR		32	/* MAL SERR */
#define BL_MAL_TXDE		33	/* MAL TXDE */
#define BL_MAL_RXDE		34	/* MAL RXDE */
#define BL_MAL_TXEOB		10	/* MAL TX EOB */
#define BL_MAL_RXEOB		11	/* MAL RX EOB */
#define BL_MAC_ETH0		60	/* MAC */

/* 
 * I2C
 */
#define CXD4707_IIC0_ADDR	0xE8000400ULL
#define CXD4707_IIC1_ADDR	0xE8000500ULL
#define CXD4707_IIC_SIZE	0x20

#define IIC_OWN			0x55
#define IIC_CLOCK		50

/* 
 * GPIO
 */
#define CXD4707_GPIO0_ADDR	0xe0000700ULL
#define CXD4707_GPIO1_ADDR	0xe0000780ULL
#define CXD4707_GPIO_SIZE	0x80

/* GPIO initial/multiplex info */
#define PPC4xx_GPIO0_BASE	CXD4707_GPIO0_ADDR
#define PPC4xx_GPIO1_BASE	CXD4707_GPIO1_ADDR
#define PPC4xx_IRQ_GPIO0(x)	(64 + (x))
#define PPC4xx_IRQ_GPIO1(x)	(66 + (x))

/* 
 * Genreral Purpose Timer
 */
#define CXD4707_GPT_ADDR	0xE0000A00ULL
#define CXD4707_GPT_SIZE	0x100

/* 
 * External Device 
 */
#define CXD4707_EXTDEV		0xf0000000ULL

#ifdef CONFIG_CXD4707_ES_2_0
/* 
 * SSI
 */
#define CXD4707_SSI_ADDR	0xc8000000ULL
#define CXD4707_SSI_SIZE	0x100

/* 
 * SCI
 */
#define CXD4707_SCI_ADDR	0xe0000400ULL
#define CXD4707_SCI_SIZE	0x20

#endif /* CONFIG_CXD4707_ES_2_0 */

/* 
 * SCP
 */
#define CXD4707_SCP0_ADDR	0xe8000000ULL
#define CXD4707_SCP1_ADDR	0xe8000100ULL
#define CXD4707_SCP_SIZE	0x8

/* 
 * I2C
 */
#define CXD4707_I2C0_ADDR	0xe8000400ULL
#define CXD4707_I2C1_ADDR	0xe8000500ULL
#define CXD4707_I2C_SIZE	0x20

/* 
 * PIO2FIFO
 */
#ifdef CONFIG_CXD4707_ES_2_0
#define CXD4707_PIO2FIFO0_ADDR	0xe8000080ULL
#define CXD4707_PIO2FIFO1_ADDR	0xe8000180ULL
#define CXD4707_PIO2FIFO2_ADDR	0xe8000480ULL
#else  /* CONFIG_CXD4707_ES_2_0 */
#define CXD4707_SCP0FIFO_ADDR	0xe8000080ULL
#define CXD4707_SCP1FIFO_ADDR	0xe8000180ULL
#define CXD4707_I2C0FIFO_ADDR	0xe8000480ULL
#define CXD4707_I2C1FIFO_ADDR	0xe8000580ULL
#endif /* CONFIG_CXD4707_ES_2_0 */
#define CXD4707_PIO2FIFO_SIZE	0x64

/* 
 * Linux dependednt part description
 */

#define EMAC_NUMS		1
#define UART_NUMS		4
#define IIC_NUMS		2
#define IIC0_IRQ		2
#define IIC1_IRQ		3
#define GPIO_NUMS		2

/* UART */
#define CXD4707_UART_OP(num)					\
	{							\
		.magic		= 0,				\
		.baud_base	= BASE_BAUD,			\
		.port		= 0,				\
		.irq		= UART##num##_INT,		\
		.flags		= (ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST), \
		.iomem_base	= (u8 *)CXD4707_UART##num##_ADDR, \
		.io_type	= SERIAL_IO_MEM,		\
	},

#ifdef CONFIG_CXD4707_UART_0
#define CXD4707_UART_OP0	CXD4707_UART_OP(0)
#define CXD4707_UART_NUM0	1
#else
#define CXD4707_UART_OP0
#define CXD4707_UART_NUM0	0
#endif

#ifdef CONFIG_CXD4707_UART_1
#define CXD4707_UART_OP1	CXD4707_UART_OP(1)
#define CXD4707_UART_NUM1	1
#else
#define CXD4707_UART_OP1
#define CXD4707_UART_NUM1	0
#endif
#ifdef CONFIG_CXD4707_UART_2
#define CXD4707_UART_OP2	CXD4707_UART_OP(2)
#define CXD4707_UART_NUM2	1
#else
#define CXD4707_UART_OP2
#define CXD4707_UART_NUM2	0
#endif
#ifdef CONFIG_CXD4707_UART_3
#define CXD4707_UART_OP3	CXD4707_UART_OP(3)
#define CXD4707_UART_NUM3	1
#else
#define CXD4707_UART_OP3
#define CXD4707_UART_NUM3	0
#endif

#if !defined(CONFIG_CXD4707_UART_0) && !defined(CONFIG_CXD4707_UART_1) && !defined(CONFIG_CXD4707_UART_2) && !defined(CONFIG_CXD4707_UART_3) 
#error CXD4707 UART: At least one UART is needed.
#endif

/* all list of uarts */
#define NUM_CXD4707_UART 	(CXD4707_UART_NUM0 + CXD4707_UART_NUM1 + CXD4707_UART_NUM2 + CXD4707_UART_NUM3)

#define CXD4707_UART_OPS	\
	CXD4707_UART_OP0	\
	CXD4707_UART_OP1	\
	CXD4707_UART_OP2	\
	CXD4707_UART_OP3

/* Interrupt Preset value */
#ifdef CONFIG_CXD4707_ES_2_0
#define UIC0_INITSENSE \
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 0: UART 0 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 1: UART 1 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 2: IIC 0 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 3: IIC 1 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 4: PCI-X Message In */	\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 5: PCI-X Cmd Wrt */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 6: PCI-X PM */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 7: PIO2FIFO  Channel 2*/	\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 8: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 9: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 10: MAL TX EOB */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 11: MAL RX EOB */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 12: DMA0 Chan 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 13: DMA0 Chan 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 14: DMA0 Chan 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 15: DMA0 Chan 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 16: GPT Timer 8 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 17: GPT Timer 9 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 18: GPT Timer 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 19: GPT Timer 4 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 20: GPT Timer 5 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 21: GPT Timer 6 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 22: GPT Timer 7 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 23: DMA1 Chan 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 24: DMA1 Chan 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 25: DMA1 Chan 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 26: DMA1 Chan 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 27: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 28: UIC2 NC Int */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 29: UIC2 Crit Int */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 30: UIC1 NC Int */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 31: UIC1 Crit Int */	

#define UIC1_INITSENSE \
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 32: MAL SERR */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 33: MAL TXDE */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 34: MAL RXDE */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 35: UART 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 36: UART 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 37: Ext Bus Ctrl */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 38: RTC */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 39: OPB->PLB 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 40: PCI INT 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 41: PCI INT 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 42: PCI INT 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 43: PCI INT 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 44: Ext Bus Mstr */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 45: AHB->PLB */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 46: OPB->PLB 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 47: SSI */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 48: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 49: SCP 0 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 50: SCP 1 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 51: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 52: PIO2FIFO Channel 0 */	\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 53: PIO2FIFO Channel 1 */	\
  (IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_NEG),	/* 54: NAND ND_INT_N */		\
  (IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 55: NAND ND_RDY */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 56: SCI */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 57: Reserved */		\
  (IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 58: PCI-X VPD */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 59: PCI-X System Err */	\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 60: EMAC 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 61: EMAC 0 WOL */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 62: USB Host */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 63: USB Device */
#else
#define UIC0_INITSENSE \
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 0: UART 0 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 1: UART 1 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 2: IIC 0 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 3: IIC 1 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 4: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 5: PCI Cmd Wrt */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 6: PCI PM */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 7: IIC 0 PIO2FIFO */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 8: IIC 0 PIO2FIFO */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 9: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 10: MAL TX EOB */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 11: MAL RX EOB */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 12: DMA0 Chan 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 13: DMA0 Chan 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 14: DMA0 Chan 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 15: DMA0 Chan 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 16: GPT Timer 8 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 17: GPT Timer 9 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 18: GPT Timer 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 19: GPT Timer 4 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 20: GPT Timer 5 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 21: GPT Timer 6 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 22: GPT Timer 7 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 23: DMA1 Chan 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 24: DMA1 Chan 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 25: DMA1 Chan 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 26: DMA1 Chan 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 27: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 28: UIC2 NC Int */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 29: UIC2 Crit Int */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 30: UIC1 NC Int */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 31: UIC1 Crit Int */	

#define UIC1_INITSENSE \
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 32: MAL SERR */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 33: MAL TXDE */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 34: MAL RXDE */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 35: UART 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 36: UART 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 37: Ext Bus Ctrl */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 38: RTC */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 39: OPB->PLB 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 40: PCI INT 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 41: PCI INT 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 42: PCI INT 2 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 43: PCI INT 3 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 44: Ext Bus Mstr */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 45: AHB->PLB */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 46: OPB->PLB 1 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 47: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 48: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 49: SCP 0 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 50: SCP 1 */			\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 51: ATA/ATAPI */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 52: SCP 0 PIO2FIFO */	\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 53: SCP 1 PIO2FIFO */	\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 54: Reserved */		\
  (IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 55: NAND ND_RDY */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 56: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 57: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 58: Reserved */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 59: PCI System Err */	\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 60: EMAC 0 */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 61: EMAC 0 WOL */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 62: USB Host */		\
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 63: USB Device */
#endif

#define BD_EMAC_ADDR(e,i) bi_enetaddr[e][i]

#include <platforms/ibm440.h>

struct ibm440_clocks cxd4707_get_clocks(void);
void _cxd4707_init_uart_clk(int port);
void cxd4707_init_uart(int line, int rs_num);
void cxd4707_read_pfc(void);
void cxd4707_setup_pci(void);
void cxd4707_todc_setup(void);
u_char cxd4707_todc_read_val(int addr);
void cxd4707_todc_write_val(int addr, unsigned char val);
unsigned long cxd4707_find_end_of_memory(void);
unsigned int cxd4707_get_rev(void);
unsigned int cxd4707_get_config_rev(void);
void cxd4707_probe_cpu(void);
void cxd4707_setup_hose(void);
void cxd4707_calibrate_decr(void);
void cxd4707_parse_bootinfo(unsigned long r3);
void cxd4707_platform_init(unsigned long r3, unsigned long r4,
			   unsigned long r5, unsigned long r6, unsigned long r7);

#ifdef CONFIG_SNSC_ADD_NBL_TLB
#include <asm/mmu.h>

#define CXD4707_ROM_BASE		0xfff00000ULL
#define CXD4707_ROM_SIZE		0x00100000
#define CXD4707_ROM_MAPPED_BASE		0xffd00000

#define CXD4707_AHB_BASE		0xc0000000ULL
#define CXD4707_AHB_SIZE		0x80000
#define CXD4707_AHB_MAPPED_BASE		0xffe00000

#define CXD4707_OPB0_BASE		0xe0000000ULL
#define CXD4707_OPB0_SIZE		0x1000
#define CXD4707_OPB0_MAPPED_BASE	0xffe80000

#define CXD4707_OPB1_BASE		0xe8000000ULL
#define CXD4707_OPB1_SIZE		0x1000
#define CXD4707_OPB1_MAPPED_BASE	0xffe81000

void *cxd4707_ioremap(phys_addr_t paddr, unsigned long size);
int cxd4707_iounmap(void *addr);
#endif

#endif /* __ASM_CXD4707_H__ */
#endif /* __KERNEL__ */
