/*
 * arch/ppc/platforms/dtgnexus.c
 *
 *  NEXUS board specific routines
 *
 *  Copyright 2003-2006 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/serialP.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/time.h>
#include <asm/todc.h>
#include <asm/bootinfo.h>
#include <asm/ppc4xx_pic.h>
#include <asm/ppc4xx_gpio.h>

#include <platforms/ibm_ocp.h>

#if defined   (CONFIG_DTGNEXUS_GENRI)
#  include <platforms/dtgnexus_genri.h>
#elif defined (CONFIG_DTGNEXUS_TEDUKURI)
#  include <platforms/dtgnexus_tedukuri.h>
#elif defined (CONFIG_DTGNEXUS_RYOUSHI)
#  include <platforms/dtgnexus_ryoushi.h>
#elif defined (CONFIG_SELECAO_TEDUKURI)
#  include <platforms/selecao_tedukuri.h>
#elif defined (CONFIG_SELECAO_RYOUSHI)
#  include <platforms/selecao_ryoushi.h>
#else
#  error Unknown NEXUS target configration.
#endif

#define NAME "NEXUS: " 

/* All clocks in Hz */
struct ibm440_clocks ibm440_clks;

#if !defined(CONFIG_DUMMY_KEYB)
extern int pckbd_setkeycode(unsigned int scancode, unsigned int keycode);
extern int pckbd_getkeycode(unsigned int scancode);
extern int pckbd_translate(unsigned char scancode, unsigned char *keycode,
		                           char raw_mode);
#endif

extern void abort(void);

/* Global Variables */
unsigned char __res[sizeof (bd_t)];

static int
dtgnexus_show_cpuinfo(struct seq_file *m)
{
	seq_printf(m, "vendor\t\t: Sony\n");
	seq_printf(m, "machine\t\t: CXD-4707\n");
	seq_printf(m, "board\t\t: %s\n", PROC_CPUINFO_BOARD);
	return 0;
}

static void __init
dtgnexus_setup_hose(void)
{
	cxd4707_setup_hose();
	ppc_md.pci_swizzle 	= common_swizzle;
#if defined   (CONFIG_DTGNEXUS_GENRI)
	ppc_md.pci_map_irq 	= dtgnexus_genri_map_irq;
#elif defined (CONFIG_DTGNEXUS_TEDUKURI)
	ppc_md.pci_map_irq 	= dtgnexus_tedukuri_map_irq;
#elif defined (CONFIG_DTGNEXUS_RYOUSHI)
	ppc_md.pci_map_irq 	= dtgnexus_ryoushi_map_irq;
#elif defined (CONFIG_SELECAO_TEDUKURI)
	ppc_md.pci_map_irq 	= selecao_tedukuri_map_irq;
#elif defined (CONFIG_SELECAO_RYOUSHI)
	ppc_md.pci_map_irq 	= selecao_ryoushi_map_irq;
#else
#  error Unknown NEXUS target configration.
#endif
}

TODC_ALLOC();

static void __init
dtgnexus_early_serial_map(void)
{
	int rs_num = 0;

	/* setup uart struct used by drivers */
#ifdef CONFIG_CXD4707_UART_0
	cxd4707_init_uart(0, rs_num++);
#else
	_cxd4707_init_uart_clk(0);
#endif
#ifdef CONFIG_CXD4707_UART_1
	cxd4707_init_uart(1, rs_num++);
#else
	_cxd4707_init_uart_clk(1);
#endif
#ifdef CONFIG_CXD4707_UART_2
	cxd4707_init_uart(2, rs_num++);
#else
	_cxd4707_init_uart_clk(2);
#endif
#ifdef CONFIG_CXD4707_UART_3
	cxd4707_init_uart(3, rs_num++);
#else
	_cxd4707_init_uart_clk(3);
#endif
}

#ifdef CONFIG_SNSC_PANIC_HOOK
static int
dtgnexus_panic_event(struct notifier_block *this, unsigned long event, void *ptr)
{
	printk("%s()\n", __FUNCTION__);
        return NOTIFY_DONE;
}

static struct notifier_block dtgnexus_panic_block = {
	dtgnexus_panic_event,
	NULL,
        INT_MIN,	/* this is called at last */
};
#endif

static void __init
dtgnexus_setup_arch(void)
{
	bd_t *bip = (bd_t *)__res;
#ifdef CONFIG_SNSC_PANIC_HOOK
	extern int panic_timeout;
#endif

#if !defined(CONFIG_BDI_SWITCH)
	/*
	 * The Abatron BDI JTAG debugger does not tolerate others
	 * mucking with the debug registers.
	 */
        mtspr(SPRN_DBCR0, (DBCR0_TDE | DBCR0_IDM));
#endif

	ibm440_clks = cxd4707_get_clocks();
	bip->bi_opb_busfreq = ibm440_clks.opb;
	
	/* Use IIC in fast (400 kHz) mode */
	bip->bi_iic_fast[0] = bip->bi_iic_fast[1] = 1;

	/* init to some ~sane value until calibrate_delay() runs */
        loops_per_jiffy = 50000000/HZ;

	/* Setup PCI host bridge */
#ifdef CONFIG_PCI
	dtgnexus_setup_hose();
#endif

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0); /* /dev/ram */
	else
#endif
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x1601);	/* /dev/hdc1 */
#endif

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif

	dtgnexus_early_serial_map();

	/* Identify the system */
	cxd4707_probe_cpu();

	/* 
	 * Initialize value for UIC
	 */
#if defined   (CONFIG_DTGNEXUS_GENRI)
	ibm4xxPIC_InitSenses = dtgnexus_genri_IRQ_initsenses;
	ibm4xxPIC_NumInitSenses = sizeof(dtgnexus_genri_IRQ_initsenses);
#elif defined (CONFIG_DTGNEXUS_TEDUKURI)
	ibm4xxPIC_InitSenses = dtgnexus_tedukuri_IRQ_initsenses;
	ibm4xxPIC_NumInitSenses = sizeof(dtgnexus_tedukuri_IRQ_initsenses);
#elif defined (CONFIG_DTGNEXUS_RYOUSHI)
	ibm4xxPIC_InitSenses = dtgnexus_ryoushi_IRQ_initsenses;
	ibm4xxPIC_NumInitSenses = sizeof(dtgnexus_ryoushi_IRQ_initsenses);
#elif defined (CONFIG_SELECAO_TEDUKURI)
	ibm4xxPIC_InitSenses = selecao_tedukuri_IRQ_initsenses;
	ibm4xxPIC_NumInitSenses = sizeof(selecao_tedukuri_IRQ_initsenses);
#elif defined (CONFIG_SELECAO_RYOUSHI)
	ibm4xxPIC_InitSenses = selecao_ryoushi_IRQ_initsenses;
	ibm4xxPIC_NumInitSenses = sizeof(selecao_ryoushi_IRQ_initsenses);
#else
#  error Unknown NEXUS target configration.
#endif

	/* print pin-mux configs */
	cxd4707_read_pfc();

#ifdef CONFIG_SNSC_PANIC_HOOK
	panic_timeout = CONFIG_SNSC_PANIC_TIMEOUT;
	notifier_chain_register(&panic_notifier_list, &dtgnexus_panic_block);
#endif
}

static void
dtgnexus_restart(char *cmd)
{
	__cli();
	abort();
}

static void
dtgnexus_power_off(void)
{
	__cli();
	for(;;);
}

static void
dtgnexus_halt(void)
{
	__cli();
	for(;;);
}

static void __init
dtgnexus_init_irq(void)
{
	int i;

	ppc4xx_pic_init();

	for (i = 0; i < NR_IRQS; i++)
 		irq_desc[i].handler = ppc4xx_pic;

}

#ifdef CONFIG_PPC4xx_GPIO

static void __init
dtgnexus_init_gpio(int port, void *base, void *pgpio)
{
	ocp_gpio_t *gpio_base = base;
	struct ppc4xx_gpio_t *gpio = pgpio;
	int i;

	printk("%s(): port=%d, gpio_base=0x%p, gpio=0x%p\n", __FUNCTION__,
	       port, gpio_base, gpio);

	/* start with all unavailable */
	gpio->no_gpio = 0xffffffff;
	gpio->no_out  = 0xffffffff;
	gpio->no_hiz  = 0xffffffff;
	gpio->no_in   = 0xffffffff;
	gpio->no_int  = 0xffffffff;
	for (i = 0; i < 3; i++) {
		gpio->no_altout[i] = 0xffffffff;
		gpio->no_altin[i]  = 0xffffffff;
	}

	switch (port) {
	case GPIO_PORT_0:
		/* These pins can be used at any multiplex */
		gpio->no_gpio     &= ~0x0000ff00;
		gpio->no_out      &= ~0x0000ff00;
		gpio->no_hiz      &= ~0x0000ff00;
		gpio->no_in       &= ~0x0000ff00;
		gpio->no_int      &= ~0x0000f000;
		gpio->no_altin[0] &= ~0x0000f000;
#ifdef CONFIG_CXD4707_ES_2_0
		gpio->no_altin[0] &= ~0x00000800;
#endif

		/* 
		 * Multiplex Group0 == Func2 (to use NAND)
		 */
		gpio->no_gpio     &= ~0x00070000;
		gpio->no_out      &= ~0x00070000;
		gpio->no_hiz      &= ~0x00070000;
		gpio->no_in       &= ~0x00070000;
		gpio->no_int      &= ~0x00070000;
		gpio->no_altin[0] &= ~0x00070000;

		/* 
		 * Multiplex Group1 == Func2 (to use GPIO)
		 */
		gpio->no_gpio     &= ~0x000000ff;
		gpio->no_out      &= ~0x000000ff;
		gpio->no_hiz      &= ~0x000000ff;
		gpio->no_in       &= ~0x000000ff;
		gpio->no_int      &= ~0x00000000;
		break;

	case GPIO_PORT_1:
		/* These pins can be used at any multiplex */
		gpio->no_gpio     &= ~0x003fbf00;
		gpio->no_out      &= ~0x003fbf00;
		gpio->no_hiz      &= ~0x003fbf00;
		gpio->no_in       &= ~0x003fbf00;
#ifdef CONFIG_CXD4707_ES_2_0
		gpio->no_altin[0] &= ~0x00003f00;
		gpio->no_altout[0]&= ~0x003f8000;
#endif

		/* 
		 * Multiplex Group1 == Func2 (to use GPIO)
		 */
		gpio->no_gpio     &= ~0xffc00000;
		gpio->no_out      &= ~0xffc00000;
		gpio->no_hiz      &= ~0xffc00000;
		gpio->no_in       &= ~0xffc00000;
		gpio->no_int      &= ~0x00000000;

		/* 
		 * Multiplex Group2 == Func2 (to use UART + NAND)
		 */
		gpio->no_gpio     &= ~0x00000001;
		gpio->no_out      &= ~0x00000001;
		gpio->no_hiz      &= ~0x00000001;
		gpio->no_in       &= ~0x00000000;
		gpio->no_int      &= ~0x00000000;
		gpio->no_altout[0]&= ~0x00000001;

		/* 
		 * Multiplex Group3 == Func0 (to use USB Host)
		 */
			/* do nothing */

#ifdef CONFIG_CXD4707_ES_2_0
		/* 
		 * Multiplex Group4 == Func1 (to use UART1 + GPIO)
		 */
		gpio->no_gpio     &= ~0x000040c0;
		gpio->no_out      &= ~0x000040c0;
		gpio->no_hiz      &= ~0x000040c0;
		gpio->no_in       &= ~0x000040c0;
		gpio->no_altout[0]&= ~0x00004080;
		gpio->no_altin[0] &= ~0x00000040;

		/* 
		 * Multiplex Group5 == Func1, but has no GPIO
		 */

		/* 
		 * Multiplex Group6 == Func0 (to use GPIO)
		 */
		gpio->no_gpio     &= ~0x0000000c;
		gpio->no_out      &= ~0x0000000c;
		gpio->no_hiz      &= ~0x0000000c;
		gpio->no_in       &= ~0x0000000c;

		/* 
		 * Multiplex Group7 == Func0 (to use GPIO)
		 */
		gpio->no_gpio     &= ~0x00000030;
		gpio->no_out      &= ~0x00000030;
		gpio->no_hiz      &= ~0x00000030;
		gpio->no_in       &= ~0x00000030;
#endif

		break;
	}
	return ;
}

static int 
dtgnexus_irq_to_gpio_port(int irq)
{
	if (irq < 64 || irq > 83)
		return -1;
	else
		return 0;	/* all INTs are in GPIO port 0 */
}

static int 
dtgnexus_irq_to_gpio_bit(int irq)
{
	if (irq < 64 || irq > 83)
		return -1;
	else
		return irq - 64;
}
#endif /* CONFIG_PPC4xx_GPIO */

void __init platform_init(unsigned long r3, unsigned long r4,
		unsigned long r5, unsigned long r6, unsigned long r7)
{
	cxd4707_platform_init(r3, r4, r5, r6, r7);

	ppc_md.setup_arch	= dtgnexus_setup_arch;
	ppc_md.show_cpuinfo	= dtgnexus_show_cpuinfo;
	ppc_md.init_IRQ		= dtgnexus_init_irq;
	ppc_md.get_irq		= NULL;		/* Set in ppc4xx_pic_init() */

	ppc_md.restart		= dtgnexus_restart;
	ppc_md.power_off	= dtgnexus_power_off;
	ppc_md.halt		= dtgnexus_halt;

#ifdef CONFIG_KGDB
	ppc_md.early_serial_map	= dtgnexus_early_serial_map;
#endif
#ifdef CONFIG_PPC4xx_GPIO
	ppc_md.init_gpio	= dtgnexus_init_gpio;
	ppc_md.irq_to_gpio_port	= dtgnexus_irq_to_gpio_port;
	ppc_md.irq_to_gpio_bit	= dtgnexus_irq_to_gpio_bit;
#endif
}

