/*
 * arch/ppc/platforms/dtgnexus.h
 *
 * NEXUS board definitions
 *
 *  Copyright 2003-2005 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __KERNEL__
#ifndef __ASM_DTGNEXUS_H__
#define __ASM_DTGNEXUS_H__

#include <linux/config.h>
#include <platforms/cxd4707gb.h>

#ifndef __ASSEMBLY__
/*
 * Data structure defining board information maintained by the boot
 * ROM on IBM's "Ebony" evaluation board. An effort has been made to
 * keep the field names consistent with the 8xx 'bd_t' board info
 * structures.
 *
 * Ebony firmware stores MAC addresses in the F/W VPD area. The
 * firmware must store the other dynamic values in NVRAM like on
 * the previous 40x systems so they  should be accessible if we
 * really want them.
 */
typedef struct board_info {
	unsigned char	bi_enetaddr[2][6];	/* EMAC addresses */
	unsigned int	bi_opb_busfreq;		/* OPB clock in Hz */
	int		bi_iic_fast[2];		/* Use fast i2c mode */
	int             bi_enetaddr_valid[2];   /* enetaddr is valid or not */
} bd_t;
#endif /* __ASSEMBLY__ */

#ifndef NR_BOARD_IRQS
#define NR_BOARD_IRQS 0
#endif

#define _IO_BASE	isa_io_base
#define _ISA_MEM_BASE	isa_mem_base
#define PCI_DRAM_OFFSET	pci_dram_offset


/*
 * Serial port defines
 */

/* uart base clock */
#define UART_DIV		12
#define BASE_BAUD		CXD4707_PLBCLK/UART_DIV/16 
#define SERIAL_IRQ0_VALID 	TRUE

/* number of CXD4707 uart to use */
#define DTGNEXUS_CXD4707_UART_OPS	\
	CXD4707_UART_OP0	\
	CXD4707_UART_OP1	\
	CXD4707_UART_OP2	\
	CXD4707_UART_OP3

#define SERIAL_PORT_DFNS	\
	DTGNEXUS_CXD4707_UART_OPS



#define RS_TABLE_SIZE	(NUM_CXD4707_UART)
extern struct serial_state rs_table[];

/* 
 * USB keyboard support 
 */
#define kbd_read_input()	0
#define kbd_read_status()	0
#define kbd_write_output(val)	{}
#define kbd_write_command(val)	{}

/* 
 * External peripheral definition 
 */
/* Peripheral CS# physical address (defined by boot loader) */
#define DTGNEXUS_PER_CS1_PADDR	(CXD4707_EXTDEV + 0x01000000)	
#define DTGNEXUS_PER_CS2_PADDR	(CXD4707_EXTDEV + 0x00100000)	
#define DTGNEXUS_PER_CS3_PADDR	(CXD4707_EXTDEV + 0x00300000)	
#define DTGNEXUS_PER_CS4_PADDR	(CXD4707_EXTDEV + 0x00000000)	
#define DTGNEXUS_PER_CS5_PADDR	(CXD4707_EXTDEV + 0x00400000)
#define DTGNEXUS_PER_CS6_PADDR	(CXD4707_EXTDEV + 0x00500000)
#define DTGNEXUS_PER_CS7_PADDR	(CXD4707_EXTDEV + 0x00600000)

/* 
 * 440GP Clock value
 */
extern struct ibm440_clocks ibm440_clks;

/* 
 * Boot time default address
 */
#define BOOT_TIME_SIZE		(0x1000)	/* 4KB */
#define BOOT_TIME_BASE		(0x00800000 - BOOT_TIME_SIZE) /* +8MB offset */


/*
 * SDM and NAND flash
 */
#define CXD4707_NAND_CHIPS		1


#endif	/* __ASM_DTGNEXUS_H__ */
#endif	/* __KERNEL__ */
