/*
 * arch/ppc/platforms/ibm440gp.h
 *
 * PPC440GP definitions
 *
 * Roland Dreier <roland@digitalvampire.org>
 *
 * Copyright 2002 Roland Dreier
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * This file contains code that was originally in the files ibm440.h
 * and ebony.h, which were written by Matt Porter of MontaVista Software Inc.
 *
 * 	Version 1.2 07/22/02 - Armin
 * 	Added default power managment settings
 */

#ifdef __KERNEL__
#ifndef __PPC_PLATFORMS_IBM440GP_H
#define __PPC_PLATFORMS_IBM440GP_H

#include <linux/config.h>
#include <platforms/ibm_ocp.h>

/*
 * Standard 4GB "page" definitions
 */
#define	PPC440_IO_PAGE		0x0000000100000000
#define	PPC440_PCICFG_PAGE	0x0000000200000000
#define	PPC440_PCIIO_PAGE	PPC440_PCICFG_PAGE
#define	PPC440_PCIMEM_PAGE	0x0000000300000000

/*
 * 36-bit trap ranges
 */
#define PPC440_IO_LO		0x40000000
#define PPC440_IO_HI		0x40001000
#define PPC440_PCICFG_LO	0x0ec00000
#define PPC440_PCICFG_HI	0x0ec7ffff
#define PPC440_PCIMEM_LO	0x80002000
#define PPC440_PCIMEM_HI	0xffffffff

/*
 * SPRN definitions
 */

#define SPRN_CPC0_GPIO		0xe5/BEARLRL

/*
 * DCRN definitions
 */ 
/* Base DCRNs */
#define DCRN_DMA0_BASE		0x100
#define DCRN_DMA1_BASE		0x108
#define DCRN_DMA2_BASE		0x110
#define DCRN_DMA3_BASE		0x118
#define DCRN_DMA0_SR_BASE	0x120
#define DCRNCAP_DMA_SG		1	/* have DMA scatter/gather capability */
#define DCRN_MAL_BASE		0x180
#define DCRN_CPMFR_BASE		0x0b2
#define DCRN_CPMSR_BASE		0x0b0

/* UIC */
#define DCRN_UIC0_BASE	0xc0
#define DCRN_UIC1_BASE	0xd0
#define UIC0		DCRN_UIC0_BASE
#define UIC1		DCRN_UIC1_BASE

#define UIC0_UIC1NC      30	/* UIC1 non-critical interrupt */
#define UIC0_UIC1CR      31	/* UIC1 critical interrupt */

/* 440GP PLB Arbiter DCRs */
#define DCRN_PLB0_REVID		0x082		/* PLB Arbiter Revision ID */
#define DCRN_PLB0_ACR		0x083		/* PLB Arbiter Control */
#define DCRN_PLB0_BESR		0x084		/* PLB Error Status */
#define DCRN_PLB0_BEARL		0x086		/* PLB Error Address Low */
#define DCRN_PLB0_BEAR		DCRN_PLB0_BEARL	/* 40x compatibility */
#define DCRN_PLB0_BEARH		0x087		/* PLB Error Address High */

/* 440GP DRAM controller DCRs */
#define DCRN_SDRAM0_CFGADDR		0x010
#define DCRN_SDRAM0_CFGDATA		0x011

#define SDRAM0_B0CR	0x40
#define SDRAM0_B1CR	0x44
#define SDRAM0_B2CR	0x48
#define SDRAM0_B3CR	0x4c

#define SDRAM_CONFIG_BANK_ENABLE	0x00000001
#define SDRAM_CONFIG_SIZE_MASK		0x000e0000
#define SDRAM_CONFIG_BANK_SIZE(reg)	((reg & SDRAM_CONFIG_SIZE_MASK) >> 17)
#define SDRAM_CONFIG_SIZE_8M		0x00000001
#define SDRAM_CONFIG_SIZE_16M		0x00000002
#define SDRAM_CONFIG_SIZE_32M		0x00000003
#define SDRAM_CONFIG_SIZE_64M		0x00000004
#define SDRAM_CONFIG_SIZE_128M		0x00000005
#define SDRAM_CONFIG_SIZE_256M		0x00000006
#define SDRAM_CONFIG_SIZE_512M		0x00000007

/* 440GP Clock, PM, chip control */
#define DCRN_CPC0_SR		0x0b0
#define DCRN_CPC0_ER		0x0b1
#define DCRN_CPC0_FR		0x0b2
#define DCRN_CPC0_SYS0		0x0e0
#define DCRN_CPC0_SYS1		0x0e1
#define DCRN_CPC0_CUST0		0x0e2
#define DCRN_CPC0_CUST1		0x0e3
#define DCRN_CPC0_STRP0		0x0e4
#define DCRN_CPC0_STRP1		0x0e5
#define DCRN_CPC0_STRP2		0x0e6
#define DCRN_CPC0_STRP3		0x0e7
#define DCRN_CPC0_GPIO		0x0e8
#define DCRN_CPC0_PLB		0x0e9
#define DCRN_CPC0_CR1		0x0ea
#define DCRN_CPC0_CR0		0x0eb
#define DCRN_CPC0_MIRQ0		0x0ec
#define DCRN_CPC0_MIRQ1		0x0ed
#define DCRN_CPC0_JTAGID	0x0ef

/*
 * PCI-X definitions
 */
#define PCIX0_REG_BASE		0x20ec80000
#define PCIX0_REG_SIZE		0x200

#define PCIX0_VENDID		0x000
#define PCIX0_DEVID		0x002
#define PCIX0_COMMAND		0x004
#define PCIX0_STATUS		0x006
#define PCIX0_REVID		0x008
#define PCIX0_CLS		0x009
#define PCIX0_CACHELS		0x00c
#define PCIX0_LATTIM		0x00d
#define PCIX0_HDTYPE		0x00e
#define PCIX0_BIST		0x00f
#define PCIX0_BAR0L		0x010
#define PCIX0_BAR0H		0x014
#define PCIX0_BAR1		0x018
#define PCIX0_BAR2L		0x01c
#define PCIX0_BAR2H		0x020
#define PCIX0_BAR3		0x024
#define PCIX0_CISPTR		0x028
#define PCIX0_SBSYSVID		0x02c
#define PCIX0_SBSYSID		0x02e
#define PCIX0_EROMBA		0x030
#define PCIX0_CAP		0x034
#define PCIX0_RES0		0x035
#define PCIX0_RES1		0x036
#define PCIX0_RES2		0x038
#define PCIX0_INTLN		0x03c
#define PCIX0_INTPN		0x03d
#define PCIX0_MINGNT		0x03e
#define PCIX0_MAXLTNCY		0x03f
#define PCIX0_BRDGOPT1		0x040
#define PCIX0_BRDGOPT2		0x044
#define PCIX0_ERREN		0x050
#define PCIX0_ERRSTS		0x054
#define PCIX0_PLBBESR		0x058
#define PCIX0_PLBBEARL		0x05c
#define PCIX0_PLBBEARH		0x060
#define PCIX0_POM0LAL		0x068
#define PCIX0_POM0LAH		0x06c
#define PCIX0_POM0SA		0x070
#define PCIX0_POM0PCIAL		0x074
#define PCIX0_POM0PCIAH		0x078
#define PCIX0_POM1LAL		0x07c
#define PCIX0_POM1LAH		0x080
#define PCIX0_POM1SA		0x084
#define PCIX0_POM1PCIAL		0x088
#define PCIX0_POM1PCIAH		0x08c
#define PCIX0_POM2SA		0x090
#define PCIX0_PIM0SA		0x098
#define PCIX0_PIM0LAL		0x09c
#define PCIX0_PIM0LAH		0x0a0
#define PCIX0_PIM1SA		0x0a4
#define PCIX0_PIM1LAL		0x0a8
#define PCIX0_PIM1LAH		0x0ac
#define PCIX0_PIM2SA		0x0b0
#define PCIX0_PIM2LAL		0x0b4
#define PCIX0_PIM2LAH		0x0b8
#define PCIX0_OMCAPID		0x0c0
#define PCIX0_OMNIPTR		0x0c1
#define PCIX0_OMMC		0x0c2
#define PCIX0_OMMA		0x0c4
#define PCIX0_OMMUA		0x0c8
#define PCIX0_OMMDATA		0x0cc
#define PCIX0_OMMEOI		0x0ce
#define PCIX0_PMCAPID		0x0d0
#define PCIX0_PMNIPTR		0x0d1
#define PCIX0_PMC		0x0d2
#define PCIX0_PMCSR		0x0d4
#define PCIX0_PMCSRBSE		0x0d6
#define PCIX0_PMDATA		0x0d7
#define PCIX0_PMSCRR		0x0d8
#define PCIX0_CAPID		0x0dc
#define PCIX0_NIPTR		0x0dd
#define PCIX0_CMD		0x0de
#define PCIX0_STS		0x0e0
#define PCIX0_IDR		0x0e4
#define PCIX0_CID		0x0e8
#define PCIX0_RID		0x0ec
#define PCIX0_MSGIL		0x100
#define PCIX0_MSGIH		0x104
#define PCIX0_MSGOL		0x108
#define PCIX0_MSGOH		0x10c
#define PCIX0_IM		0x1f8


/* 
 * IBM 440GP depend part description
 */

#define EMAC_NUMS		2
#define UART_NUMS		2
#define ZMII_NUMS		1
#define IIC_NUMS		2
#define IIC0_IRQ		2
#define IIC1_IRQ		3
#define GPIO_NUMS		1

/* UART location */
#define PPC440GP_UART0_ADDR	0x0000000140000200
#define PPC440GP_UART1_ADDR	0x0000000140000300

/* EMAC location */
#define PPC440GP_EMAC0_ADDR	0x0000000140000800
#define PPC440GP_EMAC1_ADDR	0x0000000140000900
#define PPC440GP_EMAC_SIZE	0x70

/* EMAC IRQ's */
#define BL_MAC_WOL	61	/* WOL */
#define BL_MAC_WOL1	63	/* WOL */
#define BL_MAL_SERR	32	/* MAL SERR */
#define BL_MAL_TXDE	33	/* MAL TXDE */
#define BL_MAL_RXDE	34	/* MAL RXDE */
#define BL_MAL_TXEOB	10	/* MAL TX EOB */
#define BL_MAL_RXEOB	11	/* MAL RX EOB */
#define BL_MAC_ETH0	60	/* MAC */
#define BL_MAC_ETH1	62	/* MAC */

/* DMAC0 Int */
#define DMA0_INT	12
#define DMA1_INT	13
#define DMA2_INT	14
#define DMA3_INT	15

/* ZMII location */
#define PPC440GP_ZMII_ADDR	0x0000000140000780
#define PPC440GP_ZMII_SIZE	0x0c

/* I2C location */
#define PPC440GP_IIC0_ADDR	0x40000400
//XXX:takuzo #define PPC440GP_IIC0_ADDR	0x0000000140000400
#define PPC440GP_IIC1_ADDR	0x40000500

/* GPIO location */
#define PPC440GP_GPIO0_ADDR	0x0000000140000700

/* GPIO initial/multiplex info */
#define PPC440GP_GPIO0_MSK		0xffff8000
#define PPC440GP_GPIO0_MSK_OUT		0xffff8000
#define PPC440GP_GPIO0_MSK_HIZ		0xffff8000
#define PPC440GP_GPIO0_MSK_IN		0xffff8000
#define PPC440GP_GPIO0_MSK_INT		0xffc00000

#define PPC4xx_GPIO0_BASE	PPC440GP_GPIO0_ADDR
#define PPC4xx_IRQ_GPIO0(x)	(((x)<7) ? (x)+23 : (x)+50)

/* Clock and Power Management */
#define IBM_CPM_IIC0		0x80000000	/* IIC interface */
#define IBM_CPM_IIC1		0x40000000	/* IIC interface */
#define IBM_CPM_PCI		0x20000000	/* PCI bridge */
#define IBM_CPM_CPU		0x02000000	/* processor core */
#define IBM_CPM_DMA		0x01000000	/* DMA controller */
#define IBM_CPM_BGO		0x00800000	/* PLB to OPB bus arbiter */
#define IBM_CPM_BGI		0x00400000	/* OPB to PLB bridge */
#define IBM_CPM_EBC		0x00200000	/* External Bux Controller */
#define IBM_CPM_EBM		0x00100000	/* Ext Bus Master Interface */
#define IBM_CPM_DMC		0x00080000	/* SDRAM peripheral controller */
#define IBM_CPM_PLB		0x00040000	/* PLB bus arbiter */
#define IBM_CPM_SRAM		0x00020000	/* SRAM memory controller */
#define IBM_CPM_PPM		0x00002000	/* PLB Performance Monitor */
#define IBM_CPM_UIC1		0x00001000	/* Universal Interrupt Controller */
#define IBM_CPM_GPIO0		0x00000800	/* General Purpose IO (??) */
#define IBM_CPM_GPT		0x00000400	/* General Purpose Timers  */
#define IBM_CPM_UART0		0x00000200	/* serial port 0 */
#define IBM_CPM_UART1		0x00000100	/* serial port 1 */
#define IBM_CPM_UIC0		0x00000080	/* Universal Interrupt Controller */
#define IBM_CPM_TMRCLK		0x00000040	/* CPU timers */

#define DFLT_IBM4xx_PM		~(IBM_CPM_UIC | IBM_CPM_UIC1 | IBM_CPM_CPU \
				| IBM_CPM_EBC | IBM_CPM_SRAM | IBM_CPM_BGO \
				| IBM_CPM_EBM | IBM_CPM_PLB | IBM_CPM_OPB \
				| IBM_CPM_TMRCLK | IBM_CPM_DMA | IBM_CPM_PCI)

/* 440GP Clock and Power Management DCRs */
#ifdef DCRN_CPMFR_BASE
#define DCRN_CPMFR	(DCRN_CPMFR_BASE + 0x0)	/* CPM Force */
#endif

#ifdef DCRN_CPMSR_BASE
#define DCRN_CPMSR	(DCRN_CPMSR_BASE + 0x0)	/* CPM Status */
#define DCRN_CPMER	(DCRN_CPMSR_BASE + 0x1)	/* CPM Enable */
#endif

#undef NR_UICS
#define NR_UICS 2
#define UIC_CASCADE_MASK	0x0003		/* bits 30 & 31 */

#define IIC_OWN			0x55
#define IIC_CLOCK		50

#define BD_EMAC_ADDR(e,i) bi_enetaddr[e][i]

/*
 * Serial port defines
 */
#if !defined(CONFIG_KAISER) 
#define RS_TABLE_SIZE	2
#endif

#include <platforms/ibm440.h>

/*
 * Please, refer to the Figure 13.1 in 440GP user manual
 * 
 * if internal UART clock is used, ser_clk is ignored
 */
void ibm440gp_get_clocks(struct ibm440_clocks*, unsigned int sys_clk, 
	unsigned int ser_clk);

#endif /* __PPC_PLATFORMS_IBM440GP_H */

#endif /* __KERNEL__ */
