/*
 * arch/ppc/platforms/kaiser.c
 *
 * Kaiser board specific routines
 *
 * Matt Porter <mporter@mvista.com>
 *
 * Copyright 2002 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/serialP.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/time.h>
#include <asm/todc.h>
#include <asm/bootinfo.h>
#include <asm/ppc4xx_pic.h>

#ifdef CONFIG_SNSC_MPU4xx_MACADDR
#include <platforms/snsc-mpu4xx_macaddr.h>
#endif

#ifdef CONFIG_SNSC_CXD9790
#include <linux/cxd9790.h>
#ifdef CONFIG_SNSC_CXD9790_UART
#include <linux/serial_reg.h>
#include <asm/serial.h>
#endif /* CONFIG_SNSC_CXD9790_UART */
void *mpu4xx_cxd9790_base_vaddr;
#endif /* CONFIG_SNSC_CXD9790 */

#ifdef CONFIG_RTC7301_RTC
#include <asm/rtc7301.h>
void *ppc_rtc7301_base_vaddr;
extern unsigned long rtc7301_rtc_rtc_get_time(void);
#endif

/* All clocks in Hz */
struct ibm440_clocks ibm440_clks;


#if !defined(CONFIG_DUMMY_KEYB)
extern int pckbd_setkeycode(unsigned int scancode, unsigned int keycode);
extern int pckbd_getkeycode(unsigned int scancode);
extern int pckbd_translate(unsigned char scancode, unsigned char *keycode,
		                           char raw_mode);
#endif

extern void gen550_progress(char *, unsigned short);
extern void gen550_init(int, struct serial_struct *);


/*
 * Kaiser IRQ triggering/polarity settings
 */
static u_char kaiser_IRQ_initsenses[] __initdata = {
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 0: UART 0 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 1: UART 1 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 2: IIC 0 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 3: IIC 1 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 4: PCI Inb Mess */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 5: PCI Cmd Wrt */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 6: PCI PM */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 7: PCI MSI 0 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 8: PCI MSI 1 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 9: PCI MSI 2 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 10: MAL TX EOB */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 11: MAL RX EOB */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 12: DMA Chan 0 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 13: DMA Chan 1 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 14: DMA Chan 2 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 15: DMA Chan 3 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 16: Reserved */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 17: Reserved */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 18: GPT Timer 0 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 19: GPT Timer 1 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 20: GPT Timer 2 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 21: GPT Timer 3 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 22: GPT Timer 4 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 23: Ext Int 0 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 24: Ext Int 1 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 25: Ext Int 2 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 26: Ext Int 3 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 27: Ext Int 4 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 28: Ext Int 5 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 29: Ext Int 6 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 30: UIC1 NC Int */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 31: UIC1 Crit Int */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 32: MAL SERR */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 33: MAL TXDE */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 34: MAL RXDE */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 35: ECC Unc Err */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 36: ECC Corr Err */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 37: Ext Bus Ctrl */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 38: Ext Bus Mstr */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 39: OPB->PLB */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 40: PCI MSI 3 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 41: PCI MSI 4 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 42: PCI MSI 5 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 43: PCI MSI 6 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 44: PCI MSI 7 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 45: PCI MSI 8 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 46: PCI MSI 9 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 47: PCI MSI 10 */
	(IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS),	/* 48: PCI MSI 11 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 49: PLB Perf Mon */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 50: Ext Int 7 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 51: Ext Int 8 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 52: Ext Int 9 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 53: Ext Int 10 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 54: Ext Int 11 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 55: Ext Int 12 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 56: Ser ROM Err */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 57: Reserved */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 58: Reserved */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 59: PCI Async Err */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 60: EMAC 0 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 61: EMAC 0 WOL */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 62: EMAC 1 */
	(IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 63: EMAC 1 WOL */
};

extern void abort(void);

/* Global Variables */
unsigned char __res[sizeof (bd_t)];

static void __init
kaiser_calibrate_decr(void)
{
	unsigned int freq;

	/*
	 * Determine system clock speed
	 *
	 * If we are on Rev. B silicon, then use
	 * default external system clock.  If we are
	 * on Rev. C silicon then errata forces us to
	 * use the internal clock.
	 */
	switch (PVR_REV(mfspr(PVR))) {
		case PVR_REV(PVR_440GP_RB):
			freq = KAISER_440GP_RB_SYSCLK;
			break;
		case PVR_REV(PVR_440GP_RC1):
		default:
			freq = KAISER_440GP_RC_SYSCLK;
			break;
	}
	
	tb_ticks_per_jiffy = freq / HZ;
	tb_to_us = mulhwu_scale_factor(freq, 1000000);
#ifdef CONFIG_ILATENCY
	{
		extern long ticks_per_usec;
		ticks_per_usec = freq / 1000000;
	}
#endif

	/* Clear any pending timer interrupts */
	mtspr(SPRN_TSR, TSR_ENW | TSR_WIS | TSR_DIS | TSR_FIS);

	/* Enable decrementer interrupt */
	mtspr(SPRN_TCR, TCR_DIE);
}

static int
kaiser_show_cpuinfo(struct seq_file *m)
{
	seq_printf(m, "vendor\t\t: IBM\n");
	seq_printf(m, "machine\t\t: Kaiser\n");

	return 0;
}

static inline int
kaiser_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
	static char pci_irq_table[][4] =
	/*
	 *	PCI IDSEL/INTPIN->INTLINE 
	 * 	   A   B   C   D
	 */
	{
		{ 24, 25, 26, 23 },     /* IDSEL 1 - PCI Device 0 (Maybe GOKU-S) */
		{ 25, 26, 23, 24 },     /* IDSEL 2 - NSC PCI Slot 0 Device 1 */
		{ 26, 23, 24, 25 },     /* IDSEL 3 - NSC PCI Slot 0 Device 2 */
		{ 23, 24, 25, 26 },     /* IDSEL 4 - NSC PCI Slot 1 Device 3 */
		{ 24, 25, 26, 23 },     /* IDSEL 5 - NSC PCI Slot 1 Device 4 */
		{ 25, 26, 23, 24 },     /* IDSEL 6 - NSC PCI Slot 2 Device 5 */
	};

	const long min_idsel = 1, max_idsel = 6, irqs_per_slot = 4;
	return PCI_IRQ_TABLE_LOOKUP;
}

#define PCIX_WRITEL(value, offset) \
	(writel(value, (u32)pcix_reg_base+offset))

/*
 * FIXME: This is only here to "make it work".  This will move
 * to a ibm_pcix.c which will contain a generic IBM PCIX bridge
 * configuration library. -Matt
 */
static void __init
kaiser_setup_pcix(void)
{
	void *pcix_reg_base;

	pcix_reg_base = ioremap64(PCIX0_REG_BASE, PCIX0_REG_SIZE);

	printk("pcix_reg_base = 0x%08x\n", pcix_reg_base);

	/* Disable all windows */
	PCIX_WRITEL(0, PCIX0_POM0SA);
	PCIX_WRITEL(0, PCIX0_POM1SA);
	PCIX_WRITEL(0, PCIX0_POM2SA);
	PCIX_WRITEL(0, PCIX0_PIM0SA);
	PCIX_WRITEL(0, PCIX0_PIM1SA);
	PCIX_WRITEL(0, PCIX0_PIM2SA);
	
	/* Setup 2GB PLB->PCI outbound mem window (3_8000_0000->0_8000_0000) */
	PCIX_WRITEL(0x00000003, PCIX0_POM0LAH);
	PCIX_WRITEL(0x80000000, PCIX0_POM0LAL);
	PCIX_WRITEL(0x00000000, PCIX0_POM0PCIAH);
	PCIX_WRITEL(0x80000000, PCIX0_POM0PCIAL);
	PCIX_WRITEL(0x80000001, PCIX0_POM0SA);

	/* Setup 2GB PCI->PLB inbound memory window at 0, enable MSIs */
	PCIX_WRITEL(0x00000000, PCIX0_PIM0LAH);
	PCIX_WRITEL(0x00000000, PCIX0_PIM0LAL);
	PCIX_WRITEL(0xe0000007, PCIX0_PIM0SA);

	eieio();
}

static void __init
kaiser_setup_hose(void)
{
	struct pci_controller *hose;

	/* Configure windows on the PCI-X host bridge */
	kaiser_setup_pcix();

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	hose->pci_mem_offset = KAISER_PCI_MEM_OFFSET;

	pci_init_resource(&hose->io_resource,
			KAISER_PCI_LOWER_IO,
			KAISER_PCI_UPPER_IO,
			IORESOURCE_IO,
			"PCI host bridge");

	pci_init_resource(&hose->mem_resources[0],
			KAISER_PCI_LOWER_MEM,
			KAISER_PCI_UPPER_MEM,
			IORESOURCE_MEM,
			"PCI host bridge");

	hose->io_space.start = KAISER_PCI_LOWER_IO;
	hose->io_space.end = KAISER_PCI_UPPER_IO;
	hose->mem_space.start = KAISER_PCI_LOWER_MEM;
	hose->mem_space.end = KAISER_PCI_UPPER_MEM;
	isa_io_base =
		(unsigned long)ioremap64(KAISER_PCI_IO_BASE, KAISER_PCI_IO_SIZE);
	hose->io_base_virt = (void *)isa_io_base;

	setup_indirect_pci(hose,
			KAISER_PCI_CFGA_PLB32,
			KAISER_PCI_CFGD_PLB32);
	hose->set_cfg_type = 1;

	hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

	ppc_md.pci_swizzle = common_swizzle;
	ppc_md.pci_map_irq = kaiser_map_irq;
}

TODC_ALLOC();

static void __init
kaiser_early_serial_map(void)
{
	struct serial_struct serial_req;
	unsigned int regs;

#ifdef CONFIG_UART_CLK_INTERNAL
	/* Use Internal Clock (OPB=133.3 Mhz) */
	regs = mfdcr(DCRN_CPC0_CR0) & ~0x005f0000; /* mask U0EC and UDIV */
	mtdcr(DCRN_CPC0_CR0, regs | 0x00170000); /* UDIV = div 24 */
#else
	/* Use External Clock (11.059200 Mhz) div 1 */
	regs = mfdcr(DCRN_CPC0_CR0);
	mtdcr(DCRN_CPC0_CR0, regs | 0x00400000);
#endif

	/* Setup ioremapped serial port access */
	memset(&serial_req, 0, sizeof(serial_req));
	serial_req.line = 0;
	serial_req.baud_base = BASE_BAUD;
	serial_req.port = 0;
	serial_req.irq = 0;
	serial_req.flags = ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST;
	serial_req.io_type = SERIAL_IO_MEM;
	serial_req.iomem_base = ioremap64(PPC440GP_UART0_ADDR, 8);
	serial_req.iomem_reg_shift = 0;

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	/* Configure debug serial access */
	gen550_init(0, &serial_req);
#endif

	if (early_serial_setup(&serial_req) != 0) {
		printk("Early serial init of port 0 failed\n");
	}

#ifdef USE_ONCHIP_UART1
	/* Assume early_serial_setup() doesn't modify serial_req */
	serial_req.line = 1;
	serial_req.port = 1;
	serial_req.irq = 1; 
	serial_req.iomem_base = ioremap64(PPC440GP_UART1_ADDR, 8);

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	/* Configure debug serial access */
	gen550_init(1, &serial_req);
#endif

	if (early_serial_setup(&serial_req) != 0) {
		printk("Early serial init of port 1 failed\n");
	}
#endif
}

#ifdef CONFIG_LEDS
/* 
 * ARM-style led funcs
 */
#include <asm/leds.h>

#define LED_BASE	((u8 *)mpu4xx_cxd9790_base_vaddr)
#define LED_SET(x)	writeb(x, LED_BASE)
static unsigned char saved_leds = 0;

void 
kaiser_leds_event(led_event_t ledevt)
{
	unsigned long flags;

	local_irq_save(flags);
	switch (ledevt) {
	case led_idle_start:	/* ...X .... */
		break;
	case led_idle_end:	/* ...X .... */
		break;
	case led_timer:		/* X... .... */
		saved_leds ^= 0x80;
		break;
	case led_start:
		break;
	case led_stop:
		break;
	case led_claim:	/* override idle & timer leds */
		break;
	case led_release: /* restore idle & timer leds */
		break;
	case led_green_on:
		break;
	case led_green_off:
		break;
	case led_amber_on:
		break;
	case led_amber_off:
		break;
	case led_red_on:
		break;
	case led_red_off:
		break;
	default:
		if (ledevt < 16) {
			saved_leds &= ~((unsigned char)ledevt);
		}
		break;
	}
	local_irq_restore(flags);

	LED_SET(saved_leds);
}
#endif /* CONFIG_LEDS */

static void __init
kaiser_setup_arch(void)
{
	bd_t *bip = (bd_t *)__res;
	unsigned char * vpd_base;

#if !defined(CONFIG_BDI_SWITCH)
	/*
	 * The Abatron BDI JTAG debugger does not tolerate others
	 * mucking with the debug registers.
	 */
        mtspr(SPRN_DBCR0, (DBCR0_TDE | DBCR0_IDM));
#endif

	/*
	 * Determine various clocks.
	 * To be completely correct we should get SysClk
	 * from FPGA, because it can be changed by on-board switches
	 * --ebs
	 */
	ibm440gp_get_clocks(&ibm440_clks, 33333333, 6 * 1843200);
	bip->bi_opb_busfreq = ibm440_clks.opb;
	
	/* Use IIC in standard (100 kHz) mode */
	bip->bi_iic_fast[0] = bip->bi_iic_fast[1] = 0;

#ifdef CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM
	/* 
	 * Retrieve MAC addresses from OCP IIC.
	 * See:
	 *    drivers/net/ibm_ocp/ibm_ocp_enet.c
	 *    drivers/i2c/i2c-ibm_iic.c
	 */
#else
	/* Retrieve MAC addresses from bootrom */
	vpd_base = ioremap64(KAISER_VPD_BASE, KAISER_VPD_SIZE);
	memcpy(bip->bi_enetaddr[0],KAISER_NA0_ADDR(vpd_base),6);
	memcpy(bip->bi_enetaddr[1],KAISER_NA1_ADDR(vpd_base),6);
#endif

#ifdef CONFIG_PPC_RTC
	/* Setup TODC access */
	TODC_INIT(TODC_TYPE_DS1743,
			0,
			0,
			ioremap64(KAISER_RTC_ADDR, KAISER_RTC_SIZE),
			8);
#endif /* CONFIG_PPC_RTC */

	/* init to some ~sane value until calibrate_delay() runs */
        loops_per_jiffy = 50000000/HZ;

	/* Setup PCI host bridge */
	kaiser_setup_hose();

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0); /* /dev/ram */
	else
#endif
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x1601);	/* /dev/hdc1 */
#endif

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif

	kaiser_early_serial_map();

	ibm4xxPIC_InitSenses = kaiser_IRQ_initsenses;
	ibm4xxPIC_NumInitSenses = sizeof(kaiser_IRQ_initsenses);

#ifdef CONFIG_SNSC_CXD9790
	mpu4xx_cxd9790_base_vaddr = ioremap64(MPU4xx_CXD9790_PADDR, MPU4xx_CXD9790_SIZE);
#ifdef CONFIG_SNSC_CXD9790_UART
	/* setup iomem_base for UART1-4 */
	rs_table[NUM_IBM440GP_UART+0].iomem_base = MPU4xx_CXD9790_BASE + 0x80;
	rs_table[NUM_IBM440GP_UART+1].iomem_base = MPU4xx_CXD9790_BASE + 0x88;
	rs_table[NUM_IBM440GP_UART+2].iomem_base = MPU4xx_CXD9790_BASE + 0x90;
	rs_table[NUM_IBM440GP_UART+3].iomem_base = MPU4xx_CXD9790_BASE + 0x98;
#endif /* CONFIG_SNSC_CXD9790_UART */
#endif /* CONFIG_SNSC_CXD9790 */

#ifdef CONFIG_LEDS
	saved_leds = 0xff;
	LED_SET(saved_leds);
	leds_event = kaiser_leds_event;
#endif

#ifdef CONFIG_RTC7301_RTC
	ppc_rtc7301_init();
#endif

	/* Identify the system */
	printk("Sony NSC Kaiser port (C) 2003 Sony Co.\n");


#ifdef CONFIG_MPU4xx_OPB_DISABLE_PARITY
	/* 
	 * For C-tail/USB-Host: disable EXPB Parity Checking
	 */
	{
		unsigned int ctl;

		/* read and set */
		mtdcr(DCRN_EBM0_CFGADDR, EBM0_CTL);
		ctl = mfdcr(DCRN_EBM0_CFGDATA);
		printk("OPB Disable Parity Check: EBM0_CTL = 0x%08x", ctl);

		mtdcr(DCRN_EBM0_CFGADDR, EBM0_CTL);
		mtdcr(DCRN_EBM0_CFGDATA, ctl | EBM0_CTL_PDIS);

		/* read again for check */
		mtdcr(DCRN_EBM0_CFGADDR, EBM0_CTL);
		ctl = mfdcr(DCRN_EBM0_CFGDATA);
		printk(" -> 0x%08x\n", ctl);
	}
#endif
}

static void
kaiser_restart(char *cmd)
{
	__cli();
	abort();
}

static void
kaiser_power_off(void)
{
	__cli();
	for(;;);
}

static void
kaiser_halt(void)
{
	__cli();
	for(;;);
}

/*
 * Read the 440GP memory controller to get size of system memory.
 */
static unsigned long __init
kaiser_find_end_of_memory(void)
{
	u32 i, bank_config;
	u32 mem_size = 0;

	for (i=0; i<4; i++)
	{
		switch (i)
		{
			case 0:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B0CR);
				break;
			case 1:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B1CR);
				break;
			case 2:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B2CR);
				break;
			case 3:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B3CR);
				break;
		}

		bank_config = mfdcr(DCRN_SDRAM0_CFGDATA);

		if (!(bank_config & SDRAM_CONFIG_BANK_ENABLE))
			continue;
		switch (SDRAM_CONFIG_BANK_SIZE(bank_config))
		{
			case SDRAM_CONFIG_SIZE_8M:
				mem_size += PPC440_MEM_SIZE_8M;
				break;
			case SDRAM_CONFIG_SIZE_16M:
				mem_size += PPC440_MEM_SIZE_16M;
				break;
			case SDRAM_CONFIG_SIZE_32M:
				mem_size += PPC440_MEM_SIZE_32M;
				break;
			case SDRAM_CONFIG_SIZE_64M:
				mem_size += PPC440_MEM_SIZE_64M;
				break;
			case SDRAM_CONFIG_SIZE_128M:
				mem_size += PPC440_MEM_SIZE_128M;
				break;
			case SDRAM_CONFIG_SIZE_256M:
				mem_size += PPC440_MEM_SIZE_256M;
				break;
			case SDRAM_CONFIG_SIZE_512M:
				mem_size += PPC440_MEM_SIZE_512M;
				break;
		}
	}
	return mem_size;
}

#ifdef CONFIG_SNSC_CXD9790_INTR
/*
 * Interrup handler for CXD9790 IROUT0/1
 *
 * determin CXD9790 irq# and dispatch handler
 *
 */
static void kaiser_handle_cxd9790_intr(int irq, void *dev_id, struct pt_regs *regs)
{
        int cxd9790_irq = -1;

        if(irq == MPU4xx_CXD9790_IRQ_INTO0)
                cxd9790_irq = cxd9790_get_irq(IROUT0);
        else if (irq == MPU4xx_CXD9790_IRQ_INTO1)
                cxd9790_irq = cxd9790_get_irq(IROUT1);

        if(cxd9790_irq < 0)
                return;

        ppc_irq_dispatch_handler(regs, cxd9790_irq);
}
#endif /* CONFIG_SNSC_CXD9790_INTR */

static void __init
kaiser_init_irq(void)
{
	int i;

	ppc4xx_pic_init();

	for (i = 0; i < NR_IRQS; i++)
		irq_desc[i].handler = ppc4xx_pic;

#ifdef CONFIG_SNSC_CXD9790_INTR
        /* initialize CXD9790 interrupt */
        if(mpu4xx_cxd9790_base_vaddr && !cxd9790_irq_init()) {
                request_irq(MPU4xx_CXD9790_IRQ_INTO0, kaiser_handle_cxd9790_intr, 
                        SA_INTERRUPT, "CXD9790 IROUT0", NULL);
                request_irq(MPU4xx_CXD9790_IRQ_INTO1, kaiser_handle_cxd9790_intr, 
                        SA_INTERRUPT, "CXD9790 IROUT1", NULL);
        }
#endif /* CONFIG_SNSC_CXD9790_INTR */
}


void __init platform_init(unsigned long r3, unsigned long r4,
		unsigned long r5, unsigned long r6, unsigned long r7)
{
	parse_bootinfo((struct bi_record *) (r3 + KERNELBASE));

	ppc_md.setup_arch = kaiser_setup_arch;
	ppc_md.show_cpuinfo = kaiser_show_cpuinfo;
	ppc_md.init_IRQ = kaiser_init_irq;
	ppc_md.get_irq = NULL;		/* Set in ppc4xx_pic_init() */

	ppc_md.find_end_of_memory = kaiser_find_end_of_memory;
	ppc_md.setup_io_mappings = NULL;

	ppc_md.restart = kaiser_restart;
	ppc_md.power_off = kaiser_power_off;
	ppc_md.halt = kaiser_halt;

	ppc_md.calibrate_decr = kaiser_calibrate_decr;
#ifdef CONFIG_PPC_RTC
	ppc_md.time_init = todc_time_init;
	ppc_md.set_rtc_time = todc_set_rtc_time;
	ppc_md.get_rtc_time = todc_get_rtc_time;

	ppc_md.nvram_read_val = todc_direct_read_val;
	ppc_md.nvram_write_val = todc_direct_write_val;
#endif /* CONFIG_PPC_RTC */
#ifdef CONFIG_RTC7301_RTC
	ppc_md.get_rtc_time = rtc7301_rtc_rtc_get_time;
#endif /* CONFIG_RTC7301_RTC */

#ifdef CONFIG_SERIAL_TEXT_DEBUG
	/* ppc_md.progress_init = ? */
	ppc_md.progress = gen550_progress;
#endif /* CONFIG_SERIAL_TEXT_DEBUG */
#ifdef CONFIG_KGDB
	ppc_md.early_serial_map = kaiser_early_serial_map;
#endif
#if defined(CONFIG_VT) && !defined(CONFIG_DUMMY_KEYB)
	/* USB keyboard support */
	ppc_md.kbd_setkeycode = pckbd_setkeycode;
	ppc_md.kbd_getkeycode = pckbd_getkeycode;
	ppc_md.kbd_translate = pckbd_translate;
#endif
#ifndef CONFIG_SNSC_BOOT_TIME_USE_NBLARGS
	/* 
	 * Set the time base to zero.
	 *
	 * This routine is called earlier at boot-time's mach_init()
	 * if CONFIG_SNSC_BOOT_TIME is defined.
	 */
	mtspr(SPRN_TBWL, 0);
	mtspr(SPRN_TBWU, 0);
#endif

#ifdef CONFIG_SNSC_MPU4xx_MACADDR
	ppc_md.get_enetaddr = mpu4xx_get_enetaddr;
#endif
}
