/*
 * arch/ppc/platforms/kaiser.h
 *
 * Kaiser board definitions
 *
 * Matt Porter <mporter@mvista.com>
 *
 * Copyright 2002 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 *  05/07/02 - Armin
 *  removed IIC_PORT_DFNS 
 */

#ifdef __KERNEL__
#ifndef __ASM_KAISER_H__
#define __ASM_KAISER_H__

#include <linux/config.h>
#include <platforms/ibm440gp.h>
#include <platforms/snsc-mpu4xx_cxd9790.h>

#ifndef __ASSEMBLY__
/*
 * Data structure defining board information maintained by the boot
 * ROM on IBM's "Ebony" evaluation board. An effort has been made to
 * keep the field names consistent with the 8xx 'bd_t' board info
 * structures.
 *
 * Ebony firmware stores MAC addresses in the F/W VPD area. The
 * firmware must store the other dynamic values in NVRAM like on
 * the previous 40x systems so they  should be accessible if we
 * really want them.
 */
typedef struct board_info {
	unsigned char	bi_enetaddr[2][6];	/* EMAC addresses */
	unsigned int	bi_opb_busfreq;		/* OPB clock in Hz */
	int		bi_iic_fast[2];		/* Use fast i2c mode */
	int             bi_enetaddr_valid[2];   /* enetaddr is valid or not */
} bd_t;
#endif /* __ASSEMBLY__ */

#ifndef NR_BOARD_IRQS
#define NR_BOARD_IRQS 0
#endif

#define _IO_BASE	isa_io_base
#define _ISA_MEM_BASE	isa_mem_base
#define PCI_DRAM_OFFSET	pci_dram_offset



/* F/W TLB mapping used in bootloader glue to reset EMAC */
#define KAISER_EMAC0_MR0		0xE0000800

/* Macros to get at Kaiser VPD info */
#define KAISER_VPD_BASE		0x00000001fffffe00
#define KAISER_VPD_SIZE		0x24
#define KAISER_NA0_OFFSET	0x0c
#define KAISER_NA1_OFFSET	0x18
#define KAISER_NA0_ADDR(base)	(base + KAISER_NA0_OFFSET)
#define KAISER_NA1_ADDR(base)	(base + KAISER_NA1_OFFSET)

/* Default clock rates for Rev. B and Rev. C silicon */
#define KAISER_440GP_RB_SYSCLK	33000000
#define KAISER_440GP_RC_SYSCLK	400000000

/* RTC/NVRAM location */
#define KAISER_RTC_ADDR		0x0000000148000000
#define KAISER_RTC_SIZE		0x2000

/* Flash */
#define KAISER_FPGA_ADDR		0x0000000148300000
#define KAISER_BOOT_SMALL_FLASH(x)	(x & 0x20)
#define KAISER_ONBRD_FLASH_EN(x)		(x & 0x02)
#define KAISER_FLASH_SEL(x)		(x & 0x01)
#define KAISER_SMALL_FLASH_LOW1	0x00000001ff800000
#define KAISER_SMALL_FLASH_LOW2	0x00000001ff880000
#define KAISER_SMALL_FLASH_HIGH1	0x00000001fff00000
#define KAISER_SMALL_FLASH_HIGH2	0x00000001fff80000
#define KAISER_SMALL_FLASH_SIZE	0x80000
#define KAISER_LARGE_FLASH_LOW	0x00000001ff800000
#define KAISER_LARGE_FLASH_HIGH	0x00000001ffc00000
#define KAISER_LARGE_FLASH_SIZE	0x400000

#define KAISER_SMALL_FLASH_BASE	0x00000001fff80000
#define KAISER_LARGE_FLASH_BASE	0x00000001ff800000

/*
 * Serial port defines
 */
#ifdef USE_ONCHIP_UART1	   /* MPU-400 do net have on-chip UART1 port*/
#define NUM_IBM440GP_UART 2
#else
#define NUM_IBM440GP_UART 1
#endif

#if defined (CONFIG_SNSC_CXD9790_UART)
#define NUM_CXD9790_UART	4
#else
#define NUM_CXD9790_UART	0
#endif

#define RS_TABLE_SIZE	(NUM_IBM440GP_UART + NUM_CXD9790_UART)
extern struct serial_state rs_table[];

/* OpenBIOS defined UART mappings, used before early_serial_setup */
#define UART0_IO_BASE	(u8 *) 0xE0000200
#define UART1_IO_BASE	(u8 *) 0xE0000300

#ifdef CONFIG_UART_CLK_INTERNAL
#define BASE_BAUD	133300000/24/16
#else
#define BASE_BAUD	11059200/16
#endif

#define SERIAL_IRQ0_VALID TRUE
#define UART0_INT	0
#define UART1_INT	1

#define KAISER_UART_OP(num)					\
	{ 0, BASE_BAUD, 0, UART##num##_INT,			\
		(ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST),	\
		iomem_base: UART##num##_IO_BASE,		\
		io_type: SERIAL_IO_MEM},

#ifdef USE_ONCHIP_UART1
#define IBM440GP_UART_OP	\
	KAISER_UART_OP(0)	\
	KAISER_UART_OP(1)
#else
#define IBM440GP_UART_OP	\
	KAISER_UART_OP(0)
#endif

#define SERIAL_PORT_DFNS	\
	IBM440GP_UART_OP	\
	MPU4xx_CXD9790_UART_OPS


/* USB keyboard support */
#define kbd_read_input()	0
#define kbd_read_status()	0
#define kbd_write_output(val)	{}
#define kbd_write_command(val)	{}

/* PCI support */
#define KAISER_PCI_LOWER_IO	0x00000000
#define KAISER_PCI_UPPER_IO	0x0000ffff
#define KAISER_PCI_LOWER_MEM	0x80000000
#define KAISER_PCI_UPPER_MEM	0xffffefff

#define KAISER_PCI_CFGREGS_BASE	0x000000020ec00000
#define KAISER_PCI_CFGA_PLB32	0x0ec00000
#define KAISER_PCI_CFGD_PLB32	0x0ec00004

#define KAISER_PCI_IO_BASE	0x0000000208000000
#define KAISER_PCI_IO_SIZE	0x00010000
#define KAISER_PCI_MEM_OFFSET	0x00000000


/* External peripheral definition */
/* Peripheral CS# physical address (defined by boot loader) */
#define KAISER_PER_CS1_PADDR	0x0000000149000000
#define KAISER_PER_CS2_PADDR	0x0000000148100000
#define KAISER_PER_CS3_PADDR	0x0000000148200000
#define KAISER_PER_CS4_PADDR	0x0000000148000000
#define KAISER_PER_CS5_PADDR	0x000000014e000000
#define KAISER_PER_CS6_PADDR	0x000000014c000000
#define KAISER_PER_CS7_PADDR	0x000000014a000000

#define MPU4xx_CXD9790_PADDR	KAISER_PER_CS2_PADDR
#define MPU4xx_CXD9790_SIZE	256

#define KAISER_XR1_PADDR	KAISER_PER_CS3_PADDR

/* IRQs */

#ifdef CONFIG_SNSC_CXD9790_INTR

#define CXD9790_IRQ_OFFSET	64 /* PPC440GP UIC IRQ at 0-63 */
#define CXD9790_IRQ_DEPTH	1


/* CXD9790 INTO0/1 IRQ# */
#define MPU4xx_CXD9790_IRQ_INTO0	27 /* Ext Int 4 */
#define MPU4xx_CXD9790_IRQ_INTO1	28 /* Ext Int 5 */

/* 
  NR_BOARD_IRQS defined "0" at inculde/asm/ibm440.h, but we redefine it 
  for CXD9790 interrupts handlers
*/
#ifdef NR_BOARD_IRQS
#undef NR_BOARD_IRQS
#endif
#define NR_BOARD_IRQS 48 /* number of CXD9790 irqs */
#endif /* CONFIG_SNSC_CXD9790_INTR */


/* 
 * 440GP Clock value
 */
extern struct ibm440_clocks ibm440_clks;

/* 
 * Boot time default address
 */
#define BOOT_TIME_SIZE		(0x1000)	/* 4KB */
#define BOOT_TIME_BASE		(0x00800000 - BOOT_TIME_SIZE) /* XXX: +8MB offset. check again */


/* 
 * DMA needs PLB address to r/w to PCI area 
 */
#define TO_PLBADDR(x)    (0x300000000LL | (phys_addr_t)(x))

#endif				/* __ASM_KAISER_H__ */
#endif				/* __KERNEL__ */
