/*
 *  IDE driver for TDK XR1 Kaiser specific definition
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _KAISER_XR1_H_
#define _KAISER_XR1_H_

#include <linux/config.h>
#include "kaiser.h"
#include <linux/cxd9790.h>
#include <linux/cxd9790_gpio.h>
#include <asm/io.h>
#include <asm/cache.h>

/* define if IOA-6 setting */
#ifdef CONFIG_USE_IOA_6
#  define USE_IOA_6
#else
#  undef  USE_IOA_6
#endif

#ifdef USE_IOA_6
#define XR1_INTR_GPIO_BITNUM    7
#else /* on board XR1 setting */
#define XR1_INTR_GPIO_BITNUM    3
#endif

/* interrupt definition */
#define XR1_INTR_GPIO_PORT      GPIO_PORT_A
#define XR1_IRQ  CXD9790_IRQ_GPIOA(XR1_INTR_GPIO_BITNUM)

/* base address */
static void *kaiser_xr1_base_addr;
#define XR1_BASE (kaiser_xr1_base_addr - isa_io_base)

/* XR1 burst register */
/* it must be mapped cache enabled */
static void *kaiser_xr1_burst_reg;

/* device name */
#define XR1_DEVNAME     "XR1 IDE"

static inline void
xr1_platform_init(void)
{
#ifdef USE_IOA_6
	kaiser_xr1_base_addr = ioremap64(KAISER_PER_CS5_PADDR, 1024);
	kaiser_xr1_burst_reg = __ioremap(KAISER_PER_CS5_PADDR, 512, 0); /* cache enable */
#else /* on board */
	kaiser_xr1_base_addr = ioremap64(KAISER_XR1_PADDR, 1024);
	kaiser_xr1_burst_reg = __ioremap(KAISER_XR1_PADDR, 512, 0); /* cache enable */
#endif /* USE_IOA_6 */
	if(!kaiser_xr1_base_addr || !kaiser_xr1_burst_reg)
		return;

        /* GPIO port setting */

        /* NOTE: no error check */
#ifdef CONFIG_SNSC_CXD9790
        cxd9790_gpio_init();
        cxd9790_gpio_register_bit(XR1_INTR_GPIO_PORT, XR1_INTR_GPIO_BITNUM,
                                  XR1_DEVNAME);
        cxd9790_gpio_set_mode_bit(XR1_INTR_GPIO_PORT, XR1_INTR_GPIO_BITNUM,
                                  GPIO_MODE_IN);
        cxd9790_gpio_intr_set_mode(XR1_INTR_GPIO_PORT, XR1_INTR_GPIO_BITNUM,
                                   GPIO_INTR_LEVEL_LOW);
        cxd9790_gpio_intr_unmask_bit(XR1_INTR_GPIO_PORT, XR1_INTR_GPIO_BITNUM);
#endif

        return;
}


/* platform dependent ideproc routine is supported for burst access to XR1 */
#define XR1_PLATFORM_IDEPROC

#define DCACHE_LINE_WORD (L1_CACHE_LINE_SIZE / 4)
static spinlock_t ideproc_lock = SPIN_LOCK_UNLOCKED;

static void 
xr1_platform_ideproc(ide_ide_action_t action, ide_drive_t *drive,
			void *buffer, unsigned int count)
{
	int alsize;
	unsigned long flags;

	if(!kaiser_xr1_base_addr || !kaiser_xr1_burst_reg)
		return;

	alsize = (count / DCACHE_LINE_WORD) * DCACHE_LINE_WORD;

	switch(action) {
	case ideproc_ide_input_data:
		spin_lock_irqsave(&ideproc_lock, flags);
		__asm__ __volatile__ (
			"	mtctr	%0		# set counter		\n"
			"1:							\n"
			"	dcbi	0, %1		# invalidate cache line \n"
			"	lswi	%%r8, %1, 32				\n"
			"	stswi	%%r8, %2, 32				\n"
			"	addi	%2, %2, 32				\n"
			"	bdnz	1b					\n"
			:
			: "r" ((alsize / DCACHE_LINE_WORD)),
			  "p" (kaiser_xr1_burst_reg),
			  "p" (buffer)
			: "%r8", "%r9", "%r10", "%r11", "%r12", "%r13", "%r14", "%r15"
		);
		spin_unlock_irqrestore(&ideproc_lock, flags);

		count -= alsize;
		if (count > 0) {
			buffer = (void *)((u_int32_t *)buffer + alsize);
			insw(IDE_DATA_REG, buffer, count<<1);
		}
		break;

	case ideproc_ide_output_data:
		spin_lock_irqsave(&ideproc_lock, flags);
		__asm__ __volatile__ (
			"	mtctr	%0		# set counter		\n"
			"1:							\n"
			"	dcbz	0, %1		# zero clear cache line	\n"
			"				# and mark dirty	\n"
			"				# without memory r/w op	\n"
			"	lswi	%%r8, %2, 32				\n"
			"	stswi	%%r8, %1, 32				\n"
			"	dcbf	0, %1		# flush cache line	\n"
			"	addi	%2, %2, 32				\n"
			"	bdnz	1b					\n"
			:
			: "r" ((alsize / DCACHE_LINE_WORD)),
			  "p" (kaiser_xr1_burst_reg),
			  "p" (buffer)
			: "%r8", "%r9", "%r10", "%r11", "%r12", "%r13", "%r14", "%r15"
		);
		spin_unlock_irqrestore(&ideproc_lock, flags);

		count -= alsize;
		if (count > 0) {
			buffer = (void *)((u_int32_t *)buffer + alsize);
			outsw(IDE_DATA_REG, buffer, count<<1);
		}
		break;

	case ideproc_atapi_input_bytes:
		count++;
		insw(IDE_DATA_REG, buffer, count>>1);
		if ((count & 0x03) >= 2)
			insw (IDE_DATA_REG, ((byte *)buffer) + (count & ~0x03), 1);
		break;
	case ideproc_atapi_output_bytes:
		count++;
		outsw(IDE_DATA_REG, buffer, count>>1);
		if ((count & 0x03) >= 2)
			outsw (IDE_DATA_REG, ((byte *)buffer) + (count & ~0x03), 1);
		break;
	}

}

#endif /* !_KAISER_XR1_H */
