#include <asm/system.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/module.h>

/*
 * PLB Processor Local Bus
 */

static int plb_bus_match(struct device *dev, struct device_driver *drv);

static struct constraints plb_constraints = {
	count:		0,
};

static struct bus_op_point plb_op = {
	count:		0,
};

struct bus_type plb_bus = {
        name:		"plb",
        match:		plb_bus_match,
	constraints:	&plb_constraints,
	bus_op:		&plb_op,
};

struct device plb_device = {
	name:		"PLB Processor Local Bus Controller",
	bus_id:		"plb",
	parent:		NULL,
	bus:		&plb_bus,
	driver:		NULL,
	power_state:	DPM_POWER_ON,
};

static int
plb_bus_match(struct device *dev, struct device_driver *drv)
{
	if (strcmp(dev->bus_id, "lcd") == 0)
		return (strcmp(drv->name, "ibmlcdfb") == 0);

	return 0;
}

void plb_device_register(struct device *device)
{
	device->parent = &plb_device;
	device->bus = &plb_bus;
	device_register(device);
	return;
}

void plb_driver_register(struct device_driver *driver)
{
	driver->bus = &plb_bus;
	(void) driver_register(driver);
	return;
}


/*
 * OPB Onboard Peripheral Bus
 */

static int opb_bus_match(struct device *dev, struct device_driver *drv);

static struct constraints opb_constraints = {
	count:		0,
};

static struct bus_op_point opb_op = {
	count:		0,
};

struct bus_type opb_bus = {
        name:		"opb",
        match:		opb_bus_match,
	constraints:	&opb_constraints,
	bus_op:		&opb_op,
};

struct device opb_device = {
	name:		"OPB Onboard Peripheral Bus Controller",
	bus_id:		"opb",
	parent:		NULL,
	bus:		&opb_bus,
	driver:		NULL,
	power_state:	DPM_POWER_ON,
};

static int
opb_bus_match(struct device *dev, struct device_driver *drv)
{
	if (strncmp(dev->bus_id, "uart", 4) == 0)
		return (strcmp(drv->name, "serial") == 0);

	return 0;
}

void opb_device_register(struct device *device)
{
	device->parent = &opb_device;
	device->bus = &opb_bus;
	device_register(device);
	return;
}

void opb_driver_register(struct device_driver *driver)
{
	driver->bus = &opb_bus;
	(void) driver_register(driver);
	return;
}


/*
 * EBC External Bus Controller
 */

static int ebc_bus_match(struct device *dev, struct device_driver *drv);

static struct constraints ebc_constraints = {
	count:		0,
};

static struct bus_op_point ebc_op = {
	count:		0,
};

struct bus_type ebc_bus = {
        name:		"ebc",
        match:		ebc_bus_match,
	constraints:	&ebc_constraints,
	bus_op:		&ebc_op,
};

struct device ebc_device = {
	name:		"EBC External Bus Controller",
	bus_id:		"ebc",
	parent:		NULL,
	bus:		&ebc_bus,
	driver:		NULL,
	power_state:	DPM_POWER_ON,
};

static int
ebc_bus_match(struct device *dev, struct device_driver *drv)
{
	if (strcmp(dev->bus_id, "eth") == 0)
		return (strcmp(drv->name, "cs89x0") == 0);

	return 0;
}

void ebc_device_register(struct device *device)
{
	device->parent = &ebc_device;
	device->bus = &ebc_bus;
	device_register(device);
	return;
}

void ebc_driver_register(struct device_driver *driver)
{
	driver->bus = &ebc_bus;
	(void) driver_register(driver);
	return;
}

void __init ocp_ldm_init(void)
{
	bus_register(&plb_bus);
	device_register(&plb_device);
	bus_register(&opb_bus);
	device_register(&opb_device);
        bus_register(&ebc_bus);
        device_register(&ebc_device);
	return;
}

EXPORT_SYMBOL(plb_device_register);
EXPORT_SYMBOL(plb_driver_register);
EXPORT_SYMBOL(ebc_device_register);
EXPORT_SYMBOL(ebc_driver_register);
