/*
 * arch/ppc/platforms/snsc-mpu410.c
 *
 *  MPU-410 board specific routines
 *
 *  Copyright 2003,2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/serialP.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>
#include <asm/time.h>
#include <asm/todc.h>
#include <asm/bootinfo.h>
#include <asm/ppc4xx_pic.h>
#include <asm/ppc4xx_gpio.h>

#include <platforms/ibm_ocp.h>

#define NAME "MPU-410: " 

#ifdef CONFIG_SNSC_CXD9790
#include <linux/cxd9790.h>
void *mpu4xx_cxd9790_base_vaddr;
#ifdef CONFIG_SNSC_CXD9790_UART
#include <linux/serial_reg.h>
#include <asm/serial.h>
#endif /* CONFIG_SNSC_CXD9790_UART */
#endif /* CONFIG_SNSC_CXD9790 */

#ifdef CONFIG_RTC7301_RTC
#include <asm/rtc7301.h>
void *ppc_rtc7301_base_vaddr;
extern unsigned long rtc7301_rtc_rtc_get_time(void);
#endif

/* All clocks in Hz */
struct ibm440_clocks ibm440_clks;


#if !defined(CONFIG_DUMMY_KEYB)
extern int pckbd_setkeycode(unsigned int scancode, unsigned int keycode);
extern int pckbd_getkeycode(unsigned int scancode);
extern int pckbd_translate(unsigned char scancode, unsigned char *keycode,
		                           char raw_mode);
#endif



/*
 * MPU-410 IRQ triggering/polarity settings
 */
static u_char mpu410_IRQ_initsenses[] __initdata = {
	UIC0_INITSENSE
	UIC1_INITSENSE
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 64: GPIO/ExtInt 0 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 65: GPIO/ExtInt 1 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 66: GPIO/ExtInt 2 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 67: GPIO/ExtInt 3 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 68: GPIO/ExtInt 4 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 69: GPIO/ExtInt 5 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 70: GPIO/ExtInt 6 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 71: GPIO/ExtInt 7 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 72: GPIO/ExtInt 8 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 73: GPIO/ExtInt 9 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 74: GPIO/ExtInt 10 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 75: GPIO/ExtInt 11 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 76: GPIO/ExtInt 12 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 77: GPIO/ExtInt 13 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 78: GPIO/ExtInt 14 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 79: GPIO/ExtInt 15 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 80: GPIO/ExtInt 16 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 81: GPIO/ExtInt 17 */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 82: GPIO/ExtInt 18 */
  (IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_NEG),	/* 83: GPIO/ExtInt 19 (NNMI) */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 84: ExtInt 20 (9790 INT0#) */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 85: ExtInt 21 (9790 INT1#) */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 86: ExtInt 22 (open) */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 87: ExtInt 23 (USBDPDECT) */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 88: ExtInt 24 (FLASH_BSY#) */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 89: ExtInt 25 (STB_TRG_IN#) */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 90: MS Pro Insert Detect */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG),	/* 91: MS Pro Core Int */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 92: Reserved */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 93: Reserved */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 94: Reserved */
  (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS),	/* 95: Reserved */
};



extern void abort(void);

/* Global Variables */
unsigned char __res[sizeof (bd_t)];


static int
mpu410_show_cpuinfo(struct seq_file *m)
{
	seq_printf(m, "vendor\t\t: Sony\n");
	seq_printf(m, "machine\t\t: CXD-4707\n");

	return 0;
}

/* INT routing to Slot 0: Device 0. */
#define PCI_INT_A 43
#define PCI_INT_B 40
#define PCI_INT_C 41
#define PCI_INT_D 42
static char pci_irq_table[4] = {PCI_INT_A, PCI_INT_B, PCI_INT_C, PCI_INT_D};
static inline int
mpu410_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
	return pci_irq_table[(idsel + pin - 1) % 4];
}

static void __init
mpu410_setup_hose(void)
{
	cxd4707_setup_hose();
	ppc_md.pci_swizzle 	= common_swizzle;
	ppc_md.pci_map_irq 	= mpu410_map_irq;
}

static void __init
mpu410_early_serial_map(void)
{
	cxd4707_init_uart(0);
	cxd4707_init_uart(1);
	cxd4707_init_uart(2);
	cxd4707_init_uart(3);
}

#ifdef CONFIG_SNSC_PANIC_HOOK
static int
mpu410_panic_event(struct notifier_block *this, unsigned long event, void *ptr)
{
	printk("%s()\n", __FUNCTION__);
        return NOTIFY_DONE;
}

static struct notifier_block mpu410_panic_block = {
	mpu410_panic_event,
	NULL,
        INT_MIN,	/* this is called at last */
};
#endif

#ifdef CONFIG_LEDS
/* 
 * ARM-style led funcs
 */
#include <asm/leds.h>

#define LED_BASE	((u8 *)mpu4xx_cxd9790_base_vaddr)
#define LED_SET(x)	writeb(x, LED_BASE)
static unsigned char saved_leds = 0;

#define SNSC_NBL_LOWADDR_CKSUM
#ifdef SNSC_NBL_LOWADDR_CKSUM
#include <asm/checksum.h>

static u_int32_t nbl_cksum = 0;

int
do_cksum(void)
{
	const unsigned char *buf = 0xc0000000;
	int len = 0x00100000;

	if (nbl_cksum == 0)
		nbl_cksum = csum_partial(buf, len, 0);

	if (nbl_cksum != csum_partial(buf, len, 0)) 
		return 1;
	return 0;
}
#endif /* SNSC_NBL_LOWADDR_CKSUM */

void 
mpu410_leds_event(led_event_t ledevt)
{
	unsigned long flags;

	local_irq_save(flags);
	switch (ledevt) {
	case led_idle_start:	/* ...X .... */
		break;
	case led_idle_end:	/* ...X .... */
		break;
	case led_timer:		/* X... .... */
		saved_leds ^= 0x80;
#ifdef SNSC_NBL_LOWADDR_CKSUM
		if (do_cksum())
			saved_leds &= ~0x40;
#endif
		break;
	case led_start:
		break;
	case led_stop:
		break;
	case led_claim:	/* override idle & timer leds */
		break;
	case led_release: /* restore idle & timer leds */
		break;
	case led_green_on:
		break;
	case led_green_off:
		break;
	case led_amber_on:
		break;
	case led_amber_off:
		break;
	case led_red_on:
		break;
	case led_red_off:
		break;
	default:
		if (ledevt < 16) {
			saved_leds &= ~((unsigned char)ledevt);
		}
		break;
	}
	local_irq_restore(flags);

	LED_SET(saved_leds);
}
#endif /* CONFIG_LEDS */

TODC_ALLOC();

static void __init
mpu410_setup_arch(void)
{
	bd_t *bip = (bd_t *)__res;
#ifdef CONFIG_SNSC_PANIC_HOOK
	extern int panic_timeout;
#endif

#if !defined(CONFIG_BDI_SWITCH)
	/*
	 * The Abatron BDI JTAG debugger does not tolerate others
	 * mucking with the debug registers.
	 */
        mtspr(SPRN_DBCR0, (DBCR0_TDE | DBCR0_IDM));
#endif

	ibm440_clks = cxd4707_get_clocks();
	bip->bi_opb_busfreq = ibm440_clks.opb;
	
#ifdef CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM
	/* 
	 * Retrieve MAC addresses from OCP IIC.
	 * See:
	 *    drivers/net/ibm_ocp/ibm_ocp_enet.c
	 *    drivers/i2c/i2c-ibm_iic.c
	 */
	/* Use IIC in standard (100 kHz) mode */
	bip->bi_iic_fast[0] = bip->bi_iic_fast[1] = 0;
#endif

#ifdef CONFIG_CXD4707_RTC
	/* Setup TODC access */
	cxd4707_todc_setup();
#endif

	/* init to some ~sane value until calibrate_delay() runs */
        loops_per_jiffy = 50000000/HZ;

	/* Setup PCI host bridge */
#ifdef CONFIG_PCI
	mpu410_setup_hose();
#endif

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0); /* /dev/ram */
	else
#endif
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x1601);	/* /dev/hdc1 */
#endif

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif

	mpu410_early_serial_map();

	/* Identify the system */
	printk("Sony NSC MPU-410 (C) 2003,2004 Sony Co.\n");
	cxd4707_probe_cpu();

	ibm4xxPIC_InitSenses = mpu410_IRQ_initsenses;
	ibm4xxPIC_NumInitSenses = sizeof(mpu410_IRQ_initsenses);

#ifdef CONFIG_SNSC_CXD9790
	mpu4xx_cxd9790_base_vaddr = ioremap64(MPU4xx_CXD9790_PADDR, MPU4xx_CXD9790_SIZE);
	printk(NAME "mpu4xx_cxd9790_base_vaddr = 0x%08x\n", 
	       (unsigned int)mpu4xx_cxd9790_base_vaddr);

#ifdef CONFIG_SNSC_CXD9790_UART
	/* setup iomem_base for UART1-4 */
	rs_table[NUM_CXD4707_UART+0].iomem_base = MPU4xx_CXD9790_BASE + 0x80;
	rs_table[NUM_CXD4707_UART+1].iomem_base = MPU4xx_CXD9790_BASE + 0x88;
	rs_table[NUM_CXD4707_UART+2].iomem_base = MPU4xx_CXD9790_BASE + 0x90;
	rs_table[NUM_CXD4707_UART+3].iomem_base = MPU4xx_CXD9790_BASE + 0x98;
#endif /* CONFIG_SNSC_CXD9790_UART */

#ifdef CONFIG_LEDS
	saved_leds = 0xff;
	LED_SET(saved_leds);
	leds_event = mpu410_leds_event;
#endif

#endif /* CONFIG_SNSC_CXD9790 */

#ifdef CONFIG_RTC7301_RTC
	ppc_rtc7301_init();
#endif


#ifdef CONFIG_MPU4xx_OPB_DISABLE_PARITY
	/* 
	 * For C-tail/USB-Host: disable EXPB Parity Checking
	 */
	{
		unsigned int ctl;

		/* read and set */
		mtdcr(DCRN_EBM0_CFGADDR, EBM0_CTL);
		ctl = mfdcr(DCRN_EBM0_CFGDATA);
		printk("OPB Disable Parity Check: EBM0_CTL = 0x%08x", ctl);

		mtdcr(DCRN_EBM0_CFGADDR, EBM0_CTL);
		mtdcr(DCRN_EBM0_CFGDATA, ctl | EBM0_CTL_PDIS);

		/* read again for check */
		mtdcr(DCRN_EBM0_CFGADDR, EBM0_CTL);
		ctl = mfdcr(DCRN_EBM0_CFGDATA);
		printk(" -> 0x%08x\n", ctl);
	}
#endif

	/* print pin-mux configs */
	cxd4707_read_pfc();

#ifdef CONFIG_SNSC_PANIC_HOOK
	panic_timeout = CONFIG_SNSC_PANIC_TIMEOUT;
	notifier_chain_register(&panic_notifier_list, &mpu410_panic_block);
#endif
}

static void
mpu410_restart(char *cmd)
{
	__cli();
#ifdef CONFIG_SNSC_CXD9790
	MPU4xx_CXD9790_GPIO_SETBIT(MPU4xx_SELFRST_GPIO,
				   MPU4xx_SELFRST_GPIO_BITNUM, 1);
	MPU4xx_CXD9790_GPIO_SETBIT(MPU4xx_SELFRST_GPIO,
				   MPU4xx_SELFRST_GPIO_BITNUM, 0);
#else
	abort();
#endif
}

static void
mpu410_power_off(void)
{
	__cli();
#ifdef CONFIG_SNSC_CXD9790
	MPU4xx_CXD9790_GPIO_SETBIT(MPU4xx_SHTDWNACK_GPIO, 
				   MPU4xx_SHTDWNACK_GPIO_BITNUM, 1);
	MPU4xx_CXD9790_GPIO_SETBIT(MPU4xx_PWROFF_GPIO, 
				   MPU4xx_PWROFF_GPIO_BITNUM, 1);
	MPU4xx_CXD9790_GPIO_SETBIT(MPU4xx_PWROFF_GPIO, 
				   MPU4xx_PWROFF_GPIO_BITNUM, 0);
#endif
	for(;;);
}

static void
mpu410_halt(void)
{
	__cli();
	for(;;);
}

#ifdef CONFIG_SNSC_CXD9790_INTR
/*
 * Interrup handler for CXD9790 IROUT0/1
 *
 * determin CXD9790 irq# and dispatch handler
 *
 */
static void mpu410_handle_cxd9790_intr(int irq, void *dev_id, struct pt_regs *regs)
{
        int cxd9790_irq = -1;

        if(irq == MPU4xx_CXD9790_IRQ_INTO0)
                cxd9790_irq = cxd9790_get_irq(IROUT0);
        else if (irq == MPU4xx_CXD9790_IRQ_INTO1)
                cxd9790_irq = cxd9790_get_irq(IROUT1);

        if(cxd9790_irq < 0)
                return;

        ppc_irq_dispatch_handler(regs, cxd9790_irq);
}
#endif /* CONFIG_SNSC_CXD9790_INTR */

static void __init
mpu410_init_irq(void)
{
	int i;

	ppc4xx_pic_init();

	for (i = 0; i < NR_IRQS; i++)
 		irq_desc[i].handler = ppc4xx_pic;

#ifdef CONFIG_SNSC_CXD9790_INTR
        /* initialize CXD9790 interrupt */
        if(mpu4xx_cxd9790_base_vaddr && !cxd9790_irq_init()) {
                request_irq(MPU4xx_CXD9790_IRQ_INTO0, mpu410_handle_cxd9790_intr, 
                        SA_INTERRUPT, "CXD9790 IROUT0", NULL);
                request_irq(MPU4xx_CXD9790_IRQ_INTO1, mpu410_handle_cxd9790_intr, 
                        SA_INTERRUPT, "CXD9790 IROUT1", NULL);
        }
#endif /* CONFIG_SNSC_CXD9790_INTR */
}

#ifdef CONFIG_PPC4xx_GPIO
static void __init
mpu410_init_gpio(int port, void *base, void *pgpio)
{
	ocp_gpio_t *gpio_base = base;
	struct ppc4xx_gpio_t *gpio = pgpio;
	u_int32_t sdr0_pfc0 = mfdcri(DCRN_SDR0, PFC0);
	int i;

	printk("%s(): port=%d, gpio_base=0x%p, gpio=0x%p\n", __FUNCTION__,
	       port, gpio_base, gpio);

	/* start with all unavailable */
	gpio->no_gpio 	= 0xffffffff;
	gpio->no_out  	= 0xffffffff;
	gpio->no_hiz  	= 0xffffffff;
	gpio->no_in   	= 0xffffffff;
	gpio->no_int  	= 0xffffffff;
	for (i = 0; i < 3; i++) {
		gpio->no_altout[i] = 0xffffffff;
		gpio->no_altin[i]  = 0xffffffff;
	}

	switch (port) {
	case GPIO_PORT_0:
		/* These pins can be used at any multiplex */
		gpio->no_gpio             &= ~0x0000ff00;
		gpio->no_out              &= ~0x0000ff00;
		gpio->no_hiz              &= ~0x0000ff00;
		gpio->no_in               &= ~0x0000ff00;
		gpio->no_int              &= ~0x0000f000;
		gpio->no_altin[0]         &= ~0x0000f000;
#ifdef CONFIG_CXD4707_ES_2_0
		gpio->no_altin[0]         &= ~0x00000800;
#endif
		/* 
		 * Multiplex Group0 
		 */
		switch (PFC0_GRP0_FUNC(sdr0_pfc0)) {
		case PFC0_GRP0_GPIO: /* Func0 */
			gpio->no_gpio     &= ~0xffff0000;
			gpio->no_out      &= ~0xffff0000;
			gpio->no_hiz      &= ~0xffff0000;
			gpio->no_in       &= ~0xffff0000;
			gpio->no_int      &= ~0xffff0000;
			gpio->no_altin[0] &= ~0xffff0000;
			break;
		case PFC0_GRP0_rsvd: /* Func1 */
			/* do nothing */
			break;
#ifdef CONFIG_CXD4707_ES_2_0
		case PFC0_GRP0_NAND8: /* Func2 (ES2.0) */
#else
		case PFC0_GRP0_NAND:  /* Func2 */
#endif
			gpio->no_gpio     &= ~0x00070000;
			gpio->no_out      &= ~0x00070000;
			gpio->no_hiz      &= ~0x00070000;
			gpio->no_in       &= ~0x00070000;
			gpio->no_int      &= ~0x00070000;
			gpio->no_altin[0] &= ~0x00070000;
			break;
#ifdef CONFIG_CXD4707_ES_2_0
		case PFC0_GRP0_NAND16: /* Func3 (ES2.0) */
#else
		case PFC0_GRP0_IDE0:   /* Func3 */
#endif
			/* do nothing */
			break;
		} 

		/* 
		 * Multiplex Group1
		 */
		switch (PFC0_GRP1_FUNC(sdr0_pfc0)) {
		case PFC0_GRP1_EBC32: /* Func0 */
			/* do nothing */
			break;
		case PFC0_GRP1_TRACE: /* Func1 */
			gpio->no_gpio     &= ~0x000000c0;
			gpio->no_out      &= ~0x000000c0;
			gpio->no_hiz      &= ~0x000000c0;
			gpio->no_in       &= ~0x000000c0;
			gpio->no_int      &= ~0x00000000;
			break;
		case PFC0_GRP1_GPIO: /* Func2 */
			gpio->no_gpio     &= ~0x000000ff;
			gpio->no_out      &= ~0x000000ff;
			gpio->no_hiz      &= ~0x000000ff;
			gpio->no_in       &= ~0x000000ff;
			gpio->no_int      &= ~0x00000000;
			break;
		case PFC0_GRP1_rsvd: /* Func3 */
			/* do nothing */
			break;
		} 

		break;
	case GPIO_PORT_1:
		/* These pins can be used at any multiplex */
		gpio->no_gpio             &= ~0x003fbf00;
		gpio->no_out              &= ~0x003fbf00;
		gpio->no_hiz              &= ~0x003fbf00;
		gpio->no_in               &= ~0x003fbf00;
#ifdef CONFIG_CXD4707_ES_2_0
		gpio->no_altin[0]         &= ~0x00003f00;
		gpio->no_altout[0]        &= ~0x003f8000;
#endif
		/* 
		 * Multiplex Group1
		 */
		switch (PFC0_GRP1_FUNC(sdr0_pfc0)) {
		case PFC0_GRP1_EBC32: /* Func0 */
			/* do nothing */
			break;
		case PFC0_GRP1_TRACE: /* Func1 */
			/* do nothing */
			break;
		case PFC0_GRP1_GPIO: /* Func2 */
			gpio->no_gpio     &= ~0xffc00000;
			gpio->no_out      &= ~0xffc00000;
			gpio->no_hiz      &= ~0xffc00000;
			gpio->no_in       &= ~0xffc00000;
			gpio->no_int      &= ~0x00000000;
			break;
		case PFC0_GRP1_rsvd: /* Func3 */
			/* do nothing */
			break;
		} 
		/* 
		 * Multiplex Group2
		 */
		if ((PFC0_GRP0_FUNC(sdr0_pfc0) == PFC0_GRP0_NAND16) ||
		    (PFC0_GRP0_FUNC(sdr0_pfc0) == PFC0_GRP0_NAND8)) {
			/* is NAND8/NAND16 */
			gpio->no_gpio     &= ~0x00000001;
			gpio->no_out      &= ~0x00000001;
			gpio->no_hiz      &= ~0x00000001;
			gpio->no_in       &= ~0x00000000;
			gpio->no_int      &= ~0x00000000;
		} else {
			switch (PFC0_GRP2_FUNC(sdr0_pfc0)) {
			case PFC0_GRP2_MODEM:  /* Func0 */
				gpio->no_gpio     &= ~0x00000001;
				gpio->no_out      &= ~0x00000001;
				gpio->no_hiz      &= ~0x00000001;
				gpio->no_in       &= ~0x00000000;
				gpio->no_int      &= ~0x00000000;
				break;
			case PFC0_GRP2_EBC: /* Func1 */
				/* do nothing */
				break;
			} 
		}
		/* 
		 * Multiplex Group3
		 */
		switch (PFC0_GRP3_FUNC(sdr0_pfc0)) {
		case PFC0_GRP3_USBH: /* Func0 */
			/* do nothing */
			break;
		case PFC0_GRP3_USBD: /* Func1 */
			gpio->no_gpio     &= ~0x00000002;
			gpio->no_out      &= ~0x00000002;
			gpio->no_hiz      &= ~0x00000000;
			gpio->no_in       &= ~0x00000000;
			gpio->no_int      &= ~0x00000000;
			break;
		} 

#ifdef CONFIG_CXD4707_ES_2_0
		/* 
		 * Multiplex Group4 
		 */
		switch (PFC0_GRP4_FUNC(sdr0_pfc0)) {
		case PFC0_GRP4_SCI: /* Func 0 */
			/* do nothing */
			break;
		case PFC0_GRP4_UART1: /* Func 1 */
			gpio->no_gpio     &= ~0x000040c0;
			gpio->no_out      &= ~0x000040c0;
			gpio->no_hiz      &= ~0x000040c0;
			gpio->no_in       &= ~0x000040c0;
			gpio->no_altout[0]&= ~0x00004080;
			gpio->no_altin[0] &= ~0x00000040;
			break;
		}

		/* 
		 * Multiplex Group5: has no GPIO
		 */

		/* 
		 * Multiplex Group6
		 */
		switch (PFC0_GRP6_FUNC(sdr0_pfc0)) {
		case PFC0_GRP6_GPIO: /* Func 0 */
			gpio->no_gpio     &= ~0x0000000c;
			gpio->no_out      &= ~0x0000000c;
			gpio->no_hiz      &= ~0x0000000c;
			gpio->no_in       &= ~0x0000000c;
			break;
		case PFC0_GRP6_PCI: /* Func 1 */
			/* do nothing */
			break;
		}

		/* 
		 * Multiplex Group7
		 */
		switch (PFC0_GRP7_FUNC(sdr0_pfc0)) {
		case PFC0_GRP7_GPIO: /* Func 0 */
			gpio->no_gpio     &= ~0x00000030;
			gpio->no_out      &= ~0x00000030;
			gpio->no_hiz      &= ~0x00000030;
			gpio->no_in       &= ~0x00000030;
			break;
		case PFC0_GRP7_PCI: /* Func 1 */
			/* do nothing */
			break;
		}
#endif

		break;
	}
	return ;
}

static int 
mpu410_irq_to_gpio_port(int irq)
{
	if (irq < 64 || irq > 83)
		return -1;
	else
		return 0;	/* all INTs are in GPIO port 0 */
}

static int 
mpu410_irq_to_gpio_bit(int irq)
{
	if (irq < 64 || irq > 83)
		return -1;
	else
		return irq - 64;
}
#endif /* CONFIG_PPC4xx_GPIO */

void __init platform_init(unsigned long r3, unsigned long r4,
		unsigned long r5, unsigned long r6, unsigned long r7)
{
	cxd4707_platform_init(r3, r4, r5, r6, r7);

	ppc_md.setup_arch	= mpu410_setup_arch;
	ppc_md.show_cpuinfo	= mpu410_show_cpuinfo;
	ppc_md.init_IRQ		= mpu410_init_irq;
	ppc_md.get_irq		= NULL;		/* Set in ppc4xx_pic_init() */

	ppc_md.restart		= mpu410_restart;
	ppc_md.power_off	= mpu410_power_off;
	ppc_md.halt		= mpu410_halt;

#ifdef CONFIG_KGDB
	ppc_md.early_serial_map = mpu410_early_serial_map;
#endif
#ifdef CONFIG_PPC4xx_GPIO
	ppc_md.init_gpio	= mpu410_init_gpio;
	ppc_md.irq_to_gpio_port	= mpu410_irq_to_gpio_port;
	ppc_md.irq_to_gpio_bit	= mpu410_irq_to_gpio_bit;
#endif
}

