/*
 * arch/ppc/platforms/snsc-mpu410.h
 *
 * MPU-410 board definitions
 *
 *  Copyright 2003,2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __KERNEL__
#ifndef __ASM_SNSC_MPU410_H__
#define __ASM_SNSC_MPU410_H__

#include <linux/config.h>
#include <platforms/cxd4707gb.h>
#ifdef CONFIG_SNSC_CXD9790
#include <platforms/snsc-mpu4xx_cxd9790.h>
#endif
#include <asm/machdep.h>

#ifndef __ASSEMBLY__
/*
 * Data structure defining board information maintained by the boot
 * ROM on IBM's "Ebony" evaluation board. An effort has been made to
 * keep the field names consistent with the 8xx 'bd_t' board info
 * structures.
 *
 * Ebony firmware stores MAC addresses in the F/W VPD area. The
 * firmware must store the other dynamic values in NVRAM like on
 * the previous 40x systems so they  should be accessible if we
 * really want them.
 */
typedef struct board_info {
	unsigned char	bi_enetaddr[2][6];	/* EMAC addresses */
	unsigned int	bi_opb_busfreq;		/* OPB clock in Hz */
	int		bi_iic_fast[2];		/* Use fast i2c mode */
	int             bi_enetaddr_valid[2];   /* enetaddr is valid or not */
} bd_t;
#endif /* __ASSEMBLY__ */

#ifndef NR_BOARD_IRQS
#define NR_BOARD_IRQS 0
#endif

#define _IO_BASE	isa_io_base
#define _ISA_MEM_BASE	isa_mem_base
#define PCI_DRAM_OFFSET	pci_dram_offset


/*
 * Serial port defines
 */

/* temporal:uart base clock */
#define UART_DIV		12
#define BASE_BAUD		CXD4707_PLBCLK/UART_DIV/16 
#define SERIAL_IRQ0_VALID 	TRUE

/* number of CXD4707 uart to use */
#define NUM_CXD4707_UART 4
#define MPU410_CXD4707_UART_OPS	\
	CXD4707_UART_OP(0)	\
	CXD4707_UART_OP(1)	\
	CXD4707_UART_OP(2)	\
	CXD4707_UART_OP(3)

#ifdef CONFIG_SNSC_CXD9790
#define SERIAL_PORT_DFNS	\
	MPU410_CXD4707_UART_OPS	\
	MPU4xx_CXD9790_UART_OPS
#define RS_TABLE_SIZE	(NUM_CXD4707_UART + NUM_CXD9790_UART)
#else
#define SERIAL_PORT_DFNS	\
	MPU410_CXD4707_UART_OPS	
#define RS_TABLE_SIZE	(NUM_CXD4707_UART)
#endif


extern struct serial_state rs_table[];

/* 
 * USB keyboard support 
 */
#define kbd_read_input()	0
#define kbd_read_status()	0
#define kbd_write_output(val)	{}
#define kbd_write_command(val)	{}

/* 
 * External peripheral definition 
 */
/* Peripheral CS# physical address (defined by boot loader) */
#define MPU410_PER_CS1_PADDR	(CXD4707_EXTDEV + 0x01000000)	/* JIG-21 */
#define MPU410_PER_CS2_PADDR	(CXD4707_EXTDEV + 0x00100000)	/* CXD9790 */
#define MPU410_PER_CS3_PADDR	(CXD4707_EXTDEV + 0x06000000)	/* CXD9823 */
#define MPU410_PER_CS4_PADDR	(CXD4707_EXTDEV + 0x00000000)	/* RTC-7301 */
#define MPU410_PER_CS5_PADDR	(CXD4707_EXTDEV + 0x06000000)
#define MPU410_PER_CS6_PADDR	(CXD4707_EXTDEV + 0x04000000)
#define MPU410_PER_CS7_PADDR	(CXD4707_EXTDEV + 0x02000000)

/* Flash */
#define MPU410_FPGA_ADDR		0x0000000148300000
#define MPU410_BOOT_SMALL_FLASH(x)	(x & 0x20)
#define MPU410_ONBRD_FLASH_EN(x)		(x & 0x02)
#define MPU410_FLASH_SEL(x)		(x & 0x01)
#define MPU410_SMALL_FLASH_LOW1	0x00000001ff800000
#define MPU410_SMALL_FLASH_LOW2	0x00000001ff880000
#define MPU410_SMALL_FLASH_HIGH1	0x00000001fff00000
#define MPU410_SMALL_FLASH_HIGH2	0x00000001fff80000
#define MPU410_SMALL_FLASH_SIZE	0x80000
#define MPU410_LARGE_FLASH_LOW	0x00000001ff800000
#define MPU410_LARGE_FLASH_HIGH	0x00000001ffc00000
#define MPU410_LARGE_FLASH_SIZE	0x400000

#define MPU410_SMALL_FLASH_BASE	0x00000001fff80000
#define MPU410_LARGE_FLASH_BASE	0x00000001ff800000

/* CXD-9790 */
#define MPU4xx_CXD9790_PADDR	MPU410_PER_CS2_PADDR
#define MPU4xx_CXD9790_SIZE	256

/* CXD-9823 */
#define MPU410_CXD9823_PADDR	MPU410_PER_CS3_PADDR

/* RTC-7301/NVRAM location */
#define MPU410_RTC7301_ADDR		MPU410_PER_CS4_PADDR
#define MPU410_RTC7301_SIZE		0x2000

/* CXD-4707 on-chip RTC */
#if defined (CONFIG_RTC) && defined (CONFIG_CXD4707_RTC)
#define CMOS_READ(addr) (ppc_md.nvram_read_val(DCRN_RTC0_SEC + (addr)))
#define CMOS_WRITE(val, addr) (ppc_md.nvram_write_val(DCRN_RTC0_SEC + (addr), (val)))
#define RTC_IRQ 38
#endif

/* 
 * IRQs 
 */

/* CXD9790 INTO0/1 IRQ# */
#ifdef CONFIG_SNSC_CXD9790_INTR

#define CXD9790_IRQ_OFFSET	96 /* CXD4707 UIC IRQ at 0-95 */
#define CXD9790_IRQ_DEPTH	1

#define MPU4xx_CXD9790_IRQ_INTO0	84 /* Ext Int 20 */
#define MPU4xx_CXD9790_IRQ_INTO1	85 /* Ext Int 21 */

/* 
  NR_BOARD_IRQS defined "0" at inculde/asm/ibm440.h, but we redefine it 
  for CXD9790 interrupts handlers
*/
#ifdef NR_BOARD_IRQS
#undef NR_BOARD_IRQS
#endif
#define NR_BOARD_IRQS 48 /* number of CXD9790 irqs */
#endif /* CONFIG_SNSC_CXD9790_INTR */


/* 
 * 440GP Clock value
 */
extern struct ibm440_clocks ibm440_clks;

/* 
 * Boot time default address
 */
#define BOOT_TIME_SIZE		(0x1000)	/* 4KB */
#define BOOT_TIME_BASE		(0x00800000 - BOOT_TIME_SIZE) /* +8MB offset */


/*
 * SDM and NAND flash
 */
#define CXD4707_NAND_CHIPS		1

#ifdef CONFIG_MTD_NAND_CXD4707_2X
#ifdef CONFIG_MTD_NAND_CXD4707_2X_TUNING

/* Default timing */
#define CXD4707_NAND_TIM_PRM_RTIM	23
#define CXD4707_NAND_TIM_PRM_UTIMW	8
#define CXD4707_NAND_TIM_PRM_UTIMR_H	23
#define CXD4707_NAND_TIM_PRM_UTIMR_S	11

#endif	/* CONFIG_MTD_NAND_CXD4707_2X_TUNING */
#else	/* CONFIG_MTD_NAND_CXD4707_2X */

/* Default timing */
#define CXD4707_NAND_PRM_RTIM		23
#define CXD4707_NAND_PRM_UTIMW		8
#define CXD4707_NAND_PRM_UTIMR_H	11
#define CXD4707_NAND_PRM_UTIMR_S	11

#endif	/* CONFIG_MTD_NAND_CXD4707_2X */


#endif				/* __ASM_SNSC_MPU410_H__ */
#endif				/* __KERNEL__ */
