/*
 *  arch/ppc/kernel/ppc4xx_boot-time.c
 *
 *  Copyright 2002,2003 Sony Corporation
 *
 *  This program is free software; you can redistribute	 it and/or modify it
 *  under  the terms of	 the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the	License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 *  WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO	EVENT  SHALL   THE AUTHOR  BE	 LIABLE FOR ANY	  DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 *  USE, DATA,	OR PROFITS; OR	BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>

#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/module.h>
#include <linux/seq_file.h>
#include <linux/init.h>
#include <linux/bootmem.h>

#include <asm/processor.h>
#include <asm/io.h>
#include <asm/string.h>
#include <asm/uaccess.h>
#include <asm/div64.h>

#include <linux/snsc_boot_time.h>
#include <linux/snsc_nblargs.h>

#define NAME "BootTime: "

/* because it is big-endian machine, hi-32bit must come first */
struct u64_time {
	u32	hi;
	u32	lo;
};

static void
mach_init(struct boot_time *pBootTime, int rom_init)
{
 	if (rom_init) {
		printk("pBootTime        = 0x%p\n", pBootTime);
		return ;
	}
	return ;
}


static void
mach_count_set(struct boot_time_entry *pEntry)
{
	/* Read Time Base Read Lower/Upper Register (user, R/O) */
	pEntry->count_lo = mfspr(SPRN_TBRL);
	pEntry->count_hi = mfspr(SPRN_TBRU);
}

static void
mach_print(struct seq_file *m, struct boot_time_entry *pEntry)
{
	struct u64_time time;
	u64 *dividend;
	u32 ms;

	/* Time Base counter runs at CPU core clock speed */
	time.lo = pEntry->count_lo;
	time.hi = pEntry->count_hi;
	dividend = (u64 *)&time;

	do_div(*dividend, (u32)(ibm440_clks.cpu / 1000));

	ms  = time.lo;		/* can count up to 49 days */

	seq_printf(m, " : %5u [ms]\n", ms);
}

static struct boot_time_md btmd = {
	base:      NULL, 
	init:      mach_init,
	count_set: mach_count_set,
	print:     mach_print,
};

static inline void
reserve_default_boottime_region(void)
{
	/* default value if without nblargs */
	reserve_bootmem(BOOT_TIME_BASE, BOOT_TIME_SIZE);
	printk(KERN_INFO "reserved memory at 0x%08x size 0x%x\n",
	       (unsigned int)BOOT_TIME_BASE, BOOT_TIME_SIZE);
}

void
mach_boot_time_init(void)
{
	static int initialized = 0;
#ifdef CONFIG_SNSC_BOOT_TIME_USE_NBLARGS
	struct nblargs_entry na;
#endif

	if (initialized)
		return;

	initialized = 1;

#ifdef CONFIG_SNSC_BOOT_TIME_USE_NBLARGS
	if (nblargs_get_key(SNSC_BOOT_TIME_KEY, &na) < 0) {
		printk("%s(): no key \"" SNSC_BOOT_TIME_KEY "\" found in NBLArgs.\n",
		       __FUNCTION__);
		return ;
	} else {
		printk(KERN_INFO "NBLArgs key \"" SNSC_BOOT_TIME_KEY
		       "\" found(0x%08x)", na.addr);
		if (na.size < ENTRY_DATA_SIZE) {
			printk(", but size(0x%x) is too small\n", na.size);
			return;
		}
		printk("\n");
		btmd.base = nbl_to_va(na.addr);
		/* specified boottime region is already reserved by
		   nblargs_init() */
	}
#else
	{
		reserve_default_boottime_region();
		/* replace to VA */
		btmd.base = __va(BOOT_TIME_BASE);
	}

#endif
	boot_time_init(&btmd);
}

