/*
 *  Target specific definitions for CXD9790GG for MPU-4xx
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SNSC_MPU4xx_CXD9790_H__
#define __SNSC_MPU4xx_CXD9790_H__

/* base virtual address */
extern void *mpu4xx_cxd9790_base_vaddr;
#define MPU4xx_CXD9790_BASE ((u8 *)mpu4xx_cxd9790_base_vaddr)

/* access macros */
#define CXD9790_READ(addr)		readb(MPU4xx_CXD9790_BASE + (addr))
#define CXD9790_WRITE(addr, val)	writeb(val, MPU4xx_CXD9790_BASE + (addr))

/* IRQ definition */
/* define number of board specific IRQ */

#ifdef CONFIG_SNSC_CXD9790_UART
#include <linux/cxd9790.h>

#define STD_COM_FLAGS 		(ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST)
#define MPU4xx_CXD9790_BASE_BAUD	(24000000 / 16)
#define NUM_CXD9790_UART	4
#define MPU4xx_CXD9790_UART_OPS \
	{ baud_base: MPU4xx_CXD9790_BASE_BAUD, \
	  irq: CXD9790_IRQ_UART0, \
	  io_type: SERIAL_IO_MEM, \
	  flags: STD_COM_FLAGS, \
	  type: PORT_16550A }, \
	{ baud_base: MPU4xx_CXD9790_BASE_BAUD, \
	  irq: CXD9790_IRQ_UART1, \
	  io_type: SERIAL_IO_MEM, \
	  flags: STD_COM_FLAGS, \
	  type: PORT_16550A }, \
	{ baud_base: MPU4xx_CXD9790_BASE_BAUD, \
	  irq: CXD9790_IRQ_UART2, \
	  io_type: SERIAL_IO_MEM, \
	  flags: STD_COM_FLAGS, \
	  type: PORT_16550A }, \
	{ baud_base: MPU4xx_CXD9790_BASE_BAUD, \
	  irq: CXD9790_IRQ_UART3, \
	  io_type: SERIAL_IO_MEM, \
	  flags: STD_COM_FLAGS, \
	  type: PORT_16550A },



#else  /* CONFIG_SNSC_CXD9790_UART */

#define NUM_CXD9790_UART	0
#define MPU4xx_CXD9790_UART_OPS


#endif /* !CONFIG_SNSC_CXD9790_UART */

 /* 
    here 'bitnum' is in little endian bitnaming (as in 9790 spec)
 */
static inline void 
MPU4xx_CXD9790_GPIO_SETBIT(int gpio, int bitnum, int is_set)
{
	u_int8_t *reg_po, *reg_poen;

	reg_po   = (u_int8_t *)
		(MPU4xx_CXD9790_BASE + gpio + CXD9790_PO);
	reg_poen = (u_int8_t *)
		(MPU4xx_CXD9790_BASE + gpio + CXD9790_POEN);

	if (is_set) {
		*reg_po |=  (1 << bitnum);
	} else {
		*reg_po &= ~(1 << bitnum);
	}

	*reg_poen |= (1 << bitnum);
}

/* 
 * Power off and  Reset
 * bitnumbers are in little endian bit-naming
 */
#define MPU4xx_SHTDWNACK_GPIO		CXD9790_GPIOZ
#define MPU4xx_SHTDWNACK_GPIO_BITNUM	4
#define MPU4xx_SELFRST_GPIO		CXD9790_GPIOZ
#define MPU4xx_SELFRST_GPIO_BITNUM	2
#define MPU4xx_PWROFF_GPIO		CXD9790_GPIOZ
#define MPU4xx_PWROFF_GPIO_BITNUM	0


#endif /* __SNSC_MPU4xx_CXD9790_H__ */
