/*
 *  arch/ppc/kernel/platforms/snsc-mpu4xx_macaddr.c
 *
 *  Copyright 2004 Sony Corporation
 *
 *  This program is free software; you can redistribute	 it and/or modify it
 *  under  the terms of	 the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the	License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 *  WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO	EVENT  SHALL   THE AUTHOR  BE	 LIABLE FOR ANY	  DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 *  USE, DATA,	OR PROFITS; OR	BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/ptrace.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/types.h>

#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/irq.h>
#include <asm/ocp.h>

#include <linux/netdevice.h>
#include <linux/crc32.h>

/*
 * For old NBL which doesn't pass the MAC address via bootinfo
 */
#ifdef CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM

/*
 * Check checksum of MAC address
 */
static int mpu4xx_macaddr_check_checksum(unsigned char *buf)
{
	int i;
	u16 *cksum, tmp = 0;

#ifdef CONFIG_MACADDR_CHKSUM_COMPAT
	{
		/* calc checksum */
		cksum = (u16 *)buf;
		tmp = cksum[0] ^ cksum[1] ^ cksum[2];
	}
#else
	{
		u16 *p;

		/* calc checksum */
		cksum = p = (u16 *)buf;
		for (i=0; i<3; i++)
			tmp += p[i];
		tmp = (u16) ~((tmp & 0xffff) + (tmp >> 16));
	}
#endif
	printk(" cksum=0x%04x (0x%04x)\n", cksum[3], tmp);

	if (tmp != cksum[3]) {
		printk(KERN_ERR "ibm_ocp_enet: checksum failed.\n");
		return -1;
	}

	return 0;
}

#include <linux/i2c.h>

extern void srom_iic_init(void);
extern int srom_iic_xfer(struct ocp_dev *ocp, struct i2c_msg msgs[], int num);

#define MACADDR_ERR_IIC         1
#define MACADDR_ERR_CKSUM       2

#define IIC_BUS		CONFIG_SROM_BUS
#define IIC_ADDR	CONFIG_SROM_ADDR
#define IIC_AMD		CONFIG_SROM_EEP_ADDR_BYTES
static int srom_read_eeprom(int off, int len, unsigned char *buf)
{
	unsigned char wbuf[IIC_AMD];
	struct i2c_msg msg[2];
	int errno;

	/*  
	 * setup I2C data.
	 */
#if IIC_AMD == 2
	wbuf[0] = (off >> 8) & 0xff;
	wbuf[1] = (off & 0xff);
#elif IIC_AMD == 1
	wbuf[0] = off & 0xff;
#else
#error Unsupported EEPROM on IIC.
#endif

	msg[0].addr	= IIC_ADDR;	
	msg[0].flags	= I2C_M_WR;
	msg[0].buf	= wbuf;
	msg[0].len	= IIC_AMD;

	msg[1].addr	= IIC_ADDR;	
	msg[1].flags	= I2C_M_RD;
	msg[1].buf	= buf;
	msg[1].len	= len;

	errno = srom_iic_xfer(IIC_BUS, msg, 2);
	if (errno == 2){
		int i;
		bd_t *bip;

		printk("IIC-%d: ", IIC_BUS);
		for (i=0; i<6; i++)
			printk("0x%02x%c", buf[i], (i!=5) ? ':' : ',');
		if (mpu4xx_macaddr_check_checksum(buf) < 0) {
			printk("\n");
			printk("!!! Ethernet MAC address check sum failed.  !!!\n");
			return -MACADDR_ERR_CKSUM;
		}

		/* 
		 * Update the board info.
		 * Not directly used if CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM
		 * is set, but just for sure...
		 */
		bip = (bd_t *) __res;
		memcpy(bip->bi_enetaddr[0], buf, 6);
	} else {
		printk("ERROR: srom_iic_xfer() = %d\n", errno);
		return -MACADDR_ERR_IIC;
	}

	return 0;		/* ethernet address in *buf */
}

#define EEP_OFF		CONFIG_SROM_MAC_OFFSET
#define EEP_LEN		CONFIG_SROM_MAC_LEN
static int get_enetaddr_srom(struct net_device *dev)
{
	int i, ret;
	u8  buf[EEP_LEN];

	srom_iic_init();

	ret = srom_read_eeprom(EEP_OFF, EEP_LEN, buf);
	if (ret < 0)
		return ret;

	for (i = 0; i < 6; i++)
		dev->dev_addr[i] = buf[i];
	return 0;
}
#endif  /* CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM */

/*
 * Get MAC address from boardinfo
 * if boardinfo doesn't include valid MAC address, get MAC address
 * from eeprom when CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM=y
 */
int mpu4xx_get_enetaddr(struct net_device *dev)
{
	bd_t *bip;
	int   i;
#ifdef CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM
	int   ret;
#endif
	u8   *p;

	bip = (bd_t *) __res;
	if (bip->bi_enetaddr_valid[0]) {
		printk("MACADDR(bootinfo): ");
		p = bip->bi_enetaddr[0];
		for (i=0; i<6; i++)
			printk("0x%02x%c", *p++, (i!=5) ? ':' : '\n');

		for (i = 0; i < 6; i++)
			dev->dev_addr[i] = bip->bi_enetaddr[0][i];

		return 0;
	}
	printk("Ethernet MAC address cannot be gotten from bootinfo\n");

#ifdef CONFIG_SNSC_MPU4xx_MAC_ADDRESS_ON_SROM
	ret = get_enetaddr_srom(dev);
	if (ret != -MACADDR_ERR_CKSUM)
		return ret;
#endif

	printk("!!! Ethernet MAC address cannot be gotten. !!!\n");
	printk("!!! Please use NBL diag mode to set MAC address. !!!\n");
	printk("\n");
	do {} while (1);

	return -1;
}

/*
 * Copy MAC address from bootinfo to boardinfo
 *     This function is called from parse_bootinfo() in arch/ppc/kernel/setup.c
 */
void mach_macaddr_init(u8 *data, int size)
{
	bd_t *bip;

	/* sanity check */
	if (size < 8)
		return;

	bip = (bd_t *) __res;
	memcpy(bip->bi_enetaddr[0], data, 6);
	bip->bi_enetaddr_valid[0] = *(data + 6);
}
