/*
 * xilinx_ml300.c
 *
 * Xilinx ML300 evaluation board initialization
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * 2002 (c) MontaVista, Software, Inc.  This file is licensed under the terms
 * of the GNU General Public License version 2.1.  This program is licensed
 * "as is" without any warranty of any kind, whether express or implied.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/pagemap.h>
#include <linux/serialP.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include <asm/keyboard.h>

#include "xilinx_ocp/xbasic_types.h"
extern u32 XWaitInAssert;
/*
 * The Xilinx drivers use something called XASSERT.  It should be
 * disabled in running systems to save time and space.  This is done by
 * defining NDEBUG when compiling anything that includes
 * xilinx_ocp/xbasic_types.h.  During development, it is handy to leave
 * NDEBUG off, but by default, the Xilinx code just tight loops.  In our
 * board_init function, we register a handler to be called that prints
 * out that we hit an XASSERT.  In addition, we tell it to not tight
 * loop by setting XWaitInAssert to zero.  This will probably lead to an
 * Oops and thus a backtrace.
 */

/* SAATODO: Make sure NDEBUG is on. */
void
reportXAssert(char *File, int Line)
{
	printk(KERN_CRIT "Xilinx OS Independent Code XAssert: %s:%d\n",
	       File, Line);
	printk(KERN_CRIT "Code may crash due to unhandled errors.\n");
}

/* Have OCP take care of the serial ports. */
struct ocp_def core_ocp[] = {
#ifdef XPAR_UARTNS550_0_BASEADDR
	{UART, XPAR_UARTNS550_0_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_0_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_1_BASEADDR
	{UART, XPAR_UARTNS550_1_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_1_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_2_BASEADDR
	{UART, XPAR_UARTNS550_2_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_2_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_3_BASEADDR
	{UART, XPAR_UARTNS550_3_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_3_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_4_BASEADDR
#error Edit this file to add more devices.
#endif				/* 4 */
#endif				/* 3 */
#endif				/* 2 */
#endif				/* 1 */
#endif				/* 0 */
	{OCP_NULL_TYPE, 0x0, OCP_IRQ_NA, OCP_CPM_NA}
};
#define NR_SER_PORTS (ARRAY_SIZE(core_ocp)-1)

/* Called from the end of platform_init(). */
void __init
board_init(void)
{
#ifdef CONFIG_MAGIC_SYSRQ
	extern unsigned char pckbd_sysrq_xlate[128];
	ppc_md.ppc_kbd_sysrq_xlate = pckbd_sysrq_xlate;
	SYSRQ_KEY = 0x54;
#endif

	XWaitInAssert = 0;
	XAssertSetCallback(reportXAssert);
}

/* Called after board_init from m4xx_map_io(). */
void __init
board_io_mapping(void)
{
	extern struct serial_state rs_table[];
	int i;

	/* Remap all the serial ports */
	for (i = 0; i < NR_SER_PORTS; i++)
		rs_table[i].iomem_base =
		    ioremap((unsigned long) rs_table[i].iomem_base, 16);
}

/* Called after board_io_mapping from ppc4xx_setup_arch(). */
void __init
board_setup_arch(void)
{
	/* Identify the system */
	printk
	    ("Xilinx Virtex-II Pro port (C) 2002 MontaVista Software, Inc. (source@mvista.com)\n");
}

/* Called after board_setup_irq from ppc4xx_init_IRQ(). */
void __init
board_setup_irq(void)
{
}
