/*
 * 8-bit Cipher Feedback mode
 *
 * Copyright 2003 Sony Corporation.
 *
 * Original Source: linux/crypto/ciphers/gen-cfb.h
 */
/*
  Cipher Feedback Block mode
 
  Written by David Bryson <mutex@mac.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software

   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <linux/string.h>
#include <linux/sched.h>

#if defined(CIPHER_BITS_128)
# define BS 16
#else
# define BS 8
#endif


static int CIPHER_NAME(_cfb8_encrypt)(struct cipher_context *cx, const u8 *in_blk,
				     u8 *out_blk, int size, int atomic, const u8 *_iv)
{
	const u32 *iv = (const u32 *) _iv;
	u8 itmp[BS], otmp[BS];
	int i;

	((u32 *)itmp)[0] = iv[0];
	((u32 *)itmp)[1] = iv[1];
#ifdef CIPHER_BITS_128
	((u32 *)itmp)[2] = iv[2];
	((u32 *)itmp)[3] = iv[3];
#endif
	while (size-- > 0) {
		CIPHER_NAME(_encrypt)(cx, itmp, otmp, BS, atomic);
		*out_blk  = *in_blk ^ otmp[0];

		for (i = 0; i < BS - 1; i++)
			itmp[i] = itmp[i+1];
		itmp[i] = *out_blk;

		in_blk++; out_blk++;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0)
		if (!atomic && current->need_resched)
			schedule();
#else
		if (!atomic)
			cond_resched();
#endif	       
	} /* end while */

	return 0;
}

static int CIPHER_NAME(_cfb8_decrypt)(struct cipher_context *cx, const u8 *in_blk,
				     u8 *out_blk, int size, int atomic, const u8 *_iv)
{
	const u32 *iv = (const u32 *) _iv;
	u8 itmp[BS], otmp[BS];
	int i;

	((u32 *)itmp)[0] = iv[0];
	((u32 *)itmp)[1] = iv[1];
#ifdef CIPHER_BITS_128
	((u32 *)itmp)[2] = iv[2];
	((u32 *)itmp)[3] = iv[3];
#endif
	while (size-- > 0) {
		CIPHER_NAME(_encrypt)(cx, itmp, otmp, BS, atomic);
		*out_blk = *in_blk ^ otmp[0];

		for (i = 0; i < BS - 1; i++)
			itmp[i] = itmp[i+1];
		itmp[i] = *in_blk;

		in_blk++; out_blk++;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0)
		if (!atomic && current->need_resched)
			schedule();
#else
		if (!atomic)
			cond_resched();
#endif	       
	} /* end while */

	return 0;
}

#undef BS
