/*
 *  cryptowrap-des.c - glue for DES function of CryptoAPI
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/module.h>
#include <linux/version.h>

#include <linux/init.h>
#include <linux/config.h>
#include <linux/string.h>
#include <linux/crypto.h>

#include <linux/cryptowrap.h>

#include "cryptowrap-cipher.h"

/*
 *  CryptoWrap for DES in CryptoAPI
 */

int 
CryptoWrap_DES_new(CryptoWrap_DES_object *objp, int mode)
{
	return CIPHER_new((cryptoapi_cipher_object **)objp, "des", mode);
}

int
CryptoWrap_DES_destroy(CryptoWrap_DES_object obj)
{
	return CIPHER_destroy((cryptoapi_cipher_object *)obj);
}

int
CryptoWrap_DES_set_key(CryptoWrap_DES_object obj, unsigned char *key)
{
	return CIPHER_set_key((cryptoapi_cipher_object *)obj, key, 8);
}

int
CryptoWrap_DES_encrypt(CryptoWrap_DES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv)
{
	return CIPHER_encrypt((cryptoapi_cipher_object *)obj, from, flen, to, tlen, iv);
}

int
CryptoWrap_DES_decrypt(CryptoWrap_DES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv)
{
	return CIPHER_decrypt((cryptoapi_cipher_object *)obj, from, flen, to, tlen, iv);
}

/* module */
static int CryptoWrap_DES_module_init(void);
static void CryptoWrap_DES_module_exit(void);

static int
CryptoWrap_DES_module_init()
{
	printk(KERN_INFO "CryptoWrap_DES: loaded\n");
	return 0;
}

static void
CryptoWrap_DES_module_exit()
{
	printk(KERN_INFO "CryptoWrap_DES: unloaded\n");
}

module_init(CryptoWrap_DES_module_init);
module_exit(CryptoWrap_DES_module_exit);

EXPORT_SYMBOL(CryptoWrap_DES_new);
EXPORT_SYMBOL(CryptoWrap_DES_destroy);
EXPORT_SYMBOL(CryptoWrap_DES_set_key);
EXPORT_SYMBOL(CryptoWrap_DES_encrypt);
EXPORT_SYMBOL(CryptoWrap_DES_decrypt);
