/*
 *  cryptowrap-digest.h - glue for digest function of CryptoAPI
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _CRYPTOWRAP_DIGEST_H_
#define _CRYPTOWRAP_DIGEST_H_

typedef struct CryptoWrap_cryptoapi_digest {
	struct digest_implementation *di;
	struct digest_context *dx;
} cryptoapi_digest_object;

static inline int
DIGEST_new(cryptoapi_digest_object **cop, char *name)
{
	cryptoapi_digest_object *co;
	struct digest_implementation *di;
	struct digest_context *dx;

	di = find_digest_by_name(name, 1);
	if (di == NULL)
		return CryptoWrap_E_NOTSUPPORTED;

	di->lock();

	dx = di->realloc_context(NULL, di);
	if (dx == NULL) {
		di->unlock();
		return CryptoWrap_E_NOMEM;
	}

	co = kmalloc(sizeof(cryptoapi_digest_object), GFP_KERNEL);
	if (co == NULL) {
		di->free_context(dx);
		di->unlock();
		return CryptoWrap_E_NOMEM;
	}

	co->di = di;
	co->dx = dx;
	*cop = (cryptoapi_digest_object *)co;
	MOD_INC_USE_COUNT;
	return CryptoWrap_SUCCESS;
}

static inline int
DIGEST_destroy(cryptoapi_digest_object *co)
{
	co->di->free_context(co->dx);
	co->di->unlock();
	kfree(co);
	MOD_DEC_USE_COUNT;
	return CryptoWrap_SUCCESS;
}

static inline void
DIGEST_Init(cryptoapi_digest_object *co)
{
	co->di->_open(co->dx, 1);
}

static inline void
DIGEST_Update(cryptoapi_digest_object *co, unsigned char *data, int len)
{
	co->di->_update(co->dx, data, len, 1);
}

static inline void
DIGEST_Final(cryptoapi_digest_object *co, unsigned char *md)
{
	co->di->_digest(co->dx, md, 1);
}

#endif /* _CRYPTOWRAP_DIGEST_H_ */
