/*
 *  cryptowrap-md5.c - glue for MD5 function of CryptoAPI
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/module.h>
#include <linux/version.h>

#include <linux/init.h>
#include <linux/config.h>
#include <linux/string.h>
#include <linux/crypto.h>

#include <linux/cryptowrap.h>

#include "cryptowrap-digest.h"

/*
 *  CryptoWrap for MD5 in CryptoAPI
 */

int
CryptoWrap_MD5_new(CryptoWrap_MD5_object *objp)
{
	return DIGEST_new((cryptoapi_digest_object **)objp, "md5");
}

int
CryptoWrap_MD5_destroy(CryptoWrap_MD5_object obj)
{
	return DIGEST_destroy((cryptoapi_digest_object *)obj);
}

void
CryptoWrap_MD5_Init(CryptoWrap_MD5_object obj)
{
	DIGEST_Init((cryptoapi_digest_object *)obj);
}

void
CryptoWrap_MD5_Update(CryptoWrap_MD5_object obj, unsigned char *data, int len)
{
	DIGEST_Update((cryptoapi_digest_object *)obj, data, len);
}

void
CryptoWrap_MD5_Final(CryptoWrap_MD5_object obj, unsigned char *md)
{
	DIGEST_Final((cryptoapi_digest_object *)obj, md);
}

static int CryptoWrap_MD5_module_init(void);
static void CryptoWrap_MD5_module_exit(void);

static int
CryptoWrap_MD5_module_init()
{
	printk(KERN_INFO "CryptoWrap_MD5: loaded\n");
	return 0;
}

static void
CryptoWrap_MD5_module_exit()
{
	printk(KERN_INFO "CryptoWrap_MD5: unloaded\n");
}

module_init(CryptoWrap_MD5_module_init);
module_exit(CryptoWrap_MD5_module_exit);

EXPORT_SYMBOL(CryptoWrap_MD5_new);
EXPORT_SYMBOL(CryptoWrap_MD5_destroy);
EXPORT_SYMBOL(CryptoWrap_MD5_Init);
EXPORT_SYMBOL(CryptoWrap_MD5_Update);
EXPORT_SYMBOL(CryptoWrap_MD5_Final);
