/*
 *  mpicorder2.c - another corder for the external representation of MPIs
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/string.h>
#include <linux/kernel.h>

#include "mpi.h"
#include "mpi-internal.h"
#include "util.h"

MPI
mpi_bin2mpi(byte *buffer, int len, int secure)
{
    int i, j;
    unsigned char c;
    unsigned nbits, nlimbs;
    mpi_limb_t a;
    MPI val;

    for (c = 0x80, nbits = 8; c && (c & *buffer) == 0; c >>= 1, nbits--);

    nlimbs = (len+BYTES_PER_MPI_LIMB-1) / BYTES_PER_MPI_LIMB;
    val = secure ? mpi_alloc_secure(nlimbs) : mpi_alloc(nlimbs);
    if (val == NULL)
        return NULL;

    i = BYTES_PER_MPI_LIMB - len % BYTES_PER_MPI_LIMB;
    i %= BYTES_PER_MPI_LIMB;
    val->nbits = (len - 1) * 8 + nbits;
    val->nlimbs = j = nlimbs;
    val->sign = 0;
    for (; j > 0; j--) {
        a = 0;
        for (; i < BYTES_PER_MPI_LIMB; i++) {
            a <<= 8;
            a |= *buffer++;
        }
        i = 0;
        val->d[j-1] = a;
    }
    return val;
}

int
mpi_mpi2bin(MPI val, byte *buffer)
{
    int i, j;
    mpi_limb_t a;
    unsigned char c;
    int skip = 1, len = 0;

    for (j = val->nlimbs; j > 0; j--) {
        a = val->d[j - 1];
        for (i = BYTES_PER_MPI_LIMB; i > 0; i--) {
            c = (unsigned char)((a >> (i - 1) * 8) & 0xff);
            if (j == val->nlimbs && skip) {
                if (c == 0x00)
                    continue;
                else
                    skip = 0;
            }
            *buffer++ = c;
            len++;
        }
    }

    return len;
}

void
mpi_mpi2buf(MPI val, byte *buffer, int buflen)
{
	int i, j;
	mpi_limb_t a;

	for (j = 0; j < val->nlimbs; j++) {
		a = val->d[j];
		for (i = 0; i < BYTES_PER_MPI_LIMB; i++) {
			if (--buflen < 0)
				break;
			*(buffer + buflen) = (unsigned char)((a >> i * 8) & 0xff);
		}
	}
}
