/*
 *  util.h - utility function header
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _CRYPTOWRAP_GNUPG_UTIL_H_
#define _CRYPTOWRAP_GNUPG_UTIL_H_

#include <linux/kernel.h>
#include "mpi.h"

/* logger macros */
#define log_fatal(...) \
	printk(KERN_ERR __VA_ARGS__)

#define log_error(...) \
	printk(KERN_WARNING __VA_ARGS__)

#define log_bug(...) \
	printk(KERN_ERR __VA_ARGS__)

#define log_debug(...) \
	printk(KERN_DEBUG __VA_ARGS__)

/* memory.c */
int m_size(void *p);
int m_is_secure(void *p);
void* m_alloc(size_t n);
void* m_alloc_secure(size_t n);
void* m_alloc_clear(size_t n);
void m_free(void *p);
void* m_realloc(void *p, size_t n);

/* random.c */
byte* get_random_bits(size_t nbits, int level, int secure);

/* primegen.c */
MPI generate_secret_prime( unsigned nbits );
MPI generate_public_prime( unsigned nbits );
MPI generate_elg_prime( int mode, unsigned pbits, unsigned qbits, MPI g, MPI **factors );

/* XXX */
//#define log_mpidump(text, a)
#define assert(x)
#define abort()
#define m_check(x)
#define DBG_MEMORY	0
#define DBG_CIPHER	0

#endif /* _CRYPTOWRAP_GNUPG_UTIL_H_ */ 
