/*
 *  it8172_gpio.c : ITE8172G GPIO driver
 *
 *  Copyright 2001,2002,2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __KERNEL__
#error "this module must be compiled with -D__KERNEL__"
#endif

#include <linux/module.h>
#include <linux/config.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <asm/addrspace.h>
#include <asm/it8172/it8172_int.h>

#include <linux/snsc_it8172_gpio.h>

#ifdef MODULE
MODULE_AUTHOR("Sony NSC");
MODULE_DESCRIPTION("ITE8172G GPIO driver");

#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif /* MODULE_LICENSE */
#endif /* MODULE */

#define	IT8172_GPIO_ADR	 (*(volatile __u8  *)(0x14013800 + KSEG1))
#define	IT8172_GPIO_BDR	 (*(volatile __u8  *)(0x14013808 + KSEG1))
#define	IT8172_GPIO_CDR	 (*(volatile __u8  *)(0x14013810 + KSEG1))
#define	IT8172_GPIO_ACR	 (*(volatile __u16 *)(0x14013802 + KSEG1))
#define	IT8172_GPIO_BCR	 (*(volatile __u16 *)(0x1401380a + KSEG1))
#define	IT8172_GPIO_CCR	 (*(volatile __u16 *)(0x14013812 + KSEG1))
#define IT8172_GPIO_AICR (*(volatile __u16 *)(0x14013804 + KSEG1))
#define	IT8172_GPIO_BICR (*(volatile __u16 *)(0x1401380c + KSEG1))
#define	IT8172_GPIO_CICR (*(volatile __u16 *)(0x14013814 + KSEG1))
#define	IT8172_GPIO_AISR (*(volatile __u8  *)(0x14013806 + KSEG1))
#define	IT8172_GPIO_BISR (*(volatile __u8  *)(0x1401380e + KSEG1))
#define	IT8172_GPIO_CISR (*(volatile __u8  *)(0x14013816 + KSEG1))
#define	IT8172_GPIO_GCR	 (*(volatile __u8  *)(0x14013818 + KSEG1))

#define MAX_PORT         3
#define MAX_BIT          8
#define OWNER_NAME_LEN   16

#define PORT_CHECK(port) do {                                                 \
                              if ((0 > port) || (MAX_PORT <= port)){          \
                                  printk("%s: port %d is not supported\n",    \
                                             __FUNCTION__, (port));           \
                                  return -EINVAL;                             \
                              }                                               \
                         } while (0);

#define BITNUM_CHECK(bit_num) do {                                            \
                                 if ((0 > bit_num) || (MAX_BIT <= bit_num)){  \
                                     printk("%s: invalid pin number %d\n",    \
                                                __FUNCTION__, (bit_num));     \
                                     return -EINVAL;                          \
                                 }                                            \
                              } while (0);

static spinlock_t it8172_gpio_lock;
static __u8 used_bits[MAX_PORT] = {0x00, 0x00, 0x00};
static char owner_name[MAX_PORT][MAX_BIT][OWNER_NAME_LEN];
static __u8 gpio_adr=0, gpio_bdr=0, gpio_cdr=0;

static struct gpio_intr_handler {
    __u32 bits; /* no used */
    void (*handler)(int irq, void *dev_id, struct pt_regs *regs);
    void *dev_id;
} intr_handlers[MAX_PORT][MAX_BIT] = {
    {
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL}
    },
    {
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL}
    },
    {
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL},
        {0, 0, NULL}
    }
};


static int set_cr_bit(int port, int bit_num, __u16 mode){
        volatile __u16 *cr;
        __u16 data;
        __u32 shift;

        /* register addr and shift */
        if (IT8172_GPIO_PORT_A == port){
                cr = &IT8172_GPIO_ACR;
        }
        else if (IT8172_GPIO_PORT_B == port){
                cr = &IT8172_GPIO_BCR;
        }
        else if (IT8172_GPIO_PORT_C == port){
                cr = &IT8172_GPIO_CCR;
        }
        else
                return -EINVAL;

        /* set data */
        shift = bit_num * 2;
        data = *cr & ~(3 << shift);
        *cr = data | (mode & 3) << shift;

        return 0;
}

static __u8 get_cr_as_func2(__u16 cr_data, __u8 mask)
{
        int i = 0;
        __u8 data = 0;

        for(i = 0; i < 8 && mask; i++, mask >>= 1, cr_data >>= 2){
                if((cr_data & 3) == IT8172_GPIO_MODE_FUNCTION2)
                        data |= 1 << i;
        }

        return data;
}

static int set_icr_bit(int port, int bit_num, __u32 bit_offset, __u32 value)
{
        volatile __u16 *icr;
        __u16 data;
        int shift = 0;

        /* register addr and shift */
        if (IT8172_GPIO_PORT_A == port){
                icr = &IT8172_GPIO_AICR;
        }
        else if (IT8172_GPIO_PORT_B == port){
                icr = &IT8172_GPIO_BICR;
        }
        else if (IT8172_GPIO_PORT_C == port){
                icr = &IT8172_GPIO_CICR;
        }
        else
                return -EINVAL;

        /* set data */
        shift = bit_num + bit_offset;
        data = *icr & ~(1 << shift);
        *icr = data | (value & 1) << shift;

        return 0;
}

/*
 * check used bit as function2 
 */
static __u8 it8172_gpio_query_function2(int port)
{
        if (IT8172_GPIO_PORT_A == port){
                return get_cr_as_func2(IT8172_GPIO_ACR, 0xFF);
        }
        else if (IT8172_GPIO_PORT_B == port){
                return get_cr_as_func2(IT8172_GPIO_BCR, 0xFF);
        }
        else if (IT8172_GPIO_PORT_C == port){
                return get_cr_as_func2(IT8172_GPIO_CCR, 0x1F);
        }
        else
                return -EINVAL;
}


int it8172_gpio_register_bit(int port, int bit_num, const char *name)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_register(port, (1 << bit_num), name);
}

int it8172_gpio_unregister_bit(int port, int bit_num)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_unregister(port, (1 << bit_num));
}

int it8172_gpio_register(int port, __u8 bits, const char *name)
{
        int i = 0;
        __u8 check_mask = bits;

        PORT_CHECK(port);

        /* arg check */
        for(i = 0; check_mask && i < MAX_BIT; i++, check_mask >>= 1){
                if(check_mask & 1 && used_bits[port] >> i & 1)
                        return -EBUSY;

                /* copy name */
                if(check_mask & 1){
                        memset(owner_name[port][i], '\0', OWNER_NAME_LEN);
                        strncpy(owner_name[port][i], name, OWNER_NAME_LEN);
                        owner_name[port][i][OWNER_NAME_LEN - 1] = '\0';
                }
        }
        
        /* update used_bits */
        used_bits[port] |= bits;

        return 0;
}

int it8172_gpio_unregister(int port, __u8 bits)
{
        int i = 0;
        __u8 check_mask = bits;

        PORT_CHECK(port);


        /* change to read port as default */
        for(i = 0; check_mask && i < MAX_BIT; i++, check_mask >>= 1){
                if(check_mask & 1){
                        it8172_gpio_intr_unregister_handler(port, i,
                                                            intr_handlers[port][i].dev_id);

                        /* no error because error pattern is over MAX_BIT i */
                        set_cr_bit(port, i, IT8172_GPIO_MODE_IN);
                }
        }

        /* clear used_bits */
        used_bits[port] &= ~bits;

        return 0;
}

int it8172_gpio_unregister_by_name(const char *name)
{
        int i, j;

        if (NULL == name){
                return -EINVAL;
        }

        for(j = 0; j < MAX_PORT; j++){
                for(i = 0; i < MAX_BIT; i++){
                        if (0 == strncmp(owner_name[j][i], name,
                                         OWNER_NAME_LEN)) {
                                it8172_gpio_unregister(j, (1 << i));
                        }
                }
        }
        return 0;
}

int it8172_gpio_set_mode_bit(int port, int bit_num, __u32 mode)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_set_mode(port, (1 << bit_num), mode);
}

int it8172_gpio_set_mode(int port, __u8 bits, __u32 mode)
{
        int i = 0;
        __u8 check_mask = bits;

        PORT_CHECK(port);

        /* register bits */
        for(check_mask = bits, i = 0; check_mask && i < MAX_BIT; i++, check_mask >>= 1){
                if(check_mask & 1){
                        /* no error because error pattern is over MAX_BIT i */
                        set_cr_bit(port, i, mode);

                        /* set default if OUT */
                        if ( (mode & ~3) ){
                                /* LOW */
                                it8172_gpio_set_data_bit(port, i, 0);
                        }
                        else {
                                /* HIGH */
                                it8172_gpio_set_data_bit(port, i, 1);
                        }
                }
        }

        return 0;
}

int it8172_gpio_get_data_bit(int port, int bit_num)
{
        volatile __u8 *dr;

        BITNUM_CHECK(bit_num);

        /* register addr and shift */
        if (IT8172_GPIO_PORT_A == port){
                dr = &IT8172_GPIO_ADR;
        }
        else if (IT8172_GPIO_PORT_B == port){
                dr = &IT8172_GPIO_BDR;
        }
        else if (IT8172_GPIO_PORT_C == port){
                dr = &IT8172_GPIO_CDR;
        }
        else
                return -EINVAL; /* error */

        /* get data */
        return *dr >> bit_num & 1;
}

int it8172_gpio_get_data(int port, __u8 bits, __u8 *value)
{
        if (NULL == value){
                return -EINVAL;
        }

        if (IT8172_GPIO_PORT_A == port){
                *value = IT8172_GPIO_ADR;
        }
        else if (IT8172_GPIO_PORT_B == port){
                *value = IT8172_GPIO_BDR;
        }
        else if (IT8172_GPIO_PORT_C == port){
                *value = IT8172_GPIO_CDR;
        }
        else
                return -EINVAL;

        return 0;
}

int it8172_gpio_set_data_bit(int port, int bit_num, int value)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_set_data(port, (1 << bit_num), ((value & 1) << bit_num));
}

int it8172_gpio_set_data(int port, __u8 bits, __u8 value)
{
        PORT_CHECK(port);

        if (IT8172_GPIO_PORT_A == port){
                gpio_adr &= ~bits;
                gpio_adr |= value;
                IT8172_GPIO_ADR = gpio_adr;
        }
        else if (IT8172_GPIO_PORT_B == port){
                gpio_bdr &= ~bits;
                gpio_bdr |= value;
                IT8172_GPIO_BDR = gpio_bdr;
        }
        else if(IT8172_GPIO_PORT_C == port){
                gpio_cdr &= ~bits;
                gpio_cdr |= value;
                IT8172_GPIO_CDR = gpio_cdr;
        }
        else
                return -EINVAL;

        return 0;
}

/* 
 * IT8172_GPIO_IRQ's enable/disable method 
 */
void it8172_gpio_intr_enable(void){}
void it8172_gpio_intr_disable(void){}

int it8172_gpio_intr_set_mode_bit(int port, int bit_num, __u32 mode)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_intr_set_mode(port, (1 << bit_num), mode);
}

int it8172_gpio_intr_set_mode(int port, __u8 bits, __u32 mode)
{
        int i = 0;
        __u32 flags;

        PORT_CHECK(port);

        /* check bits */
        if((used_bits[port] | bits) != used_bits[port]){
                return -EINVAL;
        }

        spin_lock_irqsave(&it8172_gpio_lock, flags);

        for(i = 0; i < MAX_BIT && bits; i++, bits >>= 1){
                if(bits & 1){
                        /* initialize gpio to read */
                        set_cr_bit(port, i, IT8172_GPIO_MODE_IN);

                        /* no error because error pattern is over MAX_BIT i */
                        set_icr_bit(port, i, 8, mode);
                        set_icr_bit(port, i, 0, 1); /* 3rd arg: 1 is intr mask */
                }
        }

        spin_unlock_irqrestore(&it8172_gpio_lock, flags);

        return 0;
}

int it8172_gpio_intr_register_handler(int port,
                                      int bit_num,
                                      void (*handler)(int irq, void *dev_id, struct pt_regs *regs),
                                      void *dev_id)
{
        PORT_CHECK(port);
        BITNUM_CHECK(bit_num);

        /* check bits */
        if(0 == (used_bits[port] & (1 << bit_num))){
                return -EINVAL;
        }

        /* check bits error: after registered bit */
        if(intr_handlers[port][bit_num].handler){
                return -EBUSY;
        }

        intr_handlers[port][bit_num].handler = handler;
        intr_handlers[port][bit_num].dev_id = dev_id;

        return 0;
}

int it8172_gpio_intr_unregister_handler(int port, int bit_num, void *dev_id)
{
        __u32 flags;

        PORT_CHECK(port);
        BITNUM_CHECK(bit_num);

        /* check bits */
        if(0 == (used_bits[port] & (1 << bit_num)))
                return -EINVAL;

        spin_lock_irqsave(&it8172_gpio_lock, flags);

        intr_handlers[port][bit_num].handler = 0;
        intr_handlers[port][bit_num].dev_id = NULL;
        set_icr_bit(port, bit_num, 0, 1); /* 3rd arg: 1 is intr mask */

        spin_unlock_irqrestore(&it8172_gpio_lock, flags);

        return 0;
}

int it8172_gpio_intr_unmask_bit(int port, int bit_num)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_intr_unmask(port, (1 << bit_num));
}

int it8172_gpio_intr_unmask(int port, __u8 bits)
{
        int i = 0;
        __u32 flags;
        __u8 shift = 0;

        PORT_CHECK(port);

        shift = bits;
        for(i = 0; shift && i < MAX_BIT; i++, shift >>= 1){
                if((shift & 1) && (0 == intr_handlers[port][i].handler))
                        return -EINVAL;
        }

        spin_lock_irqsave(&it8172_gpio_lock, flags);

        shift = bits;
        for(i = 0; shift && i < MAX_BIT; i++, shift >>= 1){
                if(shift & 1)
                        set_icr_bit(port, i, 0, 0); /* 3rd arg: 0 means intr enable */
        }

        spin_unlock_irqrestore(&it8172_gpio_lock, flags);

        return 0;
}

int it8172_gpio_intr_mask_bit(int port, int bit_num)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_intr_mask(port, (1 << bit_num));
}

int it8172_gpio_intr_mask(int port, __u8 bits)
{
        int i = 0;
        __u32 flags;

        PORT_CHECK(port);

        spin_lock_irqsave(&it8172_gpio_lock, flags);

        for(i = 0; bits && i < MAX_BIT; i++, bits >>= 1){
                if(bits & 1)
                        set_icr_bit(port, i, 0, 1); /* 3rd arg: 1 means intr mask */
        }

        spin_unlock_irqrestore(&it8172_gpio_lock, flags);

        return 0;
}

int it8172_gpio_intr_get_status_bit(int port, int bit_num)
{
        volatile __u8 *isr;

        BITNUM_CHECK(bit_num);

        /* register addr and shift */
        if (IT8172_GPIO_PORT_A == port){
                isr = &IT8172_GPIO_AISR;
        }
        else if (IT8172_GPIO_PORT_B == port){
                isr = &IT8172_GPIO_BISR;
        }
        else if (IT8172_GPIO_PORT_C == port){
                isr = &IT8172_GPIO_CISR;
        }
        else
                return -EINVAL; /* error */

        /* get data */
        return *isr >> bit_num & 1;
}

int it8172_gpio_intr_get_status(int port, __u8 *status)
{
        if (NULL == status){
                return -EINVAL;
        }

        /* register addr and shift */
        if (IT8172_GPIO_PORT_A == port){
                *status = IT8172_GPIO_AISR;
        }
        else if (IT8172_GPIO_PORT_B == port){
                *status = IT8172_GPIO_BISR;
        }
        else if (IT8172_GPIO_PORT_C == port){
                *status = IT8172_GPIO_CISR;
        }
        else
                return -EINVAL; /* error */

        return 0;
}

int it8172_gpio_intr_clear_bit(int port, int bit_num)
{
        BITNUM_CHECK(bit_num);
        return it8172_gpio_intr_clear(port, (1 << bit_num));
}

/* 
 * IT8172_GPIO_IRQ's clear method 
 */
int it8172_gpio_intr_clear(int port, __u8 bits)
{
        if (IT8172_GPIO_PORT_A == port){
                IT8172_GPIO_AISR |= bits;
        }
        else if (IT8172_GPIO_PORT_B == port){
                IT8172_GPIO_BISR |= bits;
        }
        else if (IT8172_GPIO_PORT_C == port){
                IT8172_GPIO_CISR |= bits;
        }
        else
                return -EINVAL;

        return 0;
}

static void irq_handler(int this_irq, void *dev_id, struct pt_regs *regs)
{
        volatile __u8 isr[MAX_PORT];
        __u32 i, j;

        /* get interrupt status */
        isr[IT8172_GPIO_PORT_A] = IT8172_GPIO_AISR;
        isr[IT8172_GPIO_PORT_B] = IT8172_GPIO_BISR;
        isr[IT8172_GPIO_PORT_C] = IT8172_GPIO_CISR;

        for (j = 0; j < MAX_PORT; j++){
                for (i = 0; i < MAX_BIT; i++){
                        if(!intr_handlers[j][i].handler)
                                continue;

                if(isr[j] >> i & 1)
                        intr_handlers[j][i].handler(this_irq, 
                                                    intr_handlers[j][i].dev_id, 
                                                    regs);
                }
        }

#if 0 /* must be clear by user */
        /* clear interrupt */
        IT8172_GPIO_AISR = isrA;
        IT8172_GPIO_BISR = isrB;
        IT8172_GPIO_CISR = isrC;
#endif

}

int __init it8172_gpio_init(void)
{
        /* initialize private member */
        gpio_adr = IT8172_GPIO_ADR;
        gpio_bdr = IT8172_GPIO_BDR;
        gpio_cdr = IT8172_GPIO_CDR;
        /* already used bit by someone */
        used_bits[IT8172_GPIO_PORT_A] |= it8172_gpio_query_function2(IT8172_GPIO_PORT_A);
        used_bits[IT8172_GPIO_PORT_B] |= it8172_gpio_query_function2(IT8172_GPIO_PORT_B);
        used_bits[IT8172_GPIO_PORT_C] |= it8172_gpio_query_function2(IT8172_GPIO_PORT_C);
#if 1 /* pvr intr?? temporary */
        used_bits[IT8172_GPIO_PORT_B] |= 0x08;
#endif

        it8172_gpio_intr_mask(IT8172_GPIO_PORT_A, IT8172_GPIO_BITS_ALL);
        it8172_gpio_intr_mask(IT8172_GPIO_PORT_B, IT8172_GPIO_BITS_ALL);
        it8172_gpio_intr_mask(IT8172_GPIO_PORT_C, IT8172_GPIO_BITS_ALL);
        IT8172_GPIO_GCR = 0;

        if (request_irq(IT8172_GPIO_IRQ, irq_handler, 
                        SA_SHIRQ, "gpio", intr_handlers) < 0)
                return -1;

        printk("IT8172G GPIO irq = %d, used_bits = %02X:%02X:%02X\n", 
                IT8172_GPIO_IRQ,
                used_bits[IT8172_GPIO_PORT_A],
                used_bits[IT8172_GPIO_PORT_B],
                used_bits[IT8172_GPIO_PORT_C]);

        return 0;
}

void __exit it8172_gpio_exit(void)
{
        it8172_gpio_intr_mask(IT8172_GPIO_PORT_A, IT8172_GPIO_BITS_ALL);
        it8172_gpio_unregister(IT8172_GPIO_PORT_A, IT8172_GPIO_BITS_ALL);

        it8172_gpio_intr_mask(IT8172_GPIO_PORT_B, IT8172_GPIO_BITS_ALL);
        it8172_gpio_unregister(IT8172_GPIO_PORT_B, IT8172_GPIO_BITS_ALL);

        it8172_gpio_intr_mask(IT8172_GPIO_PORT_C, IT8172_GPIO_BITS_ALL);
        it8172_gpio_unregister(IT8172_GPIO_PORT_C, IT8172_GPIO_BITS_ALL);

        free_irq(IT8172_GPIO_IRQ, intr_handlers);
}


#ifndef MODULE
__initcall(it8172_gpio_init);
__exitcall(it8172_gpio_exit);
#else /* MODULE */
module_init(it8172_gpio_init);
module_exit(it8172_gpio_exit);
#endif /* MODULE */

/* IT8172_GPIO */
EXPORT_SYMBOL(it8172_gpio_register_bit);
EXPORT_SYMBOL(it8172_gpio_unregister_bit);
EXPORT_SYMBOL(it8172_gpio_register);
EXPORT_SYMBOL(it8172_gpio_unregister);
EXPORT_SYMBOL(it8172_gpio_unregister_by_name);
EXPORT_SYMBOL(it8172_gpio_set_mode_bit);
EXPORT_SYMBOL(it8172_gpio_set_mode);
EXPORT_SYMBOL(it8172_gpio_get_data_bit);
EXPORT_SYMBOL(it8172_gpio_set_data_bit);
EXPORT_SYMBOL(it8172_gpio_get_data);
EXPORT_SYMBOL(it8172_gpio_set_data);
EXPORT_SYMBOL(it8172_gpio_intr_enable);
EXPORT_SYMBOL(it8172_gpio_intr_disable);
EXPORT_SYMBOL(it8172_gpio_intr_set_mode_bit);
EXPORT_SYMBOL(it8172_gpio_intr_set_mode);
EXPORT_SYMBOL(it8172_gpio_intr_register_handler);
EXPORT_SYMBOL(it8172_gpio_intr_unregister_handler);
EXPORT_SYMBOL(it8172_gpio_intr_unmask_bit);
EXPORT_SYMBOL(it8172_gpio_intr_mask_bit);
EXPORT_SYMBOL(it8172_gpio_intr_unmask);
EXPORT_SYMBOL(it8172_gpio_intr_mask);
EXPORT_SYMBOL(it8172_gpio_intr_get_status_bit);
EXPORT_SYMBOL(it8172_gpio_intr_get_status);
EXPORT_SYMBOL(it8172_gpio_intr_clear_bit);
EXPORT_SYMBOL(it8172_gpio_intr_clear);
