/*
 *  NBLArgs
 *
 *  Copyright 2002,2003 Sony Corporation
 *
 *  This program is free software; you can redistribute	 it and/or modify it
 *  under  the terms of	 the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the	License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 *  WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO	EVENT  SHALL   THE AUTHOR  BE	 LIABLE FOR ANY	  DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 *  USE, DATA,	OR PROFITS; OR	BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/module.h>
#include <linux/seq_file.h>
#include <linux/kernel.h>
#include <linux/bootmem.h>
#include <asm/io.h>
#include <asm/string.h>
#include <asm/uaccess.h>

#include <linux/snsc_nblargs.h>
#ifdef CONFIG_SNSC_UMA
#include <linux/snsc_uma.h>
#endif

#define NAME "NBLArgs: "

/* XXX: will be removed by when ports to x86/mips are done */
#undef NBLARGS_CMDLINE		
#undef NBLARGS_MEM_BY_CMDLINE
#define NBLARGS_MEM

#define DEBUG_LVL 1

#if DEBUG_LVL > 0
#  define printk1(args...) printk(args)
#else
#  define printk1(args...)
#endif

#if DEBUG_LVL > 1
#  define printk2(args...) printk(args)
#else
#  define printk2(args...)
#endif

#define ASSERT(x) if (!(x)) {printk(NAME "assertion ("#x ") failed at " __FILE__ ":%d\n", __LINE__);}

static struct nblargs_entry entry[NBLARGS_NUM_TABLE];
static int entry_cnt = 0;

#if (defined NBLARGS_CMDLINE) || (defined NBLARGS_MEM_BY_CMDLINE)
static char tmp[NBLARGS_COMMAND_LINE_SIZE];

static size_t simple_strcspn(const char *s, const char *reject);
static int    _parse_nblargs(char *start, int len);
#endif

static int    _parse_nblargs_memory(u_int32_t addr, int len);

int
nblargs_add(struct nblargs_entry *e, int cnt)
{
	int i;
	for (i=0; i<cnt; i++) {
		memcpy(entry+entry_cnt, e+i, sizeof(struct nblargs_entry));
		entry_cnt++;
	}
	return entry_cnt;
}

#if (defined NBLARGS_CMDLINE) || (defined NBLARGS_MEM_BY_CMDLINE)
/*
 * strspn(const char *s, const char *reject)
 *  -. Calculate the length of the initial substring of @s which only
 * 	contain letters in @accept
 *
 * Arguments:
 *   s: The string to be searched
 *   reject: The string that is rejected.
 * Returns:
 *   number of length of sub-strings that are not in reject.
 */
static 
size_t simple_strcspn(const char *s, const char *reject)
{
	const char *p;
	const char *a;
	size_t count = 0;

	for (p = s; *p != '\0'; ++p) {
		for (a = reject; *a != '\0'; ++a) {
			if (*p == *a)
				return count;
		}
		++count;
	}

	return count;
}

/* 
 * parse_location(char *ptr, u_int32_t *num)
 *  -. Parse *ptr and translates to number.
 *
 *  -. SIZE or ADDR field in NBLArgs can contain : 
 *     -. Hexadecimal numbers : [0-9a-fA-F]+, if started by 0x
 *     -. Decimal numbers : [0-9]+
 *     -. Trailing [kKmMgG]
 *
 * Arguments:
 *  ptr    : pointer to beggining of string to be parsed.
 * Returns:
 *   n  : Vaule size or address is returned in num.
 */
static u_int32_t
__parse_location(char *ptr)
{
	u_int32_t num;
	char      *ret[32];

	num = simple_strtoul(ptr, ret, 0);

	switch (**ret) {
	case 'G':
	case 'g':
		num <<= 10;
	case 'M':
	case 'm':
		num <<= 10;
	case 'K':
	case 'k':
		num <<= 10;
	}

	return num;
}


#define SET_OFFSET(off,n,len) (((off) + (n) <= (len)) ? (off)+=(n) : 0)

/* 
 * _parse_nblargs():
 *   Parse
 * Returns:
 *  >=0 : A string that seems valid NBLArgs is found.
 *        Return value is string length, not including trailing whitespace.
 *   -1 : No valid NBLArgs was found.
 */
static int
_parse_nblargs(char *start, int len)
{
	char      *from  = start;
	int       n, keylen = 0, paramlen = 0;
	u_int32_t size, addr;
	int       off = 0;
	struct nblargs_entry e;
			
	bzero(&e, sizeof(struct nblargs_entry));

	if (!SET_OFFSET(off,4,len)) /* +4 for "nbl:" */
		goto next;

	/* parse until '=' */
	if ((n = simple_strcspn(from+off, "="))) {

		strncpy(e.key, from+off, n);
		keylen = (n+1 < NBLARGS_KEY_LEN) ? n+1 : NBLARGS_KEY_LEN;
		e.key[keylen] = '\0';

		if (!SET_OFFSET(off,n+1,len)) /* +1 for '=' */
			goto next;

		printk2("Key[%d]     = %s (next ->%s<-)\n", 
			entry_cnt, 
			e.key, 
			(char*)(from+off));

		printk2("KeyLen[%d]  = %d\n", entry_cnt, keylen);

	} else {
		goto next;
	}

	/* parse until '@' */
	if ((n = simple_strcspn(from+off, "@"))) {

		size = __parse_location(from+off);

		if (!SET_OFFSET(off,n+1,len)) /* +1 for '@' */
			goto next;

		printk2("Size[%d]    = %u bytes (next ->%s<-)\n",
			entry_cnt, size, from+off);

	} else {
		goto next;
	}

	/* parse until ',' or ' ' */
	if ((n = simple_strcspn(from+off, ", "))) {

		addr = __parse_location(from+off);

		if (!SET_OFFSET(off,n,len)) 
			goto next;

		printk2("Addr[%d]    = 0x%x (next ->%s<-)\n", 
			entry_cnt, addr, from+off);
	} else {
		goto next;
	}

	/* if next is ',' then parameter follows, 
	   so parse until ' ' or end of string */
	if (*(from+off) == ',') {
		off++; /* +1 for ',' */
		if ((n = simple_strcspn(from+off, " "))) { 

			strncpy(e.param, from+off, n);
			paramlen = (n+1 < NBLARGS_PARAM_LEN) ? n+1 : NBLARGS_PARAM_LEN;
			e.param[paramlen] = '\0';

			if (!SET_OFFSET(off,n,len))
				goto next;

			printk2("Param[%d]   = %s (next ->%s<-)\n", 
				entry_cnt, e.param, from+off);
			printk2("Paramlen[%d]= %d \n", 
				entry_cnt, paramlen);

		} else {
			goto next;
		}
	}

	/* entry that has non-zero keylen is valid */
	e.keylen = keylen;
	e.paramlen = paramlen;
	e.size = size;
	e.addr = addr;

	nblargs_add(&e, 1);

	return off;

next:
	/* Given string is invaid. Try next string */
	return -1;
}
#endif /* if (defined NBLARGS_CMDLINE) || (defined NBLARGS_MEM_BY_CMDLINE) */

/* 
 * _parse_nblargs_memory():
 *
 *   Parse memory region for NBLArgs.
 *
 * Returns:
 *  >=0 : A memory region that seems valid found.
 *        Return value is number of NBLArgs found.
 *   -1 : No valid NBLArgs was found.
 */
static int
_parse_nblargs_memory(u_int32_t addr, int len)
{
	struct nblargs       *pn;
	struct nblargs_entry *pe;
			
	printk1(NAME "searching for nblargs on memory(0x%x@0x%x).\n",
		addr, len);

	if (addr == 0)
		return -1;

	/* translate to virtual address */
	pn = (struct nblargs *)nbl_to_va(addr);

	/* check for MAGIC */
	if (pn->magic != NBLARGS_MAGIC) {
		printk1(NAME "No valid NBLArgs region found in 0x%08lx(0x%p)\n", nbl_to_pa(addr), pn);
		return -1;
	}
	printk1(NAME "Valid NBLArgs region found in 0x%08lx(0x%p)\n", nbl_to_pa(addr), pn);

	pe = nbl_to_va(pn->pHead);
	if (pe == nbl_to_va(pn->pNext))
		return 0;
	do {
		struct nblargs_entry na;
		if (pe->keylen != 0) {
			if (nblargs_get_key(pe->key, &na) == 0) {
				printk(NAME "Skippig duplicated nblargs key = %s\n", pe->key);
				goto next;
			}
			printk2("Key[%d]     = %s\n", entry_cnt, pe->key);
			printk2("KeyLen[%d]  = %d\n", entry_cnt, pe->keylen);
			printk2("Size[%d]    = %u bytes\n", entry_cnt, pe->size);
			printk2("Addr[%d]    = 0x%lx\n", entry_cnt, nbl_to_pa(pe->addr));
			printk2("Param[%d]   = %s\n", entry_cnt, pe->param);
			printk2("Paramlen[%d]= %d \n", entry_cnt, pe->paramlen);
			nblargs_add(pe, 1);
		}
	next:
		pe++;
	} while (pe != nbl_to_va(pn->pMax));

	return entry_cnt;
}

/* 
 * int reserve_bootmem_common(int n)
 */
static 
int reserve_bootmem_common(int n)
{
	struct nblargs_entry *e = entry+n;	
	if (!e->keylen) 
		return -1;

#if DEBUG_LVL > 0
	printk1(NAME "%s(): key=%s, 0x%x@0x%lx, param=->%s<-\n",
		__FUNCTION__, e->key, e->size, nbl_to_pa(e->addr), e->param);
#endif

	reserve_bootmem(nbl_to_pa(e->addr), e->size);

	return 0;
}


#if (defined NBLARGS_CMDLINE) || (defined NBLARGS_MEM_BY_CMDLINE)
/* 
 * Initialization routine of NBLArgs. (cmdline+memory version)
 *
 * (1) Search cmdline for NBLArgs
 * (2) Search cmdline for "nbl:args=size@addr".
 *     If found, search memory address and size provided.
 * (3) If not found, search memory address and size provided in
 *     parameter.
 *
 * !! Preference order for duplicated keys are 1>2>3
 * 
 * -. *cmdline_p is pointer to null-terminated string passed to
 *   setup_arch().
 *
 * Returns:
 *   -. Number of NBLArgs entries stored in internal table.
 */
int
nblargs_init(char **cmdline_p)
{
	int  i;
	int retval = 0;
	struct nblargs_entry na;
	char c = ' ', *from = *cmdline_p, *to = tmp;
	int  len = 0;
	int  skip_trailing_space = 0;

	/* 
	 * parse NBLArgs in command line
	 */
	ASSERT(*cmdline_p != NULL);

	printk2("Command line is =>%s<=\n", *cmdline_p);
	for (;;) {
		skip_trailing_space = 0;

		/* look for "nbl:" */
		if (c == ' ' && !memcmp(from, "nbl:", 4)) {
			int nbllen;

			nbllen = _parse_nblargs(from, simple_strcspn(from, " "));

			if (nbllen >= 0) {
				retval++;
				from += nbllen;
				if (*from == ' ')
					skip_trailing_space = 1;
			}
		}
		c = *from++;
		if (!c)
			goto exit;
		if (NBLARGS_COMMAND_LINE_SIZE <= ++len)
			break;
		if (!skip_trailing_space)
			*to++ = c;
	}
exit:
	*to = '\0';
	memcpy(*cmdline_p, tmp, NBLARGS_COMMAND_LINE_SIZE);


	/* 
	 * if NBLArgs in command line has specified NBLArgs exsists in
	 * memory, parse it.
	 */
	if (nblargs_get_key("args", &na) == 0) {
		printk1(NAME "searching for nblargs on memory(cmdline).\n");
		retval += _parse_nblargs_memory(na.addr, na.size);
	}

	/* 
	 * if memory address is speciffed, search for it.
	 */
	retval += _parse_nblargs_memory(CONFIG_SNSC_NBLARGS_STD_ADDR, 
					CONFIG_SNSC_NBLARGS_STD_LEN);

	/* 
	 * call reserve_bootmem 
	 */
	for (i=0; i<retval; i++) {
		reserve_bootmem_common(i);
	}

	return retval;
}
EXPORT_SYMBOL(nblargs_init);
#else  /* (defined NBLARGS_CMDLINE) || (defined NBLARGS_MEM_BY_CMDLINE) */

/* 
 * Initialization routine of NBLArgs. (full memory version)
 *
 * (1) Search memory address and size provided in parameter.
 *
 * Returns:
 *   -. Number of NBLArgs entries stored in internal table.
 */
int
nblargs_init(void)
{
	int  i;
	int retval = 0;

#ifdef CONFIG_SNSC_UMA
	snsc_uma_init();
#endif
	/* 
	 * if memory address is speciffed, search for it.
	 */
	retval += _parse_nblargs_memory(CONFIG_SNSC_NBLARGS_STD_ADDR, 
					CONFIG_SNSC_NBLARGS_STD_LEN);

	/* 
	 * call reserve_bootmem 
	 */
	for (i=0; i<retval; i++) {
		reserve_bootmem_common(i);
	}

	return retval;
}
EXPORT_SYMBOL(nblargs_init);


#endif /* (defined NBLARGS_CMDLINE) || (defined NBLARGS_MEM_BY_CMDLINE) */

/* 
 * Arguments:
 *  *key : null-terminated key string.
 *  *p   : pointer to memory region
 *  *l   : length of *p in bytes
 *  *param: null-terminated list of parameters.
 *
 * Returns:
 *   0 : an entry matched given key, values returned in arguments.
 *  -1 : no entry matched given key.
 */
int
nblargs_get_key(char *key, struct nblargs_entry *na)
{
	int i;
	int retval = -1;

	ASSERT(key != NULL);
	ASSERT(na  != NULL);

	for (i=0; i<NBLARGS_NUM_TABLE; i++) {
		struct nblargs_entry *e = entry+i;

		if (!e->keylen)
			continue;

		if (!strncmp(key, e->key, NBLARGS_KEY_LEN)) {
			memcpy(na, e, sizeof(struct nblargs_entry));
			retval = 0;
			break;	/* found */
		}
	}

	return retval;
}
EXPORT_SYMBOL(nblargs_get_key);


void free_nblargs_mem(unsigned long start, unsigned long end)
{
	extern unsigned long totalram_pages;

	printk (NAME "%ldk freed\n", (end - start) >> 10);
	for (; start < end; start += PAGE_SIZE) {
		ClearPageReserved(virt_to_page(start));
		set_page_count(virt_to_page(start), 1);
		free_page(start);
		totalram_pages++;
	}
}

/* 
 * - frees reserved memory arer for 'key'.
 * - deletes nblargs entry by setting keylen to 0x0.
 */
void nblargs_free_key(char *key)
{
	int i;

	ASSERT(key != NULL);

	for (i=0; i<NBLARGS_NUM_TABLE; i++) {
		struct nblargs_entry *e = entry+i;

		if (!e->keylen)
			continue;

		if (!strncmp(key, e->key, NBLARGS_KEY_LEN)) {
			printk(NAME "freeing key=%s, 0x%x@0x%lx\n",
			       e->key, e->size, nbl_to_pa(e->addr));

			/* 0 means not valid entry*/
			e->keylen = 0; 
			free_nblargs_mem(nbl_to_va(e->addr), 
					 nbl_to_va(e->addr) + e->size);
			break;
		}
	}
	return ;
}
EXPORT_SYMBOL(nblargs_free_key);

/* 
 * This is for debug purpose
 */
int 
nblargs_get_num(int num, char *key)
{
	if (num < NBLARGS_NUM_TABLE) {
		if (entry[num].keylen) {
			strncpy(key, entry[num].key, entry[num].keylen);
			return 0;
		} 
	}
	return -1;
}


