/*
 *  i2c-cxd9790.c
 *
 *  i2c adapter for CXD9790
 *
 *  Copyright 2001-2003 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/ioport.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include <linux/hfs_sysdep.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <linux/i2c-id.h>
#include <linux/i2c-dev.h>
#include <linux/delay.h>
#include <asm/irq.h>
#include <linux/cxd9790.h>
#include "i2c-cxd.h"




#define I2C_CXD9790_DEBUG


#ifdef I2C_CXD9790_DEBUG
#define M_LEVEL 5
#define H_LEVEL 3
static int debug = 2;
MODULE_PARM (debug, "i");
MODULE_PARM_DESC (debug, "Debug enabled or not");
#define PDEBUG(level, format, arg...) \
          if (level <= debug) \
                 printk(KERN_INFO "%s: %s: " format , __FILE__ , __FUNCTION__ , ##arg)
#else
#define PDEBUG(level, format, arg...)
#endif

static unsigned int status;
static wait_queue_head_t i2c_cxd9790_wait;
static int intr_key = 0;
//static struct proc_dir_entry *i2c_cxd9790_proc,*i2c_cxd9790_dir;

static void i2c_cxd9790_inc (struct i2c_adapter *adap);
static void i2c_cxd9790_dec (struct i2c_adapter *adap);


static void
cxd_cxd9790_setiic (void *data, int reg, short val)
{
  if(reg >= 0xfe)
    reg = reg - 0xde;
  PDEBUG (M_LEVEL, "write reg  = 0x%x 0x%x\n", reg + 0xa0, val);
  CXD9790_WRITE (reg + 0xa0, val);
  return;
}

static int
cxd_cxd9790_getiic (void *data, int reg)
{
  int val;
  if(reg >= 0xfe)
    reg = reg - 0xde;
  val = CXD9790_READ (reg + 0xa0);
  PDEBUG (M_LEVEL, "read reg  = 0x%x 0x%x\n", reg + 0xa0, val);
  //  return (CXD9790_READ(reg + red 0xa0));
  return val;
}

static int
cxd_cxd9790_getown (void *data)
{
  return 0;
}

static int
cxd_cxd9790_getclock (void *data)
{
  return 0;
}



static void
cxd_cxd9790_waitforpin (void *data)
{
  if(intr_key == 0)
    {
      PDEBUG (M_LEVEL, "wait\n");
      //   wait_event_interruptible(i2c_cxd9790_wait,intr_key);
      interruptible_sleep_on_timeout (&i2c_cxd9790_wait,1*HZ);
      PDEBUG (M_LEVEL, "wait end\n");
    }
  intr_key = 0;
  return;
}



static struct i2c_algo_cxd_data cxd_cxd9790_data = {
  NULL,
  cxd_cxd9790_setiic,
  cxd_cxd9790_getiic,
  cxd_cxd9790_getown,
  cxd_cxd9790_getclock,
  cxd_cxd9790_waitforpin,
  80, 80, 100,			/*      waits, timeout */
};


static struct i2c_adapter i2c_cxd9790_ops = {
  "CXD9790 I2C",
  I2C_ALGO_BIT | I2C_HW_B_SER,
  NULL,
  &cxd_cxd9790_data,
  i2c_cxd9790_inc,
  i2c_cxd9790_dec,
  NULL,
  NULL,
};






static void
i2c_cxd9790_int (int irq, void *dev_id, struct pt_regs *regs)
{
  int flag;
  PDEBUG (M_LEVEL, "int \n");
  flag = CXD_ICINT_INTATM & cxd_cxd9790_getiic (&cxd_cxd9790_data, CXD_ICINT);
  cxd_cxd9790_setiic (&cxd_cxd9790_data, CXD_ICINTC,
		      (CXD_ICINTC_INTI2CSC | CXD_ICINTC_INTATMC |
		       CXD_ICINTC_INTI2CC));
  cxd_cxd9790_setiic (&cxd_cxd9790_data, CXD_ICINT, 0);
  if (flag != 0)
    {
      intr_key = 1;
      wake_up_interruptible (&i2c_cxd9790_wait);
    }
  return;
}


static void
i2c_cxd9790_inc (struct i2c_adapter *adap)
{
  MOD_INC_USE_COUNT;
  return;
}

static void
i2c_cxd9790_dec (struct i2c_adapter *adap)
{
  MOD_DEC_USE_COUNT;
  return;
}

/*
#ifdef CONFIG_DRAGONBALL_SNSC_MPU110
static int proc_read_i2c_cxd9790(char *page, char **start,
			      off_t off, int count, 
			      int *eof, void *data)
{
	short i;
	int len = 0;
	
	struct i2c_msg my_msg;
	char addr_data[2];
	char i2c_cxd9790_proc_data[70];

	addr_data[0] = 0;
	addr_data[1] = 0;

	my_msg.addr = MPU110_EEPROM_ADDR;
	my_msg.flags = I2C_M_WR;
	my_msg.buf = addr_data;
	my_msg.len = 2;
	i2c_cxd9790_transfer(&my_msg,1);
	my_msg.flags = I2C_M_RD;
	my_msg.buf = i2c_cxd9790_proc_data;
	my_msg.len = 64;
	i2c_cxd9790_transfer(&my_msg,1);
	
	len += sprintf(page+len, "EEPRPM TOP ADDR DATA\n");
	for(i=0;i < 4;i++){
		len += sprintf(page+len, "%c %c %c %c %c %c %c %c %c %c %c %c %c %c %c %c\n"
			       ,i2c_cxd9790_proc_data[(i*16)],i2c_cxd9790_proc_data[(i*16)+1]
			       ,i2c_cxd9790_proc_data[(i*16)+2],i2c_cxd9790_proc_data[(i*16)+3],
			       i2c_cxd9790_proc_data[(i*16)+4],i2c_cxd9790_proc_data[(i*16)+5],
			       i2c_cxd9790_proc_data[(i*16)+6],i2c_cxd9790_proc_data[(i*16)+7],
			       i2c_cxd9790_proc_data[(i*16)+8],i2c_cxd9790_proc_data[(i*16)+9],
			       i2c_cxd9790_proc_data[(i*16)+10],i2c_cxd9790_proc_data[(i*16)+11],
			       i2c_cxd9790_proc_data[(i*16)+12],i2c_cxd9790_proc_data[(i*16)+13],
			       i2c_cxd9790_proc_data[(i*16)+14],i2c_cxd9790_proc_data[(i*16)+15]);
	}
	return(len);
}

static int proc_write_i2c_cxd9790(struct file *file,
			     const char *buffer,
			     unsigned long count, 
			     void *data)
{
	unsigned long n;
	char *start;
	struct i2c_msg my_msg;
	char i2c_cxd9790_proc_data[100];
       
	n = count;
	if(n > 65)
	    n = 65;
	
        start = &i2c_cxd9790_proc_data[2];
	memset(i2c_cxd9790_proc_data,'0',100);
	if(copy_from_user(start,buffer,n-1))
	  return(-EFAULT);
	
	i2c_cxd9790_proc_data[0] = 0;
	i2c_cxd9790_proc_data[1] = 0;
	my_msg.addr = MPU110_EEPROM_ADDR;
	my_msg.flags = I2C_M_WR;
	my_msg.buf = i2c_cxd9790_proc_data;
	my_msg.len = 66;
	i2c_cxd9790_transfer(&my_msg,1);

	return n;
}
#endif

*/

static int __init
i2c_cxd9790_init (void)
{
  int result = 0;

  if (!cxd9790_present ())
    return -ENODEV;

  init_waitqueue_head (&i2c_cxd9790_wait);


  if (request_irq
      (CXD9790_IRQ_I2C, i2c_cxd9790_int, SA_SHIRQ | SA_INTERRUPT,
       "i2c_cxd9790", &status) < 0)
    {
      printk ("Failed request_irq!!\n");
      result = -EFAULT;
      goto irq_fault;
    }

  i2c_cxd_add_bus (&i2c_cxd9790_ops);
  /*
     #ifdef CONFIG_DRAGONBALL_SNSC_MPU110
     i2c_cxd9790_proc = create_proc_entry("dragonball_mx1/i2c", 0,i2c_cxd9790_dir );
     if(i2c_cxd9790_proc == NULL) {
     result = -ENOMEM;
     goto proc_ent;
     }

     i2c_cxd9790_proc->read_proc = proc_read_i2c_cxd9790;
     i2c_cxd9790_proc->write_proc = proc_write_i2c_cxd9790;
     i2c_cxd9790_proc->owner = THIS_MODULE;
     #endif
   */
  return (result);
  /*
     #ifdef CONFIG_DRAGONBALL_SNSC_MPU110
     proc_ent:
     remove_proc_entry("dragonball_mx1/i2c",i2c_cxd9790_dir);
     #endif
   */
irq_fault:
  free_irq (CXD9790_IRQ_I2C, &status);

  return (result);
}

static void __exit
i2c_cxd9790_cleanup (void)
{
  //i2c_del_adapter(&i2c_csi_adapter);
  //      dragonball_unregister_gpios(PORT_A, (3 << 15));
  //    remove_proc_entry("eeprom",i2c_cxd9790_dir);
  //    remove_proc_entry("i2c_cxd9790", NULL);
  i2c_cxd_del_bus (&i2c_cxd9790_ops);
  free_irq (CXD9790_IRQ_I2C, &status);
}

module_init(i2c_cxd9790_init);

module_exit(i2c_cxd9790_cleanup); 

#ifdef MODULE
/*
int
init_module (void)
{
  printk (KERN_INFO "I2C driver for CXD9790\n");
  return i2c_cxd9790_init ();
}




void
cleanup_module (void)
{
  i2c_cxd9790_cleanup ();
}
*/

#ifdef MODULE_LICENSE
MODULE_LICENSE ("GPL");
#endif
#endif
