/*
   -------------------------------------------------------------------------
   i2c-adap-goku-s.c i2c-hw access for the I2C peripheral
      on the GOKU-S MIPS system
   -------------------------------------------------------------------------

   Copyright 2002 TOSHIBA CORPRATION SEMICONDUCTOR COMPANY

   ----------------------------------------------------------------------------
   This file was highly leveraged from i2c-elektor.c, which was created
   by Simon G. Vogl and Hans Berglund:

 
     Copyright (C) 1995-97 Simon G. Vogl
                   1998-99 Hans Berglund

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
/* ------------------------------------------------------------------------- */


#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/completion.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-goku-s.h>
#include "i2c-goku-s.h"
#include <linux/i2c-id.h>

#define GOKU_S_DEFAULT_OWN   0x55

#if defined(CONFIG_GOKU_S_I2C_396KHZ)
    #define GOKU_S_DEFAULT_CLOCK (0x00)
#elif defined(CONFIG_GOKU_S_I2C_225KHZ)
    #define GOKU_S_DEFAULT_CLOCK (0x01)
#elif defined(CONFIG_GOKU_S_I2C_120KHZ)
    #define GOKU_S_DEFAULT_CLOCK (0x02)
#elif defined(CONFIG_GOKU_S_I2C_62KHZ)
    #define GOKU_S_DEFAULT_CLOCK (0x03)
#elif defined(CONFIG_GOKU_S_I2C_31KHZ)
    #define GOKU_S_DEFAULT_CLOCK (0x04)
#elif defined(CONFIG_GOKU_S_I2C_16KHZ)
    #define GOKU_S_DEFAULT_CLOCK (0x05)
#elif defined(CONFIG_GOKU_S_I2C_8KHZ)
    #define GOKU_S_DEFAULT_CLOCK (0x06)
#endif

#if defined(CONFIG_TOSHIBA_GOKU_S_MODE0)
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE0
#elif defined(CONFIG_TOSHIBA_GOKU_S_MODE1)
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE1
#elif defined(CONFIG_TOSHIBA_GOKU_S_MODE2)
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE2
#elif defined(CONFIG_TOSHIBA_GOKU_S_MODE3)
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE3
#elif defined(CONFIG_TOSHIBA_GOKU_S_MODE4)
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE4
#elif defined(CONFIG_TOSHIBA_GOKU_S_MODE5)
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE5
#elif defined(CONFIG_TOSHIBA_GOKU_S_MODE6)
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE6
#endif
    #define GOKU_S_DEFAULT_MODE		GOKU_S_GPIO_MODE3


static int own   = 0;
static int i2c_debug = 0;

static struct goku_s_i2c gpi;

static wait_queue_head_t i2c_wait;
static int i2c_pending;

/* ----- global defines -----------------------------------------------	*/
#define DEB(x)	if (i2c_debug>=1) x
#define DEB2(x) if (i2c_debug>=2) x
#define DEB3(x) if (i2c_debug>=3) x
#define DEBE(x)	x	/* error messages 				*/

/*#define GOKU_I2C_DEBUG*/

#if defined(GOKU_I2C_DEBUG)
#define DEB_START	printk(__FUNCTION__": start\n");
#define DEB_END		printk(__FUNCTION__": end\n");
#define DEB_SNAP(x...)	printk(__FUNCTION__": "##x);
#define DEB_SNAP2(x...)	printk(##x);

#include <linux/proc_fs.h>
#include <linux/ctype.h>
static int goku_s_i2c_read_proc(char *page, char **start, off_t off,
					int count, int *eof, void *data);
#else
#define DEB_START
#define DEB_END
#define DEB_SNAP(x...)
#define DEB_SNAP2(x...)
#endif

/* ----- local functions ----------------------------------------------	*/

static void goku_s_gbus_setbyte(void *data, __u32 ofs, __u8 val)
{
	struct goku_s_i2c* i2c = (struct goku_s_i2c*)data;

	outw(val, (i2c->gbus_base + ofs));
	DEB3(printk(": Write 0x%02x to (0x%x+0x%x)\n", val, i2c->gbus_base, ofs));
}

static __u8 goku_s_gbus_getbyte(void *data, __u32 ofs)
{
	struct goku_s_i2c* i2c = (struct goku_s_i2c*)data;
	__u16 val;

	val = inw((i2c->gbus_base + ofs));
	DEB3(printk(": Read 0x%08x from (0x%x+0x%x)\n",val, i2c->gbus_base, ofs));
	return (__u8)(val);
}

static void goku_s_gpio_setbyte(void *data, __u32 ofs, __u8 val)
{
	struct goku_s_i2c* i2c = (struct goku_s_i2c*)data;

	outl(val, (i2c->gpio_base + ofs));
	DEB3(printk(": Write 0x%02x to (0x%x+0x%x)\n", val, i2c->gpio_base, ofs));
}

static __u8 goku_s_gpio_getbyte(void *data, __u32 ofs)
{
	struct goku_s_i2c* i2c = (struct goku_s_i2c*)data;
	__u32 val;

	val = inl((i2c->gpio_base + ofs));
	DEB3(printk(": Read 0x%08x from (0x%x+0x%x)\n",val, i2c->gpio_base, ofs));
	return (__u8)(val);
}

static void goku_s_i2c_setbyte(void *data, __u32 ofs, __u8 val)
{
	struct goku_s_i2c* i2c = (struct goku_s_i2c*)data;

	outl(val, (i2c->i2c_base + ofs));
	DEB3(printk(": Write 0x%02x to (0x%x+0x%x)\n", val, i2c->i2c_base, ofs));
}

static __u8 goku_s_i2c_getbyte(void *data, __u32 ofs)
{
	struct goku_s_i2c* i2c = (struct goku_s_i2c*)data;
	__u32 val;

	val = inl((i2c->i2c_base + ofs));
	DEB3(printk(": Read 0x%08x from (0x%x+0x%x)\n",val, i2c->i2c_base, ofs));
	return (__u8)(val);
}

/* Return our slave address.  This is the address
 * put on the I2C bus when another master on the bus wants to address us
 * as a slave
 */
static int goku_s_i2c_getown(void *data)
{
DEB_START;
	return ((struct goku_s_i2c*)data)->i2c_own;
}


static int goku_s_i2c_getclock(void *data)
{
DEB_START;
	return ((struct goku_s_i2c*)data)->i2c_clk;
}


/* Put this process to sleep.  We will wake up when the
 * I2C controller interrupts.
 */
static void goku_s_i2c_waitforpin(long tmout)
{

DEB_START;
	if (!tmout) {
		tmout = 2;
	}
	/* If interrupts are enabled (which they are), then put the process to
	* sleep.  This process will be awakened by two events -- either the
	* the I2C peripheral interrupts or the timeout expires. 
	* If interrupts are not enabled then delay for a reasonable amount
	* of time and return.
	*/
	if (gpi.i2c_irq > 0) {
		cli();
		if (i2c_pending == 0) {
			if (interruptible_sleep_on_timeout(&i2c_wait, tmout * HZ ) == 0) {
//			if (sleep_on_timeout(&i2c_wait, tmout * HZ ) == 0) {
				DEB_SNAP("timeout !!\n");
			}
		} else {
			DEB_SNAP("already pending\n");
		}
		i2c_pending = 0;
		sti();
	} else {
		udelay(100);
	}
}


/*
 * generate start condition
 */
static void goku_s_i2c_start(void)
{
	int i ;			/*	sugiyama wrote */
DEB_START;
	/* generate start condition */
	spin_lock(&gpi.lock);
	gpi.skip_intr = 1;
	DEB_SNAP2("skip_intr = %d\n", gpi.skip_intr);

#if 0	/*	sugiyama wrote */
	printk("adap-goku_s write buf =\n");
	for ( i = 0 ; i < 128 ; i ++ ){
		if ( i % 15 == 0 ){
			printk("\n");
		}
		printk("0x%x, ", gpi.tx_buf[i]);
	}
	printk("gpi_tx_count = %d, rx_count=%d\n", gpi.tx_count, gpi.rx_count);
#endif

	spin_unlock(&gpi.lock);
	goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2,
		(GOKU_S_I2C_PIN | GOKU_S_I2C_BB | GOKU_S_I2C_TRX | GOKU_S_I2C_MST));
}

/*
 * generate stop condition
 */
static void goku_s_i2c_stop(void)
{
	__u8 val;

DEB_START;
	val = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBICR2);
	if (val & GOKU_S_I2C_BB) {
		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2,
			(GOKU_S_I2C_PIN | GOKU_S_I2C_TRX | GOKU_S_I2C_MST));
	}
}

static void goku_s_i2c_reset(void)
{
	__u8 val;
DEB_START;
	val = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBICR2);
	val &= ~(GOKU_S_I2C_SWARST);

	/* exec software reset */
	goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2, val | 0x2);
	goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2, val | 0x1);

	/* wait for reset completion */
	do {
		val = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBICR1);
		printk(".");
	} while (!(val & GOKU_S_I2C_SWRMON));

#if 0
	goku_s_i2c_init(data);

	/* reset I2C mode */
	gpio_outb(adap, GOKU_S_GPIO_MSR, GOKU_S_DEFAULT_MODE);
#endif
}



static int goku_s_i2c_master_send(void)
{
DEB_START;
	spin_lock(&gpi.tx_lock);
	if (gpi.tx_count < gpi.tx_size) {

		/* write 1 word data */
#if 0
		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR1,
			(GOKU_S_I2C_BC8 | GOKU_S_I2C_ACK| gpi.i2c_clk));
#endif
		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBIDBR, gpi.tx_buf[gpi.tx_count]);
		gpi.tx_count++;
		spin_unlock(&gpi.tx_lock);
		return gpi.tx_count;
	} else {
		/* stop condition */
		spin_unlock(&gpi.tx_lock);
		goku_s_i2c_stop();
//		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBIDBR, 0x00);
		/* signal to exit transmit */
		DEB_SNAP("wake_up\n\n");
		wake_up_interruptible(&i2c_wait);
//		wake_up_interruptible(&gpi.tx_wait);
		complete(&gpi.tx_comp);
	}
//	spin_unlock(&gpi.tx_lock);
	return -1;
}


static int goku_s_i2c_master_recv(void)
{
DEB_START;
	spin_lock(&gpi.lock);
	DEB_SNAP2("skip_intr = %d\n", gpi.skip_intr);
	if (gpi.skip_intr == 1) {
		/* dummy read for start condition */
		goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBIDBR);
		gpi.skip_intr = 0;
		DEB_SNAP2("skip interrupt\n");
		spin_unlock(&gpi.lock);
		return 0;
	}
	spin_unlock(&gpi.lock);
	spin_lock(&gpi.rx_lock);
	if (gpi.rx_size > gpi.rx_count) {
		if ((gpi.rx_size - gpi.rx_count) == 2) {
			/* set no ACK mode */
			DEB_SNAP2("last 2 %d:%d\n", gpi.rx_size, gpi.rx_count);
			goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR1,
										gpi.i2c_clk);
		} else if ((gpi.rx_size - gpi.rx_count) == 1) {
			/* set 1bit transfer clock */
			DEB_SNAP2("last 1 %d:%d\n", gpi.rx_size, gpi.rx_count);
			goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR1,
							(GOKU_S_I2C_BC1 | gpi.i2c_clk));
		}
		/* read 1 word data */
		gpi.rx_buf[gpi.rx_count] = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBIDBR);
		DEB_SNAP("dat[0x%02X], ridx(%d)\n", (unsigned char)gpi.rx_buf[gpi.rx_count], gpi.rx_count);
		gpi.rx_count++;
		spin_unlock(&gpi.rx_lock);
		return gpi.rx_count;
	}

	/* stop condition */
	spin_unlock(&gpi.rx_lock);
	goku_s_i2c_stop();
	DEB_SNAP("wake_up\n\n");
	wake_up_interruptible(&i2c_wait);
//	wake_up_interruptible(&gpi.rx_wait);
	complete(&gpi.rx_comp);
	return -1;
}


static void goku_s_i2c_slave_send(void)
{
	__u8 val;

DEB_START;
	spin_lock(&gpi.slave_lock);
	if (gpi.r_ptr >= GOKU_S_SLAVE_BUFSIZE) {
		gpi.r_ptr = 0;
	}
	val = gpi.slave_buf[gpi.r_ptr++];
	DEB_SNAP("dat[0x%02X], rptr(%d)\n", val, gpi.r_ptr);
	goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBIDBR, val);
	spin_unlock(&gpi.slave_lock);
}


static void goku_s_i2c_slave_recv(void)
{
	__u8 val = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBIDBR);

DEB_START;
	spin_lock(&gpi.slave_lock);
	if (gpi.w_ptr >= GOKU_S_SLAVE_BUFSIZE) {
		gpi.w_ptr = 0;
	}
	DEB_SNAP("dat[0x%02X], wptr(%d)\n", val, gpi.w_ptr);
	gpi.slave_buf[gpi.w_ptr++] = val;
	spin_unlock(&gpi.slave_lock);
}


static void goku_s_i2c_handler(int irq, void *dev, struct pt_regs *regs)
{
	__u8 val;

	if (dev != &gpi) {
		return;
	}
	val = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBICR2);

	if (val & GOKU_S_I2C_PIN) {
		return;
	}
	cli();
DEB_START;

	gpi.i2c_sts = val;
	/* check arbitration-lost */
	if (val & GOKU_S_I2C_MST) {
		DEB_SNAP2("MASTER MODE:");
		/* reset slave buffer */
		spin_lock(&gpi.slave_lock);
		gpi.w_ptr = gpi.r_ptr = 0;
		spin_unlock(&gpi.slave_lock);

		/* master mode */
		if (val & GOKU_S_I2C_TRX) {
			DEB_SNAP2("Transmitter MODE:");
			if (val & GOKU_S_I2C_LRB) {
				DEB_SNAP2("Stop condition\n");
				goku_s_i2c_stop();
				/* signal to exit transmit */
				DEB_SNAP("wake_up\n\n");
				wake_up_interruptible(&i2c_wait);
			} else {
				DEB_SNAP2("Send next data\n");
				if(goku_s_i2c_master_send() < 0) {
					DEB_SNAP2("End of Send\n");
				}
			}
		} else {
			DEB_SNAP2("Receiver MODE:");
			if (goku_s_i2c_master_recv() < 0) {
				DEB_SNAP2("End of Recv\n");
			}
		}
//		val |= (GOKU_S_I2C_PIN);
//		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2, (val & 0xF0));
	} else {
		/* slave mode */
		DEB_SNAP2("SLAVE MODE:");
		switch (val & 0x4E) {
		case 0x4C:	/* TRX:1, AL:1, AAS:1, AD0:0 */
		case 0x44:	/* TRX:1, AL:0, AAS:1, AD0:0 */
			/* write send data to SBIDBR */
			DEB_SNAP2("transmit next data:\n");
			goku_s_i2c_slave_send();
			break;
		case 0x40:	/* TRX:1, AL:0, AAS:0, AD0:0 */
			if (val & GOKU_S_I2C_LRB) {
				DEB_SNAP2("transmit stop:\n");
				val &= ~(GOKU_S_I2C_TRX);
				val |= GOKU_S_I2C_PIN;
				goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2, val);
				/* reset slave buffer */
				spin_lock(&gpi.slave_lock);
				gpi.w_ptr = gpi.r_ptr = 0;
				spin_unlock(&gpi.slave_lock);
			} else {
				/* write send data to SBIDBR */
				DEB_SNAP2("transmit next data:\n");
				goku_s_i2c_slave_send();
			}
			break;
		case 0x0E:	/* TRX:0, AL:1, AAS:1, AD0:1 */
		case 0x0C:	/* TRX:0, AL:1, AAS:1, AD0:0 */
		case 0x08:	/* TRX:0, AL:1, AAS:0, AD0:0 */
			printk(": Detected arbitration lost!!\n");
		case 0x06:	/* TRX:0, AL:0, AAS:1, AD0:1 */
		case 0x04:	/* TRX:0, AL:0, AAS:1, AD0:0 */
			/* dummy read from SBIDBR */
			val = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBIDBR);
			/* reset slave buffer */
			spin_lock(&gpi.slave_lock);
			gpi.w_ptr = gpi.r_ptr = 0;
			spin_unlock(&gpi.slave_lock);
			break;
		case 0x02:	/* TRX:0, AL:0, AAS:0, AD0:1 */
		case 0x00:	/* TRX:0, AL:0, AAS:0, AD0:0 */
			/* read data from SBIDBR */
			DEB_SNAP2("recieve next data:\n");
			goku_s_i2c_slave_recv();
			break;
		}
		DEB_SNAP("wake_up\n\n");
		wake_up_interruptible(&i2c_wait);
	}

	i2c_pending = 1;
//	goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2, val | GOKU_S_I2C_PIN);

//	DEB2(printk("goku_s_i2c_handler: in interrupt handler\n"));
//	wake_up_interruptible(&i2c_wait);
	sti();
}

/*
 * Lock the region of memory where I/O registers exist.  Request our
 * interrupt line and register its associated handler.
 */
static int goku_s_i2c_hw_init(void)
{
	struct pci_dev *dev = NULL;

DEB_START;
	/* check PCI enable */
	if (pci_present() == 0) {
		return -ENODEV;
	}

	/* look for GOKU-S I2C device */
	dev = pci_find_device(GOKU_S_I2C_VID, GOKU_S_I2C_DID, dev);
	if (dev == NULL) {
		printk("GOKU-S: Couldn't detect I2C device!\n");
		return -ENODEV;
	}

	if (pci_enable_device(dev) != 0) {
		printk("GOKU-S: Couldn't enable I2C device!\n");
		return -ENODEV;
	}
	gpi.i2c_base = pci_resource_start(dev, 0);
	gpi.i2c_size = pci_resource_len(dev, 0);
	gpi.gpio_base = pci_resource_start(dev, 2);
	gpi.gpio_size = pci_resource_len(dev, 2);
	gpi.gbus_base = pci_resource_start(dev, 3);
	gpi.gbus_size = pci_resource_len(dev, 3);
  	if (check_region(gpi.i2c_base, gpi.i2c_size) < 0 ) {
		printk("GOKU-S: invalid resource (0x%x @ 0x%x)\n",
					gpi.i2c_base, gpi.i2c_size);
		return -ENODEV;
  	} else {
		request_region(gpi.i2c_base, gpi.i2c_size,
					"GOKU-S I2C (adapter)");
  	}
  	if (check_region(gpi.gpio_base, gpi.gpio_size) < 0 ) {
		printk("GOKU-S: invalid resource (0x%x @ 0x%x)\n",
					gpi.gpio_base, gpi.gpio_size);
		release_region(gpi.i2c_base, gpi.i2c_size);
		return -ENODEV;
  	} else {
		request_region(gpi.gpio_base, gpi.gpio_size,
					"GOKU-S GPIO");
  	}
  	if (check_region(gpi.gbus_base, gpi.gbus_size) < 0 ) {
		printk("GOKU-S: invalid resource (0x%x @ 0x%x)\n",
					gpi.gbus_base, gpi.gbus_size);
		release_region(gpi.i2c_base, gpi.i2c_size);
		release_region(gpi.gpio_base, gpi.gpio_size);
		return -ENODEV;
  	} else {
		request_region(gpi.gbus_base, gpi.gbus_size,
					"GOKU-S GBUS");
  	}

	/* reset i2c controller */
	goku_s_i2c_reset();

	gpi.i2c_irq = dev->irq;

	/* register interrupt routine */
	i2c_pending = 0;
	if (gpi.i2c_irq > 0) {
	   if (request_irq(gpi.i2c_irq, goku_s_i2c_handler,
			SA_SHIRQ, "GOKU-S I2C", &gpi) < 0) {
		gpi.i2c_irq = 0;
	   } else
		DEB3(printk("Enabled I2C IRQ %d\n", gpi.i2c_irq));
//		enable_irq(gpi.i2c_irq);
	}
	/* set clock frequency */
	gpi.i2c_clk = GOKU_S_DEFAULT_CLOCK;

	/* set Mode Select */
	goku_s_gpio_setbyte(&gpi, GOKU_S_GPIO_MSR, GOKU_S_DEFAULT_MODE);

	return 0;
}


static void goku_s_i2c_release(void)
{
DEB_START;
	if (gpi.i2c_irq > 0) {
		disable_irq(gpi.i2c_irq);
		free_irq(gpi.i2c_irq, &gpi);
	}
	release_region(gpi.i2c_base, gpi.i2c_size);
	release_region(gpi.gpio_base, gpi.gpio_size);
	release_region(gpi.gbus_base, gpi.gbus_size);
}


static int goku_s_i2c_reg(struct i2c_client *client)
{
DEB_START;
	return 0;
}


static int goku_s_i2c_unreg(struct i2c_client *client)
{
DEB_START;
	return 0;
}


static void goku_s_i2c_inc_use(struct i2c_adapter *adap)
{
DEB_START;
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}


static void goku_s_i2c_dec_use(struct i2c_adapter *adap)
{
DEB_START;
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}


#if defined(GOKU_I2C_DEBUG)
static int goku_s_i2c_read_proc(char *page, char **start, off_t off,
					int count, int *eof, void *data)
{
	static char* frq[] = {"396.8", "225.2", "120.8", "62.7", "31.9", "16.1", "8.1"};
	char *p = page;
	int len;
	__u32 sbicr1, sbicr2, sbidbr, mode, gbus;

	p += sprintf(p,
			"GOKU-S I2C Debug information\n"
			" I/O Base Addr    : 0x%08x\n"
			" IRQ Number       : %d\n"
			" Clock Frequency  : %s kHz (0x%x)\n"
			" Own Slave Addr   : 0x%x\n",
			gpi.i2c_base, gpi.i2c_irq, frq[gpi.i2c_clk],
			gpi.i2c_clk, gpi.i2c_own);

#if 1
	sbicr1 = inl(gpi.i2c_base + GOKU_S_I2C_SBICR1);
	sbicr2 = inl(gpi.i2c_base + GOKU_S_I2C_SBICR2);
	sbidbr = inl(gpi.i2c_base + GOKU_S_I2C_SBIDBR);
	mode   = inl(gpi.gpio_base + GOKU_S_GPIO_MSR);
	gbus   = inw(gpi.gbus_base);
#else
	sbicr1 = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBICR1);
	sbicr2 = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBICR2);
	sbidbr = goku_s_i2c_getbyte(&gpi, GOKU_S_I2C_SBIDBR);
	mode   = goku_s_gpio_getbyte(&gpi, GOKU_S_GPIO_MSR);
	gbus   = goku_s_gbus_getbyte(&gpi, 0);
#endif
	p += sprintf(p,
			"---------------------------------\n"
			"--- SBICR1:(0x%08x)\n"
			" SWRMON           : 0x%x\n"
			" ACK              : 0x%x\n"
			"--- SBICR2:(0x%08x)\n"
			" LRB              : 0x%x\n"
			" AD0              : 0x%x\n"
			" AAS              : 0x%x\n"
			" AL               : 0x%x\n"
			" PIN              : 0x%x\n"
			" BB               : 0x%x\n"
			" TRX              : 0x%x\n"
			" MST              : 0x%x\n"
			"--- SBIDBR:(0x%08x)\n"
			" DB               : 0x%02x\n"
			"--- GPIO MSR\n"
			" Mode             : 0x%08x\n"
			"--- GBUS MSR\n"
			" Arbiter          : 0x%08x\n",
			sbicr1,
			(sbicr1 & 0x1), ((sbicr1 >> 4) & 0x1),
			sbicr2,
			(sbicr2 & 0x1), ((sbicr2 >> 1) & 0x1),
			((sbicr2 >> 2) & 0x1), ((sbicr2 >> 3) & 0x1),
			((sbicr2 >> 4) & 0x1), ((sbicr2 >> 5) & 0x1),
			((sbicr2 >> 6) & 0x1), ((sbicr2 >> 7) & 0x1),
			sbidbr,
			(sbidbr & 0xff), mode, gbus);

	p += sprintf(p,
			"---------------------------------\n"
			" i2c_pending      : %d\n",
			i2c_pending);

	len = p - page;

	if (len <= off + count) {
		*eof = 1;
	}
	*start = page + off;
	len -= off;
	if (len > count) {
		len = count;
	}
	if (len < 0) {
		len = 0;
	}

	return len;
}

static int goku_s_i2c_getxdigit(char c)
{
	int digit;
	if (isdigit(c)) {
		digit = c - '0';
	} else if (isxdigit(c)) {
		digit = toupper(c) - 'A' + 10;
	} else {
		digit = -1;
	}
	return digit;
}

static int goku_s_i2c_setting(char *name, int val)
{
	if (strcmp("SBICR1", name) == 0) {
		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR1, val);
	} else if (strcmp("SBICR2", name) == 0) {
		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBICR2, val);
	} else if (strcmp("I2CAR", name) == 0) {
		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_I2CAR, val);
	} else if (strcmp("SBIDBR", name) == 0) {
		goku_s_i2c_setbyte(&gpi, GOKU_S_I2C_SBIDBR, val);
	} else if (strcmp("GPIO", name) == 0) {
		goku_s_gpio_setbyte(&gpi, GOKU_S_GPIO_MSR, val);
	} else if (strcmp("GBUS", name) == 0) {
		goku_s_gbus_setbyte(&gpi, 0, val);
	} else {
		printk("parameter name[%s] is not supported\n", name);
		return -1;
	}
	return 0;
}

#define MAX_LEN	(30)

static int goku_s_i2c_write_proc(char *file, char *buffer,
					unsigned long count, void *data)
{
	char name [MAX_LEN+1];
	int for_real = 0, len;
	unsigned long n;
	const char *start = NULL;

	if (!capable(CAP_SYS_ADMIN)) {
		return -EACCES;
	}

	/* skip over leading whitespace */
	while (count && isspace(*buffer)) {
		--count;
		++buffer;
	}

	do {
		const char *p;
		p = buffer;
		n = count;
		while (n > 0) {
			int d, digits;
			unsigned int val = 0;
			start = p;

			while ((n > 0) && (*p != ':')) {
				--n;
				p++;
			}
			if (*p != ':') {
				goto PARSE_ERROR;
			}
			len = min((p - start), MAX_LEN);
			strncpy(name, start, len);
			name[len] = 0;
			if (n > 0) {
				--n;
				p++;
			} else {
				goto PARSE_ERROR;
			}
			digits = 0;
			while ((n > 0) && ((d = goku_s_i2c_getxdigit(*p)) >= 0)) {
				val = (val << 4) + d;
				--n;
				++p;
				++digits;
			}
			if ((n > 0) && !isspace(*p)) {
				goto PARSE_ERROR;
			}
			while ((n > 0) && isspace(*p)) {
				--n;
				++p;
			}

			if (for_real) {
				if (goku_s_i2c_setting(name, val) < 0) {
					goto PARSE_ERROR;
				}
			}
		}
	} while (!for_real++);
	return count;

PARSE_ERROR:
	printk(": parse error\n");
	return -EINVAL;
}

#endif	/* defined(GOKU_I2C_DEBUG) */

/* ------------------------------------------------------------------------
 * Encapsulate the above functions in the correct operations structure.
 * This is only done when more than one hardware adapter is supported.
 */
static struct goku_s_i2c_algo_data goku_s_i2c_data = {
	NULL,
	goku_s_i2c_setbyte,
	goku_s_i2c_getbyte,
	goku_s_i2c_getown,
	goku_s_i2c_getclock,
	goku_s_i2c_waitforpin,
	goku_s_gpio_setbyte,
	goku_s_gpio_getbyte,
	goku_s_i2c_start,
	goku_s_i2c_stop,
	goku_s_i2c_reset,
	goku_s_i2c_master_send,
	goku_s_i2c_master_recv,
	80, 80, 100,		/*	waits, timeout */
};

static struct i2c_adapter goku_s_i2c_ops = {
	"GOKU-S I2C adapter",
	I2C_HW_GOKU_I2C,
	NULL,
	&goku_s_i2c_data,
	goku_s_i2c_inc_use,
	goku_s_i2c_dec_use,
	goku_s_i2c_reg,
	goku_s_i2c_unreg,
};

/* Called when the module is loaded.  This function starts the
 * cascade of calls up through the heirarchy of i2c modules (i.e. up to the
 *  algorithm layer and into to the core layer)
 */
int __init i2c_goku_s_init(void) 
{
#if defined(GOKU_I2C_DEBUG)
#if defined(CONFIG_PROC_FS)
	struct proc_dir_entry *ent;
#endif	/* defined(CONFIG_PROC_FS) */
	int i;
#endif	/* defined(GOKU_I2C_DEBUG) */

	struct goku_s_i2c *i2c = &gpi;

	printk(KERN_INFO "Initialize GOKU-S I2C adapter module\n");

	memset(i2c, 0, sizeof(struct goku_s_i2c));
	
	if (own) {
		gpi.i2c_own = own;
	} else {
		gpi.i2c_own = GOKU_S_DEFAULT_OWN;
	}
	goku_s_i2c_data.data = (void *)i2c;
	init_waitqueue_head(&i2c_wait);

	spin_lock_init(&i2c->lock);
	spin_lock_init(&i2c->tx_lock);
	spin_lock_init(&i2c->rx_lock);
	spin_lock_init(&i2c->slave_lock);

	if (goku_s_i2c_hw_init() == 0) {
		if (i2c_iic_add_bus(&goku_s_i2c_ops) < 0)
			return -ENODEV;
	} else {
		return -ENODEV;
	}

//	init_waitqueue_head(&i2c->tx_wait);
//	init_waitqueue_head(&i2c->rx_wait);

	init_completion(&i2c->tx_comp);
	init_completion(&i2c->rx_comp);

#if defined(GOKU_I2C_DEBUG)
	for (i = 0; i < GOKU_S_SLAVE_BUFSIZE; i++) {
		i2c->slave_buf[i] = i;
	}
#endif	/* defined(GOKU_I2C_DEBUG) */

#if defined(GOKU_I2C_DEBUG)
#if defined(CONFIG_PROC_FS)
#if 0
	create_proc_read_entry("driver/goku-s-i2c", 0, 0, goku_s_i2c_read_proc, NULL);
#else
	ent = create_proc_entry("goku-s-i2c", S_IFREG|S_IRUSR|S_IWUSR, NULL);
	if (!ent) {
		printk(" can't create proc entry\n");
		return -1;
	}
	ent->nlink = 1;
	ent->data = &gpi;
	ent->read_proc = goku_s_i2c_read_proc;
	ent->write_proc = goku_s_i2c_write_proc;

#endif
#endif	/* defined(CONFIG_PROC_FS) */
#endif	/* defined(GOKU_I2C_DEBUG) */
	printk(KERN_INFO " found device at %#x irq %d.\n", 
		i2c->i2c_base, i2c->i2c_irq);
	return 0;
}


static void i2c_goku_s_exit(void)
{
DEB_START;

#if defined(GOKU_I2C_DEBUG)
#if defined(CONFIG_PROC_FS)
	remove_proc_entry("goku-s-i2c", NULL);
#endif	/* defined(CONFIG_PROC_FS) */
#endif	/* defined(GOKU_I2C_DEBUG) */

	i2c_iic_del_bus(&goku_s_i2c_ops);
        goku_s_i2c_release();
}

EXPORT_NO_SYMBOLS;

/* If modules is NOT defined when this file is compiled, then the MODULE_*
 * macros will resolve to nothing
 */
MODULE_AUTHOR("TOSHIBA CORPRATION SEMICONDUCTOR COMPANY");
MODULE_DESCRIPTION("I2C-Bus adapter routines for GOKU-S I2C bus adapter");
MODULE_LICENSE("GPL");

MODULE_PARM(own, "i");
MODULE_PARM(i2c_debug,"i");

MODULE_PARM_DESC(own, "slave address");
MODULE_PARM_DESC(i2c_debug,
        "debug level - 0 off; 1 normal; 2,3 more verbose; 9 iic-protocol");

/* Called when module is loaded or when kernel is intialized.
 * If MODULES is defined when this file is compiled, then this function will
 * resolve to init_module (the function called when insmod is invoked for a
 * module).  Otherwise, this function is called early in the boot, when the
 * kernel is intialized.  Check out /include/init.h to see how this works.
 */
module_init(i2c_goku_s_init);

/* Resolves to module_cleanup when MODULES is defined. */
module_exit(i2c_goku_s_exit); 
