/*
 *  i2c-cxd.c
 *
 *  i2c algorithm  for SONY_LSI I2C IP
 *
 *  Copyright 2001-2003 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/timer.h>
#include <linux/ioport.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include <linux/hfs_sysdep.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <linux/i2c-id.h>
#include <linux/i2c-dev.h>
#include <asm/irq.h>
#include "i2c-cxd.h"


#define I2C_DEBUG


#ifdef I2C_DEBUG
#define M_LEVEL 5
#define H_LEVEL 3
static int debug = 2;
MODULE_PARM (debug, "i");
MODULE_PARM_DESC (debug, "Debug enabled or not");
#define PDEBUG(level, format, arg...) \
          if (level <= debug) \
                 printk(KERN_DEBUG "%s: %s: " format , __FILE__ , __FUNCTION__ , ##arg)
#else
#define PDEBUG(level, format, arg...)
#endif


#define cxd_get_clock(adap) adap->getclock(adap->data)
#define cxd_outb(adap, reg, val) adap->setiic(adap->data, reg, val)
#define cxd_inb(adap, reg) adap->getiic(adap->data, reg)
#define cxd_wait_event_interruptible(adap) adap->waitforpin(adap->data)

static char *bus_speed = "hi";
MODULE_PARM(bus_speed,"s");

//static wait_queue_head_t i2c_cxd9790_wait;
static wait_queue_head_t i2c_cxd_wait;

static void
check_bb (unsigned long data)
{
  wake_up_interruptible (&i2c_cxd_wait);
}

static struct timer_list wait_bb = {
  function:check_bb,
};


//static struct proc_dir_entry *i2c_cxd_proc,*i2c_cxd_dir;

static int i2c_cxd_xfer (struct i2c_adapter *i2c_adap, struct i2c_msg msgs[],
			 int num);
static int i2c_cxd_ioctl (struct i2c_adapter *adapter, unsigned int cmd,
			  unsigned long arg);

static int __init i2c_cxd_init (struct i2c_adapter *adap);


static u32
i2c_cxd_func (struct i2c_adapter *adap)
{
  return I2C_FUNC_PROTOCOL_MANGLING | I2C_FUNC_SMBUS_EMUL | I2C_M_TEN;
}


static struct i2c_algorithm i2c_cxd_algo = {
  /* name */ "CXD I2C",
  /* id */ I2C_ALGO_CXD,
  /* master_xfer */ i2c_cxd_xfer,
  /*  smbus_access */ NULL,
  /* slave_send */ NULL,
  /* slave_rcv */ NULL,
  /* algo_control */ i2c_cxd_ioctl,
  /* functionality */ i2c_cxd_func,
};


/* 
 * registering functions to load algorithms at runtime 
 */
int
i2c_cxd_add_bus (struct i2c_adapter *adap)
{
  PDEBUG (M_LEVEL, "add_bus\n");
  adap->id |= i2c_cxd_algo.id;
  adap->algo = &i2c_cxd_algo;


#ifdef MODULE
  MOD_INC_USE_COUNT;
#endif

  if (i2c_add_adapter (adap))
    {
      goto i2c_adap_fault;
    }

  i2c_cxd_init (adap);
 
  return 0;

i2c_adap_fault:
  i2c_del_adapter (adap);

#ifdef MODULE
  MOD_DEC_USE_COUNT;
#endif
  return -1;

}

int
i2c_cxd_del_bus (struct i2c_adapter *adap)
{
  int res;
  if ((res = i2c_del_adapter (adap)) < 0)
    return res;
  printk ("i2c-algo-cxd: adapter unregistered: %s\n", adap->name);

#ifdef MODULE
  MOD_DEC_USE_COUNT;
#endif
  return 0;
}


static int
i2c_cxd_ioctl (struct i2c_adapter *adapter, unsigned int cmd,
	       unsigned long arg)
{
  /*
     switch (cmd) {
     case I2C_CXD_CHANGE_FREQ:

     break;
     case I2C_CXD_GET_STATUS:

     break;
     default:
     break;
     }
   */
  return 0;
}

int 
i2c_cxd_9790_init (struct i2c_adapter *i2c_adap)
{
	struct i2c_algo_cxd_data *adap = i2c_adap->algo_data;
	PDEBUG(M_LEVEL, "bus_speed = %s\n",bus_speed);
	if(strcmp(bus_speed ,"lo") == 0 || strcmp(bus_speed,"Lo") == 0 )
		{
			cxd_outb(adap, CXD_ICCCL, 0x76);
			cxd_outb(adap, CXD_ICCCH, 0x78);
		}
	else if(strcmp(bus_speed ,"nazca"))
		{
			cxd_outb (adap, CXD_ICCCL, 0x40);
			cxd_outb (adap, CXD_ICCCH, 0x2a);
		}
	else
		{
			cxd_outb (adap, CXD_ICCCL, 0x1b);
			cxd_outb (adap, CXD_ICCCH, 0x20);
		}
	cxd_outb (adap, CXD_ICDS, 0x10);
	cxd_outb (adap, CXD_ICDA, 0xaa);
	/*
	  PDEBUG(M_LEVEL, "IRCS0  = 0x%x\n", CXD9790_READ(0x08));
	  PDEBUG(M_LEVEL, "IRCS1  = 0x%x\n", CXD9790_READ(0x09));
	  PDEBUG(M_LEVEL, "IROUT  = 0x%x\n", CXD9790_READ(0x0a));
	*/
		 
	return 0;
}


static int
i2c_cxd_xfer (struct i2c_adapter *i2c_adap, struct i2c_msg msgs[], int num)
{
  short i, j;
  int error;
  struct i2c_algo_cxd_data *adap = i2c_adap->algo_data;
  //  int sram = *(int *) (adap->data);
  //  sram = CXD_SRAM;

  for (i = 0; i < num; i++)
    {
      PDEBUG (M_LEVEL, "flag  = 0x%x\n", msgs[i].flags);
      if (!(msgs[i].flags & (unsigned short) I2C_M_NOSTART))
	{

	  if (((msgs[i].flags) & (unsigned short) I2C_M_RD))
		  {
			    // Read
			  //	  msgs[i].addr |= I2C_PROTCOL_READ;
			  cxd_outb (adap, CXD_SRAM, (msgs[i].addr << 1) | I2C_PROTCOL_READ);
			  cxd_outb (adap, CXD_ICINTC,
				    (CXD_ICINTC_INTI2CSC | CXD_ICINTC_INTATMC |
				     CXD_ICINTC_INTI2CC));
			  cxd_outb (adap, CXD_ICINT, 0);
			  //    cxd_outb (adap, CXD_ICINT, CXD_ICINT_ATMEN);
			  if(msgs[i].len == 0)
				  {
					  PDEBUG (H_LEVEL," Length = 0 \n");
					  return -EIO;
				  }
			    if(msgs[i].len > 61)
			    {
			    PDEBUG (H_LEVEL," Length > sram size \n");
			    return -EIO;
			    }
			  cxd_outb (adap, CXD_ICTCR, msgs[i].len);
			  cxd_outb (adap, CXD_ICCM,
				    (CXD_ICCM_ACK | CXD_ICCM_ATM | CXD_ICCM_SDA |
				     CXD_ICCM_SCL));
			  while (CXD_ICCS_BB & cxd_inb (adap, CXD_ICCS))
				  {
					   // bus busy $B$J$i(B $BBT$A(B ($BL$3NG'(B)
					  wait_bb.expires = jiffies + 1;
					  add_timer (&wait_bb);
					  wait_event_interruptible (i2c_cxd_wait,
					  			    !(CXD_ICCS_BB &
					  			      cxd_inb (adap, CXD_ICCS)));

				  }
			  cxd_outb (adap, CXD_ICATMC, (CXD_ICATMC_AT0 | CXD_ICATMC_AT1));
			  /*
			    while (!(CXD_ICINT_INTATM & cxd_inb(adap,CXD_ICINT)))
			    {
			    PDEBUG(M_LEVEL, "read wait\n");
			    wait_bb.expires = jiffies + 1;
			    add_timer(&wait_bb);
			    wait_event_interruptible(i2c_cxd_wait,cxd_inb(adap,CXD_ICINT) & CXD_ICINT_INTATM);
			    }
			  */
			  /*
			    flag = cxd_inb (adap, CXD_ICINT);
			    PDEBUG (H_LEVEL," flag = 0x%p \n",flag);
			    PDEBUG (H_LEVEL," flag = 0x%p \n",CXD_ICINT_INTATM);
			    flag = CXD_ICINT_INTATM  & flag;
			    PDEBUG (H_LEVEL," and flag = 0x%p \n",flag);
			  */
			  if ((!(CXD_ICINT_INTATM & cxd_inb (adap, CXD_ICINT))))
				  {
					  cxd_outb (adap, CXD_ICINT, CXD_ICINT_ATMEN);
					  cxd_wait_event_interruptible (adap);
				  }
			  for (j = 0; j < msgs[i].len; j++)
				  {
					  msgs[i].buf[j] = cxd_inb (adap, CXD_SRAM + j + 1);
				  }
			  error = cxd_inb (adap, CXD_ICATMS);
			  PDEBUG (M_LEVEL, "err = %d\n",error);
			  if (error != 0)
				  {
					  // err $B$J$i(B RST
					  cxd_outb (adap, CXD_ICATMC, (CXD_ICATMC_APC | CXD_ICATMC_EXT |
								       CXD_ICATMC_ERRST | CXD_ICATMC_IRST | 
								       CXD_ICATMC_AT0 | CXD_ICATMC_AT1));
					  cxd_outb (adap, CXD_ICATMC, (CXD_ICATMC_AT0 | CXD_ICATMC_AT1));
					  return -1;
				  }
			  cxd_outb (adap, CXD_ICINTC,
				    (CXD_ICINTC_INTI2CSC | CXD_ICINTC_INTATMC |
				     CXD_ICINTC_INTI2CC));
			  cxd_outb (adap, CXD_ICINT, 0);
		  }
	  else 
		  {
			  // Write
			  cxd_outb (adap, CXD_SRAM, msgs[i].addr << 1);
			  if(msgs[i].len == 0)
				  {
					  PDEBUG (H_LEVEL," Length = 0 \n");
					  return -EIO;
				  }
			 
			    if(msgs[i].len > 61)
			    {
			    PDEBUG (H_LEVEL," Length > sram size \n");
			    return -EIO;
			    }
			 
			  for (j = 0; j < msgs[i].len; j++)
				  {
					  cxd_outb (adap, CXD_SRAM + j + 1, msgs[i].buf[j]);
				  }
			  cxd_outb (adap, CXD_ICINTC,
				    (CXD_ICINTC_INTI2CSC | CXD_ICINTC_INTATMC |
				     CXD_ICINTC_INTI2CC));
			  cxd_outb (adap, CXD_ICINT, 0);
			  //      cxd_outb (adap, CXD_ICINT, CXD_ICINT_ATMEN);
			  cxd_outb (adap, CXD_ICTCR, msgs[i].len);
			  cxd_outb (adap, CXD_ICCM,
				    (CXD_ICCM_ACK | CXD_ICCM_ATM | CXD_ICCM_SDA |
				     CXD_ICCM_SCL));
			  if (CXD_ICCS_BB & cxd_inb (adap, CXD_ICCS))
				  {
					  // bus busy $B$J$i(B $BBT$A(B ($BL$3NG'(B)
					  wait_bb.expires = jiffies + 1;
					  add_timer (&wait_bb);
					  wait_event_interruptible (i2c_cxd_wait,
					 		    !(CXD_ICCS_BB &
					  		      cxd_inb (adap, CXD_ICCS)));
					 

				  }
			  cxd_outb (adap, CXD_ICATMC, (CXD_ICATMC_AT0 | CXD_ICATMC_AT1));
			  /*
			    while(!(CXD_ICINT_INTATM & cxd_inb(adap,CXD_ICINT)))
			    {
			    PDEBUG(M_LEVEL, "write wait\n");
			    //       cxd_wait_event_interruptible(adap);
			    wait_bb.expires = jiffies + 1;
			    PDEBUG(M_LEVEL, "1\n");
			    add_timer(&wait_bb);
			    PDEBUG(M_LEVEL, "2\n");
			    wait_event_interruptible(i2c_cxd_wait,cxd_inb(adap,CXD_ICINT) & CXD_ICINT_INTATM);
			    }
			  */
			  /*
			    flag = cxd_inb (adap, CXD_ICINT);
			    PDEBUG (H_LEVEL," flag = 0x%p \n",flag);
			    PDEBUG (H_LEVEL," flag = 0x%p \n",CXD_ICINT_INTATM);
			    flag = CXD_ICINT_INTATM & flag;
			    PDEBUG (H_LEVEL," and flag = 0x%p \n",flag);
			  */
			  if (!(CXD_ICINT_INTATM & cxd_inb(adap, CXD_ICINT)))
				  {
					  cxd_outb (adap, CXD_ICINT, CXD_ICINT_ATMEN);
					  cxd_wait_event_interruptible (adap);
				  }
			  error = cxd_inb (adap, CXD_ICATMS);
			  PDEBUG (M_LEVEL, "err = %d\n",error);
			  if (error != 0)
				  {
					    // err $B$J$i(B RST
					  cxd_outb (adap, CXD_ICATMC, (CXD_ICATMC_APC | CXD_ICATMC_EXT |
								       CXD_ICATMC_ERRST | CXD_ICATMC_IRST | 
								       CXD_ICATMC_AT0 | CXD_ICATMC_AT1));
					  cxd_outb (adap, CXD_ICATMC, (CXD_ICATMC_AT0 | CXD_ICATMC_AT1));
					  return -1;
				  }
			  cxd_outb (adap, CXD_ICINTC,
				    (CXD_ICINTC_INTI2CSC | CXD_ICINTC_INTATMC |
				     CXD_ICINTC_INTI2CC));
			  cxd_outb (adap, CXD_ICINT, 0);
		  }
	}
    }
  return i;
}



int __init
i2c_cxd_init (struct i2c_adapter *i2c_adap)
{
	i2c_cxd_9790_init(i2c_adap);
	init_waitqueue_head (&i2c_cxd_wait);
	init_timer (&wait_bb);
	return 0;
}


static int __init
i2c_cxd_algo_init (void)
{

  return 0;
}

static void __exit
i2c_cxd_algo_cleanup (void)
{
  return;
}

EXPORT_SYMBOL(i2c_cxd_add_bus);
EXPORT_SYMBOL(i2c_cxd_del_bus);

#ifdef MODULE
int
init_module (void)
{
  printk (KERN_INFO "I2C driver for SONY_LSI I2C IP\n");
  return i2c_cxd_algo_init ();
}

void
cleanup_module (void)
{
  i2c_cxd_algo_cleanup ();
}

#ifdef MODULE_LICENSE
MODULE_LICENSE ("GPL");
#endif
#endif
