/*
   --------------------------------------------------------------------
   i2c-goku-s.h: Global defines for the I2C controller on board the    
                 TOSHIBA GOKU-S companion chip
   --------------------------------------------------------------------

   Copyright 2002 TOSHIBA CORPRATION SEMICONDUCTOR COMPANY.

 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.

 */

#ifndef I2C_GOKU_S_H
#define I2C_GOKU_S_H 1

#define GOKU_S_I2C_VID			0x102F
#define GOKU_S_I2C_DID			0x0108

/* I2C I/O Registers */
#define GOKU_S_I2C_SBICR1		0x00
#define GOKU_S_I2C_SBICR2		0x04
#define GOKU_S_I2C_I2CAR		0x08
#define GOKU_S_I2C_SBIDBR		0x0C

/* Control 1 Register (SBICR1) */
#define GOKU_S_I2C_SWRMON		0x00000001	/* (RO)[0] */
#define GOKU_S_I2C_SCK			0x00000007	/* (WO)[2:0] */
#define GOKU_S_I2C_ACK			0x00000010	/* (RW)[4] */
#define GOKU_S_I2C_BC			0x000000E0	/* (WO)[7:5] */

#define GOKU_S_I2C_BC8			0x00
#define GOKU_S_I2C_BC1			0x20
#define GOKU_S_I2C_BC2			0x40
#define GOKU_S_I2C_BC3			0x60
#define GOKU_S_I2C_BC4			0x80
#define GOKU_S_I2C_BC5			0xA0
#define GOKU_S_I2C_BC6			0xC0
#define GOKU_S_I2C_BC7			0xE0


/* Control 2 Register (SBICR2) */
#define GOKU_S_I2C_SWARST		0x00000003	/* (WO)[1:0] */
#define GOKU_S_I2C_LRB			0x00000001	/* (RO)[0] */
#define GOKU_S_I2C_AD0			0x00000002	/* (RO)[1] */
#define GOKU_S_I2C_AAS			0x00000004	/* (RO)[2] */
#define GOKU_S_I2C_AL			0x00000008	/* (RO)[3] */
#define GOKU_S_I2C_PIN			0x00000010	/* (RW)[4] */
#define GOKU_S_I2C_BB			0x00000020	/* (RW)[5] */
#define GOKU_S_I2C_TRX			0x00000040	/* (RW)[6] */
#define GOKU_S_I2C_MST			0x00000080	/* (RW)[7] */


/* I2C Bus Address Register (I2CAR) */
#define GOKU_S_I2C_ALS			0x00000001	/* (WO)[0] */
#define GOKU_S_I2C_SA			0x000000FE	/* (WO)[7:1] */


/* Data Register (SBIDBR) */
#define GOKU_S_I2C_DB			0x000000FF	/* (RW)[7:0] */


/*** GOKU-S GPIO Register ***/

#define GOKU_S_GPIO_GPIODO		0x00
#define GOKU_S_GPIO_GPIODI		0x04
#define GOKU_S_GPIO_GPIODIR		0x08
#define GOKU_S_GPIO_GPIOOD		0x0C
#define GOKU_S_GPIO_GPRER		0x10
#define GOKU_S_GPIO_GPFER		0x14
#define GOKU_S_GPIO_GPINTFR		0x18
#define GOKU_S_GPIO_GPINTER		0x1C
#define GOKU_S_GPIO_MSR			0x20

/* Mode Select Register */
#define GOKU_S_GPIO_MODE		0x00000007	/* (RW)[2:0] */

#define GOKU_S_GPIO_MODE0		0x0		/* (GPIO0-4) */
#define GOKU_S_GPIO_MODE1		0x1		/* (GPIO0-2,I2C) */
#define GOKU_S_GPIO_MODE2		0x2		/* (GPIO0/3/4,SIO) */
#define GOKU_S_GPIO_MODE3		0x3		/* (GPIO0,SIO,I2C) */
#define GOKU_S_GPIO_MODE4		0x4		/* (SIO[sync],I2C) */
#define GOKU_S_GPIO_MODE5		0x5		/* (GPIO0,SIO[modem]) */
#define GOKU_S_GPIO_MODE6		0x6		/* (SIO[modem]) */

/*** GOKU-S Internal G-bus Arbiter I/O Register */
#define GOKU_S_GBUS_GASR		0x00

#define GOKU_S_GBUS_FRS			0x0001
#define GOKU_S_GBUS_PRI0		0x0002
#define GOKU_S_GBUS_PRI1		0x0004
#define GOKU_S_GBUS_PRI2		0x0008
#define GOKU_S_GBUS_PRI3		0x0010
#define GOKU_S_GBUS_PRI4		0x0020
#define GOKU_S_GBUS_IBRE		0x0040
#define GOKU_S_GBUS_MBRE		0x0080


#define GOKU_S_SLAVE_BUFSIZE	256

struct goku_s_i2c {
	__u32		i2c_base;		/* I2C I/O Base Address */
	__u32		i2c_size;		/* I2C I/O Register Size */
	int			i2c_irq;		/* I2C Interrupt Number */
	int			i2c_clk;		/* I2C clock frequency */
	int			i2c_own;		/* I2C Slave Address */
	__u32		gpio_base;		/* GPIO I/O Base Address */
	__u32		gpio_size;		/* GPIO I/O Register Size */
	__u32		gbus_base;		/* GBUS I/O Base Address */
	__u32		gbus_size;		/* GBUS I/O Register Size */

	spinlock_t	lock;			/* spinlock for internal data */

	spinlock_t	tx_lock;		/* spinlock for tx complete */
//	wait_queue_head_t tx_wait;	/* wait for complete signal */
	struct completion tx_comp;
	char*		tx_buf;			/* buffer for transmit */
	__u32		tx_size;		/* size of buffer */
	int			tx_count;		/* number of transmit */

	spinlock_t	rx_lock;		/* spinlock for rx complete */
//	wait_queue_head_t rx_wait;	/* wait for complete signal */
	struct completion rx_comp;
	char*		rx_buf;			/* buffer for receive */
	__u32		rx_size;		/* size of buffer */
	int			rx_count;		/* number of receive */

	spinlock_t	slave_lock;		/* spinlock for slave mode */
	int			w_ptr;			/* write buffer index */
	int			r_ptr;			/* read buffer index */
	char		slave_buf[GOKU_S_SLAVE_BUFSIZE];	/* buffer for slave mode */

	char		skip_intr;		/* flag for interrupt skip */
	__u8		i2c_sts;		/* saved for SBICR2 */
//	__u8		i2c_data;		/* saved for SBIDBR */
};


#endif  /* I2C_GOKU_S_H */
