/*
 *  IDE driver for CXD9802
 *
 *  Copyright 2003,2004 Sony Corporation.
 *
 *  Derived from drivers/ide/tc86c001.c
 *       Copyright (C) 2002	Toshiba Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/pci.h>
#include <linux/hdreg.h>
#include <linux/ide.h>
#include <linux/delay.h>
#include <linux/init.h>

#include <asm/cxd9802/cxd9802.h>

#include <asm/io.h>

#include "ide_modes.h"

#define CXD9802_IDE_DEVNAME	"CXD9802 IDE"
#define CXD9802_IDE_NUM_HWIFS 2

//#define CXD9802_IDE_RESET 
//#undef CONFIG_BLK_DEV_IDEDMA

#define DEBUG
#define DEBUG_LEVEL DPL_NOTICE

//#define USE_MY_IDEPROC

#ifdef DEBUG
#define DPR(fmt , args...)	printk(fmt, ## args)
#endif /* DEBUG */

/* ======================================================================= *
 * Debug print function from dpr package                                   *
 * ======================================================================= */

#define DPL_EMERG	0
#define DPL_ALERT	1
#define DPL_CRIT	2
#define DPL_ERR		3
#define DPL_WARNING	4
#define DPL_NOTICE	5
#define DPL_INFO	6
#define DPL_DEBUG	7

#define DPM_PURPOSE	0x0000000f
#define DPM_ERR		0x00000001
#define DPM_TRACE	0x00000002
#define DPM_LAYER	0x000000f0
#define DPM_APPLICATION	0x00000010
#define DPM_MIDDLEWARE	0x00000020
#define DPM_DRIVER	0x00000040
#define DPM_KERNEL	0x00000080
#define DPM_USERDEF	0xffff0000

#ifdef DEBUG

#ifndef DEBUG_LEVEL
#define DEBUG_LEVEL DPL_DEBUG
#endif
#ifndef DEBUG_MASK
#define DEBUG_MASK  DPM_PURPOSE
#endif

#ifndef DPR
#include <stdio.h>
#define DPR(fmt, args...)	fprintf(stderr, fmt , ## args)
#endif

#define DPRL(level, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && DPR(fmt , ## args))
#define DPRM(mask, fmt, args...) \
	(void)(((mask) & DEBUG_MASK) && DPR(fmt , ## args))
#define DPRLM(level, mask, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && ((mask) & DEBUG_MASK) && \
		DPR(fmt , ## args))
#define DPRC(cond, fmt, args...) \
	(void)((cond) && DPR(fmt , ## args))
#define DPRF(fmt, args...)	DPR("%s: " fmt, __FUNCTION__ , ## args)
#define DPRFL(level, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && DPRF(fmt , ## args))
#define DPRFM(mask, fmt, args...) \
	(void)(((mask) & DEBUG_MASK) && DPRF(fmt , ## args))
#define DPRFLM(level, mask, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && ((mask) & DEBUG_MASK) && \
		DPRF(fmt , ## args))
#define DPRFC(cond, fmt, args...) \
	(void)((cond) && DPRF(fmt , ## args))

#define	DPRFS	DPRFLM(DPL_DEBUG, DPM_TRACE, "start\n")
#define	DPRFE	DPRFLM(DPL_DEBUG, DPM_TRACE, "end\n")
#define DPRFR	DPRFLM(DPL_DEBUG, DPM_TRACE, "return\n")

#else  /* DEBUG */

#ifndef DEBUG_LEVEL
#define DEBUG_LEVEL -1
#endif
#ifndef DEBUG_MASK
#define DEBUG_MASK  0
#endif

#ifndef DPR
#define DPR(fmt, args...)
#endif

#define DPRL(level, fmt, args...)
#define DPRM(mask, fmt, args...)
#define DPRLM(level, mask, fmt, args...)
#define DPRC(cond, fmt, args...)
#define DPRF(fmt, args...)
#define DPRFL(level, fmt, args...)
#define DPRFM(mask, fmt, args...)
#define DPRFLM(level, mask, fmt, args...)
#define DPRFC(cond, fmt, args...)

#define	DPRFS
#define	DPRFE
#define DPRFR

#endif /* DEBUG */

/* ======================================================================= *
 * ATA register                                                            *
 * ======================================================================= */

#define REG_DATA		0x000
#define REG_ERROR		0x001
#define REG_FEATURE		0x001
#define REG_NSECTOR		0x002
#define REG_SECTOR		0x003
#define REG_LCYL		0x004
#define REG_HCYL		0x005
#define REG_SELECT		0x006
#define REG_STATUS		0x007
#define REG_COMMAND		0x007

#define REG_ALTSTATUS		0x402
#define REG_CONTROL		0x402

#define REG_DMA_COMMAND		0x800
#define REG_DMA_STATUS		0x802
#define  REGM_INT_IDE		(1 << 2)
#define  REGM_ERROR		(1 << 1)
#define  REGM_BM_ACT		(1 << 0)
#define REG_PRD_TABLE_PTR	0x804

#define REG_SYS_CONTROL1	0xc00
#define  REGM_SOFT_RESET	(1 << 15)
#define  REGM_FIFO_RESET	(1 << 14)
#define  REGM_PDIAGN		(1 << 13)
#define  REGM_DASPN		(1 << 12)
#define  REGM_ATA_HARD_RESET	(1 << 11)
#define  REGM_COMM_XFER		(0x7 << 8)
#define  REGC_COMM_XFER_PIO_0	(0x0 << 8)
#define  REGC_COMM_XFER_PIO_1	(0x1 << 8)
#define  REGC_COMM_XFER_PIO_2	(0x2 << 8)
#define  REGC_COMM_XFER_PIO_3	(0x3 << 8)
#define  REGC_COMM_XFER_PIO_4	(0x4 << 8)
#define  REGM_DATA_XFER		(0xf << 4)
#define  REGC_DATA_XFER_PIO_0	(0x0 << 4)
#define  REGC_DATA_XFER_PIO_1	(0x1 << 4)
#define  REGC_DATA_XFER_PIO_2	(0x2 << 4)
#define  REGC_DATA_XFER_PIO_3	(0x3 << 4)
#define  REGC_DATA_XFER_PIO_4	(0x4 << 4)
#define  REGC_DATA_XFER_MDMA_0	(0x5 << 4)
#define  REGC_DATA_XFER_MDMA_1	(0x6 << 4)
#define  REGC_DATA_XFER_MDMA_2	(0x7 << 4)
#define  REGC_DATA_XFER_UDMA_0	(0x8 << 4)
#define  REGC_DATA_XFER_UDMA_1	(0x9 << 4)
#define  REGC_DATA_XFER_UDMA_2	(0xa << 4)
#define  REGC_DATA_XFER_UDMA_3	(0xb << 4)
#define  REGC_DATA_XFER_UDMA_4	(0xc << 4)
#define  REGC_DATA_XFER_UDMA_5	(0xd << 4)
#define  REGM_BREAK_ENABLE	(1 << 3)
#define  REGM_END_BREAK		(1 << 2)
#define  REGM_AUTO_DMA_ENABLE	(1 << 1)
#define  REGM_ACCESS_NOW	(1 << 0)
#define REG_SYS_CONTROL2	0xc02
#define REG_XFER_WORD_CNT1	0xc08
#define REG_XFER_WORD_CNT2	0xc0a
#define REG_SECTOR_CNT		0xc10
#define REG_XFER_START_LOWER	0xc18
#define REG_XFER_START_UPPER	0xc20
#define REG_ADD_CTL		0xc28
#define REG_LOWER_BURST_CNT	0xc30
#define REG_UPPER_BURST_CNT	0xc38
#define REG_PIO_ACC		0xc88
#define REG_HOST_RST_TIMER	0xc90
#define REG_INT_CTL		0xc98
#define  REGM_MASK_ADDR_ERR	(1 << 15)
#define  REGM_MASK_REACH_MULT	(1 << 14)
#define  REGM_MASK_DEV_TIMING	(1 << 13)
#define  REGM_MASK_UDMA_TERM	(1 << 12)
#define  REGM_MASK_TIMER_INT	(1 << 11)
#define  REGM_MASK_BUS_ERROR	(1 << 10)
#define  REGM_MASK_XFER_END	(1 << 9)
#define  REGM_MASK_HOST_INT	(1 << 8)
#define  REGM_INT_ADDR_ERR	(1 << 7)
#define  REGM_INT_REACH_MULT	(1 << 6)
#define  REGM_INT_DEV_TIMING	(1 << 5)
#define  REGM_INT_UDMA_TERM	(1 << 4)
#define  REGM_INT_TIMER_INT	(1 << 3)
#define  REGM_INT_BUS_ERROR	(1 << 2)
#define  REGM_INT_XFER_END	(1 << 1)
#define  REGM_INT_HOST_INT	(1 << 0)
#define REG_ATAPI_PKT_CMD	0xcb8
#define REG_BUS_XFER_CNT_H	0xcc0
#define REG_BUS_XFER_CNT_L	0xcc8
#define REG_DEV_T_ERR		0xcd0
#define REG_PKT_XFER_CTL	0xcd8
#define  REGM_PACKET_XFER_MODE	(1 << 12)
#define  REGS_PACKET_XFER_CNT(x) ((x) << 8)
#define  REGM_PACKET_START	(1 << 0)
#define REG_XFER_STRT_TOP	0xce0

static int cxd9802_ide_speeds[CXD9802_IDE_NUM_HWIFS];

static inline int
cxd9802_ide_is_master(ide_drive_t *drive)
{
	return (drive == &(HWIF(drive)->drives[0]));
}

/* ======================================================================= *
 * PIO                                                                     *
 * ======================================================================= */

static void
cxd9802_ide_tune_drive(ide_drive_t *drive, byte pio)
{
	u32 base = HWIF(drive)->io_ports[IDE_DATA_OFFSET];
	u16 xfer_mode;
	u16 sys_control;
	byte speed;
	DPRFS;

	pio = ide_get_best_pio_mode(drive, pio, 5, NULL);

	speed = XFER_PIO_0;
	switch (pio) {
	case 0:
		xfer_mode = (REGC_COMM_XFER_PIO_0 | REGC_DATA_XFER_PIO_0);
		speed = XFER_PIO_0;
		break;
	case 1:
		xfer_mode = (REGC_COMM_XFER_PIO_1 | REGC_DATA_XFER_PIO_1);
		speed = XFER_PIO_1;
		break;
	case 2:
		xfer_mode = (REGC_COMM_XFER_PIO_2 | REGC_DATA_XFER_PIO_2);
		speed = XFER_PIO_2;
		break;
	case 3:
		xfer_mode = (REGC_COMM_XFER_PIO_3 | REGC_DATA_XFER_PIO_3);
		speed = XFER_PIO_3;
		break;
	case 4:
		xfer_mode = (REGC_COMM_XFER_PIO_4 | REGC_DATA_XFER_PIO_4);
		speed = XFER_PIO_4;
		break;
	default:
		DPRFL(DPL_WARNING, "unknown PIO mode %d\n", pio);
		xfer_mode = (REGC_COMM_XFER_PIO_0 | REGC_DATA_XFER_PIO_0);
		speed = XFER_PIO_0;
		break;
	}

	if (cxd9802_ide_speeds[HWIF(drive)->channel] != 0 &&
	    speed > cxd9802_ide_speeds[HWIF(drive)->channel]) {
		speed = cxd9802_ide_speeds[HWIF(drive)->channel];
		DPRFL(DPL_NOTICE, "ATA %d speed is limited to %d\n",
		      HWIF(drive)->channel, speed);
	}

	if (cxd9802_ide_is_master(drive)) {
		sys_control = inw(base + REG_SYS_CONTROL1);
		sys_control &= ~(REGM_COMM_XFER | REGM_DATA_XFER);
		sys_control |= xfer_mode;
		outw(sys_control, base + REG_SYS_CONTROL1);
		DPRFL(DPL_DEBUG, "sys_control1=%04x\n", sys_control);
	} else {
		sys_control = inw(base + REG_SYS_CONTROL2);
		sys_control &= ~(REGM_COMM_XFER | REGM_DATA_XFER);
		sys_control |= xfer_mode;
		outw(sys_control, base + REG_SYS_CONTROL2);
		DPRFL(DPL_DEBUG, "sys_control2=%04x\n", sys_control);
	}

	drive->current_speed = speed;
	ide_config_drive_speed(drive, speed);

	DPRFE;
	return;
}

#ifdef USE_MY_IDEPROC
static void 
cxd9802_ide_ideproc(ide_ide_action_t action, ide_drive_t *drive,
		    void *buffer, unsigned int count)
{
	u32 base = HWIF(drive)->io_ports[IDE_DATA_OFFSET];
	DPRFS;

	switch(action) {
	case ideproc_ide_input_data:
		insw(IDE_DATA_REG, buffer, count * 2);
		break;
	case ideproc_ide_output_data:
		outsw(IDE_DATA_REG, buffer, count * 2);
		break;
	case ideproc_atapi_input_bytes:
		count++;
		insw(IDE_DATA_REG, buffer, count / 2);
		if ((count & 0x03) >= 2) {
			insw(IDE_DATA_REG,
			     (byte *)buffer + (count & ~0x03), 1);
		}
		break;
	case ideproc_atapi_output_bytes:
		if (count == 12) {
			outw(REGM_PACKET_XFER_MODE |
			     REGS_PACKET_XFER_CNT(count / 2 - 1),
			     base + REG_PKT_XFER_CTL);
			outsw(base + REG_ATAPI_PKT_CMD, buffer, count / 2);
			outw(REGM_PACKET_XFER_MODE |
			     REGS_PACKET_XFER_CNT(count / 2 - 1) |
			     REGM_PACKET_START,
			     base + REG_PKT_XFER_CTL);

			break;
		}
		count++;
		outsw(IDE_DATA_REG, buffer, count / 2);
		if ((count & 0x03) >= 2) {
			outsw(IDE_DATA_REG,
			      (byte *)buffer + (count & ~0x03), 1);
		}
		break;
	}

	DPRFE;
	return;
}
#endif /* USE_MY_IDEPROC */

/* ======================================================================= *
 * DMA                                                                     *
 * ======================================================================= */

#ifdef CONFIG_BLK_DEV_IDEDMA
static int
cxd9802_ide_tune_chipset(ide_drive_t *drive, byte speed)
{
	u32 base = HWIF(drive)->io_ports[IDE_DATA_OFFSET];
	int res;
	u16 data_xfer;
	u16 sys_control;
	DPRFS;

	switch (speed) {
	case XFER_UDMA_5:	data_xfer = REGC_DATA_XFER_UDMA_5;	break;
	case XFER_UDMA_4:	data_xfer = REGC_DATA_XFER_UDMA_4;	break;
	case XFER_UDMA_3:	data_xfer = REGC_DATA_XFER_UDMA_3;	break;
	case XFER_UDMA_2:	data_xfer = REGC_DATA_XFER_UDMA_2;	break;
	case XFER_UDMA_1:	data_xfer = REGC_DATA_XFER_UDMA_1;	break;
	case XFER_UDMA_0:	data_xfer = REGC_DATA_XFER_UDMA_0;	break;
	case XFER_MW_DMA_2:	data_xfer = REGC_DATA_XFER_MDMA_2;	break;
	case XFER_MW_DMA_1:	data_xfer = REGC_DATA_XFER_MDMA_1;	break;
	case XFER_MW_DMA_0:	data_xfer = REGC_DATA_XFER_MDMA_0;	break;
	default:
		DPRFL(DPL_WARNING, "unknown DMA mode %d\n", speed);
		DPRFR;
		return -1;
	}

	if (cxd9802_ide_is_master(drive)) {
		sys_control = inw(base + REG_SYS_CONTROL1);
		sys_control &= ~REGM_DATA_XFER;
		sys_control |= data_xfer;
		outw(sys_control, base + REG_SYS_CONTROL1);
		DPRFL(DPL_DEBUG, "sys_control1=%04x\n", sys_control);
	} else {
		sys_control = inw(base + REG_SYS_CONTROL2);
		sys_control &= ~REGM_DATA_XFER;
		sys_control |= data_xfer;
		outw(sys_control, base + REG_SYS_CONTROL2);
		DPRFL(DPL_DEBUG, "sys_control2=%04x\n", sys_control);
	}

	if (drive->init_speed == 0) {
		drive->init_speed = speed;
	}

	drive->current_speed = speed;
	res = ide_config_drive_speed(drive, speed);

	DPRFE;
	return res;
}

static ide_dma_action_t
cxd9802_ide_config_drive_for_dma(ide_drive_t *drive)
{
	struct hd_driveid *id = drive->id;
	int res;
	byte speed;
	byte udma_66;
	DPRFS;

	udma_66 = eighty_ninty_three(drive);
	speed = 0;
	if ((id->dma_ultra & (1 << 5)) && udma_66) {
		speed = XFER_UDMA_5;
	} else if ((id->dma_ultra & (1 << 4)) && udma_66) {
		speed = XFER_UDMA_4;
	} else if ((id->dma_ultra & (1 << 3)) && udma_66) {
		speed = XFER_UDMA_3;
	} else if (id->dma_ultra & (1 << 2)) {
		speed = XFER_UDMA_2;
	} else if (id->dma_ultra & (1 << 1)) {
		speed = XFER_UDMA_1;
	} else if (id->dma_ultra & (1 << 0)) {
		speed = XFER_UDMA_0;
	} else if (id->dma_mword & (1 << 2)) {
		speed = XFER_MW_DMA_2;
	} else if (id->dma_mword & (1 << 1)) {
		speed = XFER_MW_DMA_1;
	} else if (id->dma_mword & (1 << 0)) {
		speed = XFER_MW_DMA_0;
	} else {
		return ide_dma_off_quietly;
	}

	if (cxd9802_ide_speeds[HWIF(drive)->channel] != 0 &&
	    speed > cxd9802_ide_speeds[HWIF(drive)->channel]) {
		speed = cxd9802_ide_speeds[HWIF(drive)->channel];
		DPRFL(DPL_NOTICE, "ATA %d speed is limited to %d\n",
		      HWIF(drive)->channel, speed);
		if (speed < XFER_MW_DMA_0) {
			DPRFS;
			return ide_dma_off_quietly;
		}
	}

	cxd9802_ide_tune_chipset(drive, speed);

	if ((id->dma_ultra & (0x7 << 11)) != 0 ||
	    (id->dma_ultra & (0x7 << 8)) != 0 ||
	    (id->dma_mword & (0x7 << 8)) != 0 ||
	    (id->dma_1word & (0x7 << 8)) != 0) {
		res = ide_dma_on;
	} else {
		res = ide_dma_off_quietly;
	}

	DPRFE;
	return res;
}

static int
cxd9802_ide_dmaproc(ide_dma_action_t func, ide_drive_t *drive)
{
	u32 base = HWIF(drive)->io_ports[IDE_DATA_OFFSET];
	int res;
	DPRFS;

	DPRFL(DPL_DEBUG, "func=%d\n", func);

	switch (func) {
	case ide_dma_check:
		return ide_dmaproc(cxd9802_ide_config_drive_for_dma(drive),
				   drive);
	case ide_dma_read:
	case ide_dma_write:
	{
		u16 word_count;
		u16 sector_count;

		DPRFL(DPL_DEBUG, "media=%d, sec=%ld, nsec=%ld, nseg=%d\n",
		      drive->media, HWGROUP(drive)->rq->sector,
		      HWGROUP(drive)->rq->nr_sectors,
		      HWGROUP(drive)->rq->nr_segments);

		if (drive->media == ide_cdrom) {
			word_count = 2048 / 2;
			sector_count = HWGROUP(drive)->rq->nr_sectors / 4;
		} else {
			word_count = 512 / 2;
			sector_count = HWGROUP(drive)->rq->nr_sectors;
		}
		outw(sector_count, base + REG_SECTOR_CNT);
		if (cxd9802_ide_is_master(drive)) {
			outw(word_count, base + REG_XFER_WORD_CNT1);
		} else {
			outw(word_count, base + REG_XFER_WORD_CNT2);
		}
		break;
	}
	case ide_dma_begin:
		if (drive->media == ide_cdrom) {
			return 0;
		}
		outw(REGM_MASK_UDMA_TERM | REGM_MASK_XFER_END,
		     base + REG_INT_CTL);
		break;
	default:
		break;
	}

	res = ide_dmaproc(func, drive);

	DPRFE;
	return res;
}
#endif /* CONFIG_BLK_DEV_IDEDMA */

/* ======================================================================= *
 * kernel option                                                           *
 * ======================================================================= */

static int __init
cxd9802_ide_speed_setup(char *line)
{
	int opts[1 + CXD9802_IDE_NUM_HWIFS];
	int speed;
	int i;
	DPRFS;

	get_options(line, 1 + CXD9802_IDE_NUM_HWIFS, opts);
	for (i = 0; i < CXD9802_IDE_NUM_HWIFS; i++) {
		speed = opts[1 + i];
		if (speed == XFER_UDMA_5 || speed == XFER_UDMA_4 ||
		    speed == XFER_UDMA_3 || speed == XFER_UDMA_2 ||
		    speed == XFER_UDMA_1 || speed == XFER_UDMA_0 ||
		    speed == XFER_MW_DMA_2 || speed == XFER_MW_DMA_1 ||
		    speed == XFER_MW_DMA_0 ||
		    speed == XFER_PIO_4 || speed == XFER_PIO_3 ||
		    speed == XFER_PIO_2 || speed == XFER_PIO_1 ||
		    speed == XFER_PIO_0) {
			cxd9802_ide_speeds[i] = opts[1 + i];
		}
	}

	DPRFE;
	return 1;
}

__setup("cxd9802_ide_speed=", cxd9802_ide_speed_setup);

/* ======================================================================= *
 * IDE driver interface                                                    *
 * ======================================================================= */

static int cxd9802_idereg_offsets[CXD9802_IDE_NUM_HWIFS][IDE_NR_PORTS] = {
	{0x000, 0x001, 0x002, 0x003, 0x004, 0x005, 0x006, 0x007,
	 0x402, -1},
	{0x000, 0x001, 0x002, 0x003, 0x004, 0x005, 0x006, 0x007,
	 0x402, -1},
};

static ide_ioreg_t cxd9802_idereg_bases[CXD9802_IDE_NUM_HWIFS] = {
	CXD9802_ATA0_BASE,
	CXD9802_ATA1_BASE,
};

static int cxd9802_ideirq[CXD9802_IDE_NUM_HWIFS] = {
	CXD9802_IRQ_ATA0,
	CXD9802_IRQ_ATA1,
};

unsigned int __init
cxd9802_ide_ata66_check(ide_hwif_t *hwif)
{
	u32 base = hwif->io_ports[IDE_DATA_OFFSET];
	unsigned int ata66;
	DPRFS;

	/* bit13(PDIAGN) = 0:(80pin cable) 1:(40pin cable) */
	ata66 = ((inw(base + REG_SYS_CONTROL1) & REGM_PDIAGN) == 0);

	DPRFE;
	return ata66;
}

static void __init
cxd9802_ide_init_hwif(ide_hwif_t *hwif)
{
	DPRFS;

	/* routine to tune PIO mode for drives */
	hwif->tuneproc = &cxd9802_ide_tune_drive;
	hwif->drives[0].autotune = 1;
	hwif->drives[1].autotune = 1;

	hwif->autodma = 0;

#ifdef CONFIG_BLK_DEV_IDEDMA
	if (hwif->dma_base == 0){
		DPRFL(DPL_ERR, "dma_base is not found\n");
		return;
	}

	if (!noautodma){
		hwif->autodma = 1;
		hwif->dmaproc = &cxd9802_ide_dmaproc;
		hwif->speedproc = &cxd9802_ide_tune_chipset;
	} else {
		hwif->dmaproc = &ide_dmaproc;	/* original */
	}
#endif /* !CONFIG_BLK_DEV_IDEDMA */

	DPRFE;
	return;
}

/* another interrupt handler to clear interrupts at CXD9802's ATA interrupt
   registers */
static void
cxd9802_ide_intr_handler(int irq, void *devid, struct pt_regs *regs)
{
	/* clear interrupts */
	if (irq == CXD9802_IRQ_ATA0) {
		DPRFL(DPL_DEBUG, "ATA0:%04x\n",
		      cxd9802_reg_read16(CXD9802_ATA0_BASE + REG_INT_CTL));
		cxd9802_reg_write16(CXD9802_ATA0_BASE + REG_INT_CTL,
				    cxd9802_reg_read16(CXD9802_ATA0_BASE +
						       REG_INT_CTL));
	}
	if (irq == CXD9802_IRQ_ATA1) {
		DPRFL(DPL_DEBUG, "ATA1:%04x\n",
		      cxd9802_reg_read16(CXD9802_ATA1_BASE + REG_INT_CTL));
		cxd9802_reg_write16(CXD9802_ATA1_BASE + REG_INT_CTL,
				    cxd9802_reg_read16(CXD9802_ATA1_BASE +
						       REG_INT_CTL));
	}

	return;
}

void __init
cxd9802_ide_init(void)
{
	int res;
	hw_regs_t hw;
	ide_hwif_t *hwifp;
	int index;
	int i;
	DPRFS;

	for (i = 0; i < CXD9802_IDE_NUM_HWIFS; i++) {
#ifdef CXD9802_IDE_RESET
		u16 sysctrl;
		/* soft reset */
		sysctrl = cxd9802_reg_read16(cxd9802_idereg_bases[i] +
					     REG_SYS_CONTROL1);
		cxd9802_reg_write16(cxd9802_idereg_bases[i] + REG_SYS_CONTROL1,
				    sysctrl | REGM_SOFT_RESET);

		/* hard reset */
		cxd9802_reg_write16(cxd9802_idereg_bases[i] + REG_SYS_CONTROL1,
				    sysctrl | REGM_ATA_HARD_RESET);

		/* wait for finish of the reset */
		while (cxd9802_reg_read16(cxd9802_idereg_bases[i] +
					  REG_SYS_CONTROL1) &
		       REGM_ATA_HARD_RESET);

#ifndef CONFIG_SNSC_IDE_WAIT_HWRST
		{
			int loop;

			loop = 0;
			while (loop < 100) {
				if (!(cxd9802_reg_read8(cxd9802_idereg_bases[i]
							+ 0x07) & BUSY_STAT)) {
					break;
				}
				mdelay(50);
				loop++;
			}
		}
#endif /* !CONFIG_SNSC_IDE_WAIT_HWRST */
#endif /* CXD9802_IDE_RESET */

		res = request_irq(cxd9802_ideirq[i], cxd9802_ide_intr_handler,
				  SA_SHIRQ, CXD9802_IDE_DEVNAME,
				  cxd9802_ide_intr_handler);
		if (res < 0) {
			DPRFL(DPL_WARNING, "request_irq(%d) failed (%d)\n",
			      cxd9802_ideirq[i], -res);
		}

		ide_setup_ports(&hw,
				CXD9802_REG_BASE32 + cxd9802_idereg_bases[i] -
				mips_io_port_base,
				cxd9802_idereg_offsets[i], 0, 0, NULL,
				cxd9802_ideirq[i]);
		hw.chipset = ide_cxd9802;

		index = ide_register_hw(&hw, &hwifp);
		if (index == -1) {
			return;
		}

#ifdef CONFIG_BLK_DEV_IDEDMA
		hwifp->udma_four = cxd9802_ide_ata66_check(hwifp);

		hwifp->pci_dev = NULL;
#ifdef USE_MY_IDEPROC
		hwifp->ideproc = cxd9802_ide_ideproc;
#endif
		ide_setup_dma(hwifp,
			      hwifp->io_ports[IDE_DATA_OFFSET] + 0x800, 0x10);

		cxd9802_reg_write16(cxd9802_idereg_bases[i] +
				    REG_LOWER_BURST_CNT,
				    0x0008);
		cxd9802_reg_write16(cxd9802_idereg_bases[i] +
				    REG_UPPER_BURST_CNT,
				    0x0000);
#endif /* CONFIG_BLK_DEV_IDEDMA */

		cxd9802_ide_init_hwif(hwifp);
	}

	DPRFE;
	return;
}
