/*
 * I/O access macros.
 */
#ifdef CONFIG_SNSC_MTD_NAND_BYTE_SWAP

#ifdef CONFIG_SNSC_MTD_NAND_HOST_BE

/* Host: BE <-> NAND: LE */
#define READW(addr)	le16_to_cpu(readw(addr))
#define READL(addr)	le32_to_cpu(readl(addr))
#define WRITEW(n, addr)	writew(cpu_to_le16(n), addr)
#define WRITEL(n, addr)	writel(cpu_to_le32(n), addr)

#else	/* CONFIG_SNSC_MTD_NAND_HOST_BE */
#ifdef CONFIG_SNSC_MTD_NAND_HOST_LE

/* Host: LE <-> NAND: BE */
#define READW(addr)	be16_to_cpu(readw(addr))
#define READL(addr)	be32_to_cpu(readl(addr))
#define WRITEW(n, addr)	writew(cpu_to_be16(n), addr)
#define WRITEL(n, addr)	writel(cpu_to_be32(n), addr)

#else	/* CONFIG_SNSC_MTD_NAND_HOST_LE */

#error Endian for NAND flash data is not defined.

#endif	/* CONFIG_SNSC_MTD_NAND_HOST_LE */
#endif	/* CONFIG_SNSC_MTD_NAND_HOST_BE */

#else	/* CONFIG_SNSC_MTD_NAND_BYTE_SWAP */

#define READW(addr)	readw(addr)
#define READL(addr)	readl(addr)
#define WRITEW(n, addr)	writew(n, addr)
#define WRITEL(n, addr)	writel(n, addr)

#endif	/* CONFIG_SNSC_MTD_NAND_BYTE_SWAP */


#if CONFIG_SNSC_MTD_NAND_B1

#undef CONFIG_SNSC_MTD_NAND_B1
#undef CONFIG_SNSC_MTD_NAND_GEOMETRY

#else	/* CONFIG_SNSC_MTD_NAND_B1 */
#ifdef CONFIG_SNSC_MTD_NAND_B2

#error CONFIG_SNSC_MTD_NAND_B2 is not supported yet.

#if CONFIG_SNSC_MTD_NAND_HOST_BE

#define WORD_TO_BYTE0(n)	((n) >> 8)
#define WORD_TO_BYTE1(n)	((n) & 0xff)
#define BYTES_TO_WORD(b0, b1) 	(((u16)(b0) << 8) | (u16)(b1))

#else	/* CONFIG_SNSC_MTD_NAND_HOST_BE */
#if CONFIG_SNSC_MTD_NAND_HOST_LE

#define WORD_TO_BYTE0(n)	((n) & 0xff)
#define WORD_TO_BYTE1(n)	((n) >> 8)
#define BYTES_TO_WORD(b0, b1)	(((u16)(b1) << 8) | (u16)(b0))

#else	/* CONFIG_SNSC_MTD_NAND_HOST_LE */

#error Endian for NAND flash data is not defined.

#endif	/* CONFIG_SNSC_MTD_NAND_HOST_LE */
#endif	/* CONFIG_SNSC_MTD_NAND_HOST_BE */

#define NAND_READB(addr)	WORD_TO_BYTE0(READW(addr))
#define NAND_READB_2(addr, b0, b1) \
		do { \
			u16	_t_;             \
			_t_ = READW(addr);       \
			b0 = WORD_TO_BYTE0(_t_); \
			b1 = WORD_TO_BYTE1(_t_); \
		} while (0)
#ifdef CONFIG_SNSC_MTD_NAND_BYTE_OUTPUT
#define NAND_READB_2_BYTE(addr, b0, b1) \
		do { \
			b0 = NAND_READB(addr);   \
			b1 = NAND_READB(addr);   \
		} while (0)
#else	/* CONFIG_SNSC_MTD_NAND_BYTE_OUTPUT */
#define NAND_READB_2_BYTE(addr, b0, b1) \
		NAND_READB_2(addr, b0, b1)
#endif	/* CONFIG_SNSC_MTD_NAND_BYTE_OUTPUT */
#define NAND_READW(addr)	READW(addr)
#define NAND_WRITEW(addr, w)	WRITEW(w, addr)

#else	/* CONFIG_SNSC_MTD_NAND_B2 */
#ifdef CONFIG_SNSC_MTD_NAND_B4

#if CONFIG_SNSC_MTD_NAND_HOST_BE

#define LONG_TO_BYTE0(n)	( (n) >> 24)
#define LONG_TO_BYTE1(n)	(((n) >> 16) & 0xff)
#define LONG_TO_BYTE2(n)	(((n) >>  8) & 0xff)
#define LONG_TO_BYTE3(n)	( (n)        & 0xff)
#define BYTES_TO_LONG(b0, b1, b2, b3) \
		(((u32)(b0) << 24) | ((u32)(b1) << 16) | \
		 ((u32)(b2) <<  8) |  (u32)(b3))

#else	/* CONFIG_SNSC_MTD_NAND_HOST_BE */
#if CONFIG_SNSC_MTD_NAND_HOST_LE

#define LONG_TO_BYTE0(n)	( (n)        & 0xff)
#define LONG_TO_BYTE1(n)	(((n) >>  8) & 0xff)
#define LONG_TO_BYTE2(n)	(((n) >> 16) & 0xff)
#define LONG_TO_BYTE3(n)	( (n) >> 24)
#define BYTES_TO_LONG(b0, b1, b2, b3) \
		(((u32)(b3) << 24) | ((u32)(b2) << 16) | \
		 ((u32)(b1) <<  8) |  (u32)(b0))

#else	/* CONFIG_SNSC_MTD_NAND_HOST_LE */

#error Endian for NAND flash data is not defined.

#endif	/* CONFIG_SNSC_MTD_NAND_HOST_LE */
#endif	/* CONFIG_SNSC_MTD_NAND_HOST_BE */

#define NAND_READB(addr)	LONG_TO_BYTE0(READL(addr))
#define NAND_READB_2(addr, b0, b1) \
		do { \
			u32	_t_;             \
			_t_ = READL(addr);       \
			b0 = LONG_TO_BYTE0(_t_); \
			b1 = LONG_TO_BYTE1(_t_); \
		} while (0)
#define NAND_READB_4(addr, b0, b1, b2, b3) \
		do { \
			u32	_t_;             \
			_t_ = READL(addr);       \
			b0 = LONG_TO_BYTE0(_t_); \
			b1 = LONG_TO_BYTE1(_t_); \
			b2 = LONG_TO_BYTE2(_t_); \
			b3 = LONG_TO_BYTE3(_t_); \
		} while (0)
#ifdef CONFIG_SNSC_MTD_NAND_BYTE_OUTPUT
#define NAND_READB_2_BYTE(addr, b0, b1) \
		do { \
			b0 = NAND_READB(addr);   \
			b1 = NAND_READB(addr);   \
		} while (0)
#define NAND_READB_4_BYTE(addr, b0, b1, b2, b3) \
		do { \
			b0 = NAND_READB(addr);   \
			b1 = NAND_READB(addr);   \
			b2 = NAND_READB(addr);   \
			b3 = NAND_READB(addr);   \
		} while (0)
#else	/* CONFIG_SNSC_MTD_NAND_BYTE_OUTPUT */
#define NAND_READB_2_BYTE(addr, b0, b1) \
		NAND_READB_2(addr, b0, b1)
#define NAND_READB_4_BYTE(addr, b0, b1, b2, b3) \
		NAND_READB_4(addr, b0, b1, b2, b3)
#endif	/* CONFIG_SNSC_MTD_NAND_BYTE_OUTPUT */
#define NAND_READL(addr)	READL(addr)
#define NAND_WRITEL(addr, l)	WRITEL(l, addr)

#else	/* CONFIG_SNSC_MTD_NAND_B4 */

#error NAND flash bus width is not defined.

#endif	/* CONFIG_SNSC_MTD_NAND_B4 */
#endif	/* CONFIG_SNSC_MTD_NAND_B2 */
#endif	/* CONFIG_SNSC_MTD_NAND_B1 */
