/*
 * Copyright 2003,2004 Sony Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef ECC_H_
#define ECC_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ======================================================================= *
 * constant                                                                *
 * ======================================================================= */

#define ECC_DATASIZE_MAX	(0x10000 / 8)	/* 2^16 / 8 */

/* return code */
#define ECCCORRECTED	1023

/* error code */
#define EECCINVAL	22

#define EECCCORRECT	1101

/* ======================================================================= *
 * ECC library interface                                                   *
 * ======================================================================= */

int ecc_calc(const void *data, unsigned int size, unsigned long *ecc);
int ecc_calc_simple(const void *data, unsigned int size, unsigned long *ecc);
int ecc_correct(void *data, unsigned int size,
		unsigned long ecc1, unsigned long ecc2, long *pos);

int ecc_correct_xor(void *data, unsigned int size,
		    unsigned long ecc_pair_xor, long *pos);

int ecc_calc_interleaved(const void *data, unsigned int size,
			 unsigned long *ecc_even, unsigned long *ecc_odd);
int ecc_calc_simple_interleaved(const void *data, unsigned int size,
				unsigned long *ecc_even,
				unsigned long *ecc_odd);
int ecc_correct_interleaved(void *data, unsigned int size,
			    unsigned long ecc1_even, unsigned long ecc1_odd,
			    unsigned long ecc2_even, unsigned long ecc2_odd,
			    long *pos1, long *pos2);
int ecc_correct_xor_interleaved(void *data, unsigned int size, 
				unsigned long ecc_pair_even,
				unsigned long ecc_pair_odd,
				long *pos1, long *pos2);

#ifdef __cplusplus
}
#endif

#endif /* !ECC_H_ */
