/*
 * Copyright 2003 Sony Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SBL_H_
#define SBL_H_

#ifdef __cplusplus
extern "C" {
#endif

/* ======================================================================= *
 * constant                                                                *
 * ======================================================================= */

/* error code */
#define ESBLIO		5
#define ESBLINVAL	22
#define ESBLNOSPC	28

#define ESBLBADBLK	1001
#define ESBLERASE	1002
#define ESBLPROGRAM	1003
#define ESBLVERIFY	1004
#define ESBLECC		1005
#define ESBLNOENT	1006
#define ESBLINFO	1007

/* mode */
#define SBL_M_ACCESS_MODE_MASK	0x0000000f
#define SBL_M_ACCESS_MODE(x)	(x)
#define SBL_M_RAW		SBL_M_ACCESS_MODE(1)
#define SBL_M_SPARE		SBL_M_ACCESS_MODE(2)

#define SBL_M_DATA		(1 << 4)
#define SBL_M_EXTRA		(1 << 5)
#define SBL_M_VERIFY		(1 << 6)
#define SBL_M_ECC		(1 << 7)
#define SBL_M_SBLINFO		(1 << 8)
#define SBL_M_MOVEBACK		(1 << 9)

/* flags */
#define SBL_F_VERIFY_DATA	(1 << 16)
#define SBL_F_VERIFY_EXTRA	(1 << 17)

/* sbl version (sbl_dev.sbl_version) */
#define SBL_CURRENT_VERSION	1000

/* special logical block number */
#define SBL_SLBN_INVALID	0xffffffff

#define SBL_SLBN_BLOCK_MAX	0xfffffff7
#define SBL_SLBN_NOTCACHED	0xfffffff8
#define SBL_SLBN_BADBLK		0xfffffff9

/* others */
#define SBL_INFO_SIZE		8
#define SBL_MAGIC		0x4724

/* ======================================================================= *
 * type and structure                                                      *
 * ======================================================================= */

typedef struct sbl_dev_t sbl_dev_t;

/* device access functions (user defined) */

typedef int (*sbl_dev_is_bad_t)(sbl_dev_t *dev, unsigned long block,
				int mode);
typedef int (*sbl_dev_erase_t)(sbl_dev_t *dev, unsigned long block, int mode);
typedef int (*sbl_dev_program_t)(sbl_dev_t *dev, unsigned long block,
				 int page, int mode,
				 const void *data, const void *extra);
typedef int (*sbl_dev_read_t)(sbl_dev_t *dev, unsigned long block, int page,
			      int mode, void *data, void *extra);

/* device structure (filled by user) */

struct sbl_dev_t {
	int                sbl_version;
	int                flags;
	unsigned long      normal_blocks;
	unsigned long      spare_blocks;
	int                pages_per_block;
	unsigned long      data_area_size;
	unsigned long      extra_area_size;
	void              *page_buf;
	void              *copy_page_buf;
	void              *sbl_info_cache;
	unsigned long      spare_use_hint;
	void              *priv;
	sbl_dev_is_bad_t   is_bad;
	sbl_dev_erase_t    erase;
	sbl_dev_program_t  program;
	sbl_dev_read_t     read;
};

/* scaned info (filled by this library) */

typedef struct sbl_scan_t {
	unsigned long  used_normal_blocks;
	unsigned long  used_spare_blocks;
} sbl_scan_t;

/* ======================================================================= *
 * SBL library interface (provided by this library)                        *
 * ======================================================================= */

int sbl_dev_init(sbl_dev_t *dev);
int sbl_is_bad(sbl_dev_t *dev, unsigned long block, int mode);
int sbl_erase(sbl_dev_t *dev, unsigned long block, int mode);
int sbl_program(sbl_dev_t *dev, unsigned long block, int page, int mode,
		const void *data);
int sbl_read(sbl_dev_t *dev, unsigned long block, int page, int mode,
	     void *data);
int sbl_scan(sbl_dev_t *dev, sbl_scan_t *info);

#ifdef __cplusplus
}
#endif

#endif /* !SBL_H_ */
