/*
 *  ioa_3_eth.c
 *
 *  SMC91111 setup code for IOA-3
 *
 *  Copyright 2003 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <asm/arch/platform.h>
#include <asm/arch/gpio.h>
#include <asm/arch/mpu110_series.h>
#include <asm/arch/ioa_3_func.h>
#include "ioa_3_eth.h"

/*
 * functions for IOA-3 ether device
 */

/* 
 * returns interrupt number
 */
int ioa_3_eth_intnum(void)
{
	return IOA_3_INT_NUM;
}

/* 
 * returns base address
 */
int ioa_3_eth_baseaddr(void)
{
	return IOA_3_LAN_BASE + LAN_IO_OFFSET;
}

int ioa_3_eth_check_disable(void)
{
	if (!ioa_3_probe()) {
		printk("IOA-3 Ethernet is not found\n");
		return -ENODEV;
	}

	/* check DIP S/W No.8 */
	if (ioa_3_getdipsw() & IOA_3_DSW_ROMETHDIS) {
		printk("IOA-3 Ethernet is disabled by DIPSW\n");
		return -ENODEV;
	}
	return 0;
}

int ioa_3_eth_setup(void)
{
	return ioa_3_eim_setup();
}

int ioa_3_eth_request_irq(unsigned int irq, 
			  void (*handler)(int, void *, struct pt_regs *),
			  unsigned long irq_flags, const char * devname, 
			  void *dev_id)
{
	return ioa_3_request_irq(IOA_3_ETHER_INT, handler, dev_id);
}

int ioa_3_eth_free_irq(unsigned int irq, void *dev_id)
{
	return ioa_3_free_irq(IOA_3_ETHER_INT);
}

int ioa_3_eth_clear_intr(void *dev_id)
{
	/* interrupt clear is done by ioa_3_int_handler() */
	return 0;
}


/*
 * function table for IOA-3 ether device
 */
mpu110_series_eth_ops_t ioa_3_eth = {
	intnum        : ioa_3_eth_intnum,
	baseaddr      : ioa_3_eth_baseaddr,
	check_disable : ioa_3_eth_check_disable,
	setup         : ioa_3_eth_setup,
	request_irq   : ioa_3_eth_request_irq,
	free_irq      : ioa_3_eth_free_irq,
	clear_intr    : ioa_3_eth_clear_intr,
};
