/*
 *  mpu110_series_eth.h
 *
 *  SMC91111 setup code for MPU-110 Series
 *
 *  Copyright 2003 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _MPU110_SERIES_ETH_H_
#define _MPU110_SERIES_ETH_H_

#include <linux/netdevice.h>
#include <linux/etherdevice.h>

#ifdef CONFIG_DRAGONBALL_SNSC_MPU110
# include "mpu110_eth.h"
#elif CONFIG_DRAGONBALL_MPU110_SAMPLE
# include "mpu110_sample_eth.h"
#else
# error "Unknown MPU-110 series board."
#endif

typedef struct mpu110_series_eth_ops_t {
	int (*intnum)(void);
	int (*baseaddr)(void);
	int (*check_disable)(void);
	int (*setup)(void);
	int (*cleanup)(void);
	int (*request_irq)(unsigned int irq, 
			   void (*handler)(int, void *, struct pt_regs *),
			   unsigned long irq_flags, const char * devname, 
			   void *dev_id);
	int (*free_irq)(unsigned int irq, void *dev_id);
	int (*clear_intr)(void *dev_id);
} mpu110_series_eth_ops_t;

extern mpu110_series_eth_ops_t *mpu110_series_eths[MPU110_SERIES_ETH_NUM];

static inline mpu110_series_eth_ops_t *mpu110_series_eth(struct net_device *dev)
{
	int i;
	for (i = 0; i < MPU110_SERIES_ETH_NUM; i++) {
		if (dev->base_addr == mpu110_series_eths[i]->baseaddr())
			return mpu110_series_eths[i];
	}
}

#endif /* _MPU110_SERIES_ETH_H_ */
