/*
 *  Copyright 2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/snsc_nblargs.h>
#include <linux/sdm_nblargs.h>

/* SDM info passed by NBLArgs */
struct sdm_nblargs_t *sdm_nblargs_info = NULL;

static int __initialized = 0;

/*
 * Get SDM information by NBLArgs
 */
void
sdm_nblargs_get_info(void)
{
	struct nblargs_entry na;
	int    ret;

	if (__initialized)
		return;

	__initialized = 1;

	ret = nblargs_get_key(SDM_NBLARGS_KEY, &na);
	if (ret < 0) {
		printk(KERN_INFO SDM_NBLARGS_KEY " is not found\n");
		return;
	}
	if (na.size < sizeof(struct sdm_nblargs_t)) {
		printk(KERN_WARNING
		       "Invalid length of SDM info(%d) passed by NBLArgs\n",
		       na.size);
		return;
	}

	sdm_nblargs_info = (struct sdm_nblargs_t *)nbl_to_va(na.addr);
}

static int __init sdm_nblargs_init(void)
{
	sdm_nblargs_get_info();
	return 0;
}

module_init(sdm_nblargs_init);

EXPORT_SYMBOL(sdm_nblargs_info);
EXPORT_SYMBOL(sdm_nblargs_get_info);
