/* With non GPL files, use following license */
/*
 * Copyright 2004 Sony Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Otherwise with GPL files, use following license */
/*
 * Copyright 2004 Sony Corporation.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sdmlog_conf.h"

#ifdef CONFIG_SDMLOG_RECORD_SIZE
#define SDMLOG_RECORD_SIZE	CONFIG_SDMLOG_RECORD_SIZE
#else
#define SDMLOG_RECORD_SIZE	4096
#endif

/*
 * NOTE:  sdmlog_conf.h can define the following macro functions
 *    SDMLOG_LOCK(), SDMLOG_UNLOCK(),
 *    SDMLOG_INTR_LOCK(), SDMLOG_INTR_UNLOCK(),
 *    SDMLOG_GET_TIME(), SDMLOG_PRINTF
 * sdmlog_conf.h must include stdint.h, string.h, sdmlog.h, and sdmwl.h.
 */

#define DPR_DEBUG
#ifndef DPR_LEVEL
#define DPR_LEVEL DPL_NOTICE
#endif

#ifdef DPR_DEBUG
#undef DEBUG
#define DEBUG
#endif
#undef DEBUG_LEVEL
#define DEBUG_LEVEL DPR_LEVEL

/* ======================================================================= *
 * constant (internal)                                                     *
 * ======================================================================= */

#define SDMLOG_SDMNAME	"sdmlog"

#define SDMLOG_HEADER_SIZE	64
#define SDMLOG_COLOR_OFFSET	4

#define SDMLOG_MAGIC	0x535c
#define SDMLOG_VERSION	0x0001

/* ======================================================================= *
 * debug print function from dpr package                                   *
 * ======================================================================= */

#define DPL_EMERG	0
#define DPL_ALERT	1
#define DPL_CRIT	2
#define DPL_ERR		3
#define DPL_WARNING	4
#define DPL_NOTICE	5
#define DPL_INFO	6
#define DPL_DEBUG	7

#define DPM_PURPOSE	0x0000000f
#define DPM_ERR		0x00000001
#define DPM_TRACE	0x00000002
#define DPM_LAYER	0x000000f0
#define DPM_APPLICATION	0x00000010
#define DPM_MIDDLEWARE	0x00000020
#define DPM_DRIVER	0x00000040
#define DPM_KERNEL	0x00000080
#define DPM_USERDEF	0xffff0000

#ifdef DEBUG

#ifndef DEBUG_LEVEL
#define DEBUG_LEVEL DPL_DEBUG
#endif
#ifndef DEBUG_MASK
#define DEBUG_MASK  DPM_PURPOSE
#endif

#ifndef DPR
#include <stdio.h>
#define DPR(fmt, args...)	fprintf(stderr, fmt , ## args)
#endif

#define DPRL(level, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && DPR(fmt , ## args))

#define DPRM(mask, fmt, args...) \
	(void)(((mask) & DEBUG_MASK) && DPR(fmt , ## args))
#define DPRLM(level, mask, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && ((mask) & DEBUG_MASK) && \
		DPR(fmt , ## args))
#define DPRC(cond, fmt, args...) \
	(void)((cond) && DPR(fmt , ## args))

#define DPRF(fmt, args...)	DPR("%s: " fmt, __FUNCTION__ , ## args)
#define DPRFL(level, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && DPRF(fmt , ## args))
#define DPRFM(mask, fmt, args...) \
	(void)(((mask) & DEBUG_MASK) && DPRF(fmt , ## args))
#define DPRFLM(level, mask, fmt, args...) \
	(void)((level) <= DEBUG_LEVEL && ((mask) & DEBUG_MASK) && \
		DPRF(fmt , ## args))
#define DPRFC(cond, fmt, args...) \
	(void)((cond) && DPRF(fmt , ## args))

#define	DPRFS	DPRFLM(DPL_DEBUG, DPM_TRACE, "start\n")
#define	DPRFE	DPRFLM(DPL_DEBUG, DPM_TRACE, "end\n")
#define DPRFR	DPRFLM(DPL_DEBUG, DPM_TRACE, "return\n")

#else  /* DEBUG */

#ifndef DEBUG_LEVEL
#define DEBUG_LEVEL -1
#endif
#ifndef DEBUG_MASK
#define DEBUG_MASK  0
#endif

#ifndef DPR
#define DPR(fmt, args...)
#endif

#define DPRL(level, fmt, args...)
#define DPRM(mask, fmt, args...)
#define DPRLM(level, mask, fmt, args...)
#define DPRC(cond, fmt, args...)
#define DPRF(fmt, args...)
#define DPRFL(level, fmt, args...)
#define DPRFM(mask, fmt, args...)
#define DPRFLM(level, mask, fmt, args...)
#define DPRFC(cond, fmt, args...)

#define	DPRFS
#define	DPRFE
#define DPRFR

#endif /* DEBUG */

/* ======================================================================= *
 * type and structure                                                      *
 * ======================================================================= */

struct sdmlog_header {
	uint16_t	 magic;
	uint16_t	 version;
	uint16_t	 color;
	uint16_t	 reserved;
	uint32_t	 crc;
	uint32_t	 time;
};

struct sdmlog_datetime {
	int	sec;	/* 0 - 59 */
	int	min;	/* 0 - 59 */
	int	hour;	/* 0 - 23 */
	int	mday;	/* 1 - 31 */
	int	mon;	/* 0 - 11 */
	int	year;	/* year - 1900 */
};

/* ======================================================================= *
 * prototype declaration (internal)                                        *
 * ======================================================================= */

/* CRC32 */
static void sdmlog_make_crc32_table(void);
static uint32_t sdmlog_crc32(uint32_t crc, unsigned char *buf,
			     unsigned int len);

/* sdmwl callback */
static int sdmlog_chunk_isvalid(void *buf);
static void sdmlog_chunk_setcrc(void *buf);

/* record_buf */
static void sdmlog_write_header_to_buf_if_new_record(void);
static void sdmlog_write_data_to_buf(const char *data, size_t len);
static int sdmlog_write_buf_to_storage(void);

/* show/read */
static void sdmlog_show_record(const char *chunk_buf);
static int sdmlog_read_record(const char *chunk_buf, char *buf,
			      unsigned int count, int *write_pos, int *pos);

/* time */
static void sdmlog_sec_to_datetime(uint32_t sec,
				   struct sdmlog_datetime *datetime);

/* ======================================================================= *
 * global variable                                                         *
 * ======================================================================= */

static char sdmlog_record_buf[SDMLOG_RECORD_SIZE];
static int sdmlog_record_pos = 0;
static char sdmlog_chunk_buf[SDMLOG_RECORD_SIZE + 1] SDMWL_CACHELINE_ALIGNED;
static char sdmlog_read_buf[SDMLOG_RECORD_SIZE + 32];
static char sdmlog_write_buf[SDMLOG_RECORD_SIZE];

static int sdmlog_is_opened = 0;

static const struct sdmwl_chunk_info sdmlog_chunk_info = {
	.chunk_size	= SDMLOG_RECORD_SIZE,
	.color_offset	= SDMLOG_COLOR_OFFSET,
	.isvalid	= sdmlog_chunk_isvalid,
	.setcrc		= sdmlog_chunk_setcrc,
};
static struct sdmwl_desc sdmlog_sdmwl_desc;

/* ======================================================================= *
 * inline/macro function (internal)                                        *
 * ======================================================================= */

#ifndef SDMLOG_LOCK
#define SDMLOG_LOCK()
#endif
#ifndef SDMLOG_UNLOCK
#define SDMLOG_UNLOCK()
#endif
#ifndef SDMLOG_INTR_LOCK
#define SDMLOG_INTR_LOCK()
#endif
#ifndef SDMLOG_INTR_UNLOCK
#define SDMLOG_INTR_UNLOCK()
#endif
#ifndef SDMLOG_GET_TIME
#define SDMLOG_GET_TIME()	(0xffffffff)
#endif
#ifndef SDMLOG_PRINTF
#define SDMLOG_PRINTF	printf
#endif
#ifndef SDMLOG_READCPY
#define SDMLOG_READCPY	memcpy
#endif

/* ======================================================================= *
 * sdmlog interface                                                        *
 * ======================================================================= */

int
sdmlog_open(void)
{
	int res;
	DPRFS;

	SDMLOG_LOCK();
	if (sdmlog_is_opened) {
		SDMLOG_UNLOCK();
		return 0;
	}

	res = sdmwl_open(SDMLOG_SDMNAME, &sdmlog_chunk_info,
			 sdmlog_chunk_buf, &sdmlog_sdmwl_desc);
	if (res != SDMWL_ERR_OK) {
		DPRFL(DPL_WARNING, "sdmwl_open() failed (%d)\n", res);
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}
	sdmlog_is_opened = 1;
	SDMLOG_UNLOCK();

	DPRFE;
	return 0;
}

int
sdmlog_close(void)
{
	int res;
	DPRFS;

	SDMLOG_LOCK();
	if (!sdmlog_is_opened) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}

	res = sdmlog_write_buf_to_storage();
	if (res < 0) {
		SDMLOG_UNLOCK();
		DPRFR;
		return res;
	}

	res = sdmwl_close(&sdmlog_sdmwl_desc);
	if (res != SDMWL_ERR_OK) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}

	sdmlog_is_opened = 0;
	SDMLOG_UNLOCK();

	DPRFE;
	return 0;
}

int
sdmlog_add(const char *data)
{
	int res, res2;
	DPRFS;

	res = sdmlog_add_async(data);
	if (res < 0 && res != -ESDMLOG_NOSPC) {
		DPRFR;
		return res;
	}
	res2 = sdmlog_sync();
	if (res2 < 0) {
		DPRFR;
		return res;
	}

	DPRFE;
	return res;
}

int
sdmlog_add_async(const char *data)
{
	int res;
	size_t write_len;
	DPRFS;

	SDMLOG_LOCK();
	if (!sdmlog_is_opened) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}
	if (data == NULL) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_INVAL;
	}

	res = 0;
	write_len = strlen(data) + 1;

	SDMLOG_INTR_LOCK();
	sdmlog_write_header_to_buf_if_new_record();
	if (write_len > SDMLOG_RECORD_SIZE - sdmlog_record_pos &&
	    sdmlog_record_pos > SDMLOG_HEADER_SIZE) {
		SDMLOG_INTR_UNLOCK();

		/* flush buf */
		res = sdmlog_write_buf_to_storage();
		if (res < 0) {
			SDMLOG_UNLOCK();
			DPRFR;
			return res;
		}

		SDMLOG_INTR_LOCK();
		sdmlog_write_header_to_buf_if_new_record();
	}
	if (write_len > SDMLOG_RECORD_SIZE - sdmlog_record_pos) {
		/* NOTE: do not flush buf twice */
		write_len = SDMLOG_RECORD_SIZE - sdmlog_record_pos;
		res = -ESDMLOG_NOSPC;
	}
	sdmlog_write_data_to_buf(data, write_len);
	SDMLOG_INTR_UNLOCK();
	SDMLOG_UNLOCK();

	DPRFE;
	return res;
}

int
sdmlog_add_intr(const char *data)
{
	int res;
	size_t write_len;
	DPRFS;

	if (!sdmlog_is_opened) {
		DPRFR;
		return -ESDMLOG_NODEV;
	}
	if (data == NULL) {
		DPRFR;
		return -ESDMLOG_INVAL;
	}

	res = 0;
	write_len = strlen(data) + 1;

	SDMLOG_INTR_LOCK();
	sdmlog_write_header_to_buf_if_new_record();
	if (write_len > SDMLOG_RECORD_SIZE - sdmlog_record_pos) {
		write_len = SDMLOG_RECORD_SIZE - sdmlog_record_pos;
		res = -ESDMLOG_NOSPC;
	}
	sdmlog_write_data_to_buf(data, write_len);
	SDMLOG_INTR_UNLOCK();

	DPRFE;
	return res;
}

int
sdmlog_sync(void)
{
	int res;
	DPRFS;

	SDMLOG_LOCK();
	if (!sdmlog_is_opened) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}

	res = sdmlog_write_buf_to_storage();
	if (res < 0) {
		SDMLOG_UNLOCK();
		DPRFR;
		return res;
	}
	SDMLOG_UNLOCK();

	DPRFE;
	return 0;
}

int
sdmlog_clear(void)
{
	int res;
	int numchunk;
	int i;
	int err_occurred;
	DPRFS;

	SDMLOG_LOCK();
	if (!sdmlog_is_opened) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}

	res = sdmwl_query_numchunk(&sdmlog_sdmwl_desc);
	if (res < 0) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_IO;
	}
	numchunk = res;

	/* write invalid data to all chunks */
	err_occurred = 0;
	memset(sdmlog_chunk_buf, 0xff, SDMLOG_RECORD_SIZE);
	for (i = 0; i < numchunk; i++) {
		res = sdmwl_write_chunk(&sdmlog_sdmwl_desc, sdmlog_chunk_buf);
		if (res != SDMWL_ERR_OK) {
			err_occurred = 1;
		}
	}
	sdmlog_record_pos = 0;
	SDMLOG_UNLOCK();

	if (err_occurred) {
		DPRFR;
		return -ESDMLOG_IO;
	}

	DPRFE;
	return 0;
}

int
sdmlog_show(void)
{
	int res;
	int numchunk;
	int i;
	DPRFS;

	SDMLOG_LOCK();
	if (!sdmlog_is_opened) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}

	res = sdmwl_query_numchunk(&sdmlog_sdmwl_desc);
	if (res < 0) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_IO;
	}
	numchunk = res;

	for (i = 0; i < numchunk; i++) {
		res = sdmwl_read_chunk(&sdmlog_sdmwl_desc, numchunk - i - 1,
				       sdmlog_chunk_buf);
		if (res == SDMWL_ERR_OK) {
			sdmlog_show_record(sdmlog_chunk_buf);
		} else if (res != SDMWL_ERR_EINVALCHUNK) {
			SDMLOG_UNLOCK();
			DPRFR;
			return -ESDMLOG_IO;
		}
	}
	SDMLOG_UNLOCK();

	DPRFE;
	return 0;
}

int
sdmlog_read(char *buf, unsigned int count, int *pos)
{
	int res;
	int numchunk;
	int i;
	int write_pos;
	int nwritten;
	DPRFS;

	if (count == 0) {
		DPRFR;
		return 0;
	}

	SDMLOG_LOCK();
	if (!sdmlog_is_opened) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_NODEV;
	}

	res = sdmwl_query_numchunk(&sdmlog_sdmwl_desc);
	if (res < 0) {
		SDMLOG_UNLOCK();
		DPRFR;
		return -ESDMLOG_IO;
	}
	numchunk = res;

	nwritten = 0;
	write_pos = 0;
	for (i = 0; i < numchunk; i++) {
		res = sdmwl_read_chunk(&sdmlog_sdmwl_desc, numchunk - i - 1,
				       sdmlog_chunk_buf);
		if (res == SDMWL_ERR_OK) {
			res = sdmlog_read_record(sdmlog_chunk_buf,
						 buf + nwritten,
						 count, &write_pos, pos);
			nwritten += res;
			count -= res;
			if (count == 0) {
				break;
			}
		} else if (res != SDMWL_ERR_EINVALCHUNK) {
			SDMLOG_UNLOCK();
			DPRFR;
			return -ESDMLOG_IO;
		}
	}
	SDMLOG_UNLOCK();

	DPRFE;
	return nwritten;
}

/* ======================================================================= *
 * internal function                                                       *
 * ======================================================================= */

/* ------------------------ *
 * CRC32                    *
 * ------------------------ */

static uint32_t sdmlog_crc32_table[256];
static int sdmlog_crc32_table_computed = 0;

static void
sdmlog_make_crc32_table(void)
{
	uint32_t c;
	int i, j;

	for (i = 0; i < 256; i++) {
		c = i;
		for (j = 0; j < 8; j++) {
			if (c & 1) {
				c = (0xedb88320UL ^ (c >> 1));
			} else {
				c = (c >> 1);
			}
		}
		sdmlog_crc32_table[i] = c;
	}
	sdmlog_crc32_table_computed = 1;
}

static uint32_t
sdmlog_crc32(uint32_t crc, unsigned char *buf, unsigned int len)
{
	unsigned int i;

	if (!sdmlog_crc32_table_computed) {
		sdmlog_make_crc32_table();
	}

	crc ^= 0xffffffff;
	for (i = 0; i < len; i++) {
		crc = (sdmlog_crc32_table[(crc ^ buf[i]) & 0xff] ^ (crc >> 8));
	}
	return (crc ^ 0xffffffff);
}

/* ------------------------ *
 * sdmwl callback           *
 * ------------------------ */

static int
sdmlog_chunk_isvalid(void *buf)
{
	struct sdmlog_header *header;
	uint32_t crc;
	uint32_t crc_saved;
	DPRFS;

	header = (struct sdmlog_header *)buf;

	/* check magic */
	if (header->magic != SDMLOG_MAGIC) {
		DPRFR;
		return 0;
	}

	/* check CRC */
	crc_saved = header->crc;
	header->crc = 0;
	crc = sdmlog_crc32(0, buf, SDMLOG_RECORD_SIZE);
	header->crc = crc_saved;
	if (crc != crc_saved) {
		DPRFR;
		return 0;
	}

	DPRFE;
	return 1;
}

static void
sdmlog_chunk_setcrc(void *buf)
{
	struct sdmlog_header *header;
	uint32_t crc;
	DPRFS;

	header = (struct sdmlog_header *)buf;

	header->crc = 0;
	crc = sdmlog_crc32(0, buf, SDMLOG_RECORD_SIZE);
	header->crc = crc;

	DPRFE;
	return;
}

/* ------------------------ *
 * record_buf               *
 * ------------------------ */

/* should be called with intr_lock */
static void
sdmlog_write_header_to_buf_if_new_record(void)
{
	struct sdmlog_header *header;
	DPRFS;

	if (sdmlog_record_pos == 0) {
		memset(sdmlog_record_buf, 0xff, SDMLOG_HEADER_SIZE);
		header = (struct sdmlog_header *)sdmlog_record_buf;
		header->magic = SDMLOG_MAGIC;
		header->version = SDMLOG_VERSION;
		header->time = SDMLOG_GET_TIME();

		sdmlog_record_pos = SDMLOG_HEADER_SIZE;
	}

	DPRFE;
	return;
}

/* should be called with intr_lock */
static void
sdmlog_write_data_to_buf(const char *data, size_t len)
{
	DPRFS;

	memcpy(sdmlog_record_buf + sdmlog_record_pos, data, len);
	sdmlog_record_pos += len;

	DPRFE;
	return;
}

static int
sdmlog_write_buf_to_storage(void)
{
	static int writing = 0;
	int pos;
	int res;
	DPRFS;

	if (writing != 0) {
		/* this function is not reentrant and protected
		   by SDMLOG_LOCK(), this will never happen */
		return -ESDMLOG_IO;
	}

	SDMLOG_INTR_LOCK();

	pos = sdmlog_record_pos;

	if (pos > SDMLOG_HEADER_SIZE) {
		/* data exists in buf */
		writing = 1;
		memcpy(sdmlog_write_buf, sdmlog_record_buf, pos);
		memset(sdmlog_write_buf + pos, 0, SDMLOG_RECORD_SIZE - pos);
		sdmlog_record_pos = 0;

		SDMLOG_INTR_UNLOCK();

		res = sdmwl_write_chunk(&sdmlog_sdmwl_desc, sdmlog_write_buf);
		writing = 0;
		if (res != SDMWL_ERR_OK) {
			DPRFL(DPL_WARNING,
			      "sdmwl_write_chunk() failed (%d)\n", res);
			return -ESDMLOG_IO;
		}
	} else {
		SDMLOG_INTR_UNLOCK();
	}

	DPRFE;
	return 0;
}

/* ------------------------ *
 * show/read                *
 * ------------------------ */

static void
sdmlog_show_record(const char *chunk_buf)
{
	struct sdmlog_header *header;
	struct sdmlog_datetime datetime;
	int chunk_pos;
	DPRFS;

	header = (struct sdmlog_header *)chunk_buf;

	chunk_pos = SDMLOG_HEADER_SIZE;
	while (chunk_pos < SDMLOG_RECORD_SIZE) {
		if (*(chunk_buf + chunk_pos) == 0) {
			chunk_pos++;
			continue;
		}
		if (header->time == 0xffffffff) {
			SDMLOG_PRINTF("[sdmlog] @@ NBL @@           %s\n",
				      chunk_buf + chunk_pos);
		} else {
			sdmlog_sec_to_datetime(header->time, &datetime);
			SDMLOG_PRINTF("[sdmlog] %04d/%02d/%02d %02d:%02d:%02d "
				      "%s\n",
				      datetime.year + 1900,
				      datetime.mon + 1,
				      datetime.mday,
				      datetime.hour,
				      datetime.min,
				      datetime.sec,
				      chunk_buf + chunk_pos);
		}
		chunk_pos += strlen(chunk_buf + chunk_pos) + 1;
	}

	DPRFE;
	return;
}

static int
sdmlog_read_record(const char *chunk_buf,
		   char *buf, unsigned int count, int *write_pos, int *pos)
{
	struct sdmlog_header *header;
	struct sdmlog_datetime datetime;
	int res;
	int chunk_pos;
	int nwritten;
	int tmp_len;
	int ncopy;
	DPRFS;

	header = (struct sdmlog_header *)chunk_buf;

	nwritten = 0;
	chunk_pos = SDMLOG_HEADER_SIZE;
	while (chunk_pos < SDMLOG_RECORD_SIZE) {
		if (*(chunk_buf + chunk_pos) == 0) {
			chunk_pos++;
			continue;
		}
		if (header->time == 0xffffffff) {
			res = sprintf(sdmlog_read_buf,
				      "[sdmlog] @@ NBL @@           %s\n",
				      chunk_buf + chunk_pos);
		} else {
			sdmlog_sec_to_datetime(header->time, &datetime);
			res = sprintf(sdmlog_read_buf,
				      "[sdmlog] %04d/%02d/%02d %02d:%02d:%02d "
				      "%s\n",
				      datetime.year + 1900,
				      datetime.mon + 1,
				      datetime.mday,
				      datetime.hour,
				      datetime.min,
				      datetime.sec,
				      chunk_buf + chunk_pos);
		}
		tmp_len = res;
		chunk_pos += strlen(chunk_buf + chunk_pos) + 1;
		if (*write_pos == *pos) {
			ncopy = tmp_len;
			if (ncopy > count) {
				ncopy = count;
			}
			SDMLOG_READCPY(buf + nwritten, sdmlog_read_buf, ncopy);
			*pos += ncopy;
			nwritten += ncopy;
			count -= ncopy;
			if (count == 0) {
				break;
			}
		} else if (*write_pos < *pos && *write_pos + tmp_len > *pos) {
			ncopy = *write_pos + tmp_len - *pos;
			if (ncopy > count) {
				ncopy = count;
			}
			SDMLOG_READCPY(buf + nwritten,
				       sdmlog_read_buf + (*pos - *write_pos),
				       ncopy);
			*pos += ncopy;
			nwritten += ncopy;
			count -= ncopy;
			if (count == 0) {
				break;
			}
		}
		*write_pos += tmp_len;
	}

	DPRFE;
	return nwritten;
}

/* ------------------------ *
 * time                     *
 * ------------------------ */

static inline int
sdmlog_is_leap_year(int year)
{
	return (year % 4 == 0 && (year % 100 != 0 || year % 400 == 0));
}

static inline int
sdmlog_days_in_year(int year)
{
	if (sdmlog_is_leap_year(year)) {
		return 366;
	} else {
		return 365;
	}
}

static inline int
sdmlog_days_in_month(int year, int month)
{
	/* month = 0, 1, ..., 11 */
	static const int days_in_month[] = {
		31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
	};

	if (month == 1 && sdmlog_is_leap_year(year)) {
		return 29;
	}

	return days_in_month[month];
};

static void
sdmlog_sec_to_datetime(uint32_t sec, struct sdmlog_datetime *datetime)
{
	uint32_t day;
	uint32_t day_sec;
	uint32_t tday;
	int year, month;
	DPRFS;

	day = sec / (24 * 60 * 60);
	day_sec = sec % (24 * 60 * 60);
	datetime->sec = day_sec % 60;
	datetime->min = (day_sec / 60) % 60;
	datetime->hour = day_sec / (60 * 60);

	for (year = 1970;; year++) {
		tday = sdmlog_days_in_year(year);
		if (day < tday) {
			break;
		}
		day -= tday;
	}
	datetime->year = year - 1900;

	for (month = 0;; month++) {
		tday = sdmlog_days_in_month(year, month);
		if (day < tday) {
			break;
		}
		day -= tday;
	}
	datetime->mon = month;
	datetime->mday = day + 1;

	DPRFE;
	return;
}
