/*
 * SDM wearleveling library
 */
/* With non GPL files, use following license */
/*
 * Copyright 2004,2005 Sony Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Otherwise with GPL files, use following license */
/*
 * Copyright 2004,2005 Sony Corporation.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sdmwl_conf.h"

#define SDMWL_COLOR_NONE      0
#define SDMWL_COLOR_RED       0x00ff
#define SDMWL_COLOR_BLUE      0xff00
#define SDMWL_COLOR_XORED     (SDMWL_COLOR_RED ^ SDMWL_COLOR_BLUE)
#define SDMWL_LATEST_NONE     0xffffffff


static struct sdmwl_dev_ops devops;

static int initialized = 0;

int
sdmwl_register_dev(const struct sdmwl_dev_ops *ops)
{
	if (ops == NULL)
		return SDMWL_ERR_EINVAL;

	memcpy(&devops, ops, sizeof(*ops));

	initialized = 1;
	return SDMWL_ERR_OK;
}

static unsigned int
chunknum_to_offset(struct sdmwl_desc *desc, unsigned int chnum)
{
	unsigned int chunk_size, sector_size, ret;

	chunk_size = desc->chunk_info.chunk_size;
	sector_size = desc->sector_size;

	if (chunk_size < sector_size) {
		ret = ((chnum / desc->chunks_per_sector)<< desc->sector_shift)
			+ (chnum % desc->chunks_per_sector) * chunk_size;
	} else {
		ret = (chnum * desc->sectors_per_chunk) << desc->sector_shift;
	}

	return ret;
}

static unsigned int
offset_to_chunknum(struct sdmwl_desc *desc, unsigned int offset)
{
	unsigned int chunk_size, sector_size, ret;

	chunk_size = desc->chunk_info.chunk_size;
	sector_size = desc->sector_size;

	if (chunk_size < sector_size) {
		ret = (offset >> desc->sector_shift) * desc->chunks_per_sector
			+ (offset & (sector_size - 1)) / chunk_size;
	} else {
		ret = (offset >> desc->sector_shift) / desc->sectors_per_chunk;
	}

	return ret;
}

static unsigned int
calc_num_chunks(struct sdmwl_desc *desc)
{
	unsigned int chunk_size, sector_size, ret;

	chunk_size = desc->chunk_info.chunk_size;
	sector_size = desc->sector_size;

	if (chunk_size < sector_size) {
		ret = desc->chunks_per_sector * desc->num_sectors;
	} else {
		ret = desc->num_sectors / desc->sectors_per_chunk;
	}

	return ret;
}

static int
calc_shift(unsigned int val)
{
	int i;
	unsigned int comp = 1;

	for (i = 0; i < sizeof(unsigned int) * 8; i++, comp <<= 1) {
		if (comp == val)
			return i;
	}
	return -1;
}

#ifdef ALLOW_COLOR_BITERROR
#ifndef COLOR_BITERROR_LIMIT
#define COLOR_BITERROR_LIMIT  2
#endif /* COLOR_BITERROR_LIMIT */
static unsigned short
get_color(struct sdmwl_chunk_info *chunk_info, const void *buf)
{
	unsigned short color, x_red, x_blue;
	const unsigned char *p = (unsigned char *)buf;
	int i, red = 0, blue = 0;

	color = *(unsigned short *)(p + chunk_info->color_offset);

	x_red = color ^ SDMWL_COLOR_RED;
	x_blue = color ^ SDMWL_COLOR_BLUE;

	for (i = 0; i < sizeof(unsigned short) * 8; i++) {
		red += x_red & 1;
		blue += x_blue & 1;
		x_red >>= 1;
		x_blue >>= 1;
	}
	if (red <= COLOR_BITERROR_LIMIT)
		return SDMWL_COLOR_RED;
	if (blue <= COLOR_BITERROR_LIMIT)
		return SDMWL_COLOR_BLUE;

	return SDMWL_COLOR_NONE;
}
#else
static unsigned short
get_color(struct sdmwl_chunk_info *chunk_info, const void *buf)
{
	unsigned short color;
	const unsigned char *p = (unsigned char *)buf;

	color = *(unsigned short *)(p + chunk_info->color_offset);

	if ((color != SDMWL_COLOR_RED) && (color != SDMWL_COLOR_BLUE))
		return SDMWL_COLOR_NONE;

	return color;
}
#endif

static void
set_color_and_crc(struct sdmwl_desc *desc, void *buf, unsigned short color)
{
	const unsigned char *p = (unsigned char *)buf;

	*(unsigned short *)(p + desc->chunk_info.color_offset) = color;
	desc->color = color;
	desc->chunk_info.setcrc(buf);

	SDMWL_DBGPRINT("%s: color set to 0x%x\n", __FUNCTION__, color);
}

static int
search_latest_chunk(struct sdmwl_desc *desc)
{
	unsigned int offset;
	unsigned short color;
	int i;
	int ret;

	desc->color = SDMWL_COLOR_NONE;
	desc->latest = SDMWL_LATEST_NONE;

	for (i = 0; i < desc->num_chunks; i++) {

		offset = chunknum_to_offset(desc, i);

		ret = devops.read(desc->priv, desc->buf, offset,
				  desc->chunk_info.chunk_size);
		if (ret < 0) {
			SDMWL_PRINT("%s: read failed at 0x%x\n",
				    __FUNCTION__, offset);
			continue;
		}

		if (!desc->chunk_info.isvalid(desc->buf)) {
			SDMWL_DBGPRINT("%s: no valid chunk at 0x%x\n",
				       __FUNCTION__, offset);
			continue;
		}

		color = get_color(&desc->chunk_info, desc->buf);
		if (color == SDMWL_COLOR_NONE) {
			SDMWL_PRINT("%s: chunk at 0x%x doesn't have valid "
				    "color, ignored\n", __FUNCTION__, offset);
			continue;
		}
		if (desc->color == SDMWL_COLOR_NONE) {
			desc->color = color;
		}
		if (desc->color == color) {
			/* new candidate for the latest chunk */
			desc->latest = i;
			continue;
		}

		/* color change detected. now desc->latest is exactly
		   indicates the latest chunk, finish scanning */

		SDMWL_DBGPRINT("%s: color change detected at 0x%x\n",
			       __FUNCTION__, offset);

		/* boundary check */
		if ((offset & (desc->sector_size - 1)) != 0) {
			/* not sector boundary ! */
			SDMWL_PRINT("%s: !!warning!! color change detected on "
				    "non-sector-boundary(0x%x)!! ignored.\n",
				    __FUNCTION__, offset);
			continue;
		}

		break;
	}

#ifdef SDMWL_DEBUG
	if (desc->latest != SDMWL_LATEST_NONE) {
		SDMWL_DBGPRINT("%s: latest chunk found at 0x%x(%d)\n",
			       __FUNCTION__,
			       chunknum_to_offset(desc, desc->latest),
			       desc->latest);
	} else {
		SDMWL_DBGPRINT("%s: latest chunk not found.\n", __FUNCTION__);
	}
#endif /* SDMWL_DEBUG */

	return desc->latest;
}

static int
update_latest_chunk(struct sdmwl_desc *desc, void *buf)
{
	unsigned int num_chunks = desc->num_chunks;
	unsigned int sector_size = desc->sector_size;
	unsigned int chunk_size = desc->chunk_info.chunk_size;
	unsigned int offset;
	unsigned int first;
	unsigned int erasable;
	int i;
	int ret;

	if (desc->latest == SDMWL_LATEST_NONE) {
		desc->color = SDMWL_COLOR_RED;
		first = 0;
		erasable = num_chunks;
	} else {
		first = (desc->latest + 1) % num_chunks;
		if (chunk_size < sector_size) {
			erasable = num_chunks
				- (desc->latest % desc->chunks_per_sector + 1);
		} else {
			erasable = num_chunks - 1;
		}
	}

	SDMWL_DBGPRINT("%s: erasable chunks: %d\n", __FUNCTION__, erasable);

	if ((desc->color != SDMWL_COLOR_RED) &&
	    (desc->color != SDMWL_COLOR_BLUE)) {
		SDMWL_PRINT("%s: desc->color is not valid (0x%x)!!\n",
			    __FUNCTION__, desc->color);
		return SDMWL_ERR_EWRITECHUNK;
	}

	/*
	 * we must initialize color and crc.
	 * if first == 0, color change and crc update will be done in
	 * below loop (i == 0).
	 * here, to avoid redundant crc calculation,
	 * we initialize these values iff fisrt != 0.
	 */
	if (first != 0) {
		set_color_and_crc(desc, buf, desc->color);
	}

	for (i = 0; i < erasable; i++) {
		if (((first + i) % num_chunks) == 0) {
			/*
			 * we attempt to write chunk number 0,
			 * so we must change color and update crc.
			 * this will happen in following two cases.
			 *  1) first == 0 and i == 0
			 *  2) first != 0, but ((first + i) % num_chunks) == 0
			 * in case 1), this will also serve as initialization
			 * of color and crc value.
			 */
			set_color_and_crc(desc, buf,
					  desc->color ^ SDMWL_COLOR_XORED);
		}

		offset = chunknum_to_offset(desc, (first + i) % num_chunks);

		if ((offset & (sector_size - 1)) == 0) {
			SDMWL_DBGPRINT("%s: erase sector at 0x%x\n",
				       __FUNCTION__, offset);

			ret = devops.erase(desc->priv, offset,
					   desc->sectors_per_chunk);
			if (ret < 0) {
				/* erase failed, skip this sector */
				SDMWL_PRINT("%s: erase failed at 0x%x\n",
					    __FUNCTION__, offset);
				if (chunk_size < sector_size)
					i += desc->chunks_per_sector - 1;
				continue;
			}
		}

		ret = devops.write(desc->priv, buf, offset, chunk_size);
		if (ret < 0) {
			/* update failed, skip this chunk */
			SDMWL_PRINT("%s: write failed at 0x%x\n",
				    __FUNCTION__, offset);
			continue;
		}

		/* update success */
		desc->latest = offset_to_chunknum(desc, offset);

		if (i > 0) {
			SDMWL_PRINT("%s: write succeeded at 0x%x\n",
				    __FUNCTION__, offset);
		}

		SDMWL_DBGPRINT("%s: latest chunk updated: %d (offset:0x%x)\n",
			       __FUNCTION__, desc->latest, offset);
		return SDMWL_ERR_OK;
	}

	SDMWL_PRINT("%s: latest chunk update failed!\n", __FUNCTION__);
	return SDMWL_ERR_EWRITECHUNK;
}

int
sdmwl_open(const char *name, const struct sdmwl_chunk_info *chinfo,
	   void *buf, struct sdmwl_desc *desc)
{
	void *priv;
	unsigned int sector_size;
	unsigned int chunk_size;
	int shift;

	if (!initialized)
		return SDMWL_ERR_ENODEV;

	if (name == NULL || chinfo == NULL || buf == NULL || desc == NULL)
		return SDMWL_ERR_EINVAL;

	memset(desc, 0, sizeof(*desc));

	if (devops.open(name, &priv) < 0)
		return SDMWL_ERR_EPARTNAME;

	memcpy(&desc->chunk_info, chinfo, sizeof(*chinfo));

	chunk_size = chinfo->chunk_size;
	sector_size = devops.query_sector_size(priv);

	if ((shift = calc_shift(sector_size)) < 0) {
		devops.close(priv);
		return SDMWL_ERR_EINVAL;
	}

	desc->priv              = priv;
	desc->buf               = buf;
	desc->sector_size       = sector_size;
	desc->sector_shift      = shift;
	desc->chunks_per_sector = sector_size / chunk_size;
	desc->sectors_per_chunk = (chunk_size + sector_size - 1) / sector_size;
	desc->num_sectors       = devops.query_num_sectors(priv);
	desc->num_chunks        = calc_num_chunks(desc);
	desc->color             = SDMWL_COLOR_NONE;

	if (desc->num_sectors < 2 || desc->num_chunks < 2) {
		devops.close(priv);
		return SDMWL_ERR_EPARTSIZE;
	}

	/* search latest chunk offset */
	search_latest_chunk(desc);

	return SDMWL_ERR_OK;
}

int
sdmwl_query_numchunk(struct sdmwl_desc *desc)
{
	if (desc == NULL)
		return SDMWL_ERR_EINVAL;

	return desc->num_chunks;
}

int
sdmwl_read_chunk(struct sdmwl_desc *desc, int chunk_num, void *buf)
{
	unsigned int num_chunks = desc->num_chunks;
	unsigned int latest, target;
	unsigned int offset;
	int ret;

	if (!initialized)
		return SDMWL_ERR_ENODEV;

	if (desc == NULL || buf == NULL)
		return SDMWL_ERR_EINVAL;
	if (chunk_num < 0 || chunk_num >= num_chunks)
		return SDMWL_ERR_EINVALNUM;

	/* if latest chunk doesn't exist,
	   we use chunk 0 as chunk_num's origin */
	latest = (desc->latest != SDMWL_LATEST_NONE) ? desc->latest : 0;
	target = (latest + num_chunks - chunk_num) % num_chunks;
	offset = chunknum_to_offset(desc, target);

	ret = devops.read(desc->priv, buf, offset,
			  desc->chunk_info.chunk_size);
	if (ret < 0)
		return SDMWL_ERR_EIO;

	return (desc->chunk_info.isvalid(buf)) ?
		SDMWL_ERR_OK : SDMWL_ERR_EINVALCHUNK;
}

int
sdmwl_write_chunk(struct sdmwl_desc *desc, void *buf)
{
	if (!initialized)
		return SDMWL_ERR_ENODEV;

	if (desc == NULL || buf == NULL)
		return SDMWL_ERR_EINVAL;

	return update_latest_chunk(desc, buf);
}

int
sdmwl_close(struct sdmwl_desc *desc)
{
	if (!initialized)
		return SDMWL_ERR_ENODEV;

	if (devops.close(desc->priv) < 0)
		return SDMWL_ERR_EIO;

	return SDMWL_ERR_OK;
}

