/*
 *      tx49xx_oss.c  --  Sound driver for Toshiba Tx49xx MIPS Processor.
 *
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Module command line parameters:
 *
 *  Supported devices:
 *  /dev/dsp    standard OSS /dev/dsp device
 *  /dev/mixer  standard OSS /dev/mixer device
 *
 * Notes:
 *
 *  1. Much of the OSS buffer allocation, ioctl's, and mmap'ing are
 *     taken, slightly modified or not at all, from the ES1371 driver,
 *     so refer to the credits in es1371.c for those. The rest of the
 *     code (probe, open, read, write, the ISR, etc.) is new.
 *
 *  Revision history
 *    07.30.2003  Initial version
 *
 */
#include <linux/version.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/sound.h>
#include <linux/slab.h>
#include <linux/soundcard.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/pci.h>
#include <linux/bitops.h>
#include <linux/proc_fs.h>
#include <linux/spinlock.h>
#include <linux/smp_lock.h>
#include <linux/ac97_codec.h>
#include <linux/wrapper.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/hardirq.h>
#ifdef CONFIG_CPU_CXD9802
#include <asm/cxd9802/cxd9802.h>
#endif

static inline unsigned long
kseg_to_phys(volatile void *address)
{
	return PHYSADDR((unsigned long)address);
}

/* --------------------------------------------------------------------- */

#undef OSS_DOCUMENTED_MIXER_SEMANTICS

#define USE_COHERENT_DMA

#define TX49XX_MODULE_NAME "Tx49xx audio"
#define PFX TX49XX_MODULE_NAME

#ifdef TX49XX_DEBUG
#define dbg(format, arg...) \
    printk(KERN_DEBUG "%s: " format "\n" , __func__, ## arg)
#else
#define dbg(format, arg...) do {} while (0)
#endif
#define err(format, arg...) printk(KERN_ERR PFX ": " format "\n" , ## arg)
#define info(format, arg...) printk(KERN_INFO PFX ": " format "\n" , ## arg)
#define warn(format, arg...) printk(KERN_WARNING PFX ": " format "\n" , ## arg)


/* misc stuff */
#define POLL_COUNT   0x5000
#define AC97_EXT_DACS (AC97_EXTID_SDAC | AC97_EXTID_CDAC | AC97_EXTID_LDAC)


#include <asm/dma.h>
#include <asm/byteorder.h>
/**********/
/* TX4927 */
/**********/
/* Registers Address */
#define TX4927_REG_BASE	0xff1f0000

#if defined(CONFIG_SNSC_MPU22X) || defined(CONFIG_SNSC_MPU23X)
#define TX4927_DMA_REG		(TX4927_REG_BASE + 0xb800)
#else
#define TX4927_DMA_REG		(TX4927_REG_BASE + 0xb000)
#endif

#define TX4927_CCFG_REG		(TX4927_REG_BASE + 0xe000)
#define TX4927_ACLC_REG		(TX4927_REG_BASE + 0xf700)

/* bits for MCR */
#define TXx927_DMA_MCR_MSTEN	0x00000001

/* bits for CCRn */
#define TXx927_DMA_CCR_IMMCHN	0x20000000
#define TX4927_DMA_CCR_IMMCHN	TXx927_DMA_CCR_IMMCHN
#define TXx927_DMA_CCR_USEXFSZ	0x10000000
#define TXx927_DMA_CCR_LE	0x08000000
#define TXx927_DMA_CCR_CHRST	0x01000000
#define TXx927_DMA_CCR_EXTRQ	0x00010000
#define TXx927_DMA_CCR_INTENE	0x00001000
#define TXx927_DMA_CCR_INTENC	0x00000800
#define TXx927_DMA_CCR_INTENT	0x00000400
#define TXx927_DMA_CCR_XFACT	0x00000100
#define TXx927_DMA_CCR_SMPCHN	0x00000020
#define TX4927_DMA_CCR_SMPCHN	TXx927_DMA_CCR_SMPCHN
#define TXx927_DMA_CCR_XFSZ(order)	(((order) << 2) & 0x0000001c)
#define TXx927_DMA_CCR_XFSZ_1B	TXx927_DMA_CCR_XFSZ(0)
#define TXx927_DMA_CCR_XFSZ_1W	TXx927_DMA_CCR_XFSZ(2)
/* other XFSZ are CPU dependend... */

/* Internal I/O, 1 word size transfer mode */
#define TXx927_DMA_MODE_IIO_1W	\
	(TXx927_DMA_CCR_EXTRQ|TXx927_DMA_CCR_INTENE|TXx927_DMA_CCR_INTENT|TXx927_DMA_CCR_XFSZ_1W)

/* bits for CSR */
#define TXx927_DMA_STATUS_CHNEN	0x00000400
#define TX4927_DMA_CSR_CHNEN	TXx927_DMA_STATUS_CHNEN

/* bit for PCFG */
#define TXx927_PCFG_SEL2	0x00000200
#define TX4927_PCFG_SEL2	TXx927_PCFG_SEL2
#define TXx927_PCFG_DMASEL0_MASK	0x00000003
#define TX4927_PCFG_DMASEL0_MASK	TXx927_PCFG_DMASEL0_MASK
#define TXx927_PCFG_DMASEL0_ACL0	0x00000002
#define TX4927_PCFG_DMASEL0_ACL0	TXx927_PCFG_DMASEL0_ACL0
#define TXx927_PCFG_DMASEL1_MASK	0x0000000c
#define TX4927_PCFG_DMASEL1_MASK	TXx927_PCFG_DMASEL1_MASK
#define TXx927_PCFG_DMASEL1_ACL1	0x00000008
#define TX4927_PCFG_DMASEL1_ACL1	TXx927_PCFG_DMASEL1_ACL1
#define TXx927_PCFG_DMASEL2_MASK	0x00000030
#define TX4927_PCFG_DMASEL2_MASK	TXx927_PCFG_DMASEL2_MASK
#define TXx927_PCFG_DMASEL2_ACL0	0x00000030
#define TX4927_PCFG_DMASEL2_ACL0	TXx927_PCFG_DMASEL2_ACL0
#define TXx927_PCFG_DMASEL3_MASK	0x000000c0
#define TX4927_PCFG_DMASEL3_MASK	TXx927_PCFG_DMASEL3_MASK
#define TXx927_PCFG_DMASEL3_ACL1	0x000000c0
#define TX4927_PCFG_DMASEL3_ACL1	TXx927_PCFG_DMASEL3_ACL1

#define TXX927_DMA_CHANNEL_START 23
#define MAX_TXX927_DMA_CHANNELS 8

#ifdef __BIG_ENDIAN
#define endian_def_l2(e1,e2)	\
	volatile unsigned long e1,e2
#else
#define endian_def_l2(e1,e2)	\
	volatile unsigned long e2,e1
#endif

struct tx4927_ccfg_reg {
	volatile unsigned long long ccfg;
	volatile unsigned long long crir;
	volatile unsigned long long pcfg;
	volatile unsigned long long tear;
	volatile unsigned long long clkctr;
	volatile unsigned long long unused0;
	volatile unsigned long long garbc;
	volatile unsigned long long unused1;
	volatile unsigned long long unused2;
	volatile unsigned long long ramp;
};

struct tx49xx_dma_reg {
	struct tx49xx_dma_ch_reg {
		volatile unsigned long long cha;
		volatile unsigned long long sar;
		volatile unsigned long long dar;
		endian_def_l2(unused0, cntr);
		endian_def_l2(unused1, sair);
		endian_def_l2(unused2, dair);
		endian_def_l2(unused3, ccr);
		endian_def_l2(unused4, csr);
	} ch[4];
	volatile unsigned long long dbr[8];
	volatile unsigned long long tdhr;
	volatile unsigned long long midr;
	endian_def_l2(unused0, mcr);
};

#define tx4927_ccfgptr		((struct tx4927_ccfg_reg *)TX4927_CCFG_REG)
#define tx4927_dmaptr		((struct tx49xx_dma_reg *)TX4927_DMA_REG)
#define TX4927_REV_PCODE()	((unsigned long)tx4927_ccfgptr->crir >> 16)

#ifdef CONFIG_CPU_CXD9802
#define TX4927_IR_ACLC  21
#define TX4927_NUM_IR   40
#define TX4927_IRQ_BASE CXD9802_IRQ_PIC_BEG
#define TX4927_IR_DMA(n)        (22 + (n))
#else
#define TX4927_IR_ACLC	24
#define TX4927_NUM_IR	32
#define TX4927_IRQ_BASE	24
#define TX4927_IR_DMA(n)	(27 + (n))
#endif

/* IRQ for tx4927 */
int tx49xx_irq_to_irq(int irq)
{
	if (irq < 0 || irq > TX4927_NUM_IR)
		return -1;

	return irq + TX4927_IRQ_BASE;
}

/* TX4927 stype DMA controller */
struct tx49xx_dma_reg *tx49xx_dmaptrs[2]
#if defined(CONFIG_SNSC_MPU22X) || defined(CONFIG_SNSC_MPU23X)
= { tx4927_dmaptr }
#endif
;
/* TX4927 DMA IRQ */
static int tx49xx_dmairqs[2]
#if defined(CONFIG_SNSC_MPU22X) || defined(CONFIG_SNSC_MPU23X)
= { TX4927_IR_DMA(0) }
#endif
;

/**********/
/* TX4925 */
/**********/
#define TX4925_REG_BASE	0xff1f0000
#define TX4925_CCFG_REG		(TX4925_REG_BASE + 0xe000)
#define TX4925_ACLC_REG		(TX4925_REG_BASE + 0xf700)

/* DRQCTR : DMA Request Control */
#define TX4925_DRQCTR_ACLC0	0xc
#define TX4925_DRQCTR_ACLC1	0xd

/* PCFG : Pin Configuration */
#define TX4925_PCFG_SELACLC	0x00000008

struct tx4925_ccfg_reg {
	volatile unsigned long ccfg;
	volatile unsigned long revid;
	volatile unsigned long pcfg;
	volatile unsigned long toea;
	volatile unsigned long pdnctr;
	volatile unsigned long unused0;
	volatile unsigned long garbp;
	volatile unsigned long unused1;
	volatile unsigned long tocnt;
	volatile unsigned long drqctr;
	volatile unsigned long clkctr;
	volatile unsigned long garbc;
	volatile unsigned long ramp;
	volatile unsigned long jmpadr;
};

#define tx4925_ccfgptr		((struct tx4925_ccfg_reg *)TX4925_CCFG_REG)
#define TX4925_REV_PCODE()	(tx4925_ccfgptr->revid >> 16)

#define TX4925_IR_ACLC	26
#define TX4925_NUM_IR	32
#define TX4925_IRQ_BASE	24
/* IRQ for tx4925 */
int tx4925_irq_to_irq(int irq)
{
	if (irq < 0 || irq > TX4925_NUM_IR)
		return -1;

	return irq + TX4925_IRQ_BASE;
}

/**********/
/* TX4938 */
/**********/
#define TX4938_REG_BASE	0xff1f0000
#define TX4938_CCFG_REG		(TX4938_REG_BASE + 0xe000)
#define TX4938_ACLC_REG		(TX4938_REG_BASE + 0xf700)

/* PCFG : Pin Configuration */
#define TX4938_PCFG_ETH0_SEL	0x8000000000000000ull
#define TX4938_PCFG_SEL2	0x00000200

struct tx4938_ccfg_reg {
	volatile unsigned long long ccfg;
	volatile unsigned long long crir;
	volatile unsigned long long pcfg;
	volatile unsigned long long tear;
	volatile unsigned long long clkctr;
	volatile unsigned long long unused0;
	volatile unsigned long long garbc;
	volatile unsigned long long unused1;
	volatile unsigned long long unused2;
	volatile unsigned long long ramp;
	volatile unsigned long long unused3;
	volatile unsigned long long jmpadr;
};

#define tx4938_ccfgptr		((struct tx4938_ccfg_reg *)TX4938_CCFG_REG)
#define TX4938_REV_PCODE()	((unsigned long)tx4938_ccfgptr->crir >> 16)

#define TX4938_IR_ACLC	24

/************************/
/* Function for DMA OPS */
/************************/
/* chained DMA descriptor */
typedef void* tx49xx_chdma_desc_t;

struct tx49xxx_dma_ops {
	void (*init_dma)(unsigned int dmanr);
	void (*disable_dma)(unsigned int dmanr);
	void (*set_dma_mode)(unsigned int dmanr, unsigned int mode);
	void (*set_dma_inc)(unsigned int dmanr, unsigned int sai, unsigned int dai);
	int (*get_dma_residue)(unsigned int dmanr);
	unsigned int (*get_dma_status)(unsigned int dmanr);
	void (*clear_dma_status)(unsigned int dmanr);
	int (*get_dma_irqno)(unsigned int dmanr);
	/* for chain operation */
	void (*set_dma_chain)(unsigned int dmanr, tx49xx_chdma_desc_t cha);
	tx49xx_chdma_desc_t (*get_dma_chain)(unsigned int dmanr);
	void (*set_chdma_addr64)(tx49xx_chdma_desc_t desc, unsigned long long sa, unsigned long long da);
	void (*set_chdma_count)(tx49xx_chdma_desc_t desc, unsigned int count);
	void (*set_chdma_chain)(tx49xx_chdma_desc_t desc, tx49xx_chdma_desc_t cha);
	tx49xx_chdma_desc_t (*get_chdma_chain)(tx49xx_chdma_desc_t desc);
	tx49xx_chdma_desc_t (*alloc_chdma_cmd)(void);
	void (*free_chdma_cmd)(tx49xx_chdma_desc_t desc);
};


/******************/
/* TX4927 DMA OPS */
/******************/
/* reset a specific DMA channel */
static void
init_tx49xx_dma(unsigned int dmanr)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
	ch %= 4;

	dmaptr->ch[ch].ccr = TXx927_DMA_CCR_CHRST;
	dmaptr->ch[ch].cha = 0;
	dmaptr->ch[ch].sar = 0;
	dmaptr->ch[ch].dar = 0;
	dmaptr->ch[ch].cntr = 0;
	dmaptr->ch[ch].sair = 0;
	dmaptr->ch[ch].dair = 0;
	dmaptr->ch[ch].ccr = 0;
}

/* enable/disable a specific DMA channel */
static void
disable_tx49xx_dma(unsigned int dmanr)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("disable DMA  [%d]\n", dmanr);
#endif
	ch %= 4;
	dmaptr->ch[ch].ccr &= ~TXx927_DMA_CCR_XFACT;
}

/* set mode for a specific DMA channel */
static void
set_tx49xx_dma_mode(unsigned int dmanr, unsigned int mode)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("set DMA mode[%d], mode =0x%x\n", dmanr, mode);
#endif

	ch %= 4;
#ifdef __LITTLE_ENDIAN
	mode |= TXx927_DMA_CCR_LE;
#endif
	dmaptr->ch[ch].ccr = mode;
}

/* Set source/destination address increment size for a specific DMA channel.
 */
static void
set_tx49xx_dma_inc(unsigned int dmanr, unsigned int sai, unsigned int dai)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("set inc DMA[%d], sai=0x%x, dai=0x%x\n", dmanr, sai, dai);
#endif
	ch %= 4;
	dmaptr->ch[ch].sair = sai;
	dmaptr->ch[ch].dair = dai;
}


/* Get DMA residue count. After a DMA transfer, this
 * should return zero. Reading this while a DMA transfer is
 * still in progress will return unpredictable results.
 */
static int
get_tx49xx_dma_residue(unsigned int dmanr)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("get residue DMA[%d]\n", dmanr);
#endif
	ch %= 4;
	return dmaptr->ch[ch].cntr;
}

/* Get DMA status.
 */
static unsigned int
get_tx49xx_dma_status(unsigned int dmanr)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("get DMA status[%d]\n", dmanr);
#endif
	ch %= 4;
	return dmaptr->ch[ch].csr;
}

/* clear DMA status.
 */
static void
clear_tx49xx_dma_status(unsigned int dmanr)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("clear DMA status[%d]\n", dmanr);
#endif
	ch %= 4;
	dmaptr->ch[ch].csr = 0xffffffff;
}

/* get DMA completion interrupt number.
 */
static int
get_tx49xx_dma_irqno(unsigned int dmanr)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;

#ifdef MPU_DEBUG
	printk("get DMA irqno[%d] ch = 0x%x\n", dmanr, ch);
#endif
	if (ch >= MAX_TXX927_DMA_CHANNELS)
		return -1;

	return tx49xx_irq_to_irq(tx49xx_dmairqs[ch/4] + (ch % 4));
}

/*
 * chain DMA operations.
 * tx49xx_chdma_desc_t is nocache virtual address of the command descriptor.
 */

static void
set_tx49xx_dma_chain(unsigned int dmanr, tx49xx_chdma_desc_t cha)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("set DMA chain[%d] char =0x%x\n", dmanr, cha);
#endif
	ch %= 4;
	dmaptr->ch[ch].cha = kseg_to_phys(cha);
}

static tx49xx_chdma_desc_t
get_tx49xx_dma_chain(unsigned int dmanr)
{
	unsigned int ch = dmanr - TXX927_DMA_CHANNEL_START;
	unsigned int cha;
	struct tx49xx_dma_reg *dmaptr = tx49xx_dmaptrs[ch/4];
#ifdef MPU_DEBUG
	printk("get DMA chain[%d] ret =0x%x\n", dmanr, KSEG1ADDR(cha));
#endif
	ch %= 4;
	cha = (unsigned int)dmaptr->ch[ch].cha;
	if (cha == 0)
		return 0;
	return (tx49xx_chdma_desc_t)KSEG1ADDR(cha);
}

#define tx49xx_chdma_desc(desc)	((struct tx49xx_dma_ch_reg*)desc)

static void
set_tx49xx_chdma_addr64(tx49xx_chdma_desc_t desc, unsigned long long sa, unsigned long long da)
{
#ifdef MPU_DEBUG
	printk("set chdma addr64\n");
#endif
	tx49xx_chdma_desc(desc)->sar = sa;
	tx49xx_chdma_desc(desc)->dar = da;
}

static void
set_tx49xx_chdma_count(tx49xx_chdma_desc_t desc, unsigned int count)
{
#ifdef MPU_DEBUG
	printk("set chdma count\n");
#endif
	tx49xx_chdma_desc(desc)->cntr = count;
}

static void
set_tx49xx_chdma_chain(tx49xx_chdma_desc_t desc, tx49xx_chdma_desc_t cha)
{
#ifdef MPU_DEBUG
	printk("set chdma chain\n");
#endif
	tx49xx_chdma_desc(desc)->cha = kseg_to_phys(cha);
}

static tx49xx_chdma_desc_t
get_tx49xx_chdma_chain(tx49xx_chdma_desc_t desc)
{
	unsigned int cha = (unsigned int)tx49xx_chdma_desc(desc)->cha;
#ifdef MPU_DEBUG
	printk("get chdma chain\n");
#endif
	if (cha == 0)
		return 0;
	return (tx49xx_chdma_desc_t)KSEG1ADDR(cha);
}

static tx49xx_chdma_desc_t
alloc_tx49xx_chdma_cmd(void)
{
	int size = sizeof(struct tx49xx_dma_ch_reg);
	void *desc;
	/* kmalloc always return 32-byte aligned address... */
	desc = kmalloc(size, GFP_ATOMIC);
	if (desc) {
		if ((unsigned int)desc & 31)
			panic("dma: kmalloc returns unaligned address.");
		dma_cache_wback_inv((unsigned long)desc, size);
		desc = (void *)KSEG1ADDR(desc);
	}
	return (tx49xx_chdma_desc_t)desc;
}

static void
free_tx49xx_chdma_cmd(tx49xx_chdma_desc_t desc)
{
	if (desc)
		kfree((void *)KSEG0ADDR(desc));
}

MODULE_LICENSE("GPL");
MODULE_AUTHOR("TOSHIBA Corporation");
MODULE_DESCRIPTION("TOSHIBA TX4927,TX4926,TX4925,TX4937,TX4938");
#if 0
static int snd_index = SNDRV_DEFAULT_IDX1;	/* Index 0-MAX */
static char *snd_id = SNDRV_DEFAULT_STR1;	/* ID for this card */
static int snd_enable = SNDRV_DEFAULT_ENABLE1;	/* Enable this card */
#endif
static int snd_irq = 0;
static int snd_dma1 = TXX927_DMA_CHANNEL_START + 0;
static int snd_dma2 = TXX927_DMA_CHANNEL_START + 1;

/*
 *  Direct registers
 */

#define ACLCREG(ice, x) (*(volatile unsigned long *)((ice)->baseaddr + ACLC_REG_##x))


#define ACLC_REG_ACCTLEN		0x00	/* control enable */
#define ACLC_REG_ACCTLDIS		0x04	/* control disable */
#define   ACCTL_ENLINK		0x00000001	/* enable/disable AC-link */
#define   ACCTL_AUDODMA		0x00000100	/* AUDODMA enable/disable */
#define   ACCTL_AUDIDMA		0x00001000	/* AUDIDMA enable/disable */
#define   ACCTL_AUDOEHLT	0x00010000	/* AUDO error halt enable/disable */
#define   ACCTL_AUDIEHLT	0x00100000	/* AUDI error halt enable/disable */
#define ACLC_REG_ACREGACC		0x08	/* codec register access */
#define   ACREGACC_DAT_SHIFT	0	/* data field */
#define   ACREGACC_REG_SHIFT	16	/* address field */
#define   ACREGACC_CODECID_SHIFT	24	/* CODEC ID field */
#define   ACREGACC_READ		0x80000000	/* CODEC read */
#define   ACREGACC_WRITE	0x00000000	/* CODEC write */
#define ACLC_REG_ACINTSTS		0x10	/* interrupt status */
#define ACLC_REG_ACINTMSTS		0x14	/* interrupt masked status */
#define ACLC_REG_ACINTEN		0x18	/* interrupt enable */
#define ACLC_REG_ACINTDIS		0x1C	/* interrupt disable */
#define   ACINT_CODEC0RDY	0x00000001	/* CODEC0 ready */
#define   ACINT_CODEC1RDY	0x00000002	/* CODEC1 ready */
#define   ACINT_REGACCRDY	0x00000010	/* ACREGACC ready */
#define   ACINT_AUDOERR		0x00000100	/* AUDO underrun error */
#define   ACINT_AUDIERR		0x00001000	/* AUDI overrun error */

#define ACLC_REG_ACSLTEN		0x48	/* ACLC Slot Enable */
#define ACLC_REG_ACSLTDIS		0x4C	/* ACLC Slot Disable */
#define   ACSLT_AUDOSLT		0x00000001	/* Audio PCM L&R slot */
#define   ACSLT_SURRSLT		0x00000002	/* Audio Surround L&R slot */
#define   ACSLT_CENTSLT		0x00000004	/* Audio Center slot */
#define   ACSLT_LFESLT		0x00000008	/* Audio LFE(0.1ch) slot */
#define   ACSLT_AUDISLT		0x00000010	/* Audio reception slot */
#define   ACSLT_WRPEND		0x00010000	/* ACLC Slot Register Write Pending */
#define ACSLT_SLOT_MASK		0x0000000F	/* ACLC resgiter ACSLT Audio slots mask*/
#define ACLC_REG_ACDMASTS		0x80	/* DMA request status */
#define   ACDMA_AUDO		0x00000001	/* AUDODMA pending */
#define   ACDMA_AUDI		0x00000010	/* AUDIDMA pending */
#define ACLC_REG_ACAUDODAT		0xA0	/* audio out data */
#define ACLC_REG_ACAUDIDAT		0xB0	/* audio in data */
#define ACLC_REG_ACREVID		0xFC	/* revision ID */

#define   AC_AUDO_SHIFT		0
#define   AC_AUDI_SHIFT		4

#define SPDIF_ON	0x0004
#define SURR_ON		0x0010
#define CENTER_LFE_ON	0x0020

typedef struct {
	unsigned int dmanr;
	tx49xx_chdma_desc_t *chain;
	int outf;
	unsigned long offset;
/*        snd_pcm_substream_t *substream;*/
        dma_addr_t physbuf;
        unsigned int size;
        unsigned int fragsize;
        unsigned int fragsize1;
        unsigned int position;
        int frags;
        int lvi;
        int lvi_frag;
	int ack;
	int ack_reload;
} aclcdev_t;

#define   TX492X_NR_CHAIN		2	/* number of DMA chain desc */

/* Boot options */
static int      vra = 1;	// 0 = no VRA, 1 = use VRA if codec supports it
MODULE_PARM(vra, "i");
MODULE_PARM_DESC(vra, "if 1 use VRA if codec supports it");


/* --------------------------------------------------------------------- */

struct tx49xx_state {
	/* soundcore stuff */
	int             dev_audio;

	unsigned long baseaddr;
	unsigned long long baseaddr_phys;

	int irq_aclc;
	int irq_pbk;
	int irq_cap;

#ifdef TX49XX_DEBUG
	/* debug /proc entry */
	struct proc_dir_entry *ps;
	struct proc_dir_entry *ac97_ps;
#endif				/* TX49XX_DEBUG */


	struct ac97_codec codec;
	unsigned        codec_base_caps;// AC'97 reg 00h, "Reset Register"
	unsigned        codec_ext_caps;	// AC'97 reg 28h, "Extended Audio ID"
	int             no_vra;		// do not use VRA
	int		ac97_status;	// Enable channel & S/PDIF information */

	spinlock_t      lock;
	struct semaphore open_sem;
	struct semaphore sem;
	mode_t          open_mode;
	wait_queue_head_t open_wait;

	tx49xx_chdma_desc_t chain[TX492X_NR_CHAIN * 2];

	struct dmabuf {
		unsigned int    dmanr;	// DMA Channel number
		unsigned        sample_rate;	// Hz
		unsigned src_factor;     // SRC interp/decimation (no vra)
		unsigned        sample_size;	// 8 or 16
		int             num_channels;	// 1 = mono, 2 = stereo, 4, 6
		int dma_bytes_per_sample;// DMA bytes per audio sample frame
		int user_bytes_per_sample;// User bytes per audio sample frame
		int cnt_factor;          // user-to-DMA bytes per audio
		//  sample frame
		void           *rawbuf;
		dma_addr_t      dmaaddr;
		unsigned        buforder;
		unsigned numfrag;        // # of DMA fragments in DMA buffer
		unsigned        fragshift;
		void           *nextIn;	// ptr to next-in to DMA buffer
		void           *nextOut;// ptr to next-out from DMA buffer
		int             count;	// current byte count in DMA buffer
		unsigned        total_bytes;	// total bytes written or read
		unsigned        error;	// over/underrun
		wait_queue_head_t wait;
		/* redundant, but makes calculations easier */
		unsigned fragsize;       // user perception of fragment size
		unsigned dma_fragsize;   // DMA (real) fragment size
		unsigned dmasize;        // Total DMA buffer size
		//   (mult. of DMA fragsize)
		/* OSS stuff */
		unsigned        mapped:1;
		unsigned        ready:1;
		unsigned        stopped:1;
		unsigned        ossfragshift;
		int             ossmaxfrags;
		unsigned        subdivision;
	        int lvi;
	} dma_dac      , dma_adc;
} tx49xx_state;

/* --------------------------------------------------------------------- */


static inline unsigned ld2(unsigned int x)
{
	unsigned        r = 0;

	if (x >= 0x10000) {
		x >>= 16;
		r += 16;
	}
	if (x >= 0x100) {
		x >>= 8;
		r += 8;
	}
	if (x >= 0x10) {
		x >>= 4;
		r += 4;
	}
	if (x >= 4) {
		x >>= 2;
		r += 2;
	}
	if (x >= 2)
		r++;
	return r;
}


#ifdef USE_COHERENT_DMA
static inline void * dma_alloc(size_t size, dma_addr_t * dma_handle)
{
	void* ret = (void *)__get_free_pages(GFP_ATOMIC | GFP_DMA,
					     get_order(size));
	if (ret != NULL) {
		memset(ret, 0, size);
		*dma_handle = kseg_to_phys(ret);
	}
	return ret;
}

static inline void dma_free(size_t size, void* va, dma_addr_t dma_handle)
{
	free_pages((unsigned long)va, get_order(size));
}
#else
static inline void * dma_alloc(size_t size, dma_addr_t * dma_handle)
{
	return pci_alloc_consistent(NULL, size, dma_handle);
}

static inline void dma_free(size_t size, void* va, dma_addr_t dma_handle)
{
	pci_free_consistent(NULL, size, va, dma_handle);
}
#endif

/* --------------------------------------------------------------------- */

static void tx49xx_delay(int msec)
{
	unsigned long   tmo;
	signed long     tmo2;

	if (in_interrupt())
		return;

	tmo = jiffies + (msec * HZ) / 1000;
	for (;;) {
		tmo2 = tmo - jiffies;
		if (tmo2 <= 0)
			break;
		schedule_timeout(tmo2);
	}
}


/* --------------------------------------------------------------------- */

static int snd_tx49xx_codec_semaphore(struct tx49xx_state *chip)
{
	int time;

	/* codec ready ? */
	if ((ACLCREG(chip, ACINTSTS) & (ACINT_CODEC0RDY)) == 0){
/*		printk("codec_semaphore: semaphore is EIO ACINTSTS=0x%x, ACCTLEN=0x%x\n",ACLCREG(chip, ACINTSTS), ACLCREG(chip, ACCTLEN) );*/	/* sugiyama wrote */

		return -EIO;

	}
	/* Anyone holding a semaphore for 1 msec should be shot... */
	time = 10000;		/* default 10*/
      	do {
		if (ACLCREG(chip, ACINTSTS) & ACINT_REGACCRDY)
			return 0;
		udelay(10);
	} while (time--);

	/* access to some forbidden (non existant) ac97 registers will not
	 * reset the semaphore. So even if you don't get the semaphore, still
	 * continue the access. We don't need the semaphore anyway. */
	printk("codec_semaphore: semaphore is not ready [0x%lx]\n",
		   ACLCREG(chip, ACINTSTS));
	/* I don't care about the semaphore */
	return -EBUSY;
}

static u16 rdcodec(struct ac97_codec *codec, u8 addr)
{
	struct tx49xx_state *s = (struct tx49xx_state *)codec->private_data;
	unsigned long   flags;
	unsigned short res;
	unsigned long regdat;

	spin_lock_irqsave(&s->lock, flags);

	if (snd_tx49xx_codec_semaphore(s) < 0) {
		printk("codec_read: semaphore is not ready for register 0x%x\n", addr);
		res = 0xffff;
		goto done;
	}

	regdat = ((unsigned long)addr << ACREGACC_REG_SHIFT) + ACREGACC_READ;
	ACLCREG(s, ACREGACC) = regdat;
	if (snd_tx49xx_codec_semaphore(s) < 0) {
		printk("codec_read : semaphore is not ready for register 0x%x\n", (addr & 0xff));		/* sugiyama wrote */
		res = 0xffff;
		goto done;
	}
	regdat = ACLCREG(s, ACREGACC);
	if (((regdat >> ACREGACC_REG_SHIFT) & 0xff) != (addr & 0xff)) {
		printk("codec_read : reg mismatch %lx with %x\n", regdat, (addr & 0xff));		/* sugiyama wrote */
		res = 0xffff;
		goto done;
	}
	res = (regdat >> ACREGACC_DAT_SHIFT) & 0xffff;

done:
	spin_unlock_irqrestore(&s->lock, flags);

	return res;
}


static void wrcodec(struct ac97_codec *codec, u8 addr, u16 data)
{
	struct tx49xx_state *s = (struct tx49xx_state *)codec->private_data;
	unsigned long   flags;

	spin_lock_irqsave(&s->lock, flags);

	if (snd_tx49xx_codec_semaphore(s) < 0)
		printk("codec_write: semaphore is not ready for register 0x%x\n", addr);

//printk("codec_write %d: write ok reg=0x%x,val=0x%x\n", ac97->num, reg, val);		/* sugiyama wrote for DEBUG */

	ACLCREG(s, ACREGACC) =
		((unsigned long)(addr) << ACREGACC_REG_SHIFT)
		+ ((unsigned long)data << ACREGACC_DAT_SHIFT);

	spin_unlock_irqrestore(&s->lock, flags);
}

static void waitcodec(struct ac97_codec *codec)
{
	u16             temp;
	int             i;
	struct tx49xx_state *s = (struct tx49xx_state *)codec->private_data;

	/* codec_wait is used to wait for a ready state after
	   an AC97C_RESET. */
	tx49xx_delay(10);

	// first poll the CODEC_READY tag bit
	for (i = 0; i < POLL_COUNT; i++){
		if (ACLCREG(s, ACINTSTS) & ACINT_CODEC0RDY){
			break;
		}
	}
	if (i == POLL_COUNT) {
		err("waitcodec: CODEC_READY poll expired!");
		return;
	}
	// get AC'97 powerdown control/status register
	temp = rdcodec(codec, AC97_POWER_CONTROL);

	// If anything is powered down, power'em up
	if (temp & 0x7f00) {
		// Power on
		wrcodec(codec, AC97_POWER_CONTROL, 0);
		tx49xx_delay(100);
		// Reread
		temp = rdcodec(codec, AC97_POWER_CONTROL);
	}
    
	// Check if Codec REF,ANL,DAC,ADC ready
	if ((temp & 0x7f0f) != 0x000f)
		err("codec reg 26 status (0x%x) not ready!!", temp);
}


/* --------------------------------------------------------------------- */
static void tx49xx_set_spdif_output(struct tx49xx_state *s, int slots, int rate)
{
	int	aud_reg;
	unsigned short	val;
	int i;
		
	if ( slots == -1 ) { /* Turn off S/PDIF */
		aud_reg = rdcodec(&s->codec, AC97_EXTENDED_STATUS);
		wrcodec(&s->codec, AC97_EXTENDED_STATUS, (aud_reg & ~AC97_EA_SPDIF));
		s->ac97_status &= ~SPDIF_ON;
		return;
	}
	val = rdcodec(&s->codec, AC97_EXTENDED_STATUS);

//	wrcodec(&s->codec, AC97_EXTENDED_STATUS, 0x0000);

	/* Set S/PDIF transmitter sample rate */
	aud_reg = rdcodec(&s->codec, AC97_SPDIF_CONTROL);
	switch ( rate ) {
		case 32000:
			aud_reg = (aud_reg & AC97_SC_SPSR_MASK) | AC97_SC_SPSR_32K; 
			break;
		case 44100:
			aud_reg = (aud_reg & AC97_SC_SPSR_MASK) | AC97_SC_SPSR_44K; 
			break;
		case 48000:
		 	aud_reg = (aud_reg & AC97_SC_SPSR_MASK) | AC97_SC_SPSR_48K; 
			break;
		default:
#ifdef MPU_DEBUG
			printk(KERN_WARNING "tx49xx_audio: %d sample rate not supported by S/PDIF.\n", rate);
#endif
			/* turn off S/PDIF */
			aud_reg = rdcodec(&s->codec, AC97_EXTENDED_STATUS);
			wrcodec(&s->codec, AC97_EXTENDED_STATUS, (aud_reg & ~AC97_EA_SPDIF));
			s->ac97_status &= ~SPDIF_ON;
			return;
	}
	
	wrcodec(&s->codec, AC97_SPDIF_CONTROL, aud_reg);
	s->ac97_status |= SPDIF_ON;
	for ( i = 0 ; i < 10 ; i ++ ){
		if (aud_reg == rdcodec(&s->codec, AC97_SPDIF_CONTROL) ){
			break;
		}
		tx49xx_delay(100);
	}
		
	
	aud_reg = (val & AC97_EA_SLOT_MASK) | slots | AC97_EA_SPDIF;
	wrcodec(&s->codec, AC97_EXTENDED_STATUS, aud_reg);
	s->ac97_status |= SPDIF_ON;


	/* Check to make sure the configuration is valid */
	for ( i = 0 ; i < 10 ; i ++ ){
		if (rdcodec(&s->codec, AC97_EXTENDED_STATUS) & 0x0400) {
			break;
		}
		tx49xx_delay(100);
	}
	
}


/* stop the ADC before calling */
static void set_adc_rate(struct tx49xx_state *s, unsigned rate)
{
	struct dmabuf  *adc = &s->dma_adc;
	struct dmabuf  *dac = &s->dma_dac;
	unsigned        adc_rate, dac_rate;
	u16             ac97_extstat;


	if (s->no_vra) {
		// calc SRC factor
		adc->src_factor = ((96000 / rate) + 1) >> 1;
		adc->sample_rate = 48000 / adc->src_factor;
		return;
	}
#ifdef MPU_DEBUG
	printk("OSS set_adc_rate rate = %d\n", rate );
#endif
	
	adc->src_factor = 1;

	ac97_extstat = rdcodec(&s->codec, AC97_EXTENDED_STATUS);

	rate = rate > 48000 ? 48000 : rate;

	// enable VRA
	wrcodec(&s->codec, AC97_EXTENDED_STATUS,
		ac97_extstat | AC97_EXTSTAT_VRA);
	// now write the sample rate
	wrcodec(&s->codec, AC97_PCM_LR_ADC_RATE, (u16) rate);
	// read it back for actual supported rate
	adc_rate = rdcodec(&s->codec, AC97_PCM_LR_ADC_RATE);

	// some codec's don't allow unequal DAC and ADC rates, in which case
	// writing one rate reg actually changes both.
	dac_rate = rdcodec(&s->codec, AC97_PCM_FRONT_DAC_RATE);
	if (dac->num_channels > 2)
		wrcodec(&s->codec, AC97_PCM_SURR_DAC_RATE, dac_rate);
	if (dac->num_channels > 4)
		wrcodec(&s->codec, AC97_PCM_LFE_DAC_RATE, dac_rate);

	adc->sample_rate = adc_rate;
	dac->sample_rate = dac_rate;
#ifdef MPU_DEBUG
	printk("OSS set_adc_rate end read back rate = %d\n", adc_rate);
#endif
}

#define SPDIF_CNTRL_SPSR_MASK	0xc000

/* stop the DAC before calling */
static void set_dac_rate(struct tx49xx_state *s, unsigned rate)
{
	struct dmabuf  *dac = &s->dma_dac;
	struct dmabuf  *adc = &s->dma_adc;
	unsigned        adc_rate, dac_rate;
	u16             ac97_extstat;
	
	if ( s->ac97_status &= SPDIF_ON ){
		tx49xx_set_spdif_output ( s, -1, 0 );
	}
	
	if (s->no_vra) {
		// calc SRC factor
		dac->src_factor = ((96000 / rate) + 1) >> 1;
		dac->sample_rate = 48000 / dac->src_factor;
	
		if ( s->ac97_status &= SPDIF_ON ){
			tx49xx_set_spdif_output ( s, AC97_EA_SPSA_3_4, rate );
		}		
		return;
	}

#ifdef MPU_DEBUG
	printk("OSS set_dac_rate rate = %d\n", rate );
#endif
	dac->src_factor = 1;

	ac97_extstat = rdcodec(&s->codec, AC97_EXTENDED_STATUS);

	rate = rate > 48000 ? 48000 : rate;

	// enable VRA
	wrcodec(&s->codec, AC97_EXTENDED_STATUS,
		ac97_extstat | AC97_EXTSTAT_VRA);
	// now write the sample rate
	wrcodec(&s->codec, AC97_PCM_FRONT_DAC_RATE, (u16) rate);
	// I don't support different sample rates for multichannel,
	// so make these channels the same.
	if (dac->num_channels > 2)
		wrcodec(&s->codec, AC97_PCM_SURR_DAC_RATE, (u16) rate);
	if (dac->num_channels > 4)
		wrcodec(&s->codec, AC97_PCM_LFE_DAC_RATE, (u16) rate);
	// read it back for actual supported rate
	dac_rate = rdcodec(&s->codec, AC97_PCM_FRONT_DAC_RATE);

	// some codec's don't allow unequal DAC and ADC rates, in which case
	// writing one rate reg actually changes both.
	adc_rate = rdcodec(&s->codec, AC97_PCM_LR_ADC_RATE);

	dac->sample_rate = dac_rate;
	adc->sample_rate = adc_rate;
	
	if ( s->ac97_status &= SPDIF_ON ){
		tx49xx_set_spdif_output ( s, AC97_EA_SPSA_3_4, rate );
	}
#ifdef MPU_DEBUG
	printk("OSS set_dac_rate end read back rate = %d\n", dac_rate);
#endif
}

static void stop_dac(struct tx49xx_state *s)
{
	struct dmabuf  *db = &s->dma_dac;
	unsigned long   flags;

	if (db->stopped)
		return;

	spin_lock_irqsave(&s->lock, flags);

	disable_tx49xx_dma(db->dmanr);

	db->stopped = 1;

	spin_unlock_irqrestore(&s->lock, flags);
}

static void  stop_adc(struct tx49xx_state *s)
{
	struct dmabuf  *db = &s->dma_adc;
	unsigned long   flags;

	if (db->stopped)
		return;

	spin_lock_irqsave(&s->lock, flags);

	disable_tx49xx_dma(db->dmanr);
	
	db->stopped = 1;

	spin_unlock_irqrestore(&s->lock, flags);
}


static void set_xmit_slots(struct tx49xx_state *s, int num_channels)
{
	u32 ac97_config = ACLCREG(s, ACSLTEN) & ~ACSLT_SLOT_MASK;
	int i;

	s->ac97_status &= ~(SURR_ON | CENTER_LFE_ON);
	
	switch (num_channels) {
	case 1:	// mono
	case 2:	// stereo, slots 3,4
		ac97_config |= ACSLT_AUDOSLT;
		break;
	case 4:	// stereo with surround, slots 3,4,7,8
		ac97_config |= ACSLT_AUDOSLT | ACSLT_SURRSLT;
		s->ac97_status |= SURR_ON;
		break;
	case 6:	// stereo with surround and center/LFE, slots 3,4,6,7,8,9
		ac97_config |= ACSLT_AUDOSLT | ACSLT_SURRSLT | ACSLT_CENTSLT | ACSLT_LFESLT;
		s->ac97_status |= SURR_ON | CENTER_LFE_ON;
		break;
	}

	/* Check ACSLTEN Write Enable bit */
	for ( i = 0 ; i < POLL_COUNT ; i ++ ){
		if ( !(ACLCREG(s, ACSLTEN) & ACSLT_WRPEND) ){
			break;
		}
	}
	if ( i == POLL_COUNT ){
		printk("set_xmit_slots write enable check isn't OK \n");
		return;
	}
	ACLCREG(s, ACSLTEN) = ac97_config;		/* Enable slots */

}

static void     set_recv_slots(struct tx49xx_state *s,int num_channels)
{
	u32 ac97_config = ACLCREG(s, ACSLTEN) & ~ACSLT_AUDISLT;

	/*
	 * Always enable slots 3 and 4 (stereo). Slot 6 is
	 * optional Mic ADC, which I don't support yet.
	 */
	ac97_config |= ACSLT_AUDISLT;

	ACLCREG(s, ACSLTEN) = ac97_config;		/* Enable slots */
}

static void start_dac(struct tx49xx_state *s)
{
	struct dmabuf  *db = &s->dma_dac;
	unsigned long   flags;
	unsigned long   buf1, buf2;
	tx49xx_chdma_desc_t *chain = s->chain;
	int idx;
	unsigned long long dev_dma_addr = s->baseaddr_phys + ACLC_REG_ACAUDODAT;

	if (!db->stopped)
		return;

	spin_lock_irqsave(&s->lock, flags);

	// reset Buffer 1 and 2 pointers to nextOut and nextOut+dma_fragsize
	buf1 = kseg_to_phys(db->nextOut);
	buf2 = buf1 + ( db->dma_fragsize >> 1) ;	/* >> 1 == / 2 */
	if (buf2 >= db->dmaaddr + db->dmasize)
		buf2 -= db->dmasize;

	set_xmit_slots(s, db->num_channels);

	init_tx49xx_dma(db->dmanr);
	set_tx49xx_dma_mode(db->dmanr,
			    TXx927_DMA_MODE_IIO_1W | TXx927_DMA_CCR_INTENC |
			    TX4927_DMA_CCR_IMMCHN | TX4927_DMA_CCR_SMPCHN);
	set_tx49xx_dma_inc(db->dmanr, sizeof(long), 0);
	clear_tx49xx_dma_status(db->dmanr);


#ifdef MPU_DEBUG
	printk("db->count = 0x%d(%d),db->dma_fragsize = 0x%d(%d)\n", db->count, db->count, db->dma_fragsize, db->dma_fragsize );
#endif
	if (db->count == db->dma_fragsize) {
		for (idx = 0; idx < TX492X_NR_CHAIN; idx += 2) {
			set_tx49xx_chdma_chain(chain[idx  ], chain[(idx+1) % TX492X_NR_CHAIN]);
			set_tx49xx_chdma_chain(chain[idx+1], chain[(idx+2) % TX492X_NR_CHAIN]);

			set_tx49xx_chdma_addr64(chain[idx], buf1, dev_dma_addr);
			set_tx49xx_chdma_addr64(chain[idx+1], buf2, dev_dma_addr);

			set_tx49xx_chdma_count(chain[idx  ], db->dma_fragsize);
			set_tx49xx_chdma_count(chain[idx+1], db->dma_fragsize);
		}
	} else {
		for (idx = 0; idx < TX492X_NR_CHAIN; idx += 1) {
			dma_addr_t buf_dma_addr =
				buf1 + ((idx * db->dma_fragsize) % db->count);

			set_tx49xx_chdma_chain(chain[idx], chain[(idx+1) % TX492X_NR_CHAIN]);

			set_tx49xx_chdma_addr64(chain[idx], buf_dma_addr, dev_dma_addr);

			set_tx49xx_chdma_count(chain[idx], db->dma_fragsize);
		}
	}

	db->lvi      = TX492X_NR_CHAIN - 1;
	set_tx49xx_chdma_chain(chain[db->lvi], 0);
	db->stopped = 0;

	/* Tx49xx start DMA */
	set_tx49xx_dma_chain(db->dmanr, chain[0]);
	ACLCREG(s, ACCTLEN) = ACCTL_AUDODMA;

	spin_unlock_irqrestore(&s->lock, flags);
}

static void start_adc(struct tx49xx_state *s)
{
	struct dmabuf  *db = &s->dma_adc;
	unsigned long   flags;
	unsigned long   buf1, buf2;
	tx49xx_chdma_desc_t *chain = s->chain;
	int idx;
	unsigned long long dev_dma_addr =
		s->baseaddr_phys + ACLC_REG_ACAUDIDAT;

	if (!db->stopped)
		return;

	spin_lock_irqsave(&s->lock, flags);

	// reset Buffer 1 and 2 pointers to nextIn and nextIn+dma_fragsize
	buf1 = kseg_to_phys(db->nextIn);
	buf2 = buf1 + db->dma_fragsize;
	if (buf2 >= db->dmaaddr + db->dmasize)
		buf2 -= db->dmasize;

	set_recv_slots(s, db->num_channels);

	init_tx49xx_dma(db->dmanr);
	set_tx49xx_dma_mode(db->dmanr,
			    TXx927_DMA_MODE_IIO_1W | TXx927_DMA_CCR_INTENC |
			    TX4927_DMA_CCR_IMMCHN | TX4927_DMA_CCR_SMPCHN);
	set_tx49xx_dma_inc(db->dmanr, 0, sizeof(long));
	clear_tx49xx_dma_status(db->dmanr);

#ifdef MPU_DEBUG
	printk("db->count = 0x%d(%d),db->dma_fragsize = 0x%d(%d)\n", db->count, db->count, db->dma_fragsize, db->dma_fragsize );
#endif
	for (idx = 0; idx < TX492X_NR_CHAIN; idx += 2) {
		set_tx49xx_chdma_chain(chain[idx  ], chain[(idx+1) % TX492X_NR_CHAIN]);
		set_tx49xx_chdma_chain(chain[idx+1], chain[(idx+2) % TX492X_NR_CHAIN]);

		set_tx49xx_chdma_addr64(chain[idx], dev_dma_addr, buf1);
		set_tx49xx_chdma_addr64(chain[idx+1], dev_dma_addr, buf2);

		set_tx49xx_chdma_count(chain[idx  ], db->dma_fragsize);
		set_tx49xx_chdma_count(chain[idx+1], db->dma_fragsize);
	}
	db->lvi      = TX492X_NR_CHAIN - 1;
	set_tx49xx_chdma_chain(chain[db->lvi], 0);
	db->stopped = 0;

	/* Tx49xx start DMA */
	set_tx49xx_dma_chain(db->dmanr, chain[0]);
	ACLCREG(s, ACCTLEN) = ACCTL_AUDIDMA;

	spin_unlock_irqrestore(&s->lock, flags);

}

/* --------------------------------------------------------------------- */

#define DMABUF_DEFAULTORDER (17-PAGE_SHIFT)
#define DMABUF_MINORDER 1

static inline void dealloc_dmabuf(struct tx49xx_state *s, struct dmabuf *db)
{
	struct page    *page, *pend;

	if (db->rawbuf) {
		/* undo marking the pages as reserved */
		pend = virt_to_page(db->rawbuf +
				    (PAGE_SIZE << db->buforder) - 1);
		for (page = virt_to_page(db->rawbuf); page <= pend; page++)
			mem_map_unreserve(page);
		dma_free(PAGE_SIZE << db->buforder, db->rawbuf, db->dmaaddr);
	}
	db->rawbuf = db->nextIn = db->nextOut = NULL;
	db->mapped = db->ready = 0;
}

static int prog_dmabuf(struct tx49xx_state *s, struct dmabuf *db)
{
	int             order;
	unsigned user_bytes_per_sec;
	unsigned        bufs;
	struct page    *page, *pend;
	unsigned        rate = db->sample_rate;
#ifdef MPU_DEBUG
	printk("prog dmabuf\n");
#endif
	if (!db->rawbuf) {
#ifdef MPU_DEBUG
	printk(" !raw_buf \n");
#endif		
		db->ready = db->mapped = 0;
		for (order = DMABUF_DEFAULTORDER;
		     order >= DMABUF_MINORDER; order--)
			if ((db->rawbuf = dma_alloc(PAGE_SIZE << order,
						    &db->dmaaddr)))
				break;
		if (!db->rawbuf)
			return -ENOMEM;
		db->buforder = order;
		/* now mark the pages as reserved;
		   otherwise remap_page_range doesn't do what we want */
		pend = virt_to_page(db->rawbuf +
				    (PAGE_SIZE << db->buforder) - 1);
		for (page = virt_to_page(db->rawbuf); page <= pend; page++)
			mem_map_reserve(page);
	}
	
	db->cnt_factor = 1;
	if (db->sample_size == 8)
		db->cnt_factor *= 2;
	if (db->num_channels == 1)
		db->cnt_factor *= 2;
	db->cnt_factor *= db->src_factor;

	db->count = 0;
	db->nextIn = db->nextOut = db->rawbuf;

	db->user_bytes_per_sample = (db->sample_size>>3) * db->num_channels;
	db->dma_bytes_per_sample = 2 * ((db->num_channels == 1) ?
					2 : db->num_channels);

	user_bytes_per_sec = rate * db->user_bytes_per_sample;
	bufs = PAGE_SIZE << db->buforder;
	if (db->ossfragshift) {
		if ((1000 << db->ossfragshift) < user_bytes_per_sec)
			db->fragshift = ld2(user_bytes_per_sec/1000);
		else
			db->fragshift = db->ossfragshift;
	} else {
		db->fragshift = ld2(user_bytes_per_sec / 100 /
				    (db->subdivision ? db->subdivision : 1));
		if (db->fragshift < 3)
			db->fragshift = 3;
	}

	db->fragsize = 1 << db->fragshift;
	db->dma_fragsize = db->fragsize * db->cnt_factor;
	db->numfrag = bufs / db->dma_fragsize;
#ifdef MPU_DEBUG
	printk("fragsize = %d, fragshift = %d, dma_fragsize = %d, cnt_factor = %d, numfrag = %d\n", db->fragsize, db->fragshift, db->dma_fragsize, db->cnt_factor, db->numfrag);
#endif

	while (db->numfrag < 4 && db->fragshift > 3) {
		db->fragshift--;
		db->fragsize = 1 << db->fragshift;
		db->dma_fragsize = db->fragsize * db->cnt_factor;
		db->numfrag = bufs / db->dma_fragsize;
	}

	if (db->ossmaxfrags >= 4 && db->ossmaxfrags < db->numfrag)
		db->numfrag = db->ossmaxfrags;

	db->dmasize = db->dma_fragsize * db->numfrag;
	memset(db->rawbuf, 0, bufs);

#ifdef MPU_DEBUG
	dbg("rate=%d, samplesize=%d, channels=%d",
	    rate, db->sample_size, db->num_channels);
	dbg("fragsize=%d, cnt_factor=%d, dma_fragsize=%d",
	    db->fragsize, db->cnt_factor, db->dma_fragsize);
	dbg("numfrag=%d, dmasize=%d", db->numfrag, db->dmasize);
#endif

	db->ready = 1;
	return 0;
}

static inline int prog_dmabuf_adc(struct tx49xx_state *s)
{
	stop_adc(s);
	return prog_dmabuf(s, &s->dma_adc);

}

static inline int prog_dmabuf_dac(struct tx49xx_state *s)
{
	stop_dac(s);
	return prog_dmabuf(s, &s->dma_dac);
}

/* hold spinlock for the following */
static void dac_dma_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	struct tx49xx_state *s = (struct tx49xx_state *) dev_id;
	struct dmabuf  *db = &s->dma_dac;
	int nlvi;
	dma_addr_t buf_dma_addr;
	unsigned long long dev_dma_addr = s->baseaddr_phys + ACLC_REG_ACAUDODAT;

	spin_lock(&s->lock);

	nlvi = (db->lvi + 1) % TX492X_NR_CHAIN;
	if (get_tx49xx_dma_chain(db->dmanr) == s->chain[((nlvi + 1) % TX492X_NR_CHAIN)] &&
	    (get_tx49xx_dma_status(db->dmanr) & TX4927_DMA_CSR_CHNEN)) {
		goto _finish;
	}
	
	db->nextOut += db->dma_fragsize;
	if (db->nextOut >= db->rawbuf + db->dmasize)
		db->nextOut -= db->dmasize;

	/* update playback pointers */
	buf_dma_addr = kseg_to_phys(db->nextOut) + db->dma_fragsize ;
	if (buf_dma_addr >= db->dmaaddr + db->dmasize)
		buf_dma_addr -= db->dmasize;

#ifdef MPU_DEBUG
	printk("0x%x-0x%x\n", buf_dma_addr, db->dma_fragsize );
#endif
	db->count -= db->dma_fragsize;
	db->total_bytes += db->dma_fragsize;

	if (db->count <= 0) {
		spin_unlock(&s->lock);
		stop_dac(s);
		spin_lock(&s->lock);
		db->count = 0;
		db->nextIn = db->nextOut;
	}else{
		set_tx49xx_chdma_addr64(s->chain[nlvi], buf_dma_addr, dev_dma_addr);
		set_tx49xx_chdma_chain(s->chain[nlvi], 0);
		set_tx49xx_chdma_chain(s->chain[db->lvi], s->chain[nlvi]);
		if (!(get_tx49xx_dma_status(db->dmanr) & TX4927_DMA_CSR_CHNEN)) {
			set_tx49xx_dma_chain(db->dmanr,
					     get_tx49xx_chdma_chain(get_tx49xx_dma_chain(db->dmanr)));
		}
		db->lvi = nlvi;
	}
 _finish:

	/* wake up anybody listening */
	if (waitqueue_active(&db->wait))
		wake_up(&db->wait);

	spin_unlock(&s->lock);
	clear_tx49xx_dma_status(db->dmanr);

}


static void adc_dma_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	struct tx49xx_state *s = (struct tx49xx_state *) dev_id;
	struct dmabuf  *db = &s->dma_adc;
	int nlvi;
	dma_addr_t buf_dma_addr;
	unsigned long long dev_dma_addr = s->baseaddr_phys + ACLC_REG_ACAUDIDAT;


	spin_lock(&s->lock);

	nlvi = (db->lvi + 1) % TX492X_NR_CHAIN;
	if (get_tx49xx_dma_chain(db->dmanr) == s->chain[((nlvi + 1) % TX492X_NR_CHAIN)] &&
	    (get_tx49xx_dma_status(db->dmanr) & TX4927_DMA_CSR_CHNEN)) {
		goto _finish;
	}

	if (db->count + db->dma_fragsize > db->dmasize) {
		// Overrun. Stop ADC and log the error
		spin_unlock(&s->lock);
		stop_adc(s);
		clear_tx49xx_dma_status(db->dmanr);
		db->error++;
		err("adc overrun");
		return;
	}

	db->nextIn += db->dma_fragsize;
	if (db->nextIn >= db->rawbuf + db->dmasize)
		db->nextIn -= db->dmasize;

	/* update capture pointers */
	buf_dma_addr = kseg_to_phys(db->nextIn) + db->dma_fragsize;
	if (buf_dma_addr >= db->dmaaddr + db->dmasize)
		buf_dma_addr -= db->dmasize;

	db->count += db->dma_fragsize;
	db->total_bytes += db->dma_fragsize;
#ifdef MPU_DEBUG
	printk("[%d,%d,%d]-", db->count, db->total_bytes, db->dma_fragsize);
#endif
	set_tx49xx_chdma_addr64(s->chain[nlvi], dev_dma_addr, buf_dma_addr);
	set_tx49xx_chdma_chain(s->chain[nlvi], 0);
	set_tx49xx_chdma_chain(s->chain[db->lvi], s->chain[nlvi]);

	if (!(get_tx49xx_dma_status(db->dmanr) & TX4927_DMA_CSR_CHNEN)) {
		set_tx49xx_dma_chain(db->dmanr,
				     get_tx49xx_chdma_chain(get_tx49xx_dma_chain(db->dmanr)));
	}
	
	db->lvi = nlvi;
 _finish:

	/* wake up anybody listening */
	if (waitqueue_active(&db->wait))
		wake_up(&db->wait);

	spin_unlock(&s->lock);
	clear_tx49xx_dma_status(db->dmanr);
}

static void snd_tx49xx_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	struct tx49xx_state *s = (struct tx49xx_state *) dev_id;

	if (irq == s->irq_aclc) {
		/* no handling */
	} else if (irq == s->irq_pbk) {
		dac_dma_interrupt(irq, dev_id, regs);
	} else if (irq == s->irq_cap) {
		adc_dma_interrupt(irq, dev_id, regs);
	} else {
		printk("spurious interrupt %d.\n", irq);
	}
}

/* --------------------------------------------------------------------- */

static loff_t tx49xx_llseek(struct file *file, loff_t offset, int origin)
{
	return -ESPIPE;
}


static int tx49xx_open_mixdev(struct inode *inode, struct file *file)
{
	file->private_data = &tx49xx_state;
	return 0;
}

static int tx49xx_release_mixdev(struct inode *inode, struct file *file)
{
	return 0;
}

static int mixdev_ioctl(struct ac97_codec *codec, unsigned int cmd,
                        unsigned long arg)
{
	return codec->mixer_ioctl(codec, cmd, arg);
}

static int tx49xx_ioctl_mixdev(struct inode *inode, struct file *file,
			       unsigned int cmd, unsigned long arg)
{
	struct tx49xx_state *s = (struct tx49xx_state *)file->private_data;
	struct ac97_codec *codec = &s->codec;

	return mixdev_ioctl(codec, cmd, arg);
}

static /*const */ struct file_operations tx49xx_mixer_fops = {
	owner:THIS_MODULE,
	llseek:tx49xx_llseek,
	ioctl:tx49xx_ioctl_mixdev,
	open:tx49xx_open_mixdev,
	release:tx49xx_release_mixdev,
};

/* --------------------------------------------------------------------- */

static int drain_dac(struct tx49xx_state *s, int nonblock)
{
	unsigned long   flags;
	int             count, tmo;

	if (s->dma_dac.mapped || !s->dma_dac.ready || s->dma_dac.stopped)
		return 0;

	for (;;) {
		spin_lock_irqsave(&s->lock, flags);
		count = s->dma_dac.count;
		spin_unlock_irqrestore(&s->lock, flags);
		if (count <= 0)
			break;
		if (signal_pending(current))
			break;
		if (nonblock)
			return -EBUSY;
		tmo = 1000 * count / (s->no_vra ?
				      48000 : s->dma_dac.sample_rate);
		tmo /= s->dma_dac.dma_bytes_per_sample;
		tx49xx_delay(tmo);
	}

	if (signal_pending(current))
		return -ERESTARTSYS;
	return 0;
}

/* --------------------------------------------------------------------- */

static inline u8 S16_TO_U8(s16 ch)
{
	return (u8) (ch >> 8) + 0x80;
}
static inline s16 U8_TO_S16(u8 ch)
{
	return (s16) (ch - 0x80) << 8;
}

/*
 * Translates user samples to dma buffer suitable for AC'97 DAC data:
 *     If mono, copy left channel to right channel in dma buffer.
 *     If 8 bit samples, cvt to 16-bit before writing to dma buffer.
 *     If interpolating (no VRA), duplicate every audio frame src_factor times.
 */
 
#ifdef MPU_DEBUG
int first = 0;
#endif
static int translate_from_user(struct dmabuf *db,
			       char* dmabuf,
			       char* userbuf,
			       int dmacount)
{
	int             sample, i;
	int             interp_bytes_per_sample;
	int             num_samples;
	int             mono = (db->num_channels == 1);
	char            usersample[12];
	s16             ch, dmasample[6];

	if (db->sample_size == 16 && !mono && db->src_factor == 1) {
		// no translation necessary, just copy
		if (copy_from_user(dmabuf, userbuf, dmacount))
			return -EFAULT;
		return dmacount;
	}
	
	interp_bytes_per_sample = db->dma_bytes_per_sample * db->src_factor;
	num_samples = dmacount / interp_bytes_per_sample;

#ifdef MPU_DEBUG
	if ( first == 0 ){
		printk("translate_from_user sample = %d, mono = %d, src_factor = %d\n", db->sample_size, mono, db->src_factor);
		printk("interp bytes/sample = %d, dma bytes/sample = %d, num_samples = %d\n", interp_bytes_per_sample, db->dma_bytes_per_sample, num_samples);
		first = 1;
	}
#endif

	
	
	for (sample = 0; sample < num_samples; sample++) {
		if (copy_from_user(usersample, userbuf,
				   db->user_bytes_per_sample)) {
			dbg("fault");
			return -EFAULT;
		}

		for (i = 0; i < db->num_channels; i++) {
			if (db->sample_size == 8)
				ch = U8_TO_S16(usersample[i]);
			else
				ch = *((s16 *) (&usersample[i * 2]));
			dmasample[i] = ch;
			if (mono)
				dmasample[i + 1] = ch;	// right channel
		}

		// duplicate every audio frame src_factor times
		for (i = 0; i < db->src_factor; i++)
			memcpy(dmabuf, dmasample, db->dma_bytes_per_sample);

		userbuf += db->user_bytes_per_sample;
		dmabuf += interp_bytes_per_sample;
	}

	return num_samples * interp_bytes_per_sample;
}

/*
 * Translates AC'97 ADC samples to user buffer:
 *     If mono, send only left channel to user buffer.
 *     If 8 bit samples, cvt from 16 to 8 bit before writing to user buffer.
 *     If decimating (no VRA), skip over src_factor audio frames.
 */
static int translate_to_user(struct dmabuf *db,
			     char* userbuf,
			     char* dmabuf,
			     int dmacount)
{
	int             sample, i;
	int             interp_bytes_per_sample;
	int             num_samples;
	int             mono = (db->num_channels == 1);
	char            usersample[12];

	if (db->sample_size == 16 && !mono && db->src_factor == 1) {
		// no translation necessary, just copy
		if (copy_to_user(userbuf, dmabuf, dmacount))
			return -EFAULT;
		return dmacount;
	}

	interp_bytes_per_sample = db->dma_bytes_per_sample * db->src_factor;
	num_samples = dmacount / interp_bytes_per_sample;

	for (sample = 0; sample < num_samples; sample++) {
		for (i = 0; i < db->num_channels; i++) {
			if (db->sample_size == 8)
				usersample[i] =
					S16_TO_U8(*((s16 *) (&dmabuf[i * 2])));
			else
				*((s16 *) (&usersample[i * 2])) =
					*((s16 *) (&dmabuf[i * 2]));
		}

		if (copy_to_user(userbuf, usersample,
				 db->user_bytes_per_sample)) {
			dbg("fault");
			return -EFAULT;
		}

		userbuf += db->user_bytes_per_sample;
		dmabuf += interp_bytes_per_sample;
	}

	return num_samples * interp_bytes_per_sample;
}

/*
 * Copy audio data to/from user buffer from/to dma buffer, taking care
 * that we wrap when reading/writing the dma buffer. Returns actual byte
 * count written to or read from the dma buffer.
 */
static int copy_dmabuf_user(struct dmabuf *db, char* userbuf,
			    int count, int to_user)
{
	char           *bufptr = to_user ? db->nextOut : db->nextIn;
	char           *bufend = db->rawbuf + db->dmasize;
	int             cnt, ret;

	if (bufptr + count > bufend) {
		int             partial = (int) (bufend - bufptr);
		if (to_user) {
			if ((cnt = translate_to_user(db, userbuf,
						     bufptr, partial)) < 0)
				return cnt;
			ret = cnt;
			if ((cnt = translate_to_user(db, userbuf + partial,
						     db->rawbuf,
						     count - partial)) < 0)
				return cnt;
			ret += cnt;
		} else {
			if ((cnt = translate_from_user(db, bufptr, userbuf,
						       partial)) < 0)
				return cnt;
			ret = cnt;
			if ((cnt = translate_from_user(db, db->rawbuf,
						       userbuf + partial,
						       count - partial)) < 0)
				return cnt;
			ret += cnt;
		}
	} else {
		if (to_user)
			ret = translate_to_user(db, userbuf, bufptr, count);
		else
			ret = translate_from_user(db, bufptr, userbuf, count);
	}

	return ret;
}


static ssize_t tx49xx_read(struct file *file, char *buffer,
			   size_t count, loff_t *ppos)
{
	struct tx49xx_state *s = (struct tx49xx_state *)file->private_data;
	struct dmabuf  *db = &s->dma_adc;
	DECLARE_WAITQUEUE(wait, current);
	ssize_t         ret;
	unsigned long   flags;
	int             cnt, usercnt, avail;

	if (ppos != &file->f_pos)
		return -ESPIPE;
	if (db->mapped)
		return -ENXIO;
	if (!access_ok(VERIFY_WRITE, buffer, count))
		return -EFAULT;
	ret = 0;

	count *= db->cnt_factor;

	down(&s->sem);
	add_wait_queue(&db->wait, &wait);
	
	while (count > 0) {
		// wait for samples in ADC dma buffer
		do {
			if (db->stopped)
				start_adc(s);
			spin_lock_irqsave(&s->lock, flags);
			avail = db->count;
			if (avail <= 0)
				__set_current_state(TASK_INTERRUPTIBLE);
			spin_unlock_irqrestore(&s->lock, flags);
			if (avail <= 0) {
				if (file->f_flags & O_NONBLOCK) {
					if (!ret)
						ret = -EAGAIN;
					goto out;
				}
				up(&s->sem);
				schedule();
				if (signal_pending(current)) {
					if (!ret)
						ret = -ERESTARTSYS;
					goto out2;
				}
				down(&s->sem);
			}
		
		} while (avail <= 0);

		// copy from nextOut to user
		if ((cnt = copy_dmabuf_user(db, buffer,
					    count > avail ?
					    avail : count, 1)) < 0) {
			if (!ret)
				ret = -EFAULT;
			goto out;
		}
	
		spin_lock_irqsave(&s->lock, flags);
		db->count -= cnt;
		db->nextOut += cnt;
		if (db->nextOut >= db->rawbuf + db->dmasize)
			db->nextOut -= db->dmasize;
		spin_unlock_irqrestore(&s->lock, flags);

		count -= cnt;
		usercnt = cnt / db->cnt_factor;
		buffer += usercnt;
		ret += usercnt;
	}

out:
	up(&s->sem);
out2:
	remove_wait_queue(&db->wait, &wait);
	set_current_state(TASK_RUNNING);
	return ret;
}

static ssize_t tx49xx_write(struct file *file, const char *buffer,
	     		    size_t count, loff_t * ppos)
{
	struct tx49xx_state *s = (struct tx49xx_state *)file->private_data;
	struct dmabuf  *db = &s->dma_dac;
	DECLARE_WAITQUEUE(wait, current);
	ssize_t         ret = 0;
	unsigned long   flags;
	int             cnt, usercnt, avail;

#ifdef MPU_DEBUG
	dbg("write: count=%d", count);
#endif
	
	if (ppos != &file->f_pos)
		return -ESPIPE;
	if (db->mapped)
		return -ENXIO;
	if (!access_ok(VERIFY_READ, buffer, count))
		return -EFAULT;

	count *= db->cnt_factor;
	if ( s->ac97_status & SPDIF_ON ) 
		wrcodec( &s->codec, 0x2a, 0x0405 );

	down(&s->sem);	
	add_wait_queue(&db->wait, &wait);

	while (count > 0) {
		// wait for space in playback buffer
		do {
			spin_lock_irqsave(&s->lock, flags);
			avail = (int) db->dmasize - db->count;
			if (avail <= 0)
				__set_current_state(TASK_INTERRUPTIBLE);
			spin_unlock_irqrestore(&s->lock, flags);
			if (avail <= 0) {
				if (file->f_flags & O_NONBLOCK) {
					if (!ret)
						ret = -EAGAIN;
					goto out;
				}
				up(&s->sem);
				schedule();
				if (signal_pending(current)) {
					if (!ret)
						ret = -ERESTARTSYS;
					goto out2;
				}
				down(&s->sem);
			}
		} while (avail <= 0);

		// copy from user to nextIn
		if ((cnt = copy_dmabuf_user(db, (char *) buffer,
					    count > avail ?
					    avail : count, 0)) < 0) {
			if (!ret)
				ret = -EFAULT;
			goto out;
		}
#if 0
if ( first == 0 ){
	printk("\n ACLC Reg DUMP\n");
	printk("ACCTLEN=0x%08lx, ACREGACC=0x%08lx, ACINTMSTS=0x%08lx, ACINTSTS=0x%08lx\n", ACLCREG(s, ACCTLEN),ACLCREG(s, ACREGACC), ACLCREG(s, ACINTMSTS), ACLCREG(s, ACINTSTS));
	printk("ACINTEN=0x%08lx, ACDMASTS=0x%08lx, ACAUDODAT=0x%08lx, ACREVID=0x%08lx\n", ACLCREG(s, ACINTEN), ACLCREG(s, ACDMASTS), ACLCREG(s, ACAUDODAT), ACLCREG(s, ACREVID));
	printk("ACSLTEN=0x%08lx, ACSLTDIS=0x%08lx\n", ACLCREG(s, ACSLTEN), ACLCREG(s, ACSLTDIS));

	printk("AC97 CODEC Registers\n");
	for ( i = 0 ; i < 6 ; i ++ ){
		for ( j = 0 ; j < 8 ; j ++, regcnt += 2 ){
			printk("0x%04x, ", rdcodec(&s->codec, regcnt));
		}
		printk("\n");
	}
	first = 1;
}
	
#endif
		spin_lock_irqsave(&s->lock, flags);
		db->count += cnt;
		db->nextIn += cnt;
		if (db->nextIn >= db->rawbuf + db->dmasize)
			db->nextIn -= db->dmasize;
		spin_unlock_irqrestore(&s->lock, flags);
		if (db->stopped)
			start_dac(s);

		count -= cnt;
		usercnt = cnt / db->cnt_factor;
		buffer += usercnt;
		ret += usercnt;
	}			// while (count > 0)

out:
	up(&s->sem);
out2:
	remove_wait_queue(&db->wait, &wait);
	set_current_state(TASK_RUNNING);
	return ret;
}


/* No kernel lock - we have our own spinlock */
static unsigned int tx49xx_poll(struct file *file,
				struct poll_table_struct *wait)
{
	struct tx49xx_state *s = (struct tx49xx_state *)file->private_data;
	unsigned long   flags;
	unsigned int    mask = 0;

	if (file->f_mode & FMODE_WRITE) {
		if (!s->dma_dac.ready)
			return 0;
		poll_wait(file, &s->dma_dac.wait, wait);
	}
	if (file->f_mode & FMODE_READ) {
		if (!s->dma_adc.ready)
			return 0;
		poll_wait(file, &s->dma_adc.wait, wait);
	}

	spin_lock_irqsave(&s->lock, flags);
	
	if (file->f_mode & FMODE_READ) {
		if (s->dma_adc.count >= (signed)s->dma_adc.dma_fragsize)
			mask |= POLLIN | POLLRDNORM;
	}
	if (file->f_mode & FMODE_WRITE) {
		if (s->dma_dac.mapped) {
			if (s->dma_dac.count >=
			    (signed)s->dma_dac.dma_fragsize) 
				mask |= POLLOUT | POLLWRNORM;
		} else {
			if ((signed) s->dma_dac.dmasize >=
			    s->dma_dac.count + (signed)s->dma_dac.dma_fragsize)
				mask |= POLLOUT | POLLWRNORM;
		}
	}
	spin_unlock_irqrestore(&s->lock, flags);
	return mask;
}

static int tx49xx_mmap(struct file *file, struct vm_area_struct *vma)
{
	struct tx49xx_state *s = (struct tx49xx_state *)file->private_data;
	struct dmabuf  *db;
	unsigned long   size;
	int ret = 0;
	
	lock_kernel();
	down(&s->sem);
	if (vma->vm_flags & VM_WRITE)
		db = &s->dma_dac;
	else if (vma->vm_flags & VM_READ)
		db = &s->dma_adc;
	else {
		ret = -EINVAL;
		goto out;
	}
	if (vma->vm_pgoff != 0) {
		ret = -EINVAL;
		goto out;
	}
	size = vma->vm_end - vma->vm_start;
	if (size > (PAGE_SIZE << db->buforder)) {
		ret = -EINVAL;
		goto out;
	}
	if (remap_page_range(vma->vm_start, kseg_to_phys(db->rawbuf),
			     size, vma->vm_page_prot)) {
		ret = -EAGAIN;
		goto out;
	}
	vma->vm_flags &= ~VM_IO;
	db->mapped = 1;
out:
	up(&s->sem);
	unlock_kernel();
	return ret;
}


#ifdef MPU_DEBUG
static struct ioctl_str_t {
	unsigned int    cmd;
	const char     *str;
} ioctl_str[] = {
	{SNDCTL_DSP_RESET, "SNDCTL_DSP_RESET"},
	{SNDCTL_DSP_SYNC, "SNDCTL_DSP_SYNC"},
	{SNDCTL_DSP_SPEED, "SNDCTL_DSP_SPEED"},
	{SNDCTL_DSP_STEREO, "SNDCTL_DSP_STEREO"},
	{SNDCTL_DSP_GETBLKSIZE, "SNDCTL_DSP_GETBLKSIZE"},
	{SNDCTL_DSP_SAMPLESIZE, "SNDCTL_DSP_SAMPLESIZE"},
	{SNDCTL_DSP_CHANNELS, "SNDCTL_DSP_CHANNELS"},
	{SOUND_PCM_WRITE_CHANNELS, "SOUND_PCM_WRITE_CHANNELS"},
	{SOUND_PCM_WRITE_FILTER, "SOUND_PCM_WRITE_FILTER"},
	{SNDCTL_DSP_POST, "SNDCTL_DSP_POST"},
	{SNDCTL_DSP_SUBDIVIDE, "SNDCTL_DSP_SUBDIVIDE"},
	{SNDCTL_DSP_SETFRAGMENT, "SNDCTL_DSP_SETFRAGMENT"},
	{SNDCTL_DSP_GETFMTS, "SNDCTL_DSP_GETFMTS"},
	{SNDCTL_DSP_SETFMT, "SNDCTL_DSP_SETFMT"},
	{SNDCTL_DSP_GETOSPACE, "SNDCTL_DSP_GETOSPACE"},
	{SNDCTL_DSP_GETISPACE, "SNDCTL_DSP_GETISPACE"},
	{SNDCTL_DSP_NONBLOCK, "SNDCTL_DSP_NONBLOCK"},
	{SNDCTL_DSP_GETCAPS, "SNDCTL_DSP_GETCAPS"},
	{SNDCTL_DSP_GETTRIGGER, "SNDCTL_DSP_GETTRIGGER"},
	{SNDCTL_DSP_SETTRIGGER, "SNDCTL_DSP_SETTRIGGER"},
	{SNDCTL_DSP_GETIPTR, "SNDCTL_DSP_GETIPTR"},
	{SNDCTL_DSP_GETOPTR, "SNDCTL_DSP_GETOPTR"},
	{SNDCTL_DSP_MAPINBUF, "SNDCTL_DSP_MAPINBUF"},
	{SNDCTL_DSP_MAPOUTBUF, "SNDCTL_DSP_MAPOUTBUF"},
	{SNDCTL_DSP_SETSYNCRO, "SNDCTL_DSP_SETSYNCRO"},
	{SNDCTL_DSP_SETDUPLEX, "SNDCTL_DSP_SETDUPLEX"},
	{SNDCTL_DSP_GETODELAY, "SNDCTL_DSP_GETODELAY"},
	{SNDCTL_DSP_GETCHANNELMASK, "SNDCTL_DSP_GETCHANNELMASK"},
	{SNDCTL_DSP_BIND_CHANNEL, "SNDCTL_DSP_BIND_CHANNEL"},
	{OSS_GETVERSION, "OSS_GETVERSION"},
	{SOUND_PCM_READ_RATE, "SOUND_PCM_READ_RATE"},
	{SOUND_PCM_READ_CHANNELS, "SOUND_PCM_READ_CHANNELS"},
	{SOUND_PCM_READ_BITS, "SOUND_PCM_READ_BITS"},
	{SOUND_PCM_READ_FILTER, "SOUND_PCM_READ_FILTER"}
};
#endif

// Need to hold a spin-lock before calling this!
static int dma_count_done(struct dmabuf *db)
{
	if (db->stopped)
		return 0;

	return db->dma_fragsize - get_tx49xx_dma_residue(db->dmanr);
}



static int tx49xx_ioctl(struct inode *inode, struct file *file,
                        unsigned int cmd, unsigned long arg)
{
	struct tx49xx_state *s = (struct tx49xx_state *)file->private_data;
	unsigned long   flags;
	audio_buf_info  abinfo;
	count_info      cinfo;
	int             count;
	int             val, mapped, ret, diff;

	mapped = ((file->f_mode & FMODE_WRITE) && s->dma_dac.mapped) ||
		((file->f_mode & FMODE_READ) && s->dma_adc.mapped);

#ifdef MPU_DEBUG
	for (count=0; count<sizeof(ioctl_str)/sizeof(ioctl_str[0]); count++) {
		if (ioctl_str[count].cmd == cmd)
			break;
	}
	if (count < sizeof(ioctl_str) / sizeof(ioctl_str[0]))
		dbg("ioctl %s, arg=0x%lx", ioctl_str[count].str, arg);
	else
		dbg("ioctl 0x%x unknown, arg=0x%lx", cmd, arg);
#endif

	switch (cmd) {
	case OSS_GETVERSION:
		return put_user(SOUND_VERSION, (int *) arg);

	case SNDCTL_DSP_SYNC:
		if (file->f_mode & FMODE_WRITE)
			return drain_dac(s, file->f_flags & O_NONBLOCK);
		return 0;

	case SNDCTL_DSP_SETDUPLEX:
		return 0;

	case SNDCTL_DSP_GETCAPS:
		return put_user(DSP_CAP_DUPLEX | DSP_CAP_REALTIME |
				DSP_CAP_TRIGGER | DSP_CAP_MMAP, (int *)arg);

	case SNDCTL_DSP_RESET:
		if (file->f_mode & FMODE_WRITE) {
			stop_dac(s);
			synchronize_irq();
			s->dma_dac.count = s->dma_dac.total_bytes = 0;
			s->dma_dac.nextIn = s->dma_dac.nextOut =
				s->dma_dac.rawbuf;
		}
		if (file->f_mode & FMODE_READ) {
			stop_adc(s);
			synchronize_irq();
			s->dma_adc.count = s->dma_adc.total_bytes = 0;
			s->dma_adc.nextIn = s->dma_adc.nextOut =
				s->dma_adc.rawbuf;
		}
		return 0;

	case SNDCTL_DSP_SPEED:
		if (get_user(val, (int *) arg))
			return -EFAULT;
		if (val >= 0) {
			if (file->f_mode & FMODE_READ) {
				stop_adc(s);
				set_adc_rate(s, val);
			}
			if (file->f_mode & FMODE_WRITE) {
				stop_dac(s);
				set_dac_rate(s, val);
			}
			if (s->open_mode & FMODE_READ)
				if ((ret = prog_dmabuf_adc(s)))
					return ret;
			if (s->open_mode & FMODE_WRITE)
				if ((ret = prog_dmabuf_dac(s)))
					return ret;
		}
		return put_user((file->f_mode & FMODE_READ) ?
				s->dma_adc.sample_rate :
				s->dma_dac.sample_rate,
				(int *)arg);

	case SNDCTL_DSP_STEREO:
		if (get_user(val, (int *) arg))
			return -EFAULT;
		if (file->f_mode & FMODE_READ) {
			stop_adc(s);
			s->dma_adc.num_channels = val ? 2 : 1;
			if ((ret = prog_dmabuf_adc(s)))
				return ret;
		}
		if (file->f_mode & FMODE_WRITE) {
			stop_dac(s);
			s->dma_dac.num_channels = val ? 2 : 1;
			if (s->codec_ext_caps & AC97_EXT_DACS) {
				// disable surround and center/lfe in AC'97
				u16 ext_stat = rdcodec(&s->codec,
						       AC97_EXTENDED_STATUS);
				wrcodec(&s->codec, AC97_EXTENDED_STATUS,
					ext_stat | (AC97_EXTSTAT_PRI |
						    AC97_EXTSTAT_PRJ |
						    AC97_EXTSTAT_PRK));
			}
			if ((ret = prog_dmabuf_dac(s)))
				return ret;
		}
		return 0;

	case SNDCTL_DSP_CHANNELS:
		if (get_user(val, (int *) arg))
			return -EFAULT;
		if (val != 0) {
			if (file->f_mode & FMODE_READ) {
				if (val < 0 || val > 2)
					return -EINVAL;
				stop_adc(s);
				s->dma_adc.num_channels = val;
				if ((ret = prog_dmabuf_adc(s)))
					return ret;
			}
			if (file->f_mode & FMODE_WRITE) {
				switch (val) {
				case 1:
				case 2:
					break;
				case 3:
				case 5:
					return -EINVAL;
				case 4:
					if (!(s->codec_ext_caps &
					      AC97_EXTID_SDAC))
						return -EINVAL;
					break;
				case 6:
					if ((s->codec_ext_caps &
					     AC97_EXT_DACS) != AC97_EXT_DACS)
						return -EINVAL;
					break;
				default:
					return -EINVAL;
				}

				stop_dac(s);
				if (val <= 2 &&
				    (s->codec_ext_caps & AC97_EXT_DACS)) {
					// disable surround and center/lfe
					// channels in AC'97
					u16             ext_stat =
						rdcodec(&s->codec,
							AC97_EXTENDED_STATUS);
					wrcodec(&s->codec,
						AC97_EXTENDED_STATUS,
						ext_stat | (AC97_EXTSTAT_PRI |
							    AC97_EXTSTAT_PRJ |
							    AC97_EXTSTAT_PRK));
				} else if (val >= 4) {
					// enable surround, center/lfe
					// channels in AC'97
					u16             ext_stat =
						rdcodec(&s->codec,
							AC97_EXTENDED_STATUS);
					ext_stat &= ~AC97_EXTSTAT_PRJ;
					if (val == 6)
						ext_stat &=
							~(AC97_EXTSTAT_PRI |
							  AC97_EXTSTAT_PRK);
					wrcodec(&s->codec,
						AC97_EXTENDED_STATUS,
						ext_stat);
				}

				s->dma_dac.num_channels = val;
				if ((ret = prog_dmabuf_dac(s)))
					return ret;
			}
		}
		return put_user(val, (int *) arg);

	case SNDCTL_DSP_GETFMTS:	/* Returns a mask */
		return put_user(AFMT_S16_LE | AFMT_U8, (int *) arg);

	case SNDCTL_DSP_SETFMT:	/* Selects ONE fmt */
		if (get_user(val, (int *) arg))
			return -EFAULT;
		if (val != AFMT_QUERY) {
			if (file->f_mode & FMODE_READ) {
				stop_adc(s);
				if (val == AFMT_S16_LE)
					s->dma_adc.sample_size = 16;
				else {
					val = AFMT_U8;
					s->dma_adc.sample_size = 8;
				}
				if ((ret = prog_dmabuf_adc(s)))
					return ret;
			}
			if (file->f_mode & FMODE_WRITE) {
				stop_dac(s);
				if (val == AFMT_S16_LE)
					s->dma_dac.sample_size = 16;
				else {
					val = AFMT_U8;
					s->dma_dac.sample_size = 8;
				}
				if ((ret = prog_dmabuf_dac(s)))
					return ret;
			}
		} else {
			if (file->f_mode & FMODE_READ)
				val = (s->dma_adc.sample_size == 16) ?
					AFMT_S16_LE : AFMT_U8;
			else
				val = (s->dma_dac.sample_size == 16) ?
					AFMT_S16_LE : AFMT_U8;
		}
		return put_user(val, (int *) arg);

	case SNDCTL_DSP_POST:
		return 0;

	case SNDCTL_DSP_GETTRIGGER:
		val = 0;
		spin_lock_irqsave(&s->lock, flags);
		if (file->f_mode & FMODE_READ && !s->dma_adc.stopped)
			val |= PCM_ENABLE_INPUT;
		if (file->f_mode & FMODE_WRITE && !s->dma_dac.stopped)
			val |= PCM_ENABLE_OUTPUT;
		spin_unlock_irqrestore(&s->lock, flags);
		return put_user(val, (int *) arg);

	case SNDCTL_DSP_SETTRIGGER:
		if (get_user(val, (int *) arg))
			return -EFAULT;
		if (file->f_mode & FMODE_READ) {
			if (val & PCM_ENABLE_INPUT)
				start_adc(s);
			else
				stop_adc(s);
		}
		if (file->f_mode & FMODE_WRITE) {
			if (val & PCM_ENABLE_OUTPUT)
				start_dac(s);
			else
				stop_dac(s);
		}
		return 0;

	case SNDCTL_DSP_GETOSPACE:
		if (!(file->f_mode & FMODE_WRITE))
			return -EINVAL;
		abinfo.fragsize = s->dma_dac.fragsize;
		spin_lock_irqsave(&s->lock, flags);
		count = s->dma_dac.count;
		count -= dma_count_done(&s->dma_dac);
		spin_unlock_irqrestore(&s->lock, flags);
		if (count < 0)
			count = 0;
		abinfo.bytes = (s->dma_dac.dmasize - count) /
			s->dma_dac.cnt_factor;
		abinfo.fragstotal = s->dma_dac.numfrag;
		abinfo.fragments = abinfo.bytes >> s->dma_dac.fragshift;
#ifdef MPU_DEBUG
		dbg("bytes=%d, fragments=%d", abinfo.bytes, abinfo.fragments);
#endif
		return copy_to_user((void *) arg, &abinfo,
				    sizeof(abinfo)) ? -EFAULT : 0;

	case SNDCTL_DSP_GETISPACE:
		if (!(file->f_mode & FMODE_READ))
			return -EINVAL;
		abinfo.fragsize = s->dma_adc.fragsize;
		spin_lock_irqsave(&s->lock, flags);
		count = s->dma_adc.count;
		count += dma_count_done(&s->dma_adc);
		spin_unlock_irqrestore(&s->lock, flags);
		if (count < 0)
			count = 0;
		abinfo.bytes = count / s->dma_adc.cnt_factor;
		abinfo.fragstotal = s->dma_adc.numfrag;
		abinfo.fragments = abinfo.bytes >> s->dma_adc.fragshift;
		return copy_to_user((void *) arg, &abinfo,
				    sizeof(abinfo)) ? -EFAULT : 0;

	case SNDCTL_DSP_NONBLOCK:
		file->f_flags |= O_NONBLOCK;
		return 0;

	case SNDCTL_DSP_GETODELAY:
		if (!(file->f_mode & FMODE_WRITE))
			return -EINVAL;
		spin_lock_irqsave(&s->lock, flags);
		count = s->dma_dac.count;
		count -= dma_count_done(&s->dma_dac);
		spin_unlock_irqrestore(&s->lock, flags);
		if (count < 0)
			count = 0;
		count /= s->dma_dac.cnt_factor;
		return put_user(count, (int *) arg);

	case SNDCTL_DSP_GETIPTR:
		if (!(file->f_mode & FMODE_READ))
			return -EINVAL;
		spin_lock_irqsave(&s->lock, flags);
		cinfo.bytes = s->dma_adc.total_bytes;
		count = s->dma_adc.count;
		if (!s->dma_adc.stopped) {
			diff = dma_count_done(&s->dma_adc);
			count += diff;
			cinfo.bytes += diff;
			cinfo.ptr =  kseg_to_phys(s->dma_adc.nextIn) + diff -
				s->dma_adc.dmaaddr;
		} else
			cinfo.ptr = kseg_to_phys(s->dma_adc.nextIn) -
				s->dma_adc.dmaaddr;
		if (s->dma_adc.mapped)
			s->dma_adc.count &= (s->dma_adc.dma_fragsize-1);
		spin_unlock_irqrestore(&s->lock, flags);
		if (count < 0)
			count = 0;
		cinfo.blocks = count >> s->dma_adc.fragshift;
		return copy_to_user((void *) arg, &cinfo, sizeof(cinfo));

	case SNDCTL_DSP_GETOPTR:
		if (!(file->f_mode & FMODE_READ))
			return -EINVAL;
		spin_lock_irqsave(&s->lock, flags);
		cinfo.bytes = s->dma_dac.total_bytes;
		count = s->dma_dac.count;
		if (!s->dma_dac.stopped) {
			diff = dma_count_done(&s->dma_dac);
			count -= diff;
			cinfo.bytes += diff;
			cinfo.ptr = kseg_to_phys(s->dma_dac.nextOut) + diff -
				s->dma_dac.dmaaddr;
		} else
			cinfo.ptr = kseg_to_phys(s->dma_dac.nextOut) -
				s->dma_dac.dmaaddr;
		if (s->dma_dac.mapped)
			s->dma_dac.count &= (s->dma_dac.dma_fragsize-1);
		spin_unlock_irqrestore(&s->lock, flags);
		if (count < 0)
			count = 0;
		cinfo.blocks = count >> s->dma_dac.fragshift;
		return copy_to_user((void *) arg, &cinfo, sizeof(cinfo));

	case SNDCTL_DSP_GETBLKSIZE:
		if (file->f_mode & FMODE_WRITE)
			return put_user(s->dma_dac.fragsize, (int *) arg);
		else
			return put_user(s->dma_adc.fragsize, (int *) arg);

	case SNDCTL_DSP_SETFRAGMENT:
		if (get_user(val, (int *) arg))
			return -EFAULT;
		if (file->f_mode & FMODE_READ) {
			stop_adc(s);
			s->dma_adc.ossfragshift = val & 0xffff;
			s->dma_adc.ossmaxfrags = (val >> 16) & 0xffff;
			if (s->dma_adc.ossfragshift < 4)
				s->dma_adc.ossfragshift = 4;
			if (s->dma_adc.ossfragshift > 15)
				s->dma_adc.ossfragshift = 15;
			if (s->dma_adc.ossmaxfrags < 4)
				s->dma_adc.ossmaxfrags = 4;
			if ((ret = prog_dmabuf_adc(s)))
				return ret;
		}
		if (file->f_mode & FMODE_WRITE) {
			stop_dac(s);
			s->dma_dac.ossfragshift = val & 0xffff;
			s->dma_dac.ossmaxfrags = (val >> 16) & 0xffff;
			if (s->dma_dac.ossfragshift < 4)
				s->dma_dac.ossfragshift = 4;
			if (s->dma_dac.ossfragshift > 15)
				s->dma_dac.ossfragshift = 15;
			if (s->dma_dac.ossmaxfrags < 4)
				s->dma_dac.ossmaxfrags = 4;
			if ((ret = prog_dmabuf_dac(s)))
				return ret;
		}
		return 0;

	case SNDCTL_DSP_SUBDIVIDE:
		if ((file->f_mode & FMODE_READ && s->dma_adc.subdivision) ||
		    (file->f_mode & FMODE_WRITE && s->dma_dac.subdivision))
			return -EINVAL;
		if (get_user(val, (int *) arg))
			return -EFAULT;
		if (val != 1 && val != 2 && val != 4)
			return -EINVAL;
		if (file->f_mode & FMODE_READ) {
			stop_adc(s);
			s->dma_adc.subdivision = val;
			if ((ret = prog_dmabuf_adc(s)))
				return ret;
		}
		if (file->f_mode & FMODE_WRITE) {
			stop_dac(s);
			s->dma_dac.subdivision = val;
			if ((ret = prog_dmabuf_dac(s)))
				return ret;
		}
		return 0;

	case SOUND_PCM_READ_RATE:
		return put_user((file->f_mode & FMODE_READ) ?
				s->dma_adc.sample_rate :
				s->dma_dac.sample_rate,
				(int *)arg);

	case SOUND_PCM_READ_CHANNELS:
		if (file->f_mode & FMODE_READ)
			return put_user(s->dma_adc.num_channels, (int *)arg);
		else
			return put_user(s->dma_dac.num_channels, (int *)arg);

	case SOUND_PCM_READ_BITS:
		if (file->f_mode & FMODE_READ)
			return put_user(s->dma_adc.sample_size, (int *)arg);
		else
			return put_user(s->dma_dac.sample_size, (int *)arg);

	case SOUND_PCM_WRITE_FILTER:
	case SNDCTL_DSP_SETSYNCRO:
	case SOUND_PCM_READ_FILTER:
		return -EINVAL;
	case SNDCTL_DSP_SETSPDIF: /* Set S/PDIF Control register */
#ifdef MPU_DEBUG
		printk("SNDCTL_DSP_SETSPDIF\n");
#endif
		if (get_user(val, (int *)arg))
			return -EFAULT;

		/* mask out the transmitter speed bits so the user can't set them */
		val &= ~0x3000;
			
		/* Add the current transmitter speed bits to the passed value */
		ret = rdcodec(&s->codec, AC97_SPDIF_CONTROL);
		val |= (ret & 0x3000);

		wrcodec(&s->codec, AC97_SPDIF_CONTROL, val);
		if(rdcodec(&s->codec, AC97_SPDIF_CONTROL) != val ) {
			printk(KERN_ERR "tx49xx_audio: Unable to set S/PDIF configuration to 0x%04x.\n", val);
			return -EFAULT;
		}
		return put_user(val, (int *)arg);
	case SNDCTL_DSP_GETSPDIF: /* Get S/PDIF Control register */
#ifdef MPU_DEBUG
		printk("SNDCTL_DSP_GETSPDIF\n");
#endif
		if (get_user(val, (int *)arg))
			return -EFAULT;

		val = rdcodec(&s->codec, AC97_SPDIF_CONTROL);
		return put_user(val, (int *)arg);
   			
	case SNDCTL_DSP_GETCHANNELMASK:
#ifdef MPU_DEBUG
		printk("SNDCTL_DSP_GETCHANNELMASK\n");
#endif
		if (get_user(val, (int *)arg))
			return -EFAULT;
		
		s->codec_ext_caps = rdcodec(&s->codec, AC97_EXTENDED_ID);
		val = DSP_BIND_FRONT;
		if ( s->codec_ext_caps & 0x0004 )
			val |= DSP_BIND_SPDIF;

		if ( s->codec_ext_caps & 0x0080 )
			val |= DSP_BIND_SURR;
		if ( s->codec_ext_caps & 0x0140 )
			val |= DSP_BIND_CENTER_LFE;

		return put_user(val, (int *)arg);

	case SNDCTL_DSP_BIND_CHANNEL:
#ifdef MPU_DEBUG
		printk("SNDCTL_DSP_GETCHANNELMASK\n");
#endif
		if (get_user(val, (int *)arg))
			return -EFAULT;
		if ( val == DSP_BIND_QUERY ) {
			val = DSP_BIND_FRONT; /* Always report this as being enabled */
			if ( s->ac97_status & SPDIF_ON ) 
				val |= DSP_BIND_SPDIF;
			else {
				if ( s->ac97_status & SURR_ON )
					val |= DSP_BIND_SURR;
				if ( s->ac97_status & CENTER_LFE_ON )
					val |= DSP_BIND_CENTER_LFE;
			}
		} else {  /* Not a query, set it */
			if (!(file->f_mode & FMODE_WRITE))
				return -EINVAL;
				
			stop_dac(s);
			
			if ( val & DSP_BIND_SPDIF ) {  /* Turn on SPDIF */
				tx49xx_set_spdif_output ( s, AC97_EA_SPSA_3_4, s->dma_dac.sample_rate );
				if ( !(s->ac97_status & SPDIF_ON) )
					val &= ~DSP_BIND_SPDIF;
			} else {
				int mask;

				/* Turn off S/PDIF if it was on */
				if ( s->ac97_status & SPDIF_ON ) 
					tx49xx_set_spdif_output ( s, -1, 0 );
				
				mask = val & (DSP_BIND_FRONT | DSP_BIND_SURR | DSP_BIND_CENTER_LFE);
				switch (mask) {
					case DSP_BIND_FRONT:
						s->dma_dac.num_channels = 2;
						break;
					case DSP_BIND_FRONT|DSP_BIND_SURR:
						s->dma_dac.num_channels = 4;
						break;
					case DSP_BIND_FRONT|DSP_BIND_SURR|DSP_BIND_CENTER_LFE:
						s->dma_dac.num_channels = 6;
						break;
					default:
						val = DSP_BIND_FRONT;
						s->dma_dac.num_channels = 2;
						break;
				}
				set_xmit_slots(s, s->dma_dac.num_channels);

				/* check that they really got turned on */
				if ( !s->ac97_status & SURR_ON )
					val &= ~DSP_BIND_SURR;
				if ( !s->ac97_status & CENTER_LFE_ON )
					val &= ~DSP_BIND_CENTER_LFE;
			}
		}
		return put_user(val, (int *)arg);


/**************************/
		
		
		
	}

	return mixdev_ioctl(&s->codec, cmd, arg);
}


static int  tx49xx_open(struct inode *inode, struct file *file)
{
	int             minor = MINOR(inode->i_rdev);
	DECLARE_WAITQUEUE(wait, current);
	struct tx49xx_state *s = &tx49xx_state;
	int             ret;

#ifdef MPU_DEBUG
	if (file->f_flags & O_NONBLOCK)
		dbg("non-blocking");
	else
		dbg("blocking");
#endif
	
	file->private_data = s;
	/* wait for device to become free */
	down(&s->open_sem);
	while (s->open_mode & file->f_mode) {
		if (file->f_flags & O_NONBLOCK) {
			up(&s->open_sem);
			return -EBUSY;
		}
		add_wait_queue(&s->open_wait, &wait);
		__set_current_state(TASK_INTERRUPTIBLE);
		up(&s->open_sem);
		schedule();
		remove_wait_queue(&s->open_wait, &wait);
		set_current_state(TASK_RUNNING);
		if (signal_pending(current))
			return -ERESTARTSYS;
		down(&s->open_sem);
	}

	stop_dac(s);
	stop_adc(s);

	if (file->f_mode & FMODE_READ) {
		s->dma_adc.ossfragshift = s->dma_adc.ossmaxfrags =
			s->dma_adc.subdivision = s->dma_adc.total_bytes = 0;
		s->dma_adc.num_channels = 1;
		s->dma_adc.sample_size = 8;
		set_adc_rate(s, 48000);
		if ((minor & 0xf) == SND_DEV_DSP16)
			s->dma_adc.sample_size = 16;
	}

	if (file->f_mode & FMODE_WRITE) {
		s->dma_dac.ossfragshift = s->dma_dac.ossmaxfrags =
			s->dma_dac.subdivision = s->dma_dac.total_bytes = 0;
		s->dma_dac.num_channels = 1;
		s->dma_dac.sample_size = 8;
		set_dac_rate(s, 48000);
		if ((minor & 0xf) == SND_DEV_DSP16)
			s->dma_dac.sample_size = 16;
	}

	if (file->f_mode & FMODE_READ) {
		if ((ret = prog_dmabuf_adc(s)))
			return ret;
	}
	if (file->f_mode & FMODE_WRITE) {
		if ((ret = prog_dmabuf_dac(s)))
			return ret;
	}

	s->open_mode |= file->f_mode & (FMODE_READ | FMODE_WRITE);
	up(&s->open_sem);
	init_MUTEX(&s->sem);
	return 0;
}

static int tx49xx_release(struct inode *inode, struct file *file)
{
	struct tx49xx_state *s = (struct tx49xx_state *)file->private_data;

	lock_kernel();
	
	if (file->f_mode & FMODE_WRITE) {
		unlock_kernel();
		drain_dac(s, file->f_flags & O_NONBLOCK);
		lock_kernel();
	}

	down(&s->open_sem);
	if (file->f_mode & FMODE_WRITE) {
		stop_dac(s);
		dealloc_dmabuf(s, &s->dma_dac);
	}
	if (file->f_mode & FMODE_READ) {
		stop_adc(s);
		dealloc_dmabuf(s, &s->dma_adc);
	}
	s->open_mode &= ((~file->f_mode) & (FMODE_READ|FMODE_WRITE));
	up(&s->open_sem);

	wake_up(&s->open_wait);

	unlock_kernel();
	return 0;
}

static /*const */ struct file_operations tx49xx_audio_fops = {
	owner:		THIS_MODULE,
	llseek:		tx49xx_llseek,
	read:		tx49xx_read,
	write:		tx49xx_write,
	poll:		tx49xx_poll,
	ioctl:		tx49xx_ioctl,
	mmap:		tx49xx_mmap,
	open:		tx49xx_open,
	release:	tx49xx_release,
};


/* --------------------------------------------------------------------- */


/* --------------------------------------------------------------------- */

/*
 * for debugging purposes, we'll create a proc device that dumps the
 * CODEC chipstate
 */

#ifdef TX49XX_DEBUG
static int proc_tx49xx_dump(char *buf, char **start, off_t fpos,
			    int length, int *eof, void *data)
{
	struct tx49xx_state *s = &tx49xx_state;
	int             cnt, len = 0;

	/* print out header */
	len += sprintf(buf + len, "\n\t\tTX49XX Audio Debug\n\n");

	// print out digital controller state
	len += sprintf(buf + len, "TX49XX Audio Controller registers\n");
	len += sprintf(buf + len, "---------------------------------\n");
	len += sprintf (buf + len, "AC97C_CONFIG = %08x\n",
			au_readl(AC97C_CONFIG));
	len += sprintf (buf + len, "AC97C_STATUS = %08x\n",
			au_readl(AC97C_STATUS));
	len += sprintf (buf + len, "AC97C_CNTRL  = %08x\n",
			au_readl(AC97C_CNTRL));

	/* print out CODEC state */
	len += sprintf(buf + len, "\nAC97 CODEC registers\n");
	len += sprintf(buf + len, "----------------------\n");
	for (cnt = 0; cnt <= 0x7e; cnt += 2)
		len += sprintf(buf + len, "reg %02x = %04x\n",
			       cnt, rdcodec(&s->codec, cnt));

	if (fpos >= len) {
		*start = buf;
		*eof = 1;
		return 0;
	}
	*start = buf + fpos;
	if ((len -= fpos) > length)
		return length;
	*eof = 1;
	return len;

}
#endif /* TX49XX_DEBUG */

/* --------------------------------------------------------------------- */


/* --------------------------------------------------------------------- */

#ifdef CONFIG_SNSC_STAC9767_WORKAROUND
#define STAC9767_MAX_RESET      10
#define STAC9767_READY_TIMEOUT  10
/*
 * workaround for STAC9767 bug: double(or more) reset
 */
static int
snd_tx49xx_with_stac9767_reset(struct tx49xx_state *s)
{	
	int i, rst;

	ACLCREG(s, ACCTLEN) |= ACCTL_ENLINK;
	udelay(5);

	/*
	 *  issue clod reset until codec is ready
	 */
	for (rst = 0; rst < STAC9767_MAX_RESET; rst++) {
		/* do cold reset */
		ACLCREG(s, ACCTLDIS) |= ACCTL_ENLINK;
		udelay(10);
		ACLCREG(s, ACCTLEN) |= ACCTL_ENLINK;

		mdelay(5);

		/* check the primary coded is ready */
		for (i = 0; i < STAC9767_READY_TIMEOUT; i++) {
			if (ACLCREG(s, ACINTSTS) & ACINT_CODEC0RDY) {
				goto ret_success;
			}
			set_current_state(TASK_UNINTERRUPTIBLE);
			schedule_timeout(HZ / 20);
                }
		udelay(10);
        }
	printk(KERN_ERR "tx49xx_oss: AC'97 reset retry(%d) failed.\n", rst);
	return -EIO;

ret_success:
	if (rst > 0) {
		printk(KERN_INFO "tx49xx_oss: STAC9767 bug workaround: "
		       "extra reset was performed %d times.\n", rst);
	}

	return 0;
}
#endif /* CONFIG_SNSC_STAC9767_WORKAROUND */

static int get_cpu_devid(void){
	unsigned long device_id = 0;

	if (TX4925_REV_PCODE() == 0x4925 || TX4925_REV_PCODE() == 0x4926) {
		device_id = TX4925_REV_PCODE();
	} else if (TX4927_REV_PCODE() == 0x4927) {
		device_id = 0x4927;
		/* Master Control Register  DMA Controler enable */
		if (!(tx4927_dmaptr->mcr &= TXx927_DMA_MCR_MSTEN))
			tx4927_dmaptr->mcr |= TXx927_DMA_MCR_MSTEN;
	} else if (TX4938_REV_PCODE() == 0x4937) {
		/* Master Control Register  DMA Controler enable */
		if (!(tx4927_dmaptr->mcr &= TXx927_DMA_MCR_MSTEN))
			tx4927_dmaptr->mcr |= TXx927_DMA_MCR_MSTEN;
		device_id = 0x4937;
	} else if (TX4938_REV_PCODE() == 0x9802) {
		if (!(tx4927_dmaptr->mcr &= TXx927_DMA_MCR_MSTEN))
			tx4927_dmaptr->mcr |= TXx927_DMA_MCR_MSTEN;
		tx4927_ccfgptr->pcfg &= ~(3LL << 58);
		device_id = 0x9802;
	} else if (TX4938_REV_PCODE() == 0x4938) {
		device_id = 0x4938;
	}

	return device_id;

}

static int snd_tx49xx_chip_init(struct tx49xx_state *s, unsigned long device_id)
{
#ifndef CONFIG_SNSC_STAC9767_WORKAROUND
	signed long end_time;
	unsigned int cnt;
#endif
	unsigned long long dmasel_mask = 0, dmasel = 0;

	/* put logic to right state */
	init_tx49xx_dma(s->dma_dac.dmanr);
	init_tx49xx_dma(s->dma_adc.dmanr);

	/* disable AC-link and all FIFO DMAs */
	ACLCREG(s, ACCTLDIS) = ACCTL_ENLINK | ACCTL_AUDODMA | ACCTL_AUDIDMA;

	/* dummy R/W to clear pending DMAREQ if any */
	ACLCREG(s, ACAUDODAT) = ACLCREG(s, ACAUDIDAT);

	switch (device_id) {
	case 0x4927:
		/* setup DMASEL (playback:ACLC ch0, capture:ACLC ch1) */
		switch (s->dma_dac.dmanr - TXX927_DMA_CHANNEL_START) {
		case 0:
			dmasel_mask |= TX4927_PCFG_DMASEL0_MASK;
			dmasel |= TX4927_PCFG_DMASEL0_ACL0;
			break;
		case 2:
			dmasel_mask |= TX4927_PCFG_DMASEL2_MASK;
			dmasel |= TX4927_PCFG_DMASEL2_ACL0;
			break;
		default:
			printk("invalid DMA no for playback [%d]\n", s->dma_dac.dmanr);
			return -ENXIO;
		}
		switch (s->dma_adc.dmanr - TXX927_DMA_CHANNEL_START) {
		case 1:
			dmasel_mask |= TX4927_PCFG_DMASEL1_MASK;
			dmasel |= TX4927_PCFG_DMASEL1_ACL1;
			break;
		case 3:
			dmasel_mask |= TX4927_PCFG_DMASEL3_MASK;
			dmasel |= TX4927_PCFG_DMASEL3_ACL1;
			break;
		default:
			printk("invalid DMA no for capture [%d]\n", s->dma_adc.dmanr);
			return -ENXIO;
		}
		tx4927_ccfgptr->pcfg &= ~dmasel_mask;
		tx4927_ccfgptr->pcfg |= dmasel;
		break;
	case 0x4937:
	case 0x9802:
	case 0x4938:
		/* fixed DMA ch */
		break;
	case 0x4926:
	case 0x4925:
		/* setup DRQCTR (playback:ACLC ch0, capture:ACLC ch1) */
		if (s->dma_dac.dmanr - TXX927_DMA_CHANNEL_START < 0 ||
		    s->dma_dac.dmanr - TXX927_DMA_CHANNEL_START >= 4) {
			printk("invalid DMA no for playback [%d]\n", s->dma_dac.dmanr);
			return -ENXIO;
		}
		if (s->dma_adc.dmanr - TXX927_DMA_CHANNEL_START < 0 ||
		    s->dma_adc.dmanr - TXX927_DMA_CHANNEL_START >= 4 ||
		    s->dma_adc.dmanr == s->dma_dac.dmanr) {
			printk("invalid DMA no for capture [%d]\n", s->dma_adc.dmanr);
			return -ENXIO;
		}
		tx4925_ccfgptr->drqctr &=
			~(0xf << (s->dma_dac.dmanr - TXX927_DMA_CHANNEL_START) * 4);
		tx4925_ccfgptr->drqctr |=
			TX4925_DRQCTR_ACLC0 << (s->dma_dac.dmanr - TXX927_DMA_CHANNEL_START) * 4;
		tx4925_ccfgptr->drqctr &=
			~(0xf << (s->dma_adc.dmanr - TXX927_DMA_CHANNEL_START) * 4);
		tx4925_ccfgptr->drqctr |=
			TX4925_DRQCTR_ACLC1 << (s->dma_adc.dmanr - TXX927_DMA_CHANNEL_START) * 4;
		break;
	default:
		printk("unknown device %lx\n", device_id);
		return -ENODEV;
	}
#if 0
	/* master reset DMAC release*/
#endif

#ifdef CONFIG_SNSC_STAC9767_WORKAROUND
	{
		int res;

		res = snd_tx49xx_with_stac9767_reset(s);
		if (res < 0) {
			return res;
		}
	}
#else /* CONFIG_SNSC_STAC9767_WORKAROUND */
	cnt = ACLCREG(s, ACCTLEN);
	end_time = jiffies + (HZ >> 10);
	do {
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ / 10);
	} while (end_time - (signed long)jiffies >= 0);

	/* ACLink on, 2 channels */
	ACLCREG(s, ACCTLEN) = ACCTL_ENLINK;

/*	printk("AC'97 warm reset still in progress? [0x%x]\n", inl(ICHREG(chip, GLOB_CNT)));
 */

	/* wait for primary codec ready status.
	 * Once it becomes ready it should remain ready
	 * as long as we do not disable the ac97 link.
 	 */
	end_time = jiffies + HZ +100;
	do {
		if (ACLCREG(s, ACINTSTS) & ACINT_CODEC0RDY)
			goto __ok1;
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(1);
	} while (end_time - (signed long)jiffies >= 0);
	printk("codec_ready: primary codec is not ready [0x%lx]\n", ACLCREG(s, ACINTSTS));
	return -EIO;

      __ok1:
	/* wait for secondary codec ready status. No secondary codec? , ok */
	/* the end_time variable is not initialized again */
	do {
		if (ACLCREG(s, ACINTSTS) & ACINT_CODEC1RDY)
			break;
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(1);
	} while (end_time - (signed long)jiffies >= 0);
#endif /* CONFIG_SNSC_STAC9767_WORKAROUND */

	/* disable interrupts */
	/* reset channels */
	/* initialize Buffer Descriptor Lists */

	return 0;
}

static int snd_tx49xx_free(struct tx49xx_state *s)
{
	int i;
	if (s->irq_aclc < 0)
		goto __hw_end;
	/* disable AC-link and all FIFO DMAs */
	ACLCREG(s, ACCTLDIS) = ACCTL_ENLINK | ACCTL_AUDODMA | ACCTL_AUDIDMA;

	/* dummy R/W to clear pending DMAREQ if any */
	ACLCREG(s, ACAUDODAT) = ACLCREG(s, ACAUDIDAT);
	/* --- */
	if (s->irq_cap >= 0)
#if 0		
		synchronize_irq(s->irq_cap);
#else
		synchronize_irq();
#endif
	if (s->irq_pbk >= 0)
#if 0		
		synchronize_irq(s->irq_pbk);
#else
		synchronize_irq();
#endif
      __hw_end:
	for (i = 0; i < TX492X_NR_CHAIN * 2; i++) {
		if (s->chain[i])
			free_tx49xx_chdma_cmd(s->chain[i]);
	}
	if (s->irq_cap >= 0)
		free_irq(s->irq_cap, (void *)s);
	if (s->irq_pbk >= 0)
		free_irq(s->irq_pbk, (void *)s);
	if (s->irq_aclc >= 0)
		free_irq(s->irq_aclc, (void *)s);
	return 0;
}

#ifdef MPU_DEBUG	/* 030313 sugiyama wrote for DEBUG */
static void view_64bit_reg(int *reg){
	unsigned int val1, val2, val3, val4;
	unsigned long long *val = (unsigned long long *)reg;

		val1 = (unsigned int)((*val & 0xFFFF000000000000) >> 48);
		val2 = (unsigned int)((*val & 0x0000FFFF00000000) >> 32);
		val3 = (unsigned int)((*val & 0x00000000FFFF0000) >> 16);
		val4 = (unsigned int)((*val & 0x000000000000FFFF));
		printk("\nTX49xX: regaddr = 0x%x, 0x%04x-%04x-%04x-%04x", val, val1, val2, val3, val4);		/* sugiyama wrote */
}
#endif

static int __devinit tx49xx_probe(void)
{
	struct tx49xx_state *s = &tx49xx_state;
	int             val;
	int err, i;

	unsigned long device_id = 0;
	int dma_pbk, dma_cap;
	int irq_aclc = -1, irq_pbk = -1, irq_cap = -1;


	memset(s, 0, sizeof(struct tx49xx_state));

	init_waitqueue_head(&s->dma_adc.wait);
	init_waitqueue_head(&s->dma_dac.wait);
	init_waitqueue_head(&s->open_wait);
	init_MUTEX(&s->open_sem);
	spin_lock_init(&s->lock);
	s->codec.private_data = s;
	s->codec.id = 0;
	s->codec.codec_read = rdcodec;
	s->codec.codec_write = wrcodec;
	s->codec.codec_wait = waitcodec;

	device_id = get_cpu_devid();

	switch (device_id) {
	case 0x4927:
		if (!(tx4927_ccfgptr->pcfg & TX4927_PCFG_SEL2)) {
			printk(KERN_DEBUG "TX492X: ACLC disabled by PCFG\n");
			break;
		}
		if (!snd_irq)
			snd_irq = tx49xx_irq_to_irq(TX4927_IR_ACLC);
		break;
	case 0x4937:
	case 0x4938:
#ifdef MPU_DEBUG
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->ch[0].cha);
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->ch[0].sar);
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->ch[0].dar);
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->ch[0].cntr);
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->ch[0].dair);
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->ch[0].ccr);
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->ch[0].csr);
view_64bit_reg((int*)&tx49xx_dmaptrs[0]->mcr);

view_64bit_reg((int*)&tx4938_ccfgptr->pcfg);
view_64bit_reg((int*)&tx4938_ccfgptr->clkctr);
view_64bit_reg((int*)&tx4938_ccfgptr->ccfg);
#endif
		if (!(tx4938_ccfgptr->pcfg & TX4938_PCFG_SEL2)) {
			printk(KERN_DEBUG "TX493X: ACLC disabled by PCFG.SEL2\n");
			break;
		}
		if (tx4938_ccfgptr->pcfg & TX4938_PCFG_ETH0_SEL) {
			printk(KERN_DEBUG "TX493X: ACLC disabled by PCFG.ETH0_SEL\n");
			break;
		}
		/* TX4937 IRC and TX4927 IRC are compatible... */
		if (!snd_irq)
			snd_irq = tx49xx_irq_to_irq(TX4938_IR_ACLC);
		break;
	case 0x9802:
		if (!snd_irq)
			snd_irq = CXD9802_IRQ_ACLC;
		break;
	case 0x4926:
	case 0x4925:
		if (!(tx4925_ccfgptr->pcfg & TX4925_PCFG_SELACLC)) {
			printk(KERN_DEBUG "TX492X: ACLC disabled by PCFG\n");
			break;
		}
		if (!snd_irq)
			snd_irq = tx4925_irq_to_irq(TX4925_IR_ACLC);
		break;
	default:
		printk(KERN_DEBUG "TX492X: ACLC: Unknown device_id %lx\n", device_id);
		return -ENODEV;
	}

	irq_aclc = snd_irq;
	dma_pbk = snd_dma1;
	dma_cap = snd_dma2;

	switch (device_id) {
	case 0x4927:
		s->baseaddr = TX4927_ACLC_REG;
		s->baseaddr_phys = 0xf00000000ull | TX4927_ACLC_REG;
		if ((dma_pbk - TXX927_DMA_CHANNEL_START != 0 &&
		     dma_pbk - TXX927_DMA_CHANNEL_START != 2) ||
		    (dma_cap - TXX927_DMA_CHANNEL_START != 1 &&
		     dma_cap - TXX927_DMA_CHANNEL_START != 3))
			return -ENXIO;
		break;
	case 0x4937:
	case 0x9802:
	case 0x4938:
		/* playback:ACLC ch0, capture:ACLC ch1 */
		/* ACLC ch0 -- DMAC1 ch3, ACLC ch1 -- DMAC1 ch2 */
		dma_pbk = TXX927_DMA_CHANNEL_START + 0;
		dma_cap = TXX927_DMA_CHANNEL_START + 1;
		s->baseaddr = TX4938_ACLC_REG;
		s->baseaddr_phys = 0xf00000000ull | TX4938_ACLC_REG;
		break;
	case 0x4926:
	case 0x4925:
		s->baseaddr = TX4925_ACLC_REG;
		s->baseaddr_phys = 0xf00000000ull | TX4925_ACLC_REG;
		break;
	default:
		printk("unknown device %lx\n", device_id);
		return -ENODEV;
	}

	s->irq_aclc = -1;
	s->irq_pbk = -1;
	s->irq_cap = -1;

	if (request_irq(irq_aclc, snd_tx49xx_interrupt, SA_INTERRUPT|SA_SHIRQ, "TOSHIBA TX49xx ACLC", (void *)s)) {
		snd_tx49xx_free(s);
		printk("unable to grab IRQ %d\n", irq_aclc);
		return -EBUSY;
	}
	s->irq_aclc = irq_aclc;

	irq_pbk = get_tx49xx_dma_irqno(dma_pbk);
#ifdef MPU_DEBUG
printk("irq_pbk=%d, 0x%x\n", irq_pbk, irq_pbk);		/* 030313 sugiyama wrote */
#endif
	if (irq_pbk < 0) {
		snd_tx49xx_free(s);
		printk("unable to get IRQ num for DMA %d\n", dma_pbk);
		return -EBUSY;
	}

	irq_cap = get_tx49xx_dma_irqno(dma_cap);
#ifdef MPU_DEBUG
printk("irq_cap=%d, 0x%x\n", irq_cap, irq_cap);		/* 030313 sugiyama wrote */
#endif
	if (irq_cap < 0) {
		snd_tx49xx_free(s);
		printk("unable to get IRQ num for DMA %d\n", dma_cap);
		return -EBUSY;
	}
#ifdef MPU_DEBUG
printk("dma_pbk=%d, 0x%x\n", dma_pbk, dma_pbk);		/* 030313 sugiyama wrote */
#endif

	if (request_irq(irq_pbk, snd_tx49xx_interrupt, SA_INTERRUPT|SA_SHIRQ, "TOSHIBA TX49xx DMAC playback", (void *)s)) {
		snd_tx49xx_free(s);
		printk("unable to grab IRQ %d\n", irq_pbk);
		return -EBUSY;
	}
	s->irq_pbk = irq_pbk;

	if (request_irq(irq_cap, snd_tx49xx_interrupt, SA_INTERRUPT|SA_SHIRQ, "TOSHIBA TX49xx DMAC capture", (void *)s)) {
		snd_tx49xx_free(s);
		printk("unable to grab IRQ %d\n", irq_cap);
		return -EBUSY;
	}
	s->irq_cap = irq_cap;
#if 0
	synchronize_irq(s->irq_pbk);
	synchronize_irq(s->irq_cap);
#else
	synchronize_irq();
#endif

	/* allocate chain-DMA command descriptors */
	for (i = 0; i < TX492X_NR_CHAIN * 2; i++) {
		s->chain[i] = alloc_tx49xx_chdma_cmd();
		if (!s->chain[i]) {
			snd_tx49xx_free(s);
			printk("unable to allocate DMA command descriptors\n");
			return -ENOMEM;
		}
	}

	s->dma_dac.dmanr = dma_pbk;
	s->dma_adc.dmanr = dma_cap;

/***** create *****/

	/* register devices */

	if ((s->dev_audio = register_sound_dsp(&tx49xx_audio_fops, -1)) < 0)
		goto err_dev1;
	if ((s->codec.dev_mixer =
	     register_sound_mixer(&tx49xx_mixer_fops, -1)) < 0)
		goto err_dev2;

	/* ACLC Start up */
	if ((err = snd_tx49xx_chip_init(s, device_id)) < 0) {
		snd_tx49xx_free(s);
		return err;
	}

	/* codec init */
	if (!ac97_probe_codec(&s->codec))
		goto err_dev3;

	s->codec_base_caps = rdcodec(&s->codec, AC97_RESET);
	s->codec_ext_caps = rdcodec(&s->codec, AC97_EXTENDED_ID);
	info("AC'97 Base/Extended ID = %04x/%04x",
	     s->codec_base_caps, s->codec_ext_caps);

	/*
	 * On the Pb1000, audio playback is on the AUX_OUT
	 * channel (which defaults to LNLVL_OUT in AC'97
	 * rev 2.2) so make sure this channel is listed
	 * as supported (soundcard.h calls this channel
	 * ALTPCM). ac97_codec.c does not handle detection
	 * of this channel correctly.
	 */
	s->codec.supported_mixers |= SOUND_MASK_ALTPCM;
	/*
	 * Now set AUX_OUT's default volume.
	 */
	val = 0x4343;
	mixdev_ioctl(&s->codec, SOUND_MIXER_WRITE_ALTPCM,
		     (unsigned long) &val);
	
	s->ac97_status |= SPDIF_ON;
	wrcodec( &s->codec, AC97_EXTENDED_STATUS, 0x0405 );
	
	if (!(s->codec_ext_caps & AC97_EXTID_VRA)) {
		// codec does not support VRA
		s->no_vra = 1;
	} else if (!vra) {
		// Boot option says disable VRA
		u16 ac97_extstat = rdcodec(&s->codec, AC97_EXTENDED_STATUS);
		wrcodec(&s->codec, AC97_EXTENDED_STATUS,
			ac97_extstat & ~AC97_EXTSTAT_VRA);
		s->no_vra = 1;
	}
	if (s->no_vra)
		info("no VRA, interpolating and decimating");

	/* set mic to be the recording source */
	val = SOUND_MASK_MIC;
	mixdev_ioctl(&s->codec, SOUND_MIXER_WRITE_RECSRC,
		     (unsigned long) &val);

#ifdef TX49XX_DEBUG
	sprintf(proc_str, "driver/%s/%d/ac97", TX49XX_MODULE_NAME,
		s->codec.id);
	s->ac97_ps = create_proc_read_entry (proc_str, 0, NULL,
					     ac97_read_proc, &s->codec);
#endif

	return 0;

 err_dev3:
	unregister_sound_mixer(s->codec.dev_mixer);
 err_dev2:
	unregister_sound_dsp(s->dev_audio);
 err_dev1:
	snd_tx49xx_free(s);
	return -1;
}

static void __devinit tx49xx_remove(void)
{
	struct tx49xx_state *s = &tx49xx_state;

	if (!s)
		return;
#ifdef TX49XX_DEBUG
	if (s->ps)
		remove_proc_entry(TX49XX_MODULE_NAME, NULL);
#endif /* TX49XX_DEBUG */
	synchronize_irq();
	snd_tx49xx_free(s);
	unregister_sound_dsp(s->dev_audio);
	unregister_sound_mixer(s->codec.dev_mixer);
}

static int __init init_tx49xx(void)
{
/*	info("stevel@mvista.com, built " __TIME__ " on " __DATE__);*/
	return tx49xx_probe();
}

static void __exit cleanup_tx49xx(void)
{
	info("unloading");
	tx49xx_remove();
}

module_init(init_tx49xx);
module_exit(cleanup_tx49xx);

/* --------------------------------------------------------------------- */

#ifndef MODULE

static int __init tx49xx_setup(char *options)
{
	char           *this_opt;

	if (!options || !*options)
		return 0;

	for(this_opt=strtok(options, ",");
	    this_opt; this_opt=strtok(NULL, ",")) {
		if (!strncmp(this_opt, "vra", 3)) {
			vra = 1;
		}
	}

	return 1;
}

__setup("tx49xx_audio=", tx49xx_setup);

#endif /* MODULE */
