/*
 * linux/drivers/usb/device/bi/dbmx1.h
 *
 * DragonBall MX1 USB Device Bus Interface Driver
 *
 * Copyright 2003 Sony Corporation
 *
 * Copyright (c) 2000, 2001, 2002 Lineo
 *
 * By: 
 *      Stuart Lynne <sl@lineo.com>, 
 *      Tom Rushworth <tbr@lineo.com>, 
 *      Bruce Balden <balden@lineo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <asm/arch/mpu110_series.h>

#define EP0_PACKETSIZE       32
#define UDC_MAX_ENDPOINTS    6
#define UDC_NAME             "DBMX1 USBD"

#define EP0                  0

/* EP0 state */
#define EP0_WAIT_SETUP       0
#define EP0_DATA_STATE_RECV  1
#define EP0_DATA_STATE_XMIT  2

/* EPn status */
#define EP_IN_PROGRESS       1

/* settings for VBUS detection */
#define VBUS_TIMER_EXPIRE  1  /* 10[msec] */
#ifdef CONFIG_DRAGONBALL_SNSC_MPU110
#define IS_VBUS_ACTIVE(dat)  (dat)  /* active High */
#else
#error Unknown Target Board!!
#endif

/* dma request source */
#define DMA_REQ_SRC(ep)            (DMA_USB_ENDPOINT_0+ep)

/*
 * macros for register access
 */
#define READ_REG8(adr)             inb(adr)
#define WRITE_REG8(adr,dt)         outb((u8)(dt), adr)
#define READ_REG32(adr)            inl(adr)
#define WRITE_REG32(adr,dt)        outl((u32)(dt), adr)
#define SETBIT_REG32(adr,mask)     WRITE_REG32(adr,(READ_REG32(adr) | (mask)))
#define UNSETBIT_REG32(adr,mask)   WRITE_REG32(adr,(READ_REG32(adr) & (~(mask))))
#define MASKBIT_REG32(adr,mask)    WRITE_REG32(adr,(READ_REG32(adr) & (mask)))

/* set USB divider */
#define SET_PLL_USBDIV(dat) \
          WRITE_REG32(PLL_BASE + PLL_CSCR, \
		      (READ_REG32(PLL_BASE + PLL_CSCR) & 0xe3ffffff) | ((dat & 0x7) << 26))

/* reset UDC */
#define RESET_UDC() \
          WRITE_REG32(USBD_CTRL, USBD_CTRL_UDC_RST)

/* reset USB */
#define RESET_USB() \
          SETBIT_REG32(USBD_ENAB, USBD_ENAB_RST)

/* enable USB */
#define ENABLE_USB() \
          SETBIT_REG32(USBD_ENAB, USBD_ENAB_ENAB)

/* disable USB */
#define DISABLE_USB() \
          WRITE_REG32(USBD_ENAB, 0x00000000)

/* check whether USB is enable or not */
#define IS_USB_ENABLE() \
          (READ_REG32(USBD_ENAB) & USBD_ENAB_ENAB)

/* enable USB */
#define ENABLE_USB_PROCESS() \
          SETBIT_REG32(USBD_CTRL, USBD_CTRL_USB_SPD | \
                         	  USBD_CTRL_USB_ENA | \
                                  USBD_CTRL_AFE_ENA)

/* disable USB */
#define DISABLE_USB_PROCESS() \
          UNSETBIT_REG32(USBD_CTRL, USBD_CTRL_USB_SPD | \
                                    USBD_CTRL_USB_ENA | \
                                    USBD_CTRL_AFE_ENA)

/* get interrupt status register */
#define GET_USB_INTR() \
          READ_REG32(USBD_INTR)

/* set interrupt status register */
#define CLEAR_USB_INTR(_dat) \
          WRITE_REG32(USBD_INTR, _dat)

/* get interrupt mask register */
#define GET_USB_INTR_MASK() \
          READ_REG32(USBD_MASK)

/* enable USB interrupt */
#define ENABLE_USB_INTR(_mask) \
          UNSETBIT_REG32(USBD_MASK, _mask);

/* disable USB interrupt */
#define DISABLE_USB_INTR(_mask) \
          SETBIT_REG32(USBD_MASK, _mask);

/* enable SOF interrupt */
#define ENABLE_SOF_INTR() \
          UNSETBIT_REG32(USBD_MASK, USBD_MASK_SOF)

/* disable SOF interrupt */
#define DISABLE_SOF_INTR() \
          SETBIT_REG32(USBD_MASK, USBD_MASK_SOF)

/* enable Wakeup interrupt */
#define ENABLE_WAKEUP_INTR() \
          UNSETBIT_REG32(USBD_MASK, USBD_MASK_WAKEUP)

/* disable Wakeup interrupt */
#define DISABLE_WAKEUP_INTR() \
          SETBIT_REG32(USBD_MASK, USBD_MASK_WAKEUP)

/* enable Active-to-Suspend interrupt */
#define ENABLE_SUSPEND_INTR() \
          UNSETBIT_REG32(USBD_MASK, USBD_MASK_SUSP)

/* disable Active-to-Suspend interrupt */
#define DISABLE_SUSPEND_INTR() \
          SETBIT_REG32(USBD_MASK, USBD_MASK_SUSP)

/* enable Suspend-to-Resume interrupt */
#define ENABLE_RESUME_INTR() \
          UNSETBIT_REG32(USBD_MASK, USBD_MASK_RES)

/* enable Suspend-to-Resume interrupt */
#define DISABLE_RESUME_INTR() \
          SETBIT_REG32(USBD_MASK, USBD_MASK_RES)

/* enable Reset signaling Start interrupt */
#define ENABLE_RESET_START_INTR() \
          UNSETBIT_REG32(USBD_MASK, USBD_MASK_RESET_START)

/* disable Reset signaling Start interrupt */
#define DISABLE_RESET_STAR_INTRT() \
          SETBIT_REG32(USBD_MASK, USBD_MASK_RESET_START)

/* enable Reset signaling Stop interrupt */
#define ENABLE_RESET_STOP_INTR() \
          UNSETBIT_REG32(USBD_MASK, USBD_MASK_RESET_STOP)

/* disable Reset signaling Stop interrupt */
#define DISABLE_RESET_STOP_INTR() \
          SETBIT_REG32(USBD_MASK, USBD_MASK_RESET_STOP)

/* get endpoint interrupt status register */
#define GET_EP_INTR(ep) \
          READ_REG32(USBD_EP_INTR(ep))

/* clear endpoint interrupt */
#define CLEAR_EP_INTR(ep, _mask) \
          WRITE_REG32(USBD_EP_INTR(ep), _mask)

/* get endpoint interrupt mask register */
#define GET_EP_INTR_MASK(ep) \
          READ_REG32(USBD_EP_MASK(ep))

/* enable endpoint interrupt */
#define ENABLE_EP_INTR(ep, _mask) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), _mask)

/* disable endpoint interrupt */
#define DISABLE_EP_INTR(ep, _mask) \
          SETBIT_REG32(USBD_EP_MASK(ep), _mask)

/* enable Multiple Device Request interrupt */
#define ENABLE_MDEVREQ_INTR() \
          UNSETBIT_REG32(USBD_EP_MASK(EP0), USBD_EP_MASK_MDEVREQ)

/* disable Multiple Device Request interrupt */
#define DISABLE_MDEVREQ_INTR() \
          SETBIT_REG32(USBD_EP_MASK(EP0), USBD_EP_MASK_MDEVREQ)

/* enable Device Request interrupt */
#define ENABLE_DEVREQ_INTR() \
          UNSETBIT_REG32(USBD_EP_MASK(EP0), USBD_EP_MASK_DEVREQ)

/* disable Device Request interrupt */
#define DISABLE_DEVREQ_INTR() \
          SETBIT_REG32(USBD_EP_MASK(EP0), USBD_EP_MASK_DEVREQ)

/* enable EOF/EOT interrupt */
#define ENABLE_EOF_EOT_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_EOF | USBD_EP_MASK_EOT)

/* enable EOF/EOT interrupt */
#define DISABLE_EOF_EOT_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_EOF | USBD_EP_MASK_EOT)

/* enable EOT interrupt */
#define ENABLE_EOT_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_EOT)

/* disable EOT interrupt */
#define DISABLE_EOT_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_EOT)

/* enable EOF interrupt */
#define ENABLE_EOF_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_EOF)

/* disable EOF interrupt */
#define DISABLE_EOF_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_EOF)

/* enable FIFO Low interrupt */
#define ENABLE_FIFO_LOW_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_LOW)

/* disable FIFO Low interrupt */
#define DISABLE_FIFO_LOW_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_LOW)

/* enable FIFO High interrupt */
#define ENABLE_FIFO_HIGH_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_HIGH)

/* disable FIFO High interrupt */
#define DISABLE_FIFO_HIGH_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_HIGH)

/* enable FIFO Empty interrupt */
#define ENABLE_FIFO_EMPTY_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_EMPTY)

/* disable FIFO Empty interrupt */
#define DISABLE_FIFO_EMPTY_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_EMPTY)

/* enable FIFO Error interrupt */
#define ENABLE_FIFO_ERROR_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_ERROR)

/* disable FIFO Error interrupt */
#define DISABLE_FIFO_ERROR_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_ERROR)

/* enable FIFO Full interrupt */
#define ENABLE_FIFO_FULL_INTR(ep) \
          UNSETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_FULL)

/* disable FIFO Full interrupt */
#define DISABLE_FIFO_FULL_INTR(ep) \
          SETBIT_REG32(USBD_EP_MASK(ep), USBD_EP_MASK_FIFO_FULL)

/* enable Configuration Change interrupt */
#define ENABLE_CFG_CHG_INTR() \
          UNSETBIT_REG32(USBD_MASK, USBD_MASK_CFG_CHG)

/* disable Configuration Change interrupt */
#define DISABLE_CFG_CHG_INTR() \
          SETBIT_REG32(USBD_MASK, USBD_MASK_CFG_CHG)

/* read from FIFO by byte access */
#define READ_FIFO8(ep) \
          READ_REG8(USBD_EP_FDAT(ep))

/* read from FIFO by 32bit access */
#define READ_FIFO32(ep) \
          READ_REG32(USBD_EP_FDAT(ep))

/* write to FIFO by byte access */
#define WRITE_FIFO8(ep, _dat) \
          WRITE_REG8(USBD_EP_FDAT(ep), _dat)

/* write to FIFO by 32bit access */
#define WRITE_FIFO32(ep, _dat) \
          WRITE_REG32(USBD_EP_FDAT(ep), _dat)

/* get FIFO Byte Count */
#define GET_FIFO_CNT(ep) \
          ((READ_REG32(USBD_EP_STAT(ep)) & USBD_EP_STAT_BYTE_COUNT_MASK) \
                                        >> USBD_EP_STAT_BYTE_COUNT_BIT)
/* get FIFO status */
#define GET_FIFO_STAT(ep) \
          READ_REG32(USBD_EP_FSTAT(ep))

/* clear FIFO status */
#define CLEAR_FIFO_STAT(ep, _dat) \
          WRITE_REG32(USBD_EP_FSTAT(ep), _dat)

/* flush FIFO */
#define CLEAR_FIFO(ep) \
          SETBIT_REG32(USBD_EP_STAT(ep), USBD_EP_STAT_FLUSH)

/* set Write Frame End */
#define SET_FIFO_WFR(ep) \
          SETBIT_REG32(USBD_EP_FCTRL(ep), USBD_EP_FCTRL_WFR)

/* set GR value */
#define SET_FIFO_GR(ep, _dat) \
          WRITE_REG32(USBD_EP_FCTRL(ep), \
		      (READ_REG32(USBD_EP_FCTRL(ep)) & ~USBD_EP_FCTRL_GR_MASK) | \
                      ((_dat & 0x7) << USBD_EP_FCTRL_GR_BIT))

/* set ALRM value */
#define SET_FIFO_ALRM(ep, _dat) \
          WRITE_REG32(USBD_EP_FALRM(ep), (_dat & 0x3f))

/* set Frame Mode */
#define SET_FRAME_MODE(ep) \
          SETBIT_REG32(USBD_EP_FCTRL(ep), USBD_EP_FCTRL_FRAME);

/* send Zero Length Packet */
#define SET_ZLPS(ep) \
          SETBIT_REG32(USBD_EP_STAT(ep), USBD_EP_STAT_ZLPS)

/* set Force Stall */
#define SET_EP_STALL(ep) \
          SETBIT_REG32(USBD_EP_STAT(ep), USBD_EP_STAT_FORCE_STALL)

/* set Command Over */
#define SET_CMD_OVER() \
          SETBIT_REG32(USBD_CTRL, USBD_CTRL_CMD_OVER);

/* set Command Error */
#define SET_CMD_ERROR() \
          SETBIT_REG32(USBD_CTRL, USBD_CTRL_CMD_ERROR);

/* get USB status */
#define GET_USB_STAT() \
          READ_REG32(USBD_STAT)

/* get Confiduration */
#define GET_CFG() \
          ((READ_REG32(USBD_STAT) & USBD_STAT_CFG_MASK) >> USBD_STAT_CFG_BIT)

/* get Interface */
#define GET_INTF() \
          ((READ_REG32(USBD_STAT) & USBD_STAT_INTF_MASK) >> USBD_STAT_INTF_BIT)

/* get Alternate Setting */
#define GET_ALTSET() \
          ((READ_REG32(USBD_STAT) & USBD_STAT_ALTSET_MASK) >> USBD_STAT_ALTSET_BIT)
