/*
 *     OHCI support for CXD4707GA
 *
 * based upon :usb-ohci-nonpci.c Steve Longerbeam <stevel@mvista.com, or source@mvista.com>
 *
 * 2001 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 *
 * Copyright 2003 Sony Corporation
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/slab.h>
#include <linux/usb.h>
#include <linux/pci.h>

#include <asm/irq.h>
#include <asm/io.h>
#include <platforms/cxd4707gb.h>

#include "usb-ohci.h"
#include "usb-ohci-cxd4707.h"

#define DRIVER_VERSION "v1.0"
#define DRIVER_AUTHOR "Sony Corporation"
#define DRIVER_DESC "CXD4707 USB OHCI Host Controller Driver"

extern int hc_add_ohci(struct pci_dev *dev, int irq, void *mem_base, unsigned long flags,
		       ohci_t **ohci_p, const char *name, const char *slot_name);
extern void hc_remove_ohci(ohci_t *ohci);
ohci_t *cxd4707_ohci;

static int ohci_base =  0;
static int ohci_len  =  0;
static int ohci_irq  = -1;

static struct pci_dev cxd4707_pcidev;
static void *cxd4707_ohci_mem_base;

static int __init cxd4707_ohci_init(void)
{
	int ret;

	printk("CXD4707GB embedded OHCI controller\n"); 
#if defined(CONFIG_CXD4707_ERRATA_AHB_BYTE_SWAPPED)
	printk("AHB errata workaound enabled\n");
#endif


	switch (cxd4707_get_rev()) {
	case CXD4707_ES_1_0:
	case CXD4707_ES_1_1:
		cxd4707_ohci_mem_base = ioremap64(CXD4707_OHCI_BASE_ES1X, CXD4707_OHCI_SIZE);
		ohci_base = 0x00000000ffffffff & CXD4707_OHCI_BASE_ES1X;
		break;
	default:
		printk("Unknown CXD4707 rev. defaulted to ES2.0\n");
		/* intentionally fall thru */
	case CXD4707_ES_2_0:
		cxd4707_ohci_mem_base = ioremap64(CXD4707_OHCI_BASE_ES2X, CXD4707_OHCI_SIZE);
		ohci_base = 0x00000000ffffffff & CXD4707_OHCI_BASE_ES2X;
		break;
	}
	ohci_len = CXD4707_OHCI_SIZE;
	ohci_irq = CXD4707_OHCI_IRQ;

	if (!cxd4707_ohci_mem_base) {
		err("Error mapping OHCI register space");
		return -EFAULT;
	}

	/*
	 * Fill in some fields of the bogus pci_dev.
	 */
	memset(&cxd4707_pcidev, 0, sizeof(struct pci_dev));
	strcpy(cxd4707_pcidev.name, "CXD4707GB OHCI");
	strcpy(cxd4707_pcidev.slot_name, "cxd4707");
	cxd4707_pcidev.resource[0].name = "OHCI Operational Registers";
	cxd4707_pcidev.resource[0].start = ohci_base;
	cxd4707_pcidev.resource[0].end = ohci_base + ohci_len;
	cxd4707_pcidev.resource[0].flags = 0;
	cxd4707_pcidev.dma_mask = 0xffffffff;
	cxd4707_pcidev.irq = ohci_irq;

	/*
	 * Initialise the generic OHCI driver.
	 */
	ret = hc_add_ohci(&cxd4707_pcidev, ohci_irq, cxd4707_ohci_mem_base, 0, 
			  &cxd4707_ohci, "usb-ohci", "cxd4707");

	if (ret) {
		iounmap(cxd4707_ohci_mem_base);
	}
	return ret;
}

static void __exit cxd4707_ohci_exit(void)
{
	hc_remove_ohci(cxd4707_ohci);
}

MODULE_AUTHOR( DRIVER_AUTHOR );
MODULE_DESCRIPTION( DRIVER_DESC );
MODULE_LICENSE("GPL");

module_init(cxd4707_ohci_init);
module_exit(cxd4707_ohci_exit);
