/* With non GPL files, use following license */
/*
 * linux/drivers/video/snsc_wp.c
 *
 * Copyright 2002-2003 Sony Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Otherwise with GPL files, use following license */
/*
 *  linux/drivers/video/snsc_wp.c
 *
 *  Sony NSC Boot wallpaper
 *
 *  Copyright 2002-2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/string.h>
#include <linux/kd.h>
#include <linux/slab.h>
#include <linux/fb.h>
#include <video/fbcon.h>
#include <linux/tty.h>
#include <linux/smp.h>
#include <linux/init.h>
#include <linux/zlib.h>
#include <linux/vmalloc.h>
#ifdef CONFIG_SNSC_NBLARGS
#include <linux/snsc_nblargs.h>
#endif
#include "snsc_wp.h"

unsigned long bwp_start = 0, bwp_size = 0;
int bwp_found = 0;

#ifdef CONFIG_WALLPAPER_BUFSIZE
#define WP_BUFSIZE  CONFIG_WALLPAPER_BUFSIZE
#endif

#ifdef CONFIG_SNSC_NBLARGS
#define NBLARGS_KEY_BWPSTART "bwpstart"
#endif

const char *msghead = "snsc_wp:";

#ifdef BWP_DEBUG
#define PDEBUG(fmt, args...)    printk(fmt, ## args)
#else
#define PDEBUG(fmt, args...)
#endif

/*
 *  zlib decompression
 */

#ifdef CONFIG_WALLPAPER_LARGEBUF
static int
zlib_decompress(unsigned char *indata, unsigned char *outdata, 
                long insize, long *outsize)
{
        int err;
        z_stream strm;

        strm.next_in = indata;
        strm.avail_in = insize;
	strm.total_in = 0;

        strm.next_out = outdata;
        strm.avail_out = *outsize;
        strm.total_out = 0;
	strm.workspace = vmalloc(zlib_inflate_workspacesize());
	if (strm.workspace == NULL) {
		printk("cannot allocate zlib workspace\n");
		return -1;
	}

        err = zlib_inflateInit(&strm);
        if (err != Z_OK) {
                printk("%s inflateInit error: err=%d\n", msghead, err);
		vfree(strm.workspace);
                return -1;
        }

        for (;;) {
                err = zlib_inflate(&strm, Z_FINISH);
                if (err == Z_STREAM_END)
                        break;
                if (err != Z_OK) {
                        printk("%s inflate err=%d\n", msghead, err);
			vfree(strm.workspace);
                        return -1;
                }
        }

        err = zlib_inflateEnd(&strm);
        if (err != Z_OK) {
                printk("%s inflateEnd error: err=%d\n", msghead, err);
		vfree(strm.workspace);
                return -1;
        }

        *outsize = strm.total_out;
        vfree(strm.workspace);
        return 0;
}
#else
static int
zlib_decompress_with_fbwrite(unsigned char *indata, long insize, long outsize, 
                             unsigned char *fb)
{
        int i,err;
        z_stream strm;
        unsigned char *outdata, *poutdata;

        /* allocate buffer */
        outdata = kmalloc(WP_BUFSIZE, GFP_KERNEL);
        if (outdata == NULL) {
                printk("%s cannot allocate buffer for decompressed data\n",msghead);
                return -1;
        }

        strm.next_in = indata;
        strm.avail_in = insize;
	strm.total_in = 0;

        strm.next_out = outdata;
        strm.avail_out = WP_BUFSIZE;
	strm.total_out = 0;

	strm.workspace = vmalloc(zlib_inflate_workspacesize());
	if (strm.workspace == NULL) {
		printk("cannot allocate zlib workspace\n");
		return -1;
	}
        
        err = zlib_inflateInit(&strm);
        if (err != Z_OK) {
                printk("%s inflateInit error: err=%d\n", msghead, err);
                kfree(outdata);
		vfree(strm.workspace);
                return -1;
        }

        while (strm.total_in < insize) {
                err = zlib_inflate(&strm, Z_NO_FLUSH);
                if (err == Z_STREAM_END)
                        break;
                if (err != Z_OK) {
                        printk("%s inflate err=%d\n", msghead, err);
                        kfree(outdata);
			vfree(strm.workspace);
                        return -1;
                }

                if (strm.avail_out == 0) {
                        poutdata = outdata;
                        for (i = 0; i < WP_BUFSIZE; i++)
                                fb_writeb(*poutdata++, fb++);
                        strm.next_out = outdata;
                        strm.avail_out = WP_BUFSIZE;
                }
        }

        if (err != Z_STREAM_END)
                for (;;) {
                        err = zlib_inflate(&strm, Z_FINISH);
                        if (err == Z_STREAM_END)
                                break;
                        if (err != Z_OK) {
                                printk("%s inflate err=%d\n", msghead, err);
                                kfree(outdata);
				vfree(strm.workspace);
                                return -1;
                        }
                        if (strm.avail_out == 0) {
                                poutdata = outdata;
                                for (i = 0; i < WP_BUFSIZE; i++)
                                        fb_writeb(*poutdata++, fb++);
                                strm.next_out = outdata;
                                strm.avail_out = WP_BUFSIZE;
                        }
                }

        poutdata = outdata;
        for (i = 0; i < WP_BUFSIZE - strm.avail_out; i++)
                fb_writeb(*poutdata++, fb++);

        err = zlib_inflateEnd(&strm);
        if (err != Z_OK) {
                printk("%s inflateEnd error: err=%d\n", msghead, err);
                kfree(outdata);
		vfree(strm.workspace);
                return -1;
        }

        if (strm.total_out != outsize) {
                printk("%s total size of decompressed data(0x%lx) is invalid(0x%lx)\n",
                       msghead, strm.total_out, outsize);
                kfree(outdata);
		vfree(strm.workspace);
                return -1;
        }

        kfree(outdata);
        vfree(strm.workspace);

        return 0;
}
#endif /* CONFIG_WALLPAPER_LARGEBUF */


/* check bwp header */
static int
chk_bwpheader(struct bwp_header *header)
{
        if ((header->magic[0] != BWP_MAGIC0)
            || (header->magic[1] != BWP_MAGIC1)
            || (header->magic[2] != BWP_MAGIC2)
            || (header->magic[3] != BWP_MAGIC3)) {
                printk("%s no bwp magic\n", msghead);
                return -1;
        }

        return 0;
}

/*
 *  check whether bwp data exists or not at the specified address by 'bwpstart=' option
 *      if bwp exists, return 1
 */
int
bwp_chk(void)
{
        struct bwp_header *h;

        if ((bwp_found) || (!bwp_start))
                return 0;

        printk("%s check bwp at 0x%lx\n", msghead, bwp_start);
        h = (struct bwp_header *)bwp_start;
        if (chk_bwpheader(h) < 0) {
                PDEBUG(__FUNCTION__ ": bwp is not found at the address(0x%lx)\n", bwp_start);
                return 0;
        }

        PDEBUG(__FUNCTION__ ": bwp is found at the address(0x%lx)\n", bwp_start);
        bwp_found = 1;
        bwp_size = sizeof(struct bwp_header) + h->datalen;

        return 1;
}


/*
 *  Show boot wallpaper
 */
int
fbcon_show_wallpaper(void)
{
        struct display *p = &fb_display[fg_console]; /* draw to vt in foreground */
        unsigned char  *fb = p->screen_base;
        struct         bwp_header *head;
        unsigned char  *outdata;
        long           outlen, bwp_datalen;
        int            i, err;

        if (!bwp_found)
                return 0;
        
        head = (struct bwp_header *)bwp_start;

        if ((p->var.xres_virtual != head->width)
            || (p->var.yres_virtual < head->height)
            || (p->var.bits_per_pixel != head->bpp)) {
                printk("%s don't match resolution or pixel format(fb:%dx%d,%dbpp != wp:%dx%d,%dbpp)\n",
                       msghead,
                       p->var.xres_virtual, p->var.yres_virtual, p->var.bits_per_pixel,
                       head->width, head->height, head->bpp);

                return 0;
        }

        outlen = head->width * head->height * (head->bpp >> 3);
        bwp_datalen = head->datalen;

        /* if data is comppressed, decompress it */
        if (head->type == BWP_TYPE_COMPRESSED) {
#ifdef CONFIG_WALLPAPER_LARGEBUF
                int order;
                unsigned char *pdata;

                order = get_order(outlen);
                outdata = (unsigned char *)__get_free_pages(GFP_KERNEL, order);
                if (outdata == NULL) {
                        printk("%s cannot allocate buffer for decompressed data\n",msghead);
                        return 0;
                }

                err = zlib_decompress((unsigned char *)(bwp_start + sizeof(struct bwp_header)), 
                                      outdata, bwp_datalen, &outlen);
                if (err != 0) {
                        free_pages((unsigned long)outdata, order);
                        return 0;
                }
                pdata = outdata;
                for (i = 0; i < outlen; i++)
                        fb_writeb(*pdata++, fb++);
                free_pages((unsigned long)outdata, order);
#else
                /* write to fb decompressing data with small buffer */
                err = zlib_decompress_with_fbwrite((unsigned char *)bwp_start + sizeof(struct bwp_header),
                                                   bwp_datalen, outlen, fb);
                if (err != 0) {
                        return 0;
                }
#endif
        } else {
                /* uncompressed data */
                outdata = (unsigned char *)(bwp_start + sizeof(struct bwp_header));
                for (i = 0; i < outlen; i++)
                        fb_writeb(*outdata++, fb++);
        }

        return outlen;
}

/*
 *  parse cmdline and find 'bwpstart=' option
 *  (this function may be called before parse_options())
 */
void
parse_bwp_cmdline(const char *str)
{
#ifdef CONFIG_SNSC_NBLARGS
	struct nblargs_entry na;

	if (nblargs_get_key(NBLARGS_KEY_BWPSTART, &na) < 0)
		return;

	bwp_start = na.addr;
	if (bwp_start < PAGE_OFFSET)
		bwp_start = (unsigned long)(__va(bwp_start));

#else
        char *p, *endp;

        p = strstr(str, "bwpstart=");
        if (p) {
                p += 9;
                bwp_start = simple_strtoul(p, &endp, 0);
                if (bwp_start < PAGE_OFFSET)
                        bwp_start = (unsigned long)(__va(bwp_start));
        }
#endif /* CONFIG_SNSC_NBLARGS */
}
