/*
 * wprotect.c
 *
 * Write protection for the filesystem pages.
 *
 * Author: Steve Longerbeam <stevel@mvista.com, or source@mvista.com>
 *
 * 2003 (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 *
 * 2003 Sony Corporation
 * 2003 Matsushita Electric Industrial Co., Ltd.
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/pram_fs.h>
#include <linux/mm.h>
#include <asm/pgtable.h>
#include <asm/pgalloc.h>

// init_mm.page_table_lock must be held before calling!
static void pram_page_writeable(unsigned long addr, int rw)
{
	pgd_t *pgdp;
	pmd_t *pmdp;
	pte_t *ptep;
	
	pgdp = pgd_offset_k(addr);
	if (!pgd_none(*pgdp)) {
		pmdp = pmd_offset(pgdp, addr);
		if (!pmd_none(*pmdp)) {
			pte_t pte;
			ptep = pte_offset(pmdp, addr);
			pte = *ptep;
			if (pte_present(pte)) {
				pte = rw ? pte_mkwrite(pte) :
					pte_wrprotect(pte);
				set_pte(ptep, pte);
				// FIXME: I'd like to be able to flush
				// just this page, not the whole TLB,
				// but flush_tlb_page() only works for
				// user vm areas, and this is a kernel
				// mapping. Also should call update_mmu_cache()
				// after TLB flush, but can't for same
				// reason.
				//flush_tlb_page();
			}
		}
	}
}


// init_mm.page_table_lock must be held before calling!
void pram_writeable(void * vaddr, unsigned long size, int rw)
{
        unsigned long addr = (unsigned long)vaddr & PAGE_MASK;
	unsigned long end = (unsigned long)vaddr + size;
	
	do {
		pram_page_writeable(addr, rw);
		addr += PAGE_SIZE;
	} while (addr && (addr < end));

	// shouldn't have to do this if I can call flush_tlb_page() above
	flush_tlb_all();
}

