/*
 * linux/fs/xvfat/vfatfs_syms.c
 * based upon linux/fs/vfat/vfatfs_syms.c
 *
 * Exported kernel symbols for the XVFAT filesystem.
 * These symbols are used by xvfat
 * 
 * Copyright 2003 Sony Corporation
 */

#include <linux/module.h>
#include <linux/init.h>

#include <linux/mm.h>
#include <linux/xvfat_fs.h>

DECLARE_FSTYPE_DEV(xvfat_fs_type, "xvfat", xvfat_read_super);

static int __init init_xvfat_fs(void)
{
	extern int __init fwrq_init(void);
	extern int __init init_xfat_fs(void);
	extern int __init rdchk_init(void);
	int rval;

	rval = fwrq_init();
	if( rval != 0 )
		return rval;
	rval = rdchk_init();
	if( rval != 0 )
		return rval;
	rval = init_xfat_fs();
	if( rval != 0 )
		return rval;
	return register_filesystem(&xvfat_fs_type);
}

static void __exit exit_xvfat_fs(void)
{
	extern void __exit rdchk_exit(void);
	extern void fwrq_exit(void);

	unregister_filesystem(&xvfat_fs_type);
	rdchk_exit();
	fwrq_exit();
}

module_init(init_xvfat_fs)
module_exit(exit_xvfat_fs)
MODULE_LICENSE("GPL");
