/*
 *  PPC4XX GPIO driver
 *
 *  Copyright 2003 Sony Corporation.
 *
 *
 * With non GPL files, use following license:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * Otherwise with GPL files, use following license:
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _PPC4XX_GPIO_H_
#define _PPC4XX_GPIO_H_

#include <linux/config.h>
#include <linux/irq.h>
#include <asm/ibm4xx.h>
#include <asm/ppc4xx_pic.h>

/* 
 * input/output mode 
 *    bit 0: disable = 0 / enable = 1
 *    bit 1: in = 0 / out = 1
 *    bit 2: low = 0 / high = 1
 *    bit 3: high-z = 1 (ignores bit 1 & 2)
 *    bit 4,5: Norm = 00, Alt_1 = 01, Alt_2 = 10, Alt_3 = 11
 */
#define __MODE_USED             (1 << 0)
#define __MODE_IN               (0 << 1)
#define __MODE_OUT              (1 << 1)
#define __MODE_LOW              (0 << 2)
#define __MODE_HIGH             (1 << 2)
#define __MODE_HIZ		(1 << 3)
#define GET_ALT_FUNC(x)		(((x) & __MODE_ALT_MASK) >> 4)
#define SET_ALT_FUNC(x)		(((x) & 0x3) << 4)
#define   ALT_1			1
#define   ALT_2			2
#define   ALT_3			3
#define __MODE_ALT_1		SET_ALT_FUNC(ALT_1)
#define __MODE_ALT_2		SET_ALT_FUNC(ALT_2)
#define __MODE_ALT_3		SET_ALT_FUNC(ALT_3)
#define __MODE_ALT_MASK		SET_ALT_FUNC(3)

#define GPIO_MODE_IN            (__MODE_USED | __MODE_IN)
#define GPIO_MODE_OUT_LOW       (__MODE_USED | __MODE_OUT | __MODE_LOW)
#define GPIO_MODE_OUT_HIGH      (__MODE_USED | __MODE_OUT | __MODE_HIGH)
#define GPIO_MODE_HIZ           (__MODE_USED | __MODE_HIZ)

#define GPIO_MODE_IN_ALT1	(__MODE_USED | __MODE_IN | __MODE_ALT_1)
#define GPIO_MODE_IN_ALT2	(__MODE_USED | __MODE_IN | __MODE_ALT_2)
#define GPIO_MODE_IN_ALT3	(__MODE_USED | __MODE_IN | __MODE_ALT_3)

#define GPIO_MODE_OUT_ALT1	(__MODE_USED | __MODE_OUT | __MODE_ALT_1)
#define GPIO_MODE_OUT_ALT2	(__MODE_USED | __MODE_OUT | __MODE_ALT_2)
#define GPIO_MODE_OUT_ALT3	(__MODE_USED | __MODE_OUT | __MODE_ALT_3)



/* Interrrupt trigger mode */
#define GPIO_INTR_EDGE_HIGH     (IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_POS)
#define GPIO_INTR_EDGE_LOW      (IBM4xx_IRQ_SNS_EDG | IBM4xx_IRQ_POL_NEG)
#define GPIO_INTR_LEVEL_HIGH    (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_POS)
#define GPIO_INTR_LEVEL_LOW     (IBM4xx_IRQ_SNS_LVL | IBM4xx_IRQ_POL_NEG)

/*
 * Prototypes for kernel API
 */
#ifdef __KERNEL__

/* port number */
#define GPIO_PORT_0             0
#define GPIO_PORT_1             1

/* port name */
#if GPIO_NUMS > 1
#define PORT_NAME(port)         ("01"[port])
#else
#define PORT_NAME(port)         ("0"[port])
#endif

#define GPIO_BITS_PER_PORT	32


/* internal GPIO structure */
#define MAX_NAME_SIZE           15
struct ppc4xx_gpio_t {
	u_int32_t	used;	/* set 1 if used */
	u_int32_t	inout;	/* set 1 for output/hi-z, 0 for input */
	u_int32_t	intr;	/* set 1 if int. */
#ifdef CONFIG_CXD4707
	u_int32_t	alt;	/* set 1 if Alternate func. is used. */
#endif
	u_int32_t	no_gpio;
	u_int32_t	no_out;
	u_int32_t	no_hiz;
	u_int32_t	no_in;
	u_int32_t	no_int;
#ifdef CONFIG_CXD4707
	u_int32_t	no_altout[3];
	u_int32_t	no_altin[3];
#endif
	struct {
		char  name[MAX_NAME_SIZE + 1];
		int   irq_handler_set;
	} bit[GPIO_BITS_PER_PORT];
};


/* (de)initialization */
int ppc4xx_gpio_register_bit(int port, int bitnum, const char *name);
int ppc4xx_gpio_unregister_bit(int port, int bitnum);
int ppc4xx_gpio_register(int port, u_int32_t bitmask, const char *name);
int ppc4xx_gpio_unregister(int port, u_int32_t bitmask);
int ppc4xx_gpio_unregister_by_name(const char *name);
int ppc4xx_gpio_set_mode_bit(int port, int bitnum, u_int32_t mode);
int ppc4xx_gpio_set_mode(int port, u_int32_t bitmask, u_int32_t mode);

/* get/set GPIO values */
int ppc4xx_gpio_get_data_bit(int port, int bitnum);
int ppc4xx_gpio_set_data_bit(int port, int bitnum, int value);
int ppc4xx_gpio_get_data(int port, u_int32_t bitmask, u_int32_t *value);
int ppc4xx_gpio_set_data(int port, u_int32_t bitmask, u_int32_t value);

/* interrups */
int ppc4xx_gpio_intr_set_mode_bit(int port, int bitnum, u_int32_t mode);
int ppc4xx_gpio_intr_set_mode(int port, u_int32_t bitmask, u_int32_t mode);
int ppc4xx_gpio_intr_get_mode_bit(int port, int bitnum);
int ppc4xx_gpio_intr_register_handler(
	int port, int bitnum, 
	void (*handler)(int irq, void *dev_id, struct pt_regs *regs),
	void *dev_id);
int ppc4xx_gpio_intr_unregister_handler(int port, int bitnum,
					 void *dev_id);
int ppc4xx_gpio_intr_unmask_bit(int port, int bitnum);
int ppc4xx_gpio_intr_mask_bit(int port, int bitnum);
int ppc4xx_gpio_intr_unmask(int port, u_int32_t bitmask);
int ppc4xx_gpio_intr_mask(int port, u_int32_t bitmask);
int ppc4xx_gpio_irq_to_port(int irq);
int ppc4xx_gpio_irq_to_bit(int irq);

int ppc4xx_gpio_init(void);


#endif /* __KERNEL__ */

#endif /* _PPC4XX_GPIO_H_ */
