#ifndef _LINUX_CRYPTOWRAP_H_
#define _LINUX_CRYPTOWRAP_H_

/*
 *  Prototype
 */
typedef struct {int unused;}* CryptoWrap_DES_object;
typedef struct {int unused;}* CryptoWrap_3DES_object;
typedef struct {int unused;}* CryptoWrap_RSA_object;
typedef struct {int unused;}* CryptoWrap_SHA1_object;
typedef struct {int unused;}* CryptoWrap_MD5_object;
typedef struct {int unused;}* CryptoWrap_HMAC_object;
typedef struct {int unused;}* CryptoWrap_AES_object;

/*
 *  DES & Triple DES
 */
int CryptoWrap_DES_new(CryptoWrap_DES_object *objp, int mode);
int CryptoWrap_DES_destroy(CryptoWrap_DES_object obj);
int CryptoWrap_DES_set_key(CryptoWrap_DES_object obj, unsigned char *key);
int CryptoWrap_DES_encrypt(CryptoWrap_DES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv);
int CryptoWrap_DES_decrypt(CryptoWrap_DES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv);

int CryptoWrap_3DES_new(CryptoWrap_3DES_object *obj, int mode);
int CryptoWrap_3DES_destroy(CryptoWrap_3DES_object obj);
int CryptoWrap_3DES_set_key(CryptoWrap_3DES_object obj, unsigned char *key1, unsigned char *key2, unsigned char *key3);
int CryptoWrap_3DES_encrypt(CryptoWrap_3DES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv);
int CryptoWrap_3DES_decrypt(CryptoWrap_3DES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv);

/*
 *  AES
 */
int CryptoWrap_AES_new(CryptoWrap_AES_object *objp, int mode);
int CryptoWrap_AES_destroy(CryptoWrap_AES_object obj);
int CryptoWrap_AES_set_key(CryptoWrap_AES_object obj, unsigned char *key, int keylen);
int CryptoWrap_AES_encrypt(CryptoWrap_AES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv);
int CryptoWrap_AES_decrypt(CryptoWrap_AES_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, unsigned char *iv);

#define CryptoWrap_CIPHER_MODE_ECB	0x01
#define CryptoWrap_CIPHER_MODE_CBC	0x02
#define CryptoWrap_CIPHER_MODE_CFB	0x03
#define CryptoWrap_CIPHER_MODE_OFB	0x04

/*
 *  RSA
 */
int CryptoWrap_RSA_new(CryptoWrap_RSA_object *objp);
int CryptoWrap_RSA_init(CryptoWrap_RSA_object obj);
int CryptoWrap_RSA_destroy(CryptoWrap_RSA_object obj);
int CryptoWrap_RSA_set_public_key(CryptoWrap_RSA_object obj, unsigned char *e, int elen, unsigned char *n, int nlen);
int CryptoWrap_RSA_get_public_key(CryptoWrap_RSA_object obj, unsigned char *e, int *elen, unsigned char *n, int *nlen);
int CryptoWrap_RSA_set_private_key(CryptoWrap_RSA_object obj, unsigned char *d, int dlen, unsigned char *n, int nlen);
int CryptoWrap_RSA_get_private_key(CryptoWrap_RSA_object obj, unsigned char *e, int *elen, unsigned char *n, int *nlen);
int CryptoWrap_RSA_public_encrypt(CryptoWrap_RSA_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, int padding);
int CryptoWrap_RSA_private_decrypt(CryptoWrap_RSA_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen, int padding);
int CryptoWrap_RSA_private_encrypt(CryptoWrap_RSA_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen);
int CryptoWrap_RSA_public_decrypt(CryptoWrap_RSA_object obj, unsigned char *from, int flen, unsigned char *to, int *tlen);
int CryptoWrap_RSA_sign(CryptoWrap_RSA_object obj, unsigned char *msg, int mlen, unsigned char *sig, int *slen, int type);
int CryptoWrap_RSA_verify(CryptoWrap_RSA_object obj, unsigned char *msg, int mlen, unsigned char *sig, int slen, int type);
//CryptoWrap_RSA_generate_key()
int CryptoWrap_RSA_test(CryptoWrap_RSA_object obj);

#define CryptoWrap_RSA_PADDING_PKCS1	0x01
#define CryptoWrap_RSA_PADDING_OAEP	0x02
#define CryptoWrap_RSA_PADDING_NO	0x03

#define CryptoWrap_RSA_SIGN_SHA1_PSS	0x01
#define CryptoWrap_RSA_SIGN_SHA1_PKCS1	0x02
#define CryptoWrap_RSA_SIGN_RAW		0x03

/*
 * SHA1
 */
int CryptoWrap_SHA1_new(CryptoWrap_SHA1_object *objp);
int CryptoWrap_SHA1_destroy(CryptoWrap_SHA1_object obj);
void CryptoWrap_SHA1_Init(CryptoWrap_SHA1_object obj);
void CryptoWrap_SHA1_Update(CryptoWrap_SHA1_object obj, const unsigned char *data, int len);
void CryptoWrap_SHA1_Final(CryptoWrap_SHA1_object obj, unsigned char *md);
//SHA1()

#define CryptoWrap_SHA1_DIGEST_LEN	20

/*
 * MD5
 */
int CryptoWrap_MD5_new(CryptoWrap_MD5_object *objp);
int CryptoWrap_MD5_destroy(CryptoWrap_MD5_object obj);
void CryptoWrap_MD5_Init(CryptoWrap_MD5_object obj);
void CryptoWrap_MD5_Update(CryptoWrap_MD5_object obj, unsigned char *data, int len);
void CryptoWrap_MD5_Final(CryptoWrap_MD5_object obj, unsigned char *md);
//MD5()

#define CryptoWrap_MD5_DIGEST_LEN	16

/*
 * HMAC
 */
int CryptoWrap_HMAC_new(CryptoWrap_HMAC_object *objp, int type);
int CryptoWrap_HMAC_destroy(CryptoWrap_HMAC_object obj);
void CryptoWrap_HMAC_Init(CryptoWrap_HMAC_object obj, const unsigned char *key, int keylen);
void CryptoWrap_HMAC_Update(CryptoWrap_HMAC_object obj, const unsigned char *data, int len);
void CryptoWrap_HMAC_Final(CryptoWrap_HMAC_object obj, unsigned char *md, int *len);

#define CryptoWrap_HMAC_DIGEST_SHA1	0x01
#define CryptoWrap_HMAC_DIGEST_MD5	0x02

#define CryptoWrap_HMAC_BLOCKSIZE	64

/*
 * RAND (random number)
 */
void CryptoWrap_RAND(void *out, int size);

/*
 *  Return Code
 */
#define CryptoWrap_SUCCESS		0
#define CryptoWrap_E_NOTSUPPORTED	-1
#define CryptoWrap_E_NOMEM		-2
#define CryptoWrap_E_INTERNAL		-3
#define CryptoWrap_E_VERIFY		-4
#define CryptoWrap_E_BADPARITY		-5
#define CryptoWrap_E_WEAKKEY		-6
#define CryptoWrap_E_MSG_TOO_LONG	-7
#define CryptoWrap_E_MSG_TOO_SHORT	-8
#define CryptoWrap_E_KEY_TOO_SHORT	-9
#define CryptoWrap_E_MSG_INVALID	-10
#define CryptoWrap_E_NOSPC		-11


#endif /* _LINUX_CRYPTOWRAP_H_ */
