#ifndef _LINUX_CRYPTOWRAP_IOCTL_H_
#define _LINUX_CRYPTOWRAP_IOCTL_H_

#define CRYPTOWRAP_IOCTL_MAJOR	0	/* dynamic */

typedef union cryptowrap_ioctl_cipher_data {
	struct {
		unsigned char	from[1024];
		int		flen;
		unsigned char	to[1024];
		int		tlen;
		unsigned char	iv[8];
	} des, triple_des;
	struct {
		unsigned char	from[1024];
		int		flen;
		unsigned char	to[1024];
		int		tlen;
		unsigned char	iv[16];
	} aes;
	struct {
		unsigned char	from[1024];
		int		flen;
		unsigned char	to[1024];
		int		tlen;
		int		mode;	/* public or private */
	} rsa;
} CryptoWrap_Cipher_Data;

typedef union cryptowrap_ioctl_key {
	struct {
		unsigned char key[8];
	} des;
	struct {
		unsigned char key1[8];
		unsigned char key2[8];
		unsigned char key3[8];	
	} triple_des;
	struct {
		unsigned char key[32];
		int keylen;
	} aes;
} CryptoWrap_Cipher_Key;

typedef struct cryptowrap_ioctl_rsa_key {
	struct {
		unsigned char e[1024];
		unsigned char n[1024];
	} public;
	struct {
		unsigned char d[1024];
		unsigned char n[1024];
	} private;
} CryptoWrap_Rsa_Key;

typedef struct cryptowrap_ioctl_digest_data {
	unsigned char	data[1024];
	int		len;
} CryptoWrap_Digest_Data;

typedef struct cryptowrap_ioctl_digest_vaule {
	unsigned char	md[20];
	int		len;
} CryptoWrap_Digest_Value;

typedef struct cryptowrap_ioctl_auth_key {
	unsigned char key[1024];
	int len;
} CryptoWrap_Auth_Key;

#define CRYPTOWRAP_IOC_MAGIC	0x88

#define CRYPTOWRAP_IOCENCRYPT _IOWR(CRYPTOWRAP_IOC_MAGIC, 0x01, CryptoWrap_Cipher_Data)
#define CRYPTOWRAP_IOCDECRYPT _IOWR(CRYPTOWRAP_IOC_MAGIC, 0x02, CryptoWrap_Cipher_Data)
#define CRYPTOWRAP_IOCSETKEY _IOWR(CRYPTOWRAP_IOC_MAGIC, 0x03, CryptoWrap_Cipher_Key)
#define CRYPTOWRAP_IOCSIGN _IOWR(CRYPTOWRAP_IOC_MAGIC, 0x04, CryptoWrap_Cipher_Data)
#define CRYPTOWRAP_IOCVERIFY _IOWR(CRYPTOWRAP_IOC_MAGIC, 0x05, CryptoWrap_Cipher_Data)

#define CRYPTOWRAP_IOCINIT _IO(CRYPTOWRAP_IOC_MAGIC, 0x06)
#define CRYPTOWRAP_IOCUPDATE _IOW(CRYPTOWRAP_IOC_MAGIC, 0x07, CryptoWrap_Digest_Data)
#define CRYPTOWRAP_IOCFINAL _IOR(CRYPTOWRAP_IOC_MAGIC, 0x08, CryptoWrap_Digest_Value)
#define CRYPTOWRAP_IOCHINIT _IOW(CRYPTOWRAP_IOC_MAGIC, 0x09, CryptoWrap_Auth_Key)

#endif /* _LINUX_CRYPTOWRAP_IOCTL_H_ */
