/*
 *  CXD9790GG -- GPIO, interrupt, 4ch UART and I2C controller
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _CXD9790_H_
#define _CXD9790_H_

#include <linux/config.h>
#ifdef CONFIG_SNSC_MPU22X
#include <asm/tx4927/snsc_mpu22x/cxd9790.h>
#endif
#ifdef CONFIG_SNSC_MPU23X
#include <asm/cxd9802/snsc_mpu23x/cxd9790.h>
#endif

#if defined(__powerpc__)
#include <asm/cxd9790.h>
#endif

#include <linux/cxd9790_reg.h>

/*
 * CXD9790 common functions
 */
int cxd9790_present(void);
#ifdef CONFIG_SNSC_CXD9790_ES1_WORKAROUND
int cxd9790_is_es1(void);
#endif



/*----------------------------------------------------------------------*/
/*  CXD9790 interrupt controller                                        */
/*    supports two types of irq routins                                 */
/*      [Type1]                                                         */
/*        "irq_desc_t" and "struct hw_interrupt_type", defined by       */
/*        include/linux/irq.h, are used to manage each IRQ              */
/*        (mips/i386/ppc)                                               */
/*                                                                      */
/*      [Type2]                                                         */
/*        Type2 is arm, which locally defines "struct irqdesc" in       */
/*        include/asm-arm/mach/irq.h                                    */
/*----------------------------------------------------------------------*/

#if defined(__mips__) || defined(__i386__) || defined(__powerpc__)
#define CXD9790_IRQ_TYPE1
#undef  CXD9790_IRQ_TYPE2
#endif

#if defined(__arm__)
#undef  CXD9790_IRQ_TYPE1
#define CXD9790_IRQ_TYPE2
#endif


/*
 * Trigger type:
 *     used by cxd9790_set_trigger_mode() function
 */
#define TRIG_LOW_LEVEL          (IRDM_LEVEL | IRDM_LOW )
#define TRIG_HIGH_LEVEL         (IRDM_LEVEL | IRDM_HIGH)
#define TRIG_FALLING_EDGE       (IRDM_EDGE  | IRDM_LOW )
#define TRIG_RISING_EDGE        (IRDM_EDGE  | IRDM_HIGH)

/*
 * IROUT type:
 *     used by cxd9790_set_irout() function
 */
#define IROUT0                  0
#define IROUT1                  1


/* CXD9790 IRQS */
#define CXD9790_IRQ_UART0         (CXD9790_IRQ_OFFSET + 0)
#define CXD9790_IRQ_UART1         (CXD9790_IRQ_OFFSET + 1)
#define CXD9790_IRQ_UART2         (CXD9790_IRQ_OFFSET + 2)
#define CXD9790_IRQ_UART3         (CXD9790_IRQ_OFFSET + 3)
#define CXD9790_IRQ_I2C           (CXD9790_IRQ_OFFSET + 4)
#define CXD9790_IRQ_GPCD          (CXD9790_IRQ_OFFSET + 5)
#define CXD9790_IRQ_GPAB          (CXD9790_IRQ_OFFSET + 6)
#define CXD9790_IRQ_GPZ           (CXD9790_IRQ_OFFSET + 7)
#define CXD9790_IRQ_GPIOZ(x)      (CXD9790_IRQ_OFFSET + 8 + x)
#define CXD9790_IRQ_GPIOA(x)      (CXD9790_IRQ_OFFSET + 16 + x)
#define CXD9790_IRQ_GPIOB(x)      (CXD9790_IRQ_OFFSET + 24 + x)
#define CXD9790_IRQ_GPIOC(x)      (CXD9790_IRQ_OFFSET + 32 + x)
#define CXD9790_IRQ_GPIOD(x)      (CXD9790_IRQ_OFFSET + 40 + x)

#define CXD9790_NR_IRQS_IRCS      8
#define CXD9790_NR_IRQS_GPIO      40
#define CXD9790_NR_IRQS           (CXD9790_NR_IRQS_IRCS + CXD9790_NR_IRQS_GPIO)

/* prototypes */
int  cxd9790_irq_init(void);
int  cxd9790_set_trigger_mode(int irq, int trig);
int  cxd9790_set_irout(int irq, int irout);
int  cxd9790_get_irq(int irout);

#ifdef CXD9790_IRQ_TYPE1
extern struct irqaction cxd9790_ircs_irq_action;
#endif

#endif /* _CXD9790_H_ */
