/*
 *  CXD9790 GPIO driver
 *
 *  Copyright 2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _CXD9790_GPIO_H_
#define _CXD9790_GPIO_H_

#include <linux/config.h>
#include <linux/irq.h>
#include <linux/cxd9790.h>

/* port number */
#define GPIO_PORT_Z             0
#define GPIO_PORT_A             1
#define GPIO_PORT_B             2
#define GPIO_PORT_C             3
#define GPIO_PORT_D             4

/* bitmask */
#define GPIO_BITS_ALL           0xff

/* 
 * input/output mode 
 *    bit 0: disable = 0 / enable = 1
 *    bit 1: in = 0 / out = 1
 *    bit 2: low = 0 / high = 1
 */
#define __MODE_USED             (1 << 0)
#define __MODE_IN               (0 << 1)
#define __MODE_OUT              (1 << 1)
#define __MODE_LOW              (0 << 2)
#define __MODE_HIGH             (1 << 2)
#define GPIO_MODE_IN            (__MODE_USED | __MODE_IN)
#define GPIO_MODE_OUT_LOW       (__MODE_USED | __MODE_OUT | __MODE_LOW)
#define GPIO_MODE_OUT_HIGH      (__MODE_USED | __MODE_OUT | __MODE_HIGH)

/* Interrrupt trigger mode */
#define GPIO_INTR_EDGE_HIGH     (TRIG_RISING_EDGE)
#define GPIO_INTR_EDGE_LOW      (TRIG_FALLING_EDGE)
#define GPIO_INTR_LEVEL_HIGH    (TRIG_HIGH_LEVEL)
#define GPIO_INTR_LEVEL_LOW     (TRIG_LOW_LEVEL)


/*
 * Prototypes for kernel API
 */
#ifdef __KERNEL__
/* (de)initialization */
int cxd9790_gpio_register_bit(int port, int bitnum, const char *name);
int cxd9790_gpio_unregister_bit(int port, int bitnum);
int cxd9790_gpio_register(int port, __u8 bitmask, const char *name);
int cxd9790_gpio_unregister(int port, __u8 bitmask);
int cxd9790_gpio_unregister_by_name(const char *name);
int cxd9790_gpio_set_mode_bit(int port, int bitnum, __u32 mode);
int cxd9790_gpio_set_mode(int port, __u8 bitmask, __u32 mode);

/* get/set GPIO values */
int cxd9790_gpio_get_data_bit(int port, int bitnum);
int cxd9790_gpio_set_data_bit(int port, int bitnum, int value);
int cxd9790_gpio_get_data(int port, __u8 bitmask, __u8 *value);
int cxd9790_gpio_set_data(int port, __u8 bitmask, __u8 value);

/* interrups */
int cxd9790_gpio_intr_set_mode_bit(int port, int bitnum, __u32 mode);
int cxd9790_gpio_intr_set_mode(int port, __u8 bitmask, __u32 mode);
int cxd9790_gpio_intr_register_handler(
	int port, int bitnum, 
	void (*handler)(int irq, void *dev_id, struct pt_regs *regs),
	void *dev_id);
int cxd9790_gpio_intr_unregister_handler(int port, int bitnum,
					 void *dev_id);
int cxd9790_gpio_intr_unmask_bit(int port, int bitnum);
int cxd9790_gpio_intr_mask_bit(int port, int bitnum);
int cxd9790_gpio_intr_unmask(int port, __u8 bitmask);
int cxd9790_gpio_intr_mask(int port, __u8 bitmask);
int cxd9790_gpio_irq_to_port(int irq);
int cxd9790_gpio_irq_to_bit(int irq);

int cxd9790_gpio_init(void);
#endif /* __KERNEL__ */

#endif /* _CXD9790_GPIO_H_ */
