/*
 * Copyright 2003 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#ifndef __FWRQ_H__
#define __FWRQ_H__

#define FWRQ_ST_VALID	0
#define FWRQ_ST_BLKTOP	1
#define FWRQ_ST_INORD	2	/* available in blktop only */
#define FWRQ_ST_PEND	3	/* available in blktop only */
#define FWRQ_ST_BLOCK	4
#define FWRQ_ST_SINGLE	5

#define FWRQ_TSST_REJECT	0
#define FWRQ_TSST_LOCK		1
#define FWRQ_TSST_VALID		2
#define FWRQ_TSST_FIXED		3

#define FWRQ_DHST_EIO		0
#define FWRQ_DHST_MEDIAIN	1
#define FWRQ_DHST_MCHECK	2

#define FWRQ_TS_NOLOCK	0
#define FWRQ_TS_LOCK	1


struct fwrq_block_i {
	struct list_head block_list;	/* available in first of sequence block only */
	struct list_head hash_list;	/* available in blktop only */
	struct list_head seq_list;	/* available in blktop only */
					/* circular list of blktops */
	int b_blkcnt;			/* available in blktop only */
	unsigned long b_blkno;
};

struct fwrq_ts_i {
	struct list_head ts_head;	/* available in blktop only */
	struct list_head ts_tail;	/* available in blktop only */
};

struct fwrq_head {
	struct list_head fwrqh_list;	/* circular list of fwrq_head in */
					/* same flash memory erase block */
	struct buffer_head *f_bh;
	struct fwrq_block_i b;
	struct fwrq_ts_i t;
	struct fwrq_dev_head *f_dh;
	unsigned int f_state;
};


#define FWRQ_BLKTOP_HASH_SIZE	1024

struct fwrq_dev_head {
	struct list_head dh_removable_list;
	struct list_head dh_hash_list;
	struct list_head block_queue;
	struct list_head block_pending;
	kdev_t dh_dev;
	atomic_t dh_count;
	unsigned long dh_secbytes;	/* secter size in bytes		*/
	unsigned long block_size;	/* secters par erase block size */
	unsigned long block_offset;	/* by secter			*/
	unsigned long cluster_size;	/* secters par FAT cluster size */
	struct list_head blktop_hash_table[FWRQ_BLKTOP_HASH_SIZE];
	struct list_head transaction_head;
	struct semaphore dh_work_sem;
	struct semaphore dh_tslock_sem;
	pid_t dh_tslock_pid;
	char dh_tslock_comm[16];
	int reject_count;
	struct blkdev_io_error *bioe;
	int dh_state;
};

struct fwrq_transaction {
	struct list_head ts_list;
	struct list_head head_list;
	struct list_head tail_list;
	struct fwrq_head *t_head;
	struct fwrq_head *t_tail;
	struct fwrq_dev_head *t_dh;
	int t_state;
};

typedef struct fwrq_transaction fwrq_transaction_t;

static inline int buffer_fwrq(struct buffer_head *bh)
{
	return __buffer_state(bh, FWRQ);
}

#define __fwrq_buffer_state(fh, state)	(((fh)->f_state & (1UL << FWRQ_ST_##state)) != 0)
static inline int fwrq_buffer_valid(struct fwrq_head *fh)
{
	return __fwrq_buffer_state(fh, VALID);
}

static inline int fwrq_buffer_blktop(struct fwrq_head *fh)
{
	return __fwrq_buffer_state(fh, BLKTOP);
}

static inline int fwrq_buffer_block(struct fwrq_head *fh)
{
	return __fwrq_buffer_state(fh, BLOCK);
}

static inline int fwrq_buffer_pending(struct fwrq_head *fh)
{
	return __fwrq_buffer_state(fh, PEND);
}

static inline int fwrq_buffer_inorder(struct fwrq_head *fh)
{
	return __fwrq_buffer_state(fh, INORD);
}

#define __fwrq_ts_state(ts, state)	(((ts)->t_state & (1UL << FWRQ_TSST_##state)) != 0)

static inline int fwrq_ts_reject(fwrq_transaction_t *ts)
{
	return __fwrq_ts_state(ts, REJECT);
}

static inline int fwrq_ts_lock(fwrq_transaction_t *ts)
{
	return __fwrq_ts_state(ts, LOCK);
}

static inline int fwrq_ts_valid(fwrq_transaction_t *ts)
{
	return __fwrq_ts_state(ts, VALID);
}

static inline int fwrq_ts_fixed(fwrq_transaction_t *ts)
{
	return __fwrq_ts_state(ts, FIXED);
}

#define __fwrq_dh_state(dh, state)	(((dh)->dh_state & (1UL << FWRQ_DHST_##state)) != 0)

static inline int fwrq_dh_eio(struct fwrq_dev_head *dh)
{
	return __fwrq_dh_state(dh, EIO);
}

static inline int fwrq_dh_mediain(struct fwrq_dev_head *dh)
{
	return __fwrq_dh_state(dh, MEDIAIN);
}

static inline int fwrq_dh_mcheck(struct fwrq_dev_head *dh)
{
	return __fwrq_dh_state(dh, MCHECK);
}

extern void fwrq_init_queue(kdev_t dev, int blksiz, int blkoff, int clssiz);
extern void fwrq_finish_queue(kdev_t dev);
extern int fwrq_add_queue(struct buffer_head *bh);
extern int fwrq_flush_queue(kdev_t dev, int wait);

extern fwrq_transaction_t *fwrq_transaction_get(kdev_t dev, int flags);
extern void fwrq_transaction_add_head(fwrq_transaction_t *transaction,
				      struct buffer_head *bh);
extern void fwrq_transaction_add_tail(fwrq_transaction_t *transaction,
				      struct buffer_head *bh);
extern void fwrq_transaction_add_pos(fwrq_transaction_t *transaction,
				     struct buffer_head *bh, int pos);
extern void fwrq_transaction_concat(fwrq_transaction_t *transaction,
				    fwrq_transaction_t *append);
extern void fwrq_transaction_append1(fwrq_transaction_t *transaction,
				     fwrq_transaction_t *append);
extern int fwrq_transaction_commit_to_queue(fwrq_transaction_t *transaction);
extern int
  fwrq_transaction_commit_to_queue_and_flush(fwrq_transaction_t *transaction,
					     int wait);
extern int fwrq_transaction_fix(fwrq_transaction_t *transaction);
extern int fwrq_check_blkdev_media(kdev_t dev, int call_daemon);
extern int fwrq_revalidate_blkdev_media(kdev_t dev);
extern int fwrq_invalidate(kdev_t dev);

/* fs/fat/inode.c */
extern void fwrq_add_queue_inode(struct inode *inode);
extern void fwrq_transaction_add_tail_inode(fwrq_transaction_t *transaction, struct inode *inode);

#define print_bh(bh, msg) printk("%s: [%d] hashed=%d page=%08x b_count=%d %c%c%c%c%c%c%c%c%c\n", (msg), \
			       ((bh)->b_blocknr * ((bh)->b_size >> 9)), \
				 (bh)->b_pprev != 0, \
			       (bh)->b_page, (bh)->b_count, \
			       buffer_uptodate(bh) ? 'U' : '-', \
			       buffer_dirty(bh) ? 'D' : '-', \
			       buffer_locked(bh) ? 'L' : '-', \
			       buffer_req(bh) ? 'R' : '-', \
			       buffer_mapped(bh) ? 'M' : '-', \
			       buffer_new(bh) ? 'N' : '-', \
			       buffer_async(bh) ? 'A' : '-', \
			       buffer_delay(bh) ? 'd' : '-', \
			       __buffer_state((bh),FWRQ) ? 'F' : '-')

#undef VFAT_DEBUG
#undef VFAT_DEBUG2
#undef FWRQ_DEBUG
#undef FWRQ_DEBUG2

#ifdef VFAT_DEBUG
#define vfat_printk(x...)	printk(x)
#else
#define vfat_printk(x...)	do{}while(0)
#endif

#ifdef VFAT_DEBUG2
#define vfat_printk2(x...)	printk(x)
#else
#define vfat_printk2(x...)	do{}while(0)
#endif

#ifdef FWRQ_DEBUG
#define fwrq_printk(x...)	printk(x)
#else
#define fwrq_printk(x...)	do{}while(0)
#endif

#ifdef FWRQ_DEBUG2
#define fwrq_printk2(x...)	printk(x)
#else
#define fwrq_printk2(x...)	do{}while(0)
#endif

#endif /* ! __FWRQ_H__ */
