/* With non GPL files, use following license */
/*
 * Copyright 2004 Sony Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Otherwise with GPL files, use following license */
/*
 * Copyright 2004 Sony Corporation.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * header file for MMS driver
 */

#ifndef __MMS_H__
#define __MMS_H__

#ifdef CONFIG_MMS_MAX_DEVICES
  #define MMS_MAX_DEVICES	CONFIG_MMS_MAX_DEVICE
#else
  #define MMS_MAX_DEVICES	1
#endif

#define MMS_UNIT_SIZE			(16 * 1024 * 1024)
#define MMS_RESERVE_BLOCKS_PER_UNIT	22

/* ---------------------------------------- *
 * MMS ioctl                                *
 * ---------------------------------------- */

#define MMS_IOC_MAGIC	'm'

#define MMS_IOC_GETBLOCKSIZE	_IOR (MMS_IOC_MAGIC, 1, u_int32_t)
#define MMS_IOC_GETPAGESIZE	_IOR (MMS_IOC_MAGIC, 2, u_int32_t)
#define MMS_IOC_GETNUMPBLOCKS	_IOR (MMS_IOC_MAGIC, 3, u_int32_t)

/* ---------------------------------------- *
 * MMS block info                           *
 * ---------------------------------------- */
struct mms_info_t {
	u_int16_t state;
	u_int16_t lblock;
	u_int16_t prev_pblock;
	u_int8_t  ecc;
};

struct mms_dev_t;

/* ---------------------------------------- *
 * MMS nand device operations               *
 * ---------------------------------------- */
struct mms_nandif_ops {
	int (*read_block) (struct mms_dev_t *, u_int32_t, void *,
			   struct mms_info_t *);
	int (*read_mms_info) (struct mms_dev_t *, u_int32_t,
			      struct mms_info_t *);
	int (*write_block) (struct mms_dev_t *, u_int32_t, const void *,
			    const struct mms_info_t *);
	int (*write_mms_info) (struct mms_dev_t *, u_int32_t,
			       const struct mms_info_t *);
	int (*erase_block) (struct mms_dev_t *, u_int32_t);
	int (*mark_badblock) (struct mms_dev_t *, u_int32_t);
	int (*is_badblock) (struct mms_dev_t *, u_int32_t);
	int (*lock) (struct mms_dev_t *);
	int (*unlock) (struct mms_dev_t *);
#if 1 /* for table setup at insmod (mms_register_nand_device) */
	int (*read_page_sync)(struct mms_dev_t *, u_int32_t, void *);
	int (*read_mms_info_sync)(struct mms_dev_t *, u_int32_t,
				  struct mms_info_t *);
	int (*write_mms_info_sync)(struct mms_dev_t *, u_int32_t,
				   const struct mms_info_t *);
	int (*erase_block_sync)(struct mms_dev_t *, u_int32_t);
	int (*mark_badblock_sync)(struct mms_dev_t *, u_int32_t);
	int (*is_badblock_sync)(struct mms_dev_t *, u_int32_t);
	int (*fixup_ecc_sync)(struct mms_dev_t *, u_int32_t);
#endif
};

/* ---------------------------------------- *
 * MMS nand device info                     *
 * ---------------------------------------- */
struct mms_nand_t {
	u_int32_t offset;		/* start block on real NAND device */
	u_int32_t nblocks;		/* number of blocks */
	u_int32_t block_size;
	u_int32_t page_size;
	struct mms_nandif_ops *ops;
	void *priv;
};

/* ---------------------------------------- *
 * MMS device structure                     *
 * ---------------------------------------- */
struct mms_dev_t {
	struct mms_nand_t *nand;

	u_int32_t size;		/* size of mms device (in bytes) */

	u_int16_t *lptable;	/* logical -> phisycal translation table */
	u_int16_t *freetable;	/* free block table */
	u_int16_t nfreeblocks;	/* number of element in freetable */
	u_int16_t free_idx;	/* current element in freetable */

	void *req;		/* request identifier */
	int rw;                 /* request type (read or write) */
	u_int32_t lblock;	/* logical block number */
	u_int32_t pblock;	/* phisical block number */
	u_int32_t state;	/* sequence state */
	int nretry;		/* retry counter */

	void *buf;		/* buffer for sequence */

	void *devdata;		/* private data field for device driver */
};


/* ---------------------------------------- *
 * exported functions                       *
 * ---------------------------------------- */

int mms_register_nand_device(struct mms_nand_t *nand);
void mms_dispatch(struct mms_dev_t *dev, int result);

void mms_start_read_seq(struct mms_dev_t *dev, u_int32_t lblock, void *buf);
void mms_start_write_seq(struct mms_dev_t *dev, u_int32_t lblock, void *buf);


/* ---------------------------------------- *
 * mms_info state                           *
 * ---------------------------------------- */
#define WRITE_START	(0xffff)
#define WRITE_DONE	(0xaaaa)
#define PREV_ERASED	(0x0000)


/* ---------------------------------------- *
 * error code                               *
 * ---------------------------------------- */
enum mms_error {
	EMMS_INVAL = 1,
	EMMS_REGISTER_DEVICE,
	EMMS_DATACORRECTED,
	EMMS_ECCCORRECTED,
	EMMS_TIMEOUT,
	EMMS_READ,
	EMMS_WRITE,
	EMMS_ERASE,
	EMMS_LOCK,
	EMMS_UNLOCK,
	EMMS_MEDIA,
};


/* ---------------------------------------- *
 * debug print                              *
 * ---------------------------------------- */

#define DBG_FLG_RQ	(1 << 0)
#define DBG_FLG_DEV	(1 << 1)
#define DBG_FLG_SEQ	(1 << 2)
#define DBG_FLG_BH	(1 << 3)
#define DBG_FLG_IOC	(1 << 4)

#define DBG_FLG_MKTBL	(1 << 31)


#ifndef MMS_DYNAMIC_DEBUG
#ifndef DBG_FLG
#define DBG_FLG		0
#endif
#else
#undef  DBG_FLG
#define DBG_FLG		debug_flag
#endif

#define DPR_FLG(flg, args...)					\
do {								\
	if ((flg) & DBG_FLG) DPR(args);				\
} while (0)

#endif /* __MMS_H__ */
