/*
 *  Copyright 2002 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: sdmparse_conf.h,v 1.5 2004/06/02 07:03:46 satoru-i Exp $
 */

#ifndef _SDM_PARSE_CONF_H_
#define _SDM_PARSE_CONF_H_

#ifdef __KERNEL__

/* use CRC checking or not */
#define SDM_CRC_CHECK

/* support zlib decompression */
#define SDM_ZLIB_DECOMP
#define SDM_ZALLOC               0      /* use default func. */
#define SDM_ZFREE                0      /* use default func. */

/* adjust crc value to SDM */
/*
 * XXX: these macro currently uses JFFS2's crc32 routine, so it assumes that
 *      JFFS2 support is always enabled when sdmparse is used. 
 */
//#define sdm_crc32(buf, len)      (crc32(0xffffffff, buf, len) ^ 0xffffffff)
#define sdm_crc32(buf, len)      (crc32(0x0, buf, len))
#define gz_crc32(crc, buf, len)  (crc32(crc, buf, len))


#endif  /* __KERNEL__ */


/*
 * include platform dependent settings
 */
/* MPU-110 series */
#ifdef CONFIG_ARCH_DRAGONBALL
#include <asm/arch/mpu110_series.h>
#endif

/* MPU-210 */
#if defined(CONFIG_MTD_SNSC_MPU210) || defined(CONFIG_MTD_NAND_MPU210)
#include <asm/it8172/snsc_mpu210.h>
#endif

/* CXD4707 */
#ifdef CONFIG_MTD_NAND_CXD4707
#include <asm/cxd4707_nand.h>
#endif
#ifdef CONFIG_MTD_NAND_CXD4707_2X
#include <asm/cxd4707_2x_nand.h>
#endif

/* parameter check */
#if !defined(SDM_NOR_DEVICES)
#error "SDM_NOR_DEVICES is not defined!"
#endif

#if !defined(SDM_NAND_DEVICES)
#error "SDM_NAND_DEVICES is not defined!"
#endif

#if !defined(SDM_MASTER_DEV)
#warning "SDM_MASTER_DEV is not defined explicitly."
#warning "assume that master device is first nand device." 
#define SDM_MASTER_DEV   (SDM_NOR_DEVICES)
#endif

#define MAX_SDM_DEV (SDM_NOR_DEVICES + SDM_NAND_DEVICES)

#ifndef SDM_DEFAULT_MAXDESC
#warning "SDM_DEFAULT_MAXDESC is not defined in sdmparse_conf.h for current target platform, using default setting"
#define SDM_DEFAULT_MAXDESC     16 
#endif

#ifndef SDM_TABLE_STARTSECT
#warning "SDM_TABLE_STARTSECT is not defined in sdmparse_conf.h for current target platform, using default setting"
#define SDM_TABLE_STARTSECT      0
#endif

#endif /* _SDM_PARSE_CONF_H_ */
