/*
 * SDM wearleveling library
 */
/* With non GPL files, use following license */
/*
 * Copyright 2004 Sony Corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/* Otherwise with GPL files, use following license */
/*
 * Copyright 2004 Sony Corporation.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SDMWL_H__
#define __SDMWL_H__

/* error code */
#define SDMWL_ERR_OK            0
#define SDMWL_ERR_EINVAL       -1
#define SDMWL_ERR_EIO          -2
#define SDMWL_ERR_EPARTNAME    -3
#define SDMWL_ERR_EPARTSIZE    -4
#define SDMWL_ERR_EINVALNUM    -5
#define SDMWL_ERR_EINVALCHUNK  -6
#define SDMWL_ERR_EWRITECHUNK  -7

#define SDMWL_ERR_ENODEV       -64


struct sdmwl_chunk_info {
	unsigned int chunk_size;
	unsigned int color_offset;
	int (*isvalid)(void *buf);
	void (*setcrc)(void *buf);
};

struct sdmwl_desc {
	void *priv;
	void *buf;
	struct sdmwl_chunk_info chunk_info;
	unsigned int num_chunks;
	unsigned int num_sectors;
	unsigned int sector_size;
	unsigned int sector_shift;
	unsigned int chunks_per_sector;
	unsigned int sectors_per_chunk;
	unsigned int latest;                /* latest chunk number */
	unsigned short color;               /* color of latest chunk */
};

struct sdmwl_dev_ops {
	int (*open)(const char *name, void **priv);
	int (*close)(void *priv);
	int (*read)(void *priv, void *buf,
		    unsigned int offset, unsigned int length);
	int (*write)(void *priv, const void *buf,
		     unsigned int offset, unsigned int length);
	int (*erase)(void *priv, unsigned int offset, unsigned int sectors);
	int (*query_sector_size)(void *priv);
	int (*query_num_sectors)(void *priv);
};

int sdmwl_register_dev(const struct sdmwl_dev_ops *);

int sdmwl_open(const char *name, const struct sdmwl_chunk_info *chinfo,
	       void *buf, struct sdmwl_desc *desc);
int sdmwl_query_numchunk(struct sdmwl_desc *desc);
int sdmwl_read_chunk(struct sdmwl_desc *desc, int chunk_num, void *buf);
int sdmwl_write_chunk(struct sdmwl_desc *desc, void *buf);
int sdmwl_close(struct sdmwl_desc *desc);


/* sdmwl buffer should be cacheline aligned on some platform */
#ifdef CONFIG_SDMWL_CACHELINE_ALIGNED
#define SDMWL_CACHELINE_ALIGNED __attribute__((__aligned__(L1_CACHE_BYTES)))
#else /* CONFIG_SDMWL_CACHELINE_ALIGNED */
#define SDMWL_CACHELINE_ALIGNED
#endif /* CONFIG_SDMWL_CACHELINE_ALIGNED */

#endif /* __SDMWL_H__ */
