/*
 *  linux/include/linux/snsc_ich_gpio.h
 * 
 *  GPIO functions for ICH2
 *
 *  Copyright 2001-2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SNSC_ICH_GPIO_H__
#define __SNSC_ICH_GPIO_H__

#include <linux/ioctl.h>
#include <linux/sched.h>

struct ichgpio_iocdata {
        __u32  id;
        __u32  val;
};

#define ICHGPIO_IOC_MAGIC  'i'
#define ICHGPIO_IOC_USESEL       _IOW(ICHGPIO_IOC_MAGIC, 0, struct ichgpio_iocdata)
#define ICHGPIO_IOC_IOSEL        _IOW(ICHGPIO_IOC_MAGIC, 1, struct ichgpio_iocdata)
#define ICHGPIO_IOC_SIGINV       _IOW(ICHGPIO_IOC_MAGIC, 2, struct ichgpio_iocdata)
#define ICHGPIO_IOC_CLRSTAT      _IO(ICHGPIO_IOC_MAGIC, 3)
#define ICHGPIO_IOC_READSTAT     _IO(ICHGPIO_IOC_MAGIC, 4)
#define ICHGPIO_IOC_IN           _IO(ICHGPIO_IOC_MAGIC, 5)
#define ICHGPIO_IOC_OUT          _IO(ICHGPIO_IOC_MAGIC, 6)


/* for ichgpio_use_sel */
#define ICHGPIO_USESEL_NATIVE   0
#define ICHGPIO_USESEL_GPIO     1

/* for ichgpio_io_sel */
#define ICHGPIO_IOSEL_OUT       0
#define ICHGPIO_IOSEL_IN        1

/* for ichgpio_sig_inv */
#define ICHGPIO_SIGINV_HIGH     0
#define ICHGPIO_SIGINV_LOW      1

/* for ichgpio_ctrl_rout */
#define ICHGPIO_CTRLROUT_NO     0
#define ICHGPIO_CTRLROUT_SMI    1
#define ICHGPIO_CTRLROUT_SCI    2


/* for common GPIO interface */
#define GPIO_BITS_ALL           0xffffffff

#define GPIO_MODE_IN            0
#define GPIO_MODE_OUT_HIGH      1
#define GPIO_MODE_OUT_LOW       2

#define GPIO_INTR_LEVEL_MASK    0x000f
#define GPIO_INTR_LEVEL_HIGH    0x0000
#define GPIO_INTR_LEVEL_LOW     0x0001

#define GPIO_INTR_ROUT_MASK     0x00f0
#define GPIO_INTR_ROUT_SCI      0x0000
#define GPIO_INTR_ROUT_SMI      0x0010
#define GPIO_INTR_ROUT_NO       0x0020

#ifdef __KERNEL__
extern int ichgpio_init(void);
/* for common GPIO interface */
extern int ich_gpio_register_bit(int port, int bitnum, const char *name);
extern int ich_gpio_unregister_bit(int port, int bitnum);
extern int ich_gpio_register(int port, __u32 bitmask, const char *name);
extern int ich_gpio_unregister(int port, __u32 bitmask);
extern int ich_gpio_unregister_by_name(const char *name);
extern int ich_gpio_set_mode_bit(int port, int bitnum, __u32 mode);
extern int ich_gpio_set_mode(int port, __u32 bitmask , __u32 mode);
extern int ich_gpio_get_data_bit(int port, int bitnum);
extern int ich_gpio_set_data_bit(int port, int bitnum, int value);
extern int ich_gpio_get_data(int port, __u32 bitmask, __u32 *value);
extern int ich_gpio_set_data(int port, __u32 bitmask, __u32 value);
//extern void ich_gpio_intr_enable( void ); /* optional */
//extern void ich_gpio_intr_disable( void ); /* optional */
extern int ich_gpio_intr_set_mode_bit(int port, int bitnum, __u32 mode);
extern int ich_gpio_intr_set_mode(int port, __u32 bitmask, __u32 mode);
extern int ich_gpio_intr_register_handler(int port, int bitnum,
					  void (*handler)(int irq,
							  void *dev_id,
							  struct pt_regs *regs),
					  void *dev_id);
extern int ich_gpio_intr_unregister_handler(int port, int bitnum,
					    void *dev_id);
extern int ich_gpio_intr_mask_bit(int port, int bitnum);
extern int ich_gpio_intr_unmask_bit(int port, int bitnum);
extern int ich_gpio_intr_mask(int port, __u32 bitmask);
extern int ich_gpio_intr_unmask(int port, __u32 bitmask);
extern int ich_gpio_intr_get_status_bit(int port, int bitnum);
extern int ich_gpio_intr_get_status(int port, __u32 *status);
extern int ich_gpio_intr_clear_bit(int port, int bitnum);
extern int ich_gpio_intr_clear(int port, __u32 bitmask);
#endif /* __KERNEL__ */

#endif /* __SNSC_ICH_GPIO_H__ */
