/*
 *  snsc_ite_gpio.h : ITE 8712G GPIO driver API header file
 *
 *  Copyright 2001,2002,2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __IT8172_GPIO_H
#define __IT8172_GPIO_H

#include <asm/ptrace.h>

/* port identifier */
#define IT8172_GPIO_PORT_A              0
#define IT8172_GPIO_PORT_B              1
#define IT8172_GPIO_PORT_C              2
/* bit field */
#define IT8172_GPIO_BITS_ALL            ((__u8)0xff)
/* GPIO pin mode */
#define IT8172_GPIO_MODE_FUNCTION2      0x00
#define IT8172_GPIO_MODE_OUT_HIGH       0x01
#define IT8172_GPIO_MODE_OUT_LOW        0x11
#define IT8172_GPIO_MODE_IN             0x02
/* GPIO interrupt mode */
#define IT8172_GPIO_INTR_EDGE_HIGH      1
#define IT8172_GPIO_INTR_EDGE_LOW       0

#ifdef __KERNEL__

int  it8172_gpio_register_bit(int port, int bitnum, const char *name);
int  it8172_gpio_unregister_bit(int port, int bitnum);
int  it8172_gpio_register(int port, __u8 bitmask, const char *name);
int  it8172_gpio_unregister(int port, __u8 bitmask);
int  it8172_gpio_unregister_by_name(const char *name);
int  it8172_gpio_set_mode_bit(int port, int bitnum, __u32 mode);
int  it8172_gpio_set_mode(int port, __u8 bitmask, __u32 mode);

int  it8172_gpio_get_data_bit(int port, int bitnum);
int  it8172_gpio_set_data_bit(int port, int bitnum, int value);
int  it8172_gpio_get_data(int port, __u8 bitmask, __u8 *value);
int  it8172_gpio_set_data(int port, __u8 bitmask, __u8 value);

void it8172_gpio_intr_enable(void);
void it8172_gpio_intr_disable(void);
int  it8172_gpio_intr_set_mode_bit(int port, int bitnum, __u32 mode);
int  it8172_gpio_intr_set_mode(int port, __u8 bitmask, __u32 mode);
int  it8172_gpio_intr_register_handler(int port, int bitnum,
                                          void (*handler)(int irq, void *dev_id, struct pt_regs *regs),
                                          void *dev_id);
int  it8172_gpio_intr_unregister_handler(int port, int bitnum, void *dev_id);
int  it8172_gpio_intr_mask_bit(int port, int bitnum);
int  it8172_gpio_intr_unmask_bit(int port, int bitnum);
int  it8172_gpio_intr_mask(int port, __u8 bitmask);
int  it8172_gpio_intr_unmask(int port, __u8 bitmask);
int  it8172_gpio_intr_get_status_bit(int port, int bitnum);
int  it8172_gpio_intr_get_status(int port, __u8 *status);
int  it8172_gpio_intr_clear_bit(int port, int bitnum);
int  it8172_gpio_intr_clear(int port, __u8 bitmask);

#endif /* __KERNEL__ */

#endif /* __IT8172_GPIO_H */
