/*
 *  snsc_mpu210_siogpio.h : MPU-210 SuperIO GPIO driver API header file
 *
 *  Copyright 2002,2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef _SNSC_MPU210_SIOGPIO_H
#define _SNSC_MPU210_SIOGPIO_H

/* port identifier */
#define MPU210_SIOGPIO_PORT_PIN1        0
#define MPU210_SIOGPIO_PORT_PIN2        1
#define MPU210_SIOGPIO_PORT_PIN3        2
#define MPU210_SIOGPIO_PORT_PIN4        3
#define MPU210_SIOGPIO_PORT_PIN5        4

/* bit field */
#define MPU210_SIOGPIO_BITS_ALL         ((__u8)0xff)

/* GPIO pin mode */
#define MPU210_SIOGPIO_MODE_IN          0x0001
#define MPU210_SIOGPIO_MODE_OUT_HIGH    0x0002
#define MPU210_SIOGPIO_MODE_OUT_LOW     0x0004
#define MPU210_SIOGPIO_MODE_NOINVERT    0x0008
#define MPU210_SIOGPIO_MODE_INVERT      0x0010
#define MPU210_SIOGPIO_MODE_PULLUP      0x0020
#define MPU210_SIOGPIO_MODE_NOPULLUP    0x0040

/* GPIO interrupt mode */
#define MPU210_SIOGPIO_INTR_EDGE_HIGH   1
#define MPU210_SIOGPIO_INTR_EDGE_LOW    2
#define MPU210_SIOGPIO_INTR_LEVEL_HIGH  3
#define MPU210_SIOGPIO_INTR_LEVEL_LOW   4


#ifdef __KERNEL__
/* exported functions */

/* Initialize */
int  mpu210_siogpio_register_bit(int port, int bitnum, const char *name);
int  mpu210_siogpio_unregister_bit(int port, int bitnum);
int  mpu210_siogpio_register(int port, __u8 bitmask, const char *name);
int  mpu210_siogpio_unregister(int port, __u8 bitmask);
int  mpu210_siogpio_unregister_by_name(const char *name);
int  mpu210_siogpio_set_mode_bit(int port, int bitnum, __u32 mode);
int  mpu210_siogpio_set_mode(int port, __u8 bitmask, __u32 mode);

/* Data register access */
int  mpu210_siogpio_get_data_bit(int port, int bitnum);
int  mpu210_siogpio_set_data_bit(int port, int bitnum, int value);
int  mpu210_siogpio_get_data(int port, __u8 bitmask, __u8 *value);
int  mpu210_siogpio_set_data(int port, __u8 bitmask, __u8 value);

/* Interrupt handling */
void mpu210_siogpio_intr_enable(void);
void mpu210_siogpio_intr_disable(void);
int  mpu210_siogpio_intr_set_mode_bit(int port, int bitnum, __u32 mode);
int  mpu210_siogpio_intr_set_mode(int port, __u8 bitmask, __u32 mode);
int  mpu210_siogpio_intr_register_handler(int port, int bitnum,
                                          void (*handler)(int irq, void *dev_id, struct pt_regs *regs),
                                          void *dev_id);
int  mpu210_siogpio_intr_unregister_handler(int port, int bitnum, void *dev_id);
int  mpu210_siogpio_intr_mask_bit(int port, int bitnum);
int  mpu210_siogpio_intr_unmask_bit(int port, int bitnum);
int  mpu210_siogpio_intr_mask(int port, __u8 bitmask);
int  mpu210_siogpio_intr_unmask(int port, __u8 bitmask);
int  mpu210_siogpio_intr_get_status_bit(int port, int bitnum);
int  mpu210_siogpio_intr_get_status(int port, __u8 *status);
int  mpu210_siogpio_intr_clear_bit(int port, int bitnum);
int  mpu210_siogpio_intr_clear(int port, __u8 bitmask);

#endif /* __KERNEL__ */

#endif /* _SNSC_MPU210_SIOGPIO_H */
