/*
 *  snsc_mpu30x_siogpio.h : MPU-30X SuperIO GPIO driver header file
 *
 *  Copyright 2002,2003 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef _SNSC_MPU30X_SIOGPIO_H
#define _SNSC_MPU30X_SIOGPIO_H

/* Bit Field */
#define SIOGPIO_BITS_ALL  0xffff

/* GPIO pin mode */
#define SIOGPIO_MODE_IOMASK         0x0003
#define SIOGPIO_MODE_IN             0x0000
#define SIOGPIO_MODE_OUT_HIGH       0x0001
#define SIOGPIO_MODE_OUT_LOW        0x0002

#define SIOGPIO_MODE_INVMASK        0x0004
#define SIOGPIO_MODE_NOINVERT       0x0000
#define SIOGPIO_MODE_INVERT         0x0004

#define SIOGPIO_MODE_OUTDRVMASK     0x0008
#define SIOGPIO_MODE_PUSHPULL       0x0000
#define SIOGPIO_MODE_OPENDRAIN      0x0008

/* GPIO interrupt mode */
#define SIOGPIO_INTR_EDGE_HIGH      0
#define SIOGPIO_INTR_EDGE_LOW       1
#define SIOGPIO_INTR_EDGE_BOTH      2


#ifdef __KERNEL__
/* exported functions */

/* Initialize */
int mpu30x_siogpio_register_bit(int port, int bitnum, const char *name);
int mpu30x_siogpio_unregister_bit(int port, int bitnum);
int mpu30x_siogpio_register(int port, __u16 bitmask, const char *name);
int mpu30x_siogpio_unregister(int port, __u16 bitmask);
int mpu30x_siogpio_unregister_by_name(const char *name);
int mpu30x_siogpio_set_mode_bit(int port, int bitnum, __u32 mode);
int mpu30x_siogpio_set_mode(int port, __u16 bitmask, __u32 mode);

/* Data register access */
int mpu30x_siogpio_get_data_bit(int port, int bit_num);
int mpu30x_siogpio_set_data_bit(int port, int bit_num, int value);
int mpu30x_siogpio_get_data(int port, __u16 bitmask, __u16 *value);
int mpu30x_siogpio_set_data(int port, __u16 bitmask, __u16 value);

/* Interrupt handling */
void mpu30x_siogpio_intr_enable( void );
void mpu30x_siogpio_intr_disable( void );

int mpu30x_siogpio_intr_set_mode_bit(int port, int bitnum, __u32 mode);
int mpu30x_siogpio_intr_set_mode(int port, __u16 bitmask, __u32 mode);

int mpu30x_siogpio_intr_register_handler(int port, int bitnum,
					 void (*handler)(int irq, void *dev_id,
							 struct pt_regs *regs),
					 void *dev_id);

int mpu30x_siogpio_intr_unregister_handler(int port, int bitnum, void *dev_id);
int mpu30x_siogpio_intr_unmask_bit(int port, int bit_num);
int mpu30x_siogpio_intr_mask_bit(int port, int bit_num);
int mpu30x_siogpio_intr_unmask(int port, __u16 bitmask);
int mpu30x_siogpio_intr_mask(int port, __u16 bitmask);
int mpu30x_siogpio_intr_get_status_bit(int port, int bitnum);
int mpu30x_siogpio_intr_get_status(int port, __u16 *status);
int mpu30x_siogpio_intr_clear_bit(int port, int bitnum);
int mpu30x_siogpio_intr_clear(int port, __u16 bitmask);

#endif /* __KERNEL__ */

#endif /* _SNSC_MPU30X_SIOGPIO_H */
