/*
 *  include/linux/snsc_spi.h -- Sony NSC SPI interface
 *
 *  Copyright 2004 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _LINUX_SNSC_SPI_H_
#define _LINUX_SNSC_SPI_H_

#include <linux/snsc_major.h>
#include <linux/ioctl.h>

typedef struct snscspi_msg_t {
	int	 cs;
	int	 flags;
#define SNSCSPI_M_WR	(1 << 0)
#define SNSCSPI_M_RD	(1 << 1)
#define SNSCSPI_M_CS	(1 << 2)
	size_t	 len;
	void	*buf;
} snscspi_msg_t;

typedef struct snscspi_transaction_t {
	size_t		 nmsgs;
	snscspi_msg_t	*msgs;
} snscspi_transaction_t;

#define SNSCSPI_IOC_TRANSACTION		_IOW('S', 0x00, snscspi_transaction_t)
#define SNSCSPI_IOC_SET_FREQ		_IOW('S', 0x01, u_int32_t)
#define SNSCSPI_IOC_GET_FREQ		_IOR('S', 0x02, u_int32_t)
#define SNSCSPI_IOC_SET_IFDELAY		_IOW('S', 0x03, u_int32_t)
#define SNSCSPI_IOC_GET_IFDELAY		_IOR('S', 0x04, u_int32_t)
#define SNSCSPI_IOC_SET_SIZE		_IOW('S', 0x05, u_int32_t)
#define SNSCSPI_IOC_GET_SIZE		_IOR('S', 0x06, u_int32_t)
#define SNSCSPI_IOC_SET_MODE		_IOW('S', 0x07, int)
#define SNSCSPI_IOC_GET_MODE		_IOR('S', 0x08, int)

#define SNSCSPI_M_POL_HIGH	(1 << 0)
#define SNSCSPI_M_POL_LOW	(1 << 1)
#define SNSCSPI_M_PHA_1		(1 << 2)
#define SNSCSPI_M_PHA_2		(1 << 3)
#define SNSCSPI_M_LSB		(1 << 4)
#define SNSCSPI_M_MSB		(1 << 5)

#ifdef __KERNEL__
typedef struct snscspi_bus_t {
	char *name;
	int   busno;
	int (*open)(struct snscspi_bus_t *bus);
	int (*release)(struct snscspi_bus_t *bus);
	int (*ioctl)(struct snscspi_bus_t *bus,
		     unsigned int cmd, void *arg, int argtype);
#define SNSCSPI_T_ARG_KERNEL	(1 << 0)
#define SNSCSPI_T_BUF_USER		(1 << 1)
	void *reg;
	void *priv;
} snscspi_bus_t;

int snscspi_do_ioctl(snscspi_bus_t *bus, unsigned int cmd, void *arg);
snscspi_bus_t *snscspi_get_bus(int busno);

int snscspi_register(snscspi_bus_t *bus);
int snscspi_unregister(snscspi_bus_t *bus);
#endif /* __KERNEL__*/

#endif /* !_LINUX_SNSC_SPI_H_ */
