/*
 *  Sony NSC UMA
 *
 *  Copyright 2005 Sony Corporation.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SNSC_UMA_H__
#define __SNSC_UMA_H__

typedef u_int32_t snsc_uma_t;

typedef struct  {
	u_int32_t addr;
	u_int32_t size;
	u_int32_t flags;
} snsc_uma_desc_t;

#define SNSC_UMA_FLG_OCCUPIED		(1 << 0)
#define SNSC_UMA_FLG_FREE_DIR		(1 << 1)
#define SNSC_UMA_FLG_FREE_FROM_TAIL	(0 << 1)
#define SNSC_UMA_FLG_FREE_FROM_HEAD	(1 << 1)

#define SNSC_UMA_READ			(0)
#define SNSC_UMA_WRITE			(1)

/* Arch dependent UMA defines. */
#include <asm/snsc_uma.h>

struct snsc_uma_machdep_calls {

	/* init_range()
	 *   region: uma region specifier (starting from 0)
	 *       pd: pointer for uma region calcurated initially.
	 * returns 0 for success, -1 means there is no region.
	 */
	int (*init_range)(snsc_uma_t region);
	/* get_range() 
	 *   region: uma region specifier (starting from 0)
	 *       pd: pointer for uma region.
	 * returns 0 for success, -1 means there is no region.
	 */
	int (*get_range)(snsc_uma_t region, snsc_uma_desc_t *pd);
	int (*set_range)(snsc_uma_t region, snsc_uma_desc_t pd);
	void (*sync)(void *vaddr, u_int32_t size,
		     u_int32_t direction);
};

int snsc_uma_init(void);
int snsc_uma_getsize(snsc_uma_t region, unsigned int *size);
int snsc_uma_request(snsc_uma_t region, unsigned int pages,
		     void *vaddr, dma_addr_t *paddr);
int snsc_uma_free(snsc_uma_t region);
void snsc_uma_sync(void *addr, unsigned int size, unsigned int direction);
int snsc_uma_sync_all(snsc_uma_t region);


#endif	/* __SNSC_UMA_H__ */
