/*
 * based upon linux/include/linux/msdos_fs.h
 *
 * Copyright 2003 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#ifndef _LINUX_XVFAT_FS_H
#define _LINUX_XVFAT_FS_H

#ifdef __KERNEL__
#include <linux/msdos_fs.h>
#endif

/*
 * ioctl commands
 */
#define	XVFAT_IOCTL_GETLONGNAME		_IOR('r', 3, struct xvfat_lname_box)
#define	XVFAT_IOCTL_GETMINFO		_IOR('r', 4, struct xvfat_media_info)

/* ioctl: get a long name character code */
#define XVFAT_LNAME_ERR_NONE    0
#define XVFAT_LNAME_ERR_BADCHAR 1
#define XVFAT_LNAME_ERR_TOOLONG 2	/* not enough u_name_len size */

struct xvfat_lname_box{
	int io_name_len;	/* Set a file name length		*/
   	 			/* Not include NUL terminated character	*/
	unsigned char *io_name;	/* Set a file name			*/
	int u_name_alloc_size;	/* Set a size of u_name array length	*/ 
	unsigned short *u_name;	/* Return a file name with Unicode	*/ 
				/* This is NUL terminated		*/
	int u_name_len;		/* Return a length of string in u_name	*/
   	 			/* Not include NUL terminated		*/
				/* character(U+0000)			*/
	int error;		/* Return error code			*/ 
};

/* ioctl: get a removable disk infomation */
#define XVFAT_MINFO_CFLAG_CHANGE 1
#define XVFAT_MINFO_CFLAG_INSERT 2
struct xvfat_media_info{
	int insert;             /* 0:not inserted, 1:inserted   */
	int change;
	int capability;         /* capability flags             */
	unsigned int state_change_count;   
};


#ifdef __KERNEL__

/* xvfat/buffer.c */
extern struct buffer_head *xfat_bread(struct super_block *sb, int block);
extern struct buffer_head *xfat_getblk(struct super_block *sb, int block);
extern void xfat_brelse(struct super_block *sb, struct buffer_head *bh);
extern void xfat_mark_buffer_dirty(struct super_block *sb,
				   struct buffer_head *bh);
extern void xfat_set_uptodate(struct super_block *sb, struct buffer_head *bh,
			      int val);
extern int xfat_is_uptodate(struct super_block *sb, struct buffer_head *bh);
extern void xfat_ll_rw_block(struct super_block *sb, int opr, int nbreq,
			     struct buffer_head *bh[32]);

/* xvfat/cache.c */
extern int xfat_access(struct super_block *sb, int nr, int new_value);
extern int xfat_bmap(struct inode *inode, int sector);
extern void xfat_cache_init(void);
extern void xfat_cache_lookup(struct inode *inode, int cluster, int *f_clu,
			      int *d_clu);
extern void xfat_cache_add(struct inode *inode, int f_clu, int d_clu);
extern void xfat_cache_inval_inode(struct inode *inode);
extern void xfat_cache_inval_dev(kdev_t device);
extern int xfat_get_cluster(struct inode *inode, int cluster);
extern int xfat_free(struct inode *inode, int skip);
extern int xfat_free_ts(struct inode *inode, int skip, fwrq_transaction_t *ts);
extern int xfat_access_ts(struct super_block *sb, int nr, int new_value,
			  fwrq_transaction_t *ts);

/* xvfat/dir.c */
extern struct file_operations xfat_dir_operations;
extern int xfat_search_long(struct inode *inode, const char *name,
			    int name_len, int anycase, loff_t *spos,
			    loff_t *lpos);
extern int xfat_readdir(struct file *filp, void *dirent, filldir_t filldir);
/* extern ssize_t xfat_read_dir(struct file *filp, char *buf, size_t siz, loff_t *ppos); */
extern int xfat_dir_ioctl(struct inode * inode, struct file * filp,
			  unsigned int cmd, unsigned long arg);
extern int xfat_dir_empty(struct inode *dir);
extern int xfat_add_entries_ts(struct inode *dir,int slots,
			       struct buffer_head **bh,
			       struct msdos_dir_entry **de, int *ino,
			       fwrq_transaction_t *ts);
extern int xfat_new_dir_ts(struct inode *dir, struct inode *parent,
			   int is_vfat, fwrq_transaction_t *ts);

/* xvfat/file.c */
extern struct file_operations xfat_file_operations;
extern struct inode_operations xfat_file_inode_operations;
extern ssize_t xfat_file_read(struct file *filp, char *buf, size_t count,
			      loff_t *ppos);
extern int xfat_get_block(struct inode *inode, long iblock,
			  struct buffer_head *bh_result, int create);
extern void xfat_truncate(struct inode *inode);
extern void xfat_truncate_ts(struct inode *inode, fwrq_transaction_t *ts);

/* xvfat/inode.c */
extern void xfat_hash_init(void);
extern void xfat_attach(struct inode *inode, int i_pos);
extern void xfat_detach(struct inode *inode);
extern struct inode *xfat_iget(struct super_block *sb, int i_pos);
extern struct inode *xfat_build_inode(struct super_block *sb,
				      struct msdos_dir_entry *de, int ino,
				      int *res);
extern void xfat_delete_inode(struct inode *inode);
extern void xfat_clear_inode(struct inode *inode);
extern void xfat_put_super(struct super_block *sb);
extern struct super_block *
xfat_read_super(struct super_block *sb, void *data, int silent,
		struct inode_operations *fs_dir_inode_ops);
extern int xfat_statfs(struct super_block *sb, struct statfs *buf);
extern void xfat_write_inode(struct inode *inode, int wait);
extern int xfat_notify_change(struct dentry * dentry, struct iattr * attr);
extern void xfat_set_inode_depend(struct inode *inode, void *data,
				  struct buffer_head *dep_bh, void *dep_data);
extern void xfat_unset_inode_depend(void);

/* xvfat/misc.c */
extern void xfat_fs_panic(struct super_block *s, const char *msg);
extern int xfat_is_binary(char conversion, char *extension);
extern void lock_xfat(struct super_block *sb);
extern void unlock_xfat(struct super_block *sb);
extern void xfat_clusters_flush(struct super_block *sb);
extern int xfat_add_cluster(struct inode *inode);
extern struct buffer_head *xfat_extend_dir(struct inode *inode);
extern int xfat__get_entry(struct inode *dir, loff_t *pos,
			   struct buffer_head **bh,
			   struct msdos_dir_entry **de, int *ino);
static __inline__ int xfat_get_entry(struct inode *dir, loff_t *pos,
				     struct buffer_head **bh,
				     struct msdos_dir_entry **de, int *ino)
{
	/* Fast stuff first */
	if (*bh && *de &&
	    (*de - (struct msdos_dir_entry *)(*bh)->b_data) < MSDOS_SB(dir->i_sb)->dir_per_block - 1) {
		*pos += sizeof(struct msdos_dir_entry);
		(*de)++;
		(*ino)++;
		return 0;
	}
	return xfat__get_entry(dir,pos,bh,de,ino);
}
extern int xfat_add_cluster_ts(struct inode *inode, fwrq_transaction_t *ts);
extern struct buffer_head *xfat_extend_dir_ts(struct inode *inode,
					      fwrq_transaction_t *ts,
					      int allow_reorder);
extern int xfat_subdirs(struct inode *dir);
extern int xfat_scan(struct inode *dir, const char *name,
		     struct buffer_head **res_bh,
		     struct msdos_dir_entry **res_de, int *ino);

/* xvfat/namei.c - these are for dmsdos */
extern struct dentry *xvfat_lookup(struct inode *dir, struct dentry *);
extern int xvfat_create(struct inode *dir, struct dentry *dentry, int mode);
extern int xvfat_rmdir(struct inode *dir, struct dentry *dentry);
extern int xvfat_unlink(struct inode *dir, struct dentry *dentry);
extern int xvfat_mkdir(struct inode *dir, struct dentry *dentry, int mode);
extern int xvfat_rename(struct inode *old_dir, struct dentry *old_dentry,
			struct inode *new_dir, struct dentry *new_dentry);
extern struct super_block *xvfat_read_super(struct super_block *sb, void *data,
					    int silent);
extern int xvfat_ioctl_iochar_to_uni16(struct inode *inode,
				       struct xvfat_lname_box *u_lbox);
extern int xvfat_ioctl_get_minfo(struct inode *inode,
				 struct xvfat_media_info *u_minfo);

extern int xfat_free_ts(struct inode *inode, int skip, fwrq_transaction_t *ts);
extern int xfat_access_ts(struct super_block *sb, int nr, int new_value,
			  fwrq_transaction_t *ts);
extern int xfat_add_entries_ts(struct inode *dir,int slots,
			       struct buffer_head **bh,
			       struct msdos_dir_entry **de, int *ino,
			       fwrq_transaction_t *ts);
extern int xfat_new_dir_ts(struct inode *dir, struct inode *parent,
			  int is_vfat, fwrq_transaction_t *ts);
/* extern int xfat_file_flush(struct file *filp); */
extern void xfat_truncate_ts(struct inode *inode, fwrq_transaction_t *ts);
extern void xfat_set_inode_depend(struct inode *inode, void *data,
				  struct buffer_head *dep_bh, void *dep_data);
extern void xfat_unset_inode_depend(void);
extern int xfat_add_cluster_ts(struct inode *inode, fwrq_transaction_t *ts);
extern struct buffer_head *xfat_extend_dir_ts(struct inode *inode,
					      fwrq_transaction_t *ts,
					      int allow_reorder);
extern int xvfat_ioctl_iochar_to_uni16(struct inode *inode,
				       struct xvfat_lname_box *u_lbox);
extern int xvfat_ioctl_get_minfo(struct inode *inode,
				 struct xvfat_media_info *u_minfo);

/* xvfat/tz.c */
extern int xfat_tzset(struct tz_rule *rule, char *timezone);
extern int xfat_date_dos2unix(struct tz_rule *rule, unsigned short time,
			      unsigned short date);
extern void xfat_date_unix2dos(struct tz_rule *rule, int unix_date,
			       unsigned short *time, unsigned short *date);

/* xvfat/vfatfs_syms.c */
extern struct file_system_type xvfat_fs_type;

#endif /* __KERNEL__ */

#endif /* ! _LINUX_XVFAT_FS_H */
